/**
 * Copyright © 2016-2022 The Thingsboard Authors
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.tofly.security.component.jwt.extractor;

import com.tofly.security.component.config.BaseSecurityConfig;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;

@Component(value = "jwtHeaderTokenExtractor")
public class JwtHeaderTokenExtractor implements TokenExtractor {
    public static final String HEADER_PREFIX = "Bearer ";

    @Override
    public String extract(HttpServletRequest request) {
        String token = null;
        String header = request.getHeader(BaseSecurityConfig.JWT_TOKEN_HEADER_PARAM);
        if (StringUtils.isBlank(header)) {
            header = request.getHeader(BaseSecurityConfig.JWT_TOKEN_HEADER_PARAM_V2);
            if (StringUtils.isBlank(header)) {
                token = request.getHeader("Token");
                if (StringUtils.isBlank(token)) {
                    if (request.getParameterMap() != null && !request.getParameterMap().isEmpty()) {
                        String[] tokenParamValue = request.getParameterMap().get(BaseSecurityConfig.JWT_TOKEN_QUERY_PARAM);
                        if (tokenParamValue != null && tokenParamValue.length == 1) {
                            token = tokenParamValue[0];
                        }
                    }
                }
            }
        }else{
            token = header.substring(HEADER_PREFIX.length(), header.length());
        }

        if (StringUtils.isBlank(token)) {
            throw new AuthenticationServiceException("token cannot be blank!");
        }
        return token;
    }
}
