package com.tofly.security.component.rest;

import com.tofly.security.SecurityUser;
import com.tofly.security.UserPrincipal;
import com.tofly.security.component.exception.ErrorMessage;
import com.tofly.security.component.exception.TkDataValidationException;
import com.tofly.security.component.user.User;
import com.tofly.security.id.TenantId;
import com.tofly.security.id.UserDTO;
import com.tofly.security.id.UserId;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

@Component
@Slf4j
public class RestAuthenticationProvider implements AuthenticationProvider {

//    @Autowired
//    private TkUserService tkUserService;

    @Autowired
    private PasswordEncoder passwordEncoder;

    @Autowired
    public RestAuthenticationProvider() {

    }

    @Override
    public Authentication authenticate(Authentication authentication) throws AuthenticationException  {

        Object principal = authentication.getPrincipal();
        if (!(principal instanceof UserPrincipal)) {
            throw new BadCredentialsException("Authentication Failed. Bad user principal.");
        }

        UserPrincipal userPrincipal = (UserPrincipal) principal;
        if (userPrincipal.getType() == UserPrincipal.Type.USER_NAME) {
            String principalValue = userPrincipal.getValue();
            String password = (String) authentication.getCredentials();
//            UserDTO userDTO = tkUserService.getUserByPhoneNumberOrUsername(principalValue);
            // TODO
            UserDTO userDTO = new UserDTO();

            if (userDTO == null) {            	
            	throw new TkDataValidationException(ErrorMessage.USERNAME_PASSWORD_INCORRECT.getMessage());
            } else {
                if (!passwordEncoder.matches(password, userDTO.getPassword())) {
                	throw new TkDataValidationException(ErrorMessage.USERNAME_PASSWORD_INCORRECT.getMessage());
                }
            }
            
            if(userDTO.isEnabled()==false) {
            	throw new TkDataValidationException(ErrorMessage.ACCOUNT_DISABLED.getMessage());
            }
            
            if(null!=userDTO.getAccountExpireTime()) {
            	 if(userDTO.getAccountExpireTime().before(new Date())){
                 	throw new TkDataValidationException(ErrorMessage.ACCOUNT_HAS_EXPIRED.getMessage());
                 }
            }
            
            if(userDTO.getIsFirst()==0) {
            	throw new TkDataValidationException(ErrorMessage.ACCOUNT_IS_NOT_ACTIVE.getMessage());
            }
            
            User user = new User(UserId.fromString(userDTO.getId()));
            SecurityUser securityUser =
                    new SecurityUser(user, true, userPrincipal);
//            Set<String> roles = tkUserService.getUserRole(userDTO.getId());
            // TODO
            Set<String> roles = new HashSet<>();
                    securityUser.setRoles(roles);
            securityUser.setTenantId(TenantId.fromString(userDTO.getTenantId()));
            return new UsernamePasswordAuthenticationToken(
                    securityUser, null, securityUser.getAuthorities());
        }
        return null;
    }


    @Override
    public boolean supports(Class<?> authentication) {
        return (UsernamePasswordAuthenticationToken.class.isAssignableFrom(authentication));
    }
}