package com.tofly.security.component.rest;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.tofly.security.SecurityUser;
import com.tofly.security.component.jwt.RefreshTokenRepository;
import com.tofly.security.component.token.JwtTokenFactory;
import com.tofly.security.model.JwtToken;
import com.tofly.security.utils.Date8Util;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.WebAttributes;
import org.springframework.security.web.authentication.AuthenticationSuccessHandler;
import org.springframework.stereotype.Component;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component(value = "defaultAuthenticationSuccessHandler")
public class RestAwareAuthenticationSuccessHandler implements AuthenticationSuccessHandler {

    private final ObjectMapper mapper;
    private final JwtTokenFactory tokenFactory;
    private final RefreshTokenRepository refreshTokenRepository;
    private final RedisTemplate redisTemplate;
//    private final RolePermissionService rolePermissionService;
    

    @Autowired
    public RestAwareAuthenticationSuccessHandler(final ObjectMapper mapper, final JwtTokenFactory tokenFactory, final RefreshTokenRepository refreshTokenRepository, RedisTemplate redisTemplate/*, RolePermissionService rolePermissionService*/) {
        this.mapper = mapper;
        this.tokenFactory = tokenFactory;
        this.refreshTokenRepository = refreshTokenRepository;
        this.redisTemplate = redisTemplate;
//        this.rolePermissionService = rolePermissionService;
    }

    @Override
    public void onAuthenticationSuccess(HttpServletRequest request, HttpServletResponse response,
                                        Authentication authentication) throws IOException, ServletException {
        SecurityUser securityUser = (SecurityUser) authentication.getPrincipal();

        JwtToken accessToken = tokenFactory.createAccessJwtToken(securityUser);
        JwtToken refreshToken = refreshTokenRepository.requestRefreshToken(securityUser);

        Map<String, String> tokenMap = new HashMap<String, String>();
        String tokenValue = accessToken.getToken();
        tokenMap.put("token", tokenValue);
        tokenMap.put("refreshToken", refreshToken.getToken());

        response.setStatus(HttpStatus.OK.value());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        mapper.writeValue(response.getWriter(), tokenMap);
        ValueOperations<String, String> valueOperations = redisTemplate.opsForValue();

        //String o = valueOperations.get(SecurityConstants.REDIS_LOGIN_LOCK);
        //if (!StringUtil.isEmpty(o)) {
            //valueOperations.getOperations().delete(SecurityConstants.REDIS_LOGIN_LOCK);
        valueOperations.set("token_"+securityUser.getId(), tokenValue);
        valueOperations.set("refreshToken_"+securityUser.getId(), refreshToken.getToken());
        redisTemplate.expireAt(tokenValue, Date8Util.getEndTime());
        //}
        
        
        //存放用户数据权限
//        List<Map<String,String>> permissionList = rolePermissionService.getRolePermissionByUserId(securityUser.getId().toString());
        // TODO
        List<Map<String,String>> permissionList = new ArrayList<>();
        if(permissionList.size()!=0) {
        	Map<String,String> permissionMap =new HashMap<String,String>();
        	for(int i=0;i<permissionList.size();i++) {    
        		permissionMap.put(permissionList.get(i).get("permission_type"), permissionList.get(i).get("data_id"));
        	}
        	String s = JSONObject.toJSONString(permissionMap);   
        	valueOperations.set("dataPermission_"+securityUser.getId(),s);
        }
            

        clearAuthenticationAttributes(request);
    }

    /**
     * Removes temporary authentication-related data which may have been stored
     * in the session during the authentication process..
     */
    protected final void clearAuthenticationAttributes(HttpServletRequest request) {
        HttpSession session = request.getSession(false);

        if (session == null) {
            return;
        }

        session.removeAttribute(WebAttributes.AUTHENTICATION_EXCEPTION);
    }
}
