package com.tofly.security.component.user;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.JsonNode;
import com.tofly.security.Authority;
import com.tofly.security.component.vaildation.Length;
import com.tofly.security.component.vaildation.NoXss;
import com.tofly.security.id.*;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;


@EqualsAndHashCode(callSuper = true)
public class User extends SearchTextBasedWithAdditionalInfo<UserId>
    implements HasName, HasTenantId, HasCustomerId {

  private static final long serialVersionUID = 8250339805336035966L;

  private TenantId tenantId;
  private CustomerId customerId;
  private String email;
  private Authority authority;

  @NoXss
  @Length(fieldName = "first name")
  private String firstName;

  @NoXss
  @Length(fieldName = "last name")
  private String lastName;
  /** 平台的用户详情 */
  @Getter
  @Setter
  private UserDetailsDTO userDetailsDTO;

  public User() {
    super();
  }

  public User(UserId id) {
    super(id);
  }

  public User(User user) {
    super(user);
    this.tenantId = user.getTenantId();
    this.customerId = user.getCustomerId();
    this.email = user.getEmail();
    this.authority = user.getAuthority();
    this.firstName = user.getFirstName();
    this.lastName = user.getLastName();
  }

  @Override
  public UserId getId() {
    return super.getId();
  }

  @Override
  public long getCreatedTime() {
    return super.getCreatedTime();
  }

  public TenantId getTenantId() {
    return tenantId;
  }

  public void setTenantId(TenantId tenantId) {
    this.tenantId = tenantId;
  }

  public CustomerId getCustomerId() {
    return customerId;
  }

  public void setCustomerId(CustomerId customerId) {
    this.customerId = customerId;
  }

  public String getEmail() {
    return email;
  }

  public void setEmail(String email) {
    this.email = email;
  }

  @Override
  @JsonProperty(access = JsonProperty.Access.READ_ONLY)
  public String getName() {
    return email;
  }

  public Authority getAuthority() {
    return authority;
  }

  public void setAuthority(Authority authority) {
    this.authority = authority;
  }

  public String getFirstName() {
    return firstName;
  }

  public void setFirstName(String firstName) {
    this.firstName = firstName;
  }

  public String getLastName() {
    return lastName;
  }

  public void setLastName(String lastName) {
    this.lastName = lastName;
  }

  @Override
  public JsonNode getAdditionalInfo() {
    return super.getAdditionalInfo();
  }

  @Override
  public String getSearchText() {
    return getEmail();
  }

  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder();
    builder.append("User [tenantId=");
    builder.append(tenantId);
    builder.append(", customerId=");
    builder.append(customerId);
    builder.append(", email=");
    builder.append(email);
    builder.append(", authority=");
    builder.append(authority);
    builder.append(", firstName=");
    builder.append(firstName);
    builder.append(", lastName=");
    builder.append(lastName);
    builder.append(", additionalInfo=");
    builder.append(getAdditionalInfo());
    builder.append(", createdTime=");
    builder.append(createdTime);
    builder.append(", id=");
    builder.append(id);
    builder.append("]");
    return builder.toString();
  }

  @JsonIgnore
  public boolean isSystemAdmin() {
    return tenantId == null || EntityId.NULL_UUID.equals(tenantId.getId());
  }

  @JsonIgnore
  public boolean isTenantAdmin() {
    return !isSystemAdmin()
            && (customerId == null || EntityId.NULL_UUID.equals(customerId.getId()));
  }

  @JsonIgnore
  public boolean isCustomerUser() {
    return !isSystemAdmin() && !isTenantAdmin();
  }
}
