package com.tofly.security.id;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.DateSerializer;
import com.tofly.security.component.exception.EntityCreationException;
import com.tofly.security.constant.ModelConstants;
import com.tofly.security.utils.ReflectUtils;
import lombok.Data;
import org.springframework.beans.BeanUtils;

import javax.validation.constraints.NotEmpty;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;

@Data
@JsonInclude(JsonInclude.Include.NON_NULL)
public class BaseDTO {
  private String id;
  /**创建用户*/
  private String creator;
  /**创建时间*/
  @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
  @JsonSerialize(using = DateSerializer.class)
  private Date createTime;
  /**更新用户*/
  private String updater;
  /**更新时间*/
  @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
  @JsonSerialize(using = DateSerializer.class)
  private Date updateTime;
  @JsonIgnore
  public <E> void copyToEntity(E entity, String... ignoreProperties) {
    if (ignoreProperties.length == 0) {
      BeanUtils.copyProperties(
          this,
          entity,
          ModelConstants.TablePropertyMapping.CREATE_TIME,
          ModelConstants.TablePropertyMapping.UPDATE_TIME,
          ModelConstants.TablePropertyMapping.CREATOR,
          ModelConstants.TablePropertyMapping.UPDATER);
    } else {
      BeanUtils.copyProperties(this, entity, ignoreProperties);
    }
  }

  @JsonIgnore
  @SuppressWarnings("unchecked")
  public <E> E getEntity(Class<E> clazz) {
    Constructor<?> constructor = ReflectUtils.getEmptyParameterConstructor(clazz);
    if (constructor != null) {
      try {
        E e = (E) constructor.newInstance();
        BeanUtils.copyProperties(this, e);
        return e;
      } catch (InstantiationException | IllegalAccessException | InvocationTargetException e) {
        throw new EntityCreationException(
            "try to get entity from class [" + clazz.getName() + "] failed " + e.getMessage());
      }
    } else {
      throw new EntityCreationException(
          "try to get entity from class [" + clazz.getName() + "] failed");
    }
  }
}
