package com.tofly.security.id;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.tofly.security.component.exception.DTOCreationException;
import com.tofly.security.utils.ReflectUtils;
import lombok.Getter;
import lombok.Setter;
import org.springframework.beans.BeanUtils;

import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

@Getter
@Setter
public class BaseEntity implements Serializable {
  private static final long serialVersionUID = 8948437944054606982L;

  private String id;

  @JsonIgnore
  public <T> void copyToDTO(T dto, String... ignoreProperties) {
    BeanUtils.copyProperties(this, dto, ignoreProperties);
  }

  @JsonIgnore
  @SuppressWarnings("unchecked")
  public <T> T getDTO(Class<T> clazz) {
    Constructor<?> constructor = ReflectUtils.getEmptyParameterConstructor(clazz);
    if (constructor != null) {
      try {
        T t = (T) constructor.newInstance();
        BeanUtils.copyProperties(this, t);
        return t;
      } catch (InstantiationException | IllegalAccessException | InvocationTargetException e) {
        throw new DTOCreationException(
            "try to get dto from class [" + clazz.getName() + "] failed " + e.getMessage());
      }
    } else {
      throw new DTOCreationException("try to get dto from class [" + clazz.getName() + "] failed");
    }
  }
}
