package com.tofly.base.controller.admin;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.tofly.base.config.SysAdminConstant;
import com.tofly.base.controller.BaseController;
import com.tofly.base.entity.menu.ColumnAuthQuery;
import com.tofly.base.entity.menu.ColumnAuthVO;
import com.tofly.base.entity.user.SysUserRoleEntity;
import com.tofly.base.mapper.user.UserRoleMapper;
import com.tofly.base.server.common.data.dto.MenuDTO;
import com.tofly.base.server.common.data.security.SecurityUser;
import com.tofly.base.service.menu.MenuService;
import com.tofly.base.service.role.RoleMenuColumnService;
import com.tofly.base.service.user.TkUserService;
import com.tofly.base.utils.UserTicketUtil;
import io.swagger.annotations.Api;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
@RequestMapping("api/yt/admin")
@Api(tags = {"租户管理", "菜单管理"})
@RequiredArgsConstructor
@PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{})")
public class TkAdminController extends BaseController {

    private final MenuService menuService;

    private final RoleMenuColumnService roleMenuColumnService;

    private final TkUserService tkUserService;
    private final UserRoleMapper userRoleMapper;

    @GetMapping("/me/menus")
    public ResponseEntity<List<MenuDTO>> getMyMenus(
            @RequestParam(value = "needButton", required = false, defaultValue = "true") boolean needButton,
            @RequestParam(value = "needField", required = false, defaultValue = "true") boolean needField,
            @RequestParam(value = "roleId", required = false) String roleId
    ) {

        List<MenuDTO> list = null;
        try {

            SecurityUser user = getCurrentUser();
            if (user.getPlatformUserName().equals("sysadmin")) {
                list = menuService.getMyMenus(null,
                        null,
                        true,
                        false,
                        needButton,
                        needField);
            } else {
                list = menuService.getMyMenus(null,
                        user.getId().toString(),
                        false,
                        false,
                        needButton,
                        needField);
            }
            recursion(list, roleId);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResponseEntity.ok(list);
    }

    private void recursion(List<MenuDTO> myMenus, String roleId) {
        List<String> roleIds = new ArrayList<>();
        List<Map<String, String>> tempList = null;
        List<Map<String, String>> visibleTempList = null;
        Map<String, String> tempMap = null;
        Map<String, String> visibleTempMap = null;
        ColumnAuthQuery columnAuthQuery = null;
        roleIds.add(roleId);
        for (MenuDTO myMenu : myMenus) {
            tempList = new ArrayList<>();
            visibleTempList = new ArrayList<>();
            String menuType = myMenu.getMeta().get("menuType").asText();
            if ("3".equals(menuType)) {
                if (SysAdminConstant.SYS_ADMIN.equals(UserTicketUtil.getCurrentUser().getCurrentUserId()) || SysAdminConstant.SYS_ROLE.equals(roleId)) {
                    columnAuthQuery = new ColumnAuthQuery();
                    columnAuthQuery.setMenuId(myMenu.getId());
                    columnAuthQuery.setRoleIds(roleIds);
                    columnAuthQuery.setCreator(SysAdminConstant.SYS_ADMIN);
                    List<ColumnAuthVO> list1 = roleMenuColumnService.getColumnsByMenuAndRoleAdmin(columnAuthQuery);
                    for (ColumnAuthVO config : list1) {
                        tempMap = new HashMap<>();
                        visibleTempMap = new HashMap<>();
                        tempMap.put("propField", config.getPropField());
                        tempMap.put("propName", config.getPropName());
                        tempMap.put("id", config.getId() == null ? "" : config.getId().toString());
                        tempMap.put("configId", config.getConfigId().toString());
                        tempMap.put("menuId", config.getMenuId());
                        tempMap.put("isVisible", StringUtils.isNotEmpty(config.getIsVisible()) ? config.getIsVisible() : "0");
                        tempMap.put("roleId", config.getRoleId());
                        tempList.add(tempMap);
                        visibleTempMap.put("propField", config.getPropField());
                        visibleTempMap.put("propName", config.getPropName());
                        visibleTempMap.put("menuId", config.getMenuId());
                        visibleTempMap.put("isVisible", "1");
                        visibleTempMap.put("configId", config.getConfigId().toString());
                        visibleTempList.add(visibleTempMap);
                    }
                    myMenu.setFieldMap(tempList);
                    myMenu.setVisibleFieldMap(visibleTempList);
                } else {
                    columnAuthQuery = new ColumnAuthQuery();
                    columnAuthQuery.setMenuId(myMenu.getId());
                    List<SysUserRoleEntity> sysUserRoleEntities = userRoleMapper.selectList(new LambdaQueryWrapper<SysUserRoleEntity>().eq(SysUserRoleEntity::getUserId, UserTicketUtil.getCurrentUser().getCurrentUserId()));
                    List<String> tempRoleIds = sysUserRoleEntities.stream().map(item -> item.getRoleId()).collect(Collectors.toList());
                    if (StringUtils.isNotEmpty(roleId)) {
                        columnAuthQuery.setRoleIds(roleIds);
                        columnAuthQuery.setCreator(UserTicketUtil.getCurrentUser().getCurrentUserId());
                    } else {
                        columnAuthQuery.setRoleIds(tempRoleIds);
                        columnAuthQuery.setCreator(tkUserService.findUserInfoById(UserTicketUtil.getCurrentUser().getCurrentUserId()).getCreator());
                    }

                    List<ColumnAuthVO> list = roleMenuColumnService.getColumnsByMenuAndRole(columnAuthQuery);

                    for (ColumnAuthVO config : list) {
                        tempMap = new HashMap<>();
                        tempMap.put("propField", config.getPropField());
                        tempMap.put("propName", config.getPropName());
                        tempMap.put("id", config.getId() == null ? "" : config.getId().toString());
                        tempMap.put("menuId", config.getMenuId());
                        tempMap.put("isVisible", config.getIsVisible());
                        tempMap.put("roleId", config.getRoleId());
                        tempMap.put("configId", config.getConfigId().toString());
                        tempList.add(tempMap);
                    }

                    columnAuthQuery.setRoleIds(tempRoleIds);
                    columnAuthQuery.setCreator(tkUserService.findUserInfoById(UserTicketUtil.getCurrentUser().getCurrentUserId()).getCreator());
                    List<ColumnAuthVO> visibleList = roleMenuColumnService.getColumnsByMenuAndRoleVisible(columnAuthQuery);
                    for (ColumnAuthVO config : visibleList) {
                        visibleTempMap = new HashMap<>();
                        visibleTempMap.put("propField", config.getPropField());
                        visibleTempMap.put("propName", config.getPropName());
                        visibleTempMap.put("menuId", config.getMenuId());
                        visibleTempMap.put("isVisible", config.getIsVisible());
                        visibleTempMap.put("configId", config.getConfigId().toString());
                        visibleTempList.add(visibleTempMap);
                    }
                    myMenu.setFieldMap(tempList);
                    myMenu.setVisibleFieldMap(visibleTempList);
                }
            } else {
                if (!CollectionUtils.isEmpty(myMenu.getChildren())) {
                    recursion(myMenu.getChildren(), roleId);
                }
            }
        }
    }
}
