package com.tofly.base.controller.appdesign;

import com.tofly.base.server.common.data.core.exception.ThingsboardErrorCode;
import com.tofly.base.server.common.data.core.exception.ThingsboardException;
import com.tofly.base.server.common.data.dto.SysAppDesignDTO;
import com.tofly.base.server.common.data.id.EntityId;
import com.tofly.base.server.common.data.security.SecurityUser;
import com.tofly.base.server.common.data.utils.tools.ResponseResult;
import com.tofly.base.service.appdesign.SysAppDesignService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

/**
 * @author: 徐浩然
 * @desc: APP定制
 * @date: 2021/12/3-18:45
 */
@Api(tags = "APP定制")
@RestController
@RequestMapping("api/yt/app_design")
@RequiredArgsConstructor
public class TkAppDesignController {

  private final SysAppDesignService sysAppDesignService;

  @GetMapping("get")
  @ApiOperation("查询详情")
  public ResponseEntity<SysAppDesignDTO> get() throws ThingsboardException {
    String tenantId = EntityId.NULL_UUID.toString();
    try {
      tenantId = getCurrentUser().getCurrentTenantId();
    } catch (ThingsboardException e) {
    }
    return ResponseEntity.ok(sysAppDesignService.get(tenantId));
  }

  @PutMapping("update")
  @ApiOperation("更新")
  @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:appDesign:update:update'})")
  public ResponseEntity update(@RequestBody SysAppDesignDTO sysAppDesignDTO)
      throws ThingsboardException {
    sysAppDesignDTO.setId(null);
    return sysAppDesignService.update(sysAppDesignDTO, getCurrentUser().getCurrentTenantId())
        ? ResponseEntity.ok().build()
        : ResponseEntity.badRequest().build();
  }

  @PostMapping("data_reset")
  @PreAuthorize(
          "@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:appDesign:data_reset:reset'})")
  @ApiOperation("恢复出厂设置")
  public ResponseResult<Boolean> dataReset()
          throws ThingsboardException {
    return ResponseResult.success(
            sysAppDesignService.factoryDataReset(getCurrentUser().getCurrentTenantId()));
  }

  protected SecurityUser getCurrentUser() throws ThingsboardException {
    Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
    if (authentication != null && authentication.getPrincipal() instanceof SecurityUser) {
      return (SecurityUser) authentication.getPrincipal();
    } else {
      throw new ThingsboardException("你没有权限执行改操作!", ThingsboardErrorCode.AUTHENTICATION);
    }
  }
}
