package com.tofly.base.controller.dict;

import com.tofly.base.controller.BaseController;
import com.tofly.base.server.common.data.common.AddGroup;
import com.tofly.base.server.common.data.common.DeleteGroup;
import com.tofly.base.server.common.data.common.UpdateGroup;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.core.exception.ThingsboardException;
import com.tofly.base.server.common.data.dto.DeleteDTO;
import com.tofly.base.server.common.data.dto.SysDictDTO;
import com.tofly.base.server.common.data.enums.OrderTypeEnum;
import com.tofly.base.server.common.data.utils.tools.ResponseResult;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.service.dic.SysDictService;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;

import static com.tofly.base.server.common.data.constant.QueryConstant.*;

@RestController
@RequestMapping("api/yt/dict")
@RequiredArgsConstructor
@PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{})")
public class TkDictController extends BaseController {
    private final SysDictService sysDictService;

    @GetMapping(params = {PAGE_SIZE, PAGE})
    public TkPageData<SysDictDTO> pageSysDict(
            @RequestParam(PAGE_SIZE) int pageSize,
            @RequestParam(PAGE) int page,
            @RequestParam(value = "dictName", required = false) String dictName,
            @RequestParam(value = "dictCode", required = false) String dictCode,
            @RequestParam(value = ORDER_FILED, required = false) String orderBy,
            @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType) {

        HashMap<String, Object> queryMap = new HashMap<>();
        queryMap.put(PAGE_SIZE, pageSize);
        queryMap.put(PAGE, page);
        queryMap.put(ORDER_FILED, orderBy);
        queryMap.put("dictName", dictName);
        queryMap.put("dictCode", dictCode);
        if (orderType != null) {
            queryMap.put(ORDER_TYPE, orderType.name());
        }
        return sysDictService.page(queryMap);
    }

    @PostMapping
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:dict:post'})")
    public ResponseResult<SysDictDTO> saveSysDict(
            HttpServletRequest servletRequest,
         @RequestBody   @Validated({AddGroup.class}) SysDictDTO sysDictDTO) throws ThingsboardException {
        SysDictDTO newDTO =
                sysDictService.saveSysDict(servletRequest, sysDictDTO, getCurrentUser().getCurrentTenantId());
        return ResponseResult.success(newDTO);
    }

    @DeleteMapping
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:dict:delete'})")
    public ResponseResult<String> deleteSysDict(
            @Validated({DeleteGroup.class}) @RequestBody DeleteDTO deleteDTO) {

        return sysDictService.deleteSysDict(deleteDTO.getIds())
                ? ResponseResult.success(FastIotConstants.StateValue.DELETE_SUCCESS)
                : ResponseResult.failed(FastIotConstants.StateValue.DELETE_FAILED);
    }

    @PutMapping
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:dict:update'})")
    public ResponseResult<SysDictDTO> updateSysDict(HttpServletRequest servletRequest,
                                                    @Validated({UpdateGroup.class}) @RequestBody SysDictDTO SysDictDTO)
            throws ThingsboardException {
        return ResponseResult.success(
                sysDictService.updateSysDict(servletRequest, SysDictDTO, getCurrentUser().getCurrentTenantId()));
    }
}
