package com.tofly.base.controller.dict;

import com.tofly.base.controller.BaseController;
import com.tofly.base.server.common.data.common.DeleteGroup;
import com.tofly.base.server.common.data.common.UpdateGroup;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.core.exception.ThingsboardException;
import com.tofly.base.server.common.data.dto.DeleteDTO;
import com.tofly.base.server.common.data.dto.SysDictItemDTO;
import com.tofly.base.server.common.data.enums.OrderTypeEnum;
import com.tofly.base.server.common.data.utils.tools.ResponseResult;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.service.dic.SysDictItemService;
import com.tofly.base.service.dic.SysDictService;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static com.tofly.base.server.common.data.constant.QueryConstant.*;


@RestController
@RequestMapping("api/yt/dict_item")
@RequiredArgsConstructor
public class TkDictItemController extends BaseController {
    private final SysDictItemService sysDictItemService;
    private final SysDictService sysDictService;
    private  final  RedisTemplate redisTemplate;
    private static  final String dict_prefix="dict_prefix:";
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{})")
    @GetMapping(params = {PAGE_SIZE, PAGE})
    public TkPageData<SysDictItemDTO> pageSysDictItem(
            @RequestParam(PAGE_SIZE) int pageSize,
            @RequestParam(PAGE) int page,
            @RequestParam(value = "dictId", required = false) String dictId,
            @RequestParam(value = "itemText", required = false) String itemText,
            @RequestParam(value = "dictCode", required = false) String dictCode,
            @RequestParam(value = "itemValue", required = false) String itemValue,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = ORDER_FILED, required = false) String orderBy,
            @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType) {

        HashMap<String, Object> queryMap = new HashMap<>();
        queryMap.put(PAGE_SIZE, pageSize);
        queryMap.put(PAGE, page);
        queryMap.put(ORDER_FILED, orderBy);
        queryMap.put("itemText", itemText);
        queryMap.put("status", status);
        queryMap.put("dictId", dictId);
        if (orderType != null) {
            queryMap.put(ORDER_TYPE, orderType.name());
        }
        return sysDictItemService.page(queryMap);
    }
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{})")
    @GetMapping("itemInfo")
    public SysDictItemDTO itemInfo(

            @RequestParam(value = "dictCode", required = true) String dictCode,
            @RequestParam(value = "itemValue", required = true) String itemValue
       ) {

        //获取redis
        //从redis中获取该code的对应的值
        final Object o = redisTemplate.opsForHash().get(dict_prefix+dictCode, itemValue);
        if(o==null){
            //通过dictCode 查询id
            final List<SysDictItemDTO> dictItemByDictCode
                    = sysDictItemService.findDictItemByDictCode(dictCode);
            for (SysDictItemDTO sysDictItemDTO : dictItemByDictCode) {
                if(sysDictItemDTO.getItemValue().equals(itemValue)){
                    redisTemplate.opsForHash().put(dict_prefix+dictCode,itemValue,sysDictItemDTO);

                    return  sysDictItemDTO;
                }
            }
        }else{
            return  (SysDictItemDTO)o;
        }
        return  null;
    }

    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{})")
    @GetMapping("getDicItemText")
    public String getDicItemText(
            @RequestParam(value = "dictCode", required = true) String dictCode,
            @RequestParam(value = "itemValue", required = true) String itemValue
    ) {
        //通过dictCode 查询id
        final List<SysDictItemDTO> dictItemByDictCode
                = sysDictItemService.findDictItemByDictCode(dictCode);
        for (SysDictItemDTO sysDictItemDTO : dictItemByDictCode) {
            if(sysDictItemDTO.getItemValue().equals(itemValue)){
                return  sysDictItemDTO.getItemText();
            }
        }
        return null;
    }
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{})")
    @GetMapping("getDicItemValue")
    public String getDicItemValue(
            @RequestParam("itemText") String itemText, @RequestParam("dictCode") String dictCode
    ) {
        //通过dictCode 查询id
        final List<SysDictItemDTO> dictItemByDictCode
                = sysDictItemService.findDictItemByDictCode(dictCode);
        for (SysDictItemDTO sysDictItemDTO : dictItemByDictCode) {
            if(sysDictItemDTO.getItemText().equals(itemText)){
                return  sysDictItemDTO.getItemValue();
            }
        }
        return null;
    }
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{})")
    @GetMapping("itemInfoByName")
    public SysDictItemDTO itemInfoByName(

            @RequestParam(value = "dictCode", required = true) String dictCode,
            @RequestParam(value = "itemName", required = true) String itemName
    ) {
        //通过dictCode 查询id
        final List<SysDictItemDTO> dictItemByDictCode
                = sysDictItemService.findDictItemByDictCode(dictCode);
        for (SysDictItemDTO sysDictItemDTO : dictItemByDictCode) {
            if(sysDictItemDTO.getItemText().equals(itemName)){
                return  sysDictItemDTO;
            }
        }
        return null;
    }
    @PostMapping
    @ApiOperation("字典值新增")
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:dictItem:post'})")
    public ResponseResult<SysDictItemDTO> saveSysDictItem(
            HttpServletRequest servletRequest,
           @RequestBody SysDictItemDTO sysDictItemDTO)
            throws ThingsboardException {
        removeRedisKey();


        SysDictItemDTO newDTO =
                sysDictItemService.saveSysDictItem(servletRequest, sysDictItemDTO,null );
        return ResponseResult.success(newDTO);
    }

    @DeleteMapping
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:dictItem:delete'})")
    public ResponseResult<String> deleteSysDictItem(
            @Validated({DeleteGroup.class}) @RequestBody DeleteDTO deleteDTO) {
        removeRedisKey();

        return sysDictItemService.deleteSysDictItem(deleteDTO.getIds())
                ? ResponseResult.success(FastIotConstants.StateValue.DELETE_SUCCESS)
                : ResponseResult.failed(FastIotConstants.StateValue.DELETE_FAILED);
    }

    @PutMapping
    @ApiOperation("字典值更新")
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:dictItem:update'})")
    public ResponseResult<SysDictItemDTO> updateSysDictItem(
            HttpServletRequest servletRequest,
             @RequestBody SysDictItemDTO SysDictItemDTO) {

        removeRedisKey();

        return ResponseResult.success(
                sysDictItemService.updateSysDictItem(
                        servletRequest, SysDictItemDTO, null));
    }

    @PostMapping("/find")
    public ResponseEntity<List<SysDictItemDTO>> findDictItemByDictCode(
            @RequestBody Map<String, String> dictCode) {
        return ResponseEntity.ok(sysDictItemService.findDictItemByDictCode(dictCode.get("dictCode")));
    }

    public void removeRedisKey(){
        Set<String> keys = redisTemplate.keys(dict_prefix + "*");
        if(keys!=null &&keys.size()>0){
            redisTemplate.delete(keys);
        }

     /*   if(keys!=null && keys.size()>0){
            for (String key : keys) {

            }
        }*/

    }
}
