package com.tofly.base.controller.enterprise;


import com.tofly.base.server.common.data.core.exception.ThingsboardErrorCode;
import com.tofly.base.server.common.data.core.exception.ThingsboardException;
import com.tofly.base.server.common.data.dto.SysEnterpriseDTO;
import com.tofly.base.server.common.data.security.SecurityUser;
import com.tofly.base.service.enterprise.SysEnterpriseService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

/**
 * @author: 徐浩然
 * @desc: 企业信息
 * @date: 2021/12/3-18:45
 */
@Api(tags = "企业信息")
@RestController
@RequestMapping("api/yt/enterprise")
@RequiredArgsConstructor
public class TkEnterpriseController{

  private final SysEnterpriseService sysEnterpriseService;

  @GetMapping("get")
  @ApiOperation("查询详情")
  public ResponseEntity<SysEnterpriseDTO> get() throws ThingsboardException {
    return ResponseEntity.ok(sysEnterpriseService.get(getCurrentUser().getCurrentTenantId()));
  }

  @PutMapping("update")
  @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','TENANT_ADMIN'},{'api:yt:enterprise:update:update'})")
  @ApiOperation("更新")
  public ResponseEntity<Boolean> update(@RequestBody SysEnterpriseDTO sysEnterpriseDTO)
      throws ThingsboardException {
    return ResponseEntity.ok(
        sysEnterpriseService.update(sysEnterpriseDTO, getCurrentUser().getCurrentTenantId()));
  }

  protected SecurityUser getCurrentUser() throws ThingsboardException {
    Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
    if (authentication != null && authentication.getPrincipal() instanceof SecurityUser) {
      return (SecurityUser) authentication.getPrincipal();
    } else {
      throw new ThingsboardException("你没有权限执行改操作!", ThingsboardErrorCode.AUTHENTICATION);
    }
  }
}
