package com.tofly.base.controller.log;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.tofly.base.entity.log.SysLogEntity;
import com.tofly.base.server.common.data.EntityType;
import com.tofly.base.server.common.data.constant.ModelConstants;
import com.tofly.base.server.common.data.core.exception.ThingsboardErrorCode;
import com.tofly.base.server.common.data.core.exception.ThingsboardException;
import com.tofly.base.server.common.data.dto.SysLogDTO;
import com.tofly.base.server.common.data.enums.ActionType;
import com.tofly.base.server.common.data.enums.OrderTypeEnum;
import com.tofly.base.server.common.data.security.SecurityUser;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.service.log.TkSysLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import static com.tofly.base.server.common.data.constant.QueryConstant.*;

@RestController
@RequestMapping("api/yt/log")
@Api(tags = {"日志管理"})
@RequiredArgsConstructor
public class TkSyslogController {

    private final TkSysLogService logService;

    @GetMapping("{entityId}")
    @PreAuthorize("@check.checkPermissions({},{})")
    @ApiOperation("详情")
    public SysLogDTO detail(@PathVariable("entityId") String entityId)
            throws ThingsboardException {
        return logService.detail(entityId);
    }

    @GetMapping(value = "exception", params = {PAGE_SIZE, PAGE})
    @ApiOperation("异常日志列表")
    @PreAuthorize("@check.checkPermissions({},{})")
    public TkPageData<SysLogDTO> pageExceptions(
            @RequestParam(PAGE_SIZE) int pageSize,
            @RequestParam(PAGE) int page,
            @RequestParam(value = "startTime", required = false) Long startTime,
            @RequestParam(value = "endTime", required = false) Long endTime,
            @RequestParam(value = "actionType", required = false) ActionType actionType,
            @RequestParam(value = ORDER_FILED, required = false) String orderBy,
            @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType)
            throws ThingsboardException {


        if (StringUtils.isEmpty(orderBy)) {
            orderBy = ModelConstants.CREATED_TIME_PROPERTY;
        }
        IPage<SysLogEntity> pageInfrom = logService.getPage(page, pageSize, orderBy, orderType);
        return logService.exceptionPage(pageInfrom, startTime, endTime
                , getCurrentUser().isPtTenantAdmin() ? getCurrentUser().getCurrentTenantId() : null
                , getCurrentUser().isCustomerUser() ? getCurrentUser().getCurrentUserId() : null
                , actionType);
    }

    @GetMapping(value = "operate", params = {PAGE_SIZE, PAGE})
    @ApiOperation("操作日志列表")
    @PreAuthorize("@check.checkPermissions({},{})")
    public TkPageData<SysLogDTO> pageOperate(
            @RequestParam(PAGE_SIZE) int pageSize,
            @RequestParam(PAGE) int page,
            @RequestParam(value = "startTime", required = false) Long startTime,
            @RequestParam(value = "endTime", required = false) Long endTime,
            @RequestParam(value = "entityType", required = false) EntityType entityType,
            @RequestParam(value = "actionType", required = false) ActionType actionType,
            @RequestParam(value = ORDER_FILED, required = false) String orderBy,
            @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType)
            throws ThingsboardException {
        if (StringUtils.isEmpty(orderBy)) {
            orderBy = ModelConstants.CREATED_TIME_PROPERTY;
        }
        IPage<SysLogEntity> pageInfrom = logService.getPage(page, pageSize, orderBy, orderType);
        return logService.operatePage(pageInfrom, startTime, endTime
                , getCurrentUser().isPtTenantAdmin() ? getCurrentUser().getCurrentTenantId() : null
                , getCurrentUser().isCustomerUser() ? getCurrentUser().getCurrentUserId() : null
                , entityType, actionType);
    }

    @GetMapping(value = "user", params = {PAGE_SIZE, PAGE})
    @ApiOperation("登录登出列表")
    @PreAuthorize("@check.checkPermissions({},{})")
    public TkPageData<SysLogDTO> pageLogin(
            @RequestParam(PAGE_SIZE) int pageSize,
            @RequestParam(PAGE) int page,
            @RequestParam(value = "startTime", required = false) Long startTime,
            @RequestParam(value = "endTime", required = false) Long endTime,
            @RequestParam(value = "actionType", required = false) ActionType actionType,
            @RequestParam(value = ORDER_FILED, required = false) String orderBy,
            @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType)
            throws ThingsboardException {


        if (StringUtils.isEmpty(orderBy)) {
            orderBy = ModelConstants.CREATED_TIME_PROPERTY;
        }
        IPage<SysLogEntity> pageInfrom = logService.getPage(page, pageSize, orderBy, orderType);
        return logService.loginPage(pageInfrom, startTime, endTime
                , getCurrentUser().isPtTenantAdmin() ? getCurrentUser().getCurrentTenantId() : null
                , getCurrentUser().isCustomerUser() ? getCurrentUser().getCustomerId().toString() : null
                , actionType);
    }

    /**
     * 测评新增接口，数据库备份日志列表
     */
    @GetMapping(value = "backup", params = {PAGE_SIZE, PAGE})
    @ApiOperation("备份日志列表")
    @PreAuthorize("@check.checkPermissions({},{})")
    public TkPageData<SysLogDTO> pageBackupLog(
            @RequestParam(PAGE_SIZE) int pageSize,
            @RequestParam(PAGE) int page,
            @RequestParam(value = "startTime", required = false) Long startTime,
            @RequestParam(value = "endTime", required = false) Long endTime,
            @RequestParam(value = "actionType", required = false) ActionType actionType,
            @RequestParam(value = ORDER_FILED, required = false) String orderBy,
            @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType)
            throws ThingsboardException {


        if (StringUtils.isEmpty(orderBy)) {
            orderBy = ModelConstants.CREATED_TIME_PROPERTY;
        }
        IPage<SysLogEntity> pageInfrom = logService.getPage(page, pageSize, orderBy, orderType);
        return logService.backupLogPage(pageInfrom, startTime, endTime
                , getCurrentUser().isPtTenantAdmin() ? getCurrentUser().getCurrentTenantId() : null
                , getCurrentUser().isCustomerUser() ? getCurrentUser().getCurrentUserId() : null
                , actionType);
    }

    protected SecurityUser getCurrentUser() throws ThingsboardException {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication != null && authentication.getPrincipal() instanceof SecurityUser) {
            return (SecurityUser) authentication.getPrincipal();
        } else {
            throw new ThingsboardException("你没有权限执行改操作!", ThingsboardErrorCode.AUTHENTICATION);
        }
    }


}
