package com.tofly.base.controller.menu;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.tofly.base.config.SysAdminConstant;
import com.tofly.base.controller.BaseController;
import com.tofly.base.entity.menu.ColumnAuthQuery;
import com.tofly.base.entity.menu.ColumnAuthVO;
import com.tofly.base.entity.menu.FormVisibleConfig;
import com.tofly.base.entity.user.SysUserRoleEntity;
import com.tofly.base.mapper.user.UserRoleMapper;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.dto.MenuDTO;
import com.tofly.base.server.common.data.security.SecurityUser;
import com.tofly.base.service.menu.FormVisibleConfigService;
import com.tofly.base.service.menu.MenuService;
import com.tofly.base.service.role.RoleMenuColumnService;
import com.tofly.base.service.user.TkUserService;
import com.tofly.base.utils.UserTicketUtil;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import java.net.URI;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequiredArgsConstructor
@RequestMapping("api/yt/menu")
public class TkMenuController extends BaseController{

    private final MenuService menuService;

    private final FormVisibleConfigService formVisibleConfigService;

    private final RoleMenuColumnService roleMenuColumnService;

    private final TkUserService tkUserService;
    private final UserRoleMapper userRoleMapper;

    @GetMapping("/me/menus")
    public List<MenuDTO> getMyMenus(
            @RequestParam(value = "needButton", required = false, defaultValue = "true") boolean needButton,
            @RequestParam(value = "needField", required = false, defaultValue = "true") boolean needField,
            @RequestParam(value = "roleId", required = false, defaultValue = "true") String roleId
    ) {

        List<MenuDTO> list = null;
        try {
            SecurityUser user = getCurrentUser();
            if (user.getPlatformUserName().equals("sysadmin")) {
                list = menuService.getMyMenus(null,
                        null,
                        true,
                        false,
                        needButton,
                        needField);
            } else {
                list = menuService.getMyMenus(null,
                        user.getId().toString(),
                        false,
                        false,
                        needButton,
                        needField);
            }
            recursion(list, roleId);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return list;
    }

    private void recursion(List<MenuDTO> myMenus, String roleId){
        List<String> roleIds = new ArrayList<>();
        List<Map<String, String>> tempList = null;
        List<Map<String, String>> visibleTempList = null;
        Map<String, String> tempMap = null;
        Map<String, String> visibleTempMap = null;
        ColumnAuthQuery columnAuthQuery = null;
        roleIds.add(roleId);
        for (MenuDTO myMenu : myMenus) {
            tempList = new ArrayList<>();
            visibleTempList = new ArrayList<>();
            String menuType = myMenu.getMeta().get("menuType").asText();
            if("3".equals(menuType)){
                if(SysAdminConstant.SYS_ADMIN.equals(UserTicketUtil.getCurrentUser().getCurrentUserId()) || SysAdminConstant.SYS_ROLE.equals(roleId)){
                    columnAuthQuery = new ColumnAuthQuery();
                    columnAuthQuery.setMenuId(myMenu.getId());
                    columnAuthQuery.setRoleIds(roleIds);
                    columnAuthQuery.setCreator(SysAdminConstant.SYS_ADMIN);
                    List<ColumnAuthVO> list1 = roleMenuColumnService.getColumnsByMenuAndRoleAdmin(columnAuthQuery);
                    for (ColumnAuthVO config : list1) {
                        tempMap = new HashMap<>();
                        visibleTempMap = new HashMap<>();
                        tempMap.put("propField", config.getPropField());
                        tempMap.put("propName", config.getPropName());
                        tempMap.put("id", config.getId()==null?"":config.getId().toString());
                        tempMap.put("configId", config.getConfigId().toString());
                        tempMap.put("menuId", config.getMenuId());
                        tempMap.put("isVisible", StringUtils.isNotEmpty(config.getIsVisible()) ? config.getIsVisible() : "0");
                        tempMap.put("roleId", config.getRoleId());
                        tempList.add(tempMap);
                        visibleTempMap.put("propField", config.getPropField());
                        visibleTempMap.put("propName", config.getPropName());
                        visibleTempMap.put("menuId", config.getMenuId());
                        visibleTempMap.put("isVisible", "1");
                        visibleTempMap.put("configId", config.getConfigId().toString());
                        visibleTempList.add(visibleTempMap);
                    }
                    myMenu.setFieldMap(tempList);
                    myMenu.setVisibleFieldMap(visibleTempList);
                }else{
                    columnAuthQuery = new ColumnAuthQuery();
                    columnAuthQuery.setMenuId(myMenu.getId());
                    List<SysUserRoleEntity> sysUserRoleEntities = userRoleMapper.selectList(new LambdaQueryWrapper<SysUserRoleEntity>().eq(SysUserRoleEntity::getUserId, UserTicketUtil.getCurrentUser().getCurrentUserId()));
                    List<String> tempRoleIds = sysUserRoleEntities.stream().map(item -> item.getRoleId()).collect(Collectors.toList());
                    if(StringUtils.isNotEmpty(roleId)){
                        columnAuthQuery.setRoleIds(roleIds);
                        columnAuthQuery.setCreator(UserTicketUtil.getCurrentUser().getCurrentUserId());
                    }else {
                        columnAuthQuery.setRoleIds(tempRoleIds);
                        columnAuthQuery.setCreator(tkUserService.findUserInfoById(UserTicketUtil.getCurrentUser().getCurrentUserId()).getCreator());
                    }
                    List<ColumnAuthVO> list = roleMenuColumnService.getColumnsByMenuAndRole(columnAuthQuery);

                    for (ColumnAuthVO config : list) {
                        tempMap = new HashMap<>();
                        tempMap.put("propField", config.getPropField());
                        tempMap.put("propName", config.getPropName());
                        tempMap.put("id", config.getId()==null?"":config.getId().toString());
                        tempMap.put("menuId", config.getMenuId());
                        tempMap.put("isVisible", config.getIsVisible());
                        tempMap.put("roleId", config.getRoleId());
                        tempMap.put("configId", config.getConfigId().toString());
                        tempList.add(tempMap);
                    }
                    columnAuthQuery.setRoleIds(tempRoleIds);
                    columnAuthQuery.setCreator(tkUserService.findUserInfoById(UserTicketUtil.getCurrentUser().getCurrentUserId()).getCreator());
                    List<ColumnAuthVO> visibleList = roleMenuColumnService.getColumnsByMenuAndRoleVisible(columnAuthQuery);
                    for (ColumnAuthVO config : visibleList) {
                        visibleTempMap = new HashMap<>();
                        visibleTempMap.put("propField", config.getPropField());
                        visibleTempMap.put("propName", config.getPropName());
                        visibleTempMap.put("menuId", config.getMenuId());
                        visibleTempMap.put("isVisible", config.getIsVisible());
                        visibleTempMap.put("configId", config.getConfigId().toString());
                        visibleTempList.add(visibleTempMap);
                    }
                    myMenu.setFieldMap(tempList);
                    myMenu.setVisibleFieldMap(visibleTempList);
                }
            }else{
                if(!CollectionUtils.isEmpty(myMenu.getChildren())){
                    recursion(myMenu.getChildren(), roleId);
                }
            }
        }
    }


    @GetMapping("get_ids/{roleId}")
    public Set<String> getMenuIdsByRoleId(@PathVariable("roleId") String roleId) {
        return menuService.getMenuIdsByRoleId(roleId);
    }

    @PutMapping("/manage/assign/{roleId}")
    public void assignMenuToRole(
            @RequestBody String[] menuIds, @PathVariable("roleId") String roleId) {
        menuService.assignMenuToRole(menuIds, roleId);
    }

    @GetMapping("{id}")
    public ResponseEntity<MenuDTO> getMenus(@PathVariable("id") String id) {
        return ResponseEntity.of(menuService.getMenu(id));
    }

    @PutMapping
    public MenuDTO updateMenu(@RequestBody MenuDTO menuDTO) {
        Assert.notNull(menuDTO.getId(), "menuId cannot be null");
        MenuDTO menu = menuService.updateMenu(menuDTO);
        List<FormVisibleConfig> configs = new ArrayList<>();
        FormVisibleConfig config = null;
        if (!CollectionUtils.isEmpty(menuDTO.getFieldMap())) {
            for (Map<String, String> stringObjectMap : menuDTO.getFieldMap()) {
                config = new FormVisibleConfig();
                config.setMenuId(menuDTO.getId());
                config.setPropField(stringObjectMap.get("propField"));
                config.setPropName(stringObjectMap.get("propName"));
                configs.add(config);
            }
            formVisibleConfigService.remove(new LambdaQueryWrapper<FormVisibleConfig>().eq(FormVisibleConfig::getMenuId, menuDTO.getId()));
            formVisibleConfigService.saveBatch(configs);
        }
        return menu;
    }

    @PostMapping
    public ResponseEntity<MenuDTO> saveMenu(@RequestBody MenuDTO menuDTO) {
        MenuDTO newMenuDTO =
                menuService.saveMenu(menuDTO);
        List<FormVisibleConfig> configs = new ArrayList<>();
        FormVisibleConfig config = null;

        if (!CollectionUtils.isEmpty(menuDTO.getFieldMap())) {
            for (Map<String, String> stringObjectMap : menuDTO.getFieldMap()) {
                config = new FormVisibleConfig();
                config.setMenuId(menuDTO.getId());
                config.setPropField(stringObjectMap.get("propField"));
                config.setPropName(stringObjectMap.get("propName"));
                configs.add(config);
            }
        }
        formVisibleConfigService.saveBatch(configs);
        return Optional.ofNullable(newMenuDTO)
                .map(
                        dto -> {
                            URI location =
                                    ServletUriComponentsBuilder.fromCurrentRequest()
                                            .path("/{id}")
                                            .buildAndExpand(newMenuDTO.getId())
                                            .toUri();
                            return ResponseEntity.created(location).body(newMenuDTO);
                        })
                .orElse(ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build());
    }

    @DeleteMapping
    public void deleteMenus(@RequestBody String[] ids) {
        if (ids.length == 0) {
            throw new TkDataValidationException("需要删除的菜单不能为空");
        }
        formVisibleConfigService.remove(
                new QueryWrapper<FormVisibleConfig>().lambda()
                        .in(FormVisibleConfig::getMenuId, ids)
        );
        menuService.deleteMenus(null, ids);

    }

    @PostMapping("/me/buttons")
    public List<MenuDTO> getMyButtons() {
    	List<MenuDTO> retrunList =new ArrayList<MenuDTO>();
    	try {
    		 SecurityUser user = getCurrentUser();    		
    		 retrunList =menuService.getButtons(user.getId().toString());    		  
    	}catch (Exception e) {
    		e.printStackTrace();
		}
    	return retrunList;
    }
}