package com.tofly.base.controller.organization;

import com.alibaba.druid.sql.dialect.oracle.ast.stmt.OracleCreateTableStatement;
import com.tofly.base.controller.BaseController;
import com.tofly.base.ding.base.entity.DingDeptVo;
import com.tofly.base.ding.handle.DingDeptHandle;
import com.tofly.base.entity.organization.TkOrganizationEntity;
import com.tofly.base.server.common.data.common.DeleteGroup;
import com.tofly.base.server.common.data.common.UpdateGroup;
import com.tofly.base.server.common.data.core.exception.ThingsboardException;
import com.tofly.base.server.common.data.dto.DeleteDTO;
import com.tofly.base.server.common.data.dto.OrganizationDTO;
import com.tofly.base.server.common.data.security.SecurityUser;
import com.tofly.base.service.organization.TkOrganizationService;
import com.tofly.base.service.user.TkUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.RequiredArgsConstructor;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import java.net.URI;
import java.util.*;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("api/yt/organization")
@Api(tags = "组织管理")
@RequiredArgsConstructor
public class TkOrganizationController extends BaseController {

    private final TkOrganizationService organizationService;

    private final TkUserService tkUserService;

    private final StringRedisTemplate redisTemplate;

    @PostMapping
    @PreAuthorize("@check.checkPermissions({},{'api:yt:organization:post'})")
    public ResponseEntity<OrganizationDTO> saveGroup(@RequestBody OrganizationDTO groupDTO)
            throws ThingsboardException {
        //如果当前登录的用户不是超级管理员或者租户管理员
        boolean isCustomerOrPlatform = !getCurrentUser().isPtSysadmin() && !getCurrentUser().isPtTenantAdmin();

        SecurityUser user = getCurrentUser();

        OrganizationDTO newGroupDTO =
                organizationService.saveOrganization(groupDTO, getCurrentUser().getCurrentTenantId(), isCustomerOrPlatform, getCurrentUser().getCurrentUserId());

        // ------安全测评加入------
    /*try {

      EntityId entityId = new EntityId() {
        @Override
        public UUID getId() {
          return UUID.fromString(newGroupDTO.getId());
        }

        @Override
        public EntityType getEntityType() {
          return EntityType.ORGANIZATION_ASSET;
        }
      };

      EntityInfo entity = new EntityInfo(entityId, newGroupDTO.getName());

      logEntityAction(
              getCurrentUser(),
              entityId,
              entity,
              new CustomerId(EntityId.NULL_UUID),
              ActionType.ADDED,
              null);
    } catch (Exception e) {
      // ignore
    }*/

        return Optional.ofNullable(newGroupDTO)
                .map(
                        dto -> {
                            URI location =
                                    ServletUriComponentsBuilder.fromCurrentRequest()
                                            .path("/{id}")
                                            .buildAndExpand(newGroupDTO.getId())
                                            .toUri();
                            return ResponseEntity.created(location).body(newGroupDTO);
                        })
                .orElse(ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).build());
    }

    @ApiResponses({@ApiResponse(code = 200, message = "OK", response = ArrayList.class)})
    @GetMapping("/getChildrenId")
    public List<String> getChildrenId(@RequestParam String id) {
        return organizationService.getChildrenId(id);
    }

    @ApiOperation("数据同步")
    @GetMapping("/syncData2")
    public ResponseEntity<Boolean> syncData() {
        // ------安全测评加入------
        String redisKey = "organization:tkUser:syncData";
        // 使用SET NX命令尝试设置Key，设置成功说明是第一次请求，过期时间设为5分钟
        Boolean isSetSuccess = redisTemplate.opsForValue().setIfAbsent(redisKey, "processing", 5, TimeUnit.MINUTES);
        if (Boolean.TRUE.equals(isSetSuccess)) {

            final List<DingDeptVo> dingDeptVos = DingDeptHandle.pullDingDept();

            organizationService.saveOrUpdateOrganizationByDingDept(dingDeptVos);

            for (DingDeptVo dingDeptVo : dingDeptVos) {
                tkUserService.pullUserFromDing(dingDeptVo.getDeptId());
            }
        }

        return ResponseEntity.ok(Boolean.TRUE);
    }

    @DeleteMapping
    @PreAuthorize("@check.checkPermissions({},{'api:yt:organization:delete'})")
    public ResponseEntity<Boolean> deleteOrganizations(
            @Validated(DeleteGroup.class) @RequestBody DeleteDTO deleteDTO) throws ThingsboardException {
        // ------安全测评加入------
        Map<String, String> organizationMap = new LinkedHashMap<>();
        Set<String> ids = deleteDTO.getIds();
        try {
            organizationMap = organizationService.findOrganizationMap(ids);
        } catch (Exception e) {
            // ignore
        }

        boolean result = organizationService.deleteOrganizations(deleteDTO, getCurrentUser().getCurrentTenantId());

        // ------安全测评加入------
  /*  try {
      for (Iterator<String> iterator = ids.iterator(); iterator.hasNext(); ) {
        String id =  iterator.next();
        EntityId entityId = new EntityId() {
          @Override
          public UUID getId() {
            return UUID.fromString(id);
          }

          @Override
          public EntityType getEntityType() {
            return EntityType.ORGANIZATION_ASSET;
          }
        };
        EntityInfo entity = new EntityInfo(entityId, organizationMap.get(id));

        logEntityAction(
                getCurrentUser(),
                entityId,
                entity,
                new CustomerId(EntityId.NULL_UUID),
                ActionType.DELETED,
                null);
      }

    } catch (Exception e) {
      // ignore
    }*/
        return ResponseEntity.ok(result);
    }

    @GetMapping("/me/list")
    @ApiOperation("下拉列表")
    public ResponseEntity getMyOrganizations() throws ThingsboardException {
        //TODO 等待user迁移
        List<OrganizationDTO> result = organizationService.getMyOrganizations(
                getCurrentUser().isPtSysadmin(),
                getCurrentUser().isPtTenantAdmin(),
                getCurrentUser().getCurrentTenantId(),
                getCurrentUser().getCurrentUserId());
        return ResponseEntity.ok(result);
    }

    @PutMapping
    @PreAuthorize("@check.checkPermissions({},{'api:yt:organization:update'})")
    public ResponseEntity updateOrganization(@Validated(UpdateGroup.class) @RequestBody OrganizationDTO groupDTO) throws ThingsboardException {
        //如果当前登录的用户不是超级管理员或者租户管理员
        boolean isCustomerOrPlatform = !getCurrentUser().isPtSysadmin() && !getCurrentUser().isPtTenantAdmin();
        OrganizationDTO organization = organizationService.updateOrganization(
                groupDTO,
                getCurrentUser().getCurrentTenantId(),
                isCustomerOrPlatform,
                getCurrentUser().getCurrentUserId());

        // ------安全测评加入------
        try {
     /* EntityId entityId = new EntityId() {
        @Override
        public UUID getId() {
          return UUID.fromString(organization.getId());
        }

        @Override
        public EntityType getEntityType() {
          return EntityType.ORGANIZATION_ASSET;
        }
      };*/

    /*  EntityInfo entity = new EntityInfo(entityId, organization.getName());

      logEntityAction(
              getCurrentUser(),
              entityId,
              entity,
              new CustomerId(EntityId.NULL_UUID),
              ActionType.UPDATED,
              null);*/
        } catch (Exception e) {
            // ignore
        }

        return ResponseEntity.ok(
                organization);
    }

    @PutMapping("save_organization_mapping/{userId}")
    public void bindUserToOrganization(
            @RequestBody String[] organizationIds, @PathVariable("userId") String userId) throws ThingsboardException {
        organizationService.bindUserToOrganization(getCurrentUser().getCurrentTenantId(), userId, organizationIds);
    }
    @GetMapping("/getAllOrganization")
    public ResponseEntity<List<TkOrganizationEntity>> getAllOrganization() {
        List<TkOrganizationEntity> tkOrganizationEntities = organizationService.getAllOrganization();
        return ResponseEntity.ok(tkOrganizationEntities);
    }
}
