package com.tofly.base.controller.platform;

import com.tofly.base.server.common.data.core.exception.ThingsboardErrorCode;
import com.tofly.base.server.common.data.core.exception.ThingsboardException;
import com.tofly.base.server.common.data.dto.SysPlatformDesignDTO;
import com.tofly.base.server.common.data.id.EntityId;
import com.tofly.base.server.common.data.security.SecurityUser;
import com.tofly.base.server.common.data.utils.tools.ResponseResult;
import com.tofly.base.service.platform.SysPlatformDesignService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

/**
 * @author: 徐浩然
 * @desc: 平台定制
 * @date: 2021/12/3-18:44
 */
@Api(tags = "平台定制")
@RestController
@RequestMapping("api/yt/platform")
@RequiredArgsConstructor
public class TkPlatformDesignController {

  private final SysPlatformDesignService sysPlatformDesignService;

  @GetMapping("get")
  @ApiOperation("查询详情")
  public ResponseEntity<SysPlatformDesignDTO> get() {
    String tenantId = EntityId.NULL_UUID.toString();
    try {
      tenantId = getCurrentUser().getCurrentTenantId();
    } catch (ThingsboardException e) {
    }
    return ResponseEntity.ok(sysPlatformDesignService.get(tenantId));
  }

  @PutMapping("update")
  @PreAuthorize(
      "@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:platform:update:update'})")
  @ApiOperation("更新")
  public ResponseEntity update(@RequestBody SysPlatformDesignDTO sysPlatformDesignDTO)
      throws ThingsboardException {
    return sysPlatformDesignService.update(
            sysPlatformDesignDTO, getCurrentUser().getCurrentTenantId())
        ? ResponseEntity.ok().build()
        : ResponseEntity.badRequest().build();
  }

  @PostMapping("data_reset")
  @PreAuthorize(
      "@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:platform:data_reset:reset'})")
  @ApiOperation("恢复出厂设置")
  public ResponseResult<Boolean> dataReset(@RequestBody SysPlatformDesignDTO sysPlatformDesignDTO)
      throws ThingsboardException {
    return ResponseResult.success(
        sysPlatformDesignService.factoryDataReset(getCurrentUser().getCurrentTenantId()));
  }

  protected SecurityUser getCurrentUser() throws ThingsboardException {
    Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
    if (authentication != null && authentication.getPrincipal() instanceof SecurityUser) {
      return (SecurityUser) authentication.getPrincipal();
    } else {
      throw new ThingsboardException("你没有权限执行改操作!", ThingsboardErrorCode.AUTHENTICATION);
    }
  }
}
