package com.tofly.base.controller.role;//package com.tofly.yxBase.server.controller;

import com.tofly.base.config.SysAdminConstant;
import com.tofly.base.controller.BaseController;
import com.tofly.base.entity.role.RoleMenuColumn;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.core.cache.CacheUtils;
import com.tofly.base.server.common.data.dto.RoleDTO;
import com.tofly.base.server.common.data.dto.request.RoleReqDTO;
import com.tofly.base.server.common.data.enums.OrderTypeEnum;
import com.tofly.base.server.common.data.enums.RoleEnum;
import com.tofly.base.server.common.data.security.SecurityUser;
import com.tofly.base.server.common.data.utils.tools.ResponseResult;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.service.role.RoleMenuColumnService;
import com.tofly.base.service.role.RolePermissionService;
import com.tofly.base.service.role.RoleService;
import com.tofly.base.utils.DataConfig;
import com.tofly.base.utils.NetworkCrypt;
import com.tofly.base.utils.UserTicketUtil;
import com.tofly.base.utils.UserTokenUtil;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.RequiredArgsConstructor;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import java.util.*;

import static com.tofly.base.server.common.data.constant.QueryConstant.*;

@RestController
@RequestMapping("api/yt/role")
@RequiredArgsConstructor
public class TkRoleController extends BaseController {

    private final RoleService roleService;

    private final RolePermissionService rolePermissionService;

    private final RoleMenuColumnService roleMenuColumnService;

    private final RedisTemplate redisTemplate;

    private final DataConfig dc;
    private final CacheUtils cacheUtils;
    @GetMapping(params = {PAGE_SIZE, PAGE})
    @PreAuthorize("hasAnyAuthority('SYS_ADMIN','TENANT_ADMIN')")
    public TkPageData<RoleDTO> pageRole(
            @RequestParam(PAGE_SIZE) int pageSize,
            @RequestParam(PAGE) int page,
            @RequestParam(value = "roleName", required = false) String roleName,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = ORDER_FILED, required = false) String orderBy,
            @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType) {
        HashMap<String, Object> queryMap =
                buildQueryParam(page, pageSize, orderBy, orderType, status, roleName);
        return roleService.page(null, queryMap);
    }

    @GetMapping(
            path = "/tenant",
            params = {PAGE_SIZE, PAGE})
    @PreAuthorize("hasAnyAuthority('SYS_ADMIN','PLATFORM_ADMIN')")
    public TkPageData<RoleDTO> pageTenantRole(
            @RequestParam(PAGE_SIZE) int pageSize,
            @RequestParam(PAGE) int page,
            @RequestParam(value = "roleName", required = false) String roleName,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = ORDER_FILED, required = false) String orderBy,
            @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType) {
        HashMap<String, Object> queryMap =
                buildQueryParam(page, pageSize, orderBy, orderType, status, roleName);
        queryMap.put("roleType", RoleEnum.TENANT_ADMIN.name());
        return roleService.page(null, queryMap);
    }
    @ApiResponses({@ApiResponse(code = 200, message = "OK", response = Boolean.class)})
    @GetMapping("/getHasRoleByUserId")
    public Boolean getHasRoleByUserId(@RequestParam("userID") String userID, @RequestParam("roleName") String roleName){

        return roleService.getHasRoleByUserId(userID,roleName);
    }

    @DeleteMapping
    @PreAuthorize(
            "@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN','TENANT_ADMIN'},{'api:yt:role:delete'})")
    public void deleteRole(@RequestBody String[] ids) {
        roleService.deleteRole(ids, null);
    }

    @GetMapping("/me/permissions")
    @ApiOperation("权限测试")
    public Set<String> getPermissions() {
        final SecurityUser currentUser = UserTicketUtil.getCurrentUser();
        return roleService.getPermissions(
                false, currentUser.isTenantAdmin(), null, UserTokenUtil.getTokenUserID());
    }

    @PutMapping("update_status/{roleId}/{status}")
    public void updateRoleStatus(
            @PathVariable("roleId") String roleId, @PathVariable("status") int status) {
        Assert.isTrue(status == 0 || status == 1, "role status is not correct");
        roleService.updateRoleStatus(
                roleId, status, true, null);
    }

    @PostMapping("save_with_menu")
    @PreAuthorize(
            "@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN','TENANT_ADMIN'},{'api:yt:role:saveOrUpdateRoleInfoWithMenu:post','api:yt:role:saveOrUpdateRoleInfoWithMenu:update'})")
    public ResponseResult<RoleDTO> saveOrUpdateRoleInfoWithMenu(@RequestBody RoleReqDTO roleReqDTO) {
        RoleDTO roleDTO = null;

        roleDTO = roleService.saveOrUpdateRoleInfoWithMenu(
                roleReqDTO,
                true,
                true,
                null);
        if (!SysAdminConstant.SYS_ROLE.equals(roleReqDTO.getId())) {
            List<RoleMenuColumn> updateColumnList = new ArrayList<>();
            RoleMenuColumn roleMenuColumn = null;
            for (Map<String, Object> column : roleReqDTO.getFieldMap()) {
                roleMenuColumn = new RoleMenuColumn();
                roleMenuColumn.setConfigId(Long.valueOf(column.get("configId").toString()));
                roleMenuColumn.setIsVisible(column.get("isVisible").toString());
                roleMenuColumn.setRoleId(roleDTO.getId());
                roleMenuColumn.setCreator(UserTicketUtil.getCurrentUser().getCurrentUserId());
                roleMenuColumn.setId(column.get("id") == null || "".equals(column.get("id")) ? null : Long.parseLong(column.get("id").toString()));
                if (column.get("id") == null || "".equals(column.get("id"))) {
                    roleMenuColumnService.save(roleMenuColumn);
                } else {
                    updateColumnList.add(roleMenuColumn);
                }
            }
            roleMenuColumnService.updateBatchById(updateColumnList);
        }
        cacheUtils.invalidateCacheName(FastIotConstants.CacheConfigKey.CACHE_CONFIG_KEY);

        return ResponseResult.success(roleDTO);
    }

    @PostMapping("/find/list")
    public List<RoleDTO> findRoleInfoList(@RequestBody RoleDTO roleDTO) {
        return roleService.findRoleInfo(
                false,
                null,
                null,
                roleDTO);
    }

    private HashMap<String, Object> buildQueryParam(
            int page,
            int pageSize,
            String orderBy,
            OrderTypeEnum orderType,
            Integer status,
            String roleName) {
        HashMap<String, Object> queryMap = new HashMap<>();
        queryMap.put(PAGE_SIZE, pageSize);
        queryMap.put(PAGE, page);
        queryMap.put(ORDER_FILED, orderBy);
        queryMap.put("roleName", roleName);
        if (status != null) {
            queryMap.put("status", status == 1);
        }
        if (orderType != null) {
            queryMap.put(ORDER_TYPE, orderType.name());
        }
        return queryMap;
    }


    /**
     * 获取设备数据接口
     *
     * @return
     */
    @GetMapping("/devicesData")
    public String getDevicesData(@RequestHeader Map<String, String> headers) {


        String deData = "";
        HttpClient client = HttpClientBuilder.create().build();
        try {
//	  	    //参数json传字符串
//	  	    String queryString = JSONObject.toJSONString(queryMap);  	    
//	  	    //参数加密
//	  	    String param = NetworkCrypt.encrypt(""+queryString+"", DataConfig.ENCODEKEY);

            HttpGet get = new HttpGet(dc.getUrl() + "/zhjc/device/all");
            get.addHeader("Content-type", "application/json; charset=utf-8");
            String token = "";
            if (null == headers.get("X-Authorization")) {
                token = headers.get("x-authorization").trim();
            } else {
                token = headers.get("X-Authorization").trim();
            }
            get.setHeader("X-Access-Token", token.substring(7, token.length()));
            HttpResponse res = client.execute(get);
            if (res.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                HttpEntity entity = res.getEntity();
                String result = EntityUtils.toString(entity);
                deData = NetworkCrypt.decrypt(result.substring(1, result.length() - 1), dc.getDECODEKEY());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return deData;
    }

    /**
     * 获取传感器数据接口
     *
     * @return
     */
    @GetMapping("/sensorsData")
    public String getSensorsData(@RequestHeader Map<String, String> headers) {
        String deData = "";
        HttpClient client = HttpClientBuilder.create().build();
        try {
//		    //参数json传字符串
//		    String queryString = JSONObject.toJSONString(queryMap);  	    
//		    //参数加密
//		    String param = NetworkCrypt.encrypt(""+queryString+"", DataConfig.ENCODEKEY);

            HttpGet get = new HttpGet(dc.getUrl() + "/zhjc/sensor/all");
            get.addHeader("Content-type", "application/json; charset=utf-8");
            String token = "";
            if (null == headers.get("X-Authorization")) {
                token = headers.get("x-authorization").trim();
            } else {
                token = headers.get("X-Authorization").trim();
            }
            get.setHeader("X-Access-Token", token.substring(7, token.length()));
            HttpResponse res = client.execute(get);
            if (res.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                HttpEntity entity = res.getEntity();
                String result = EntityUtils.toString(entity);
                deData = NetworkCrypt.decrypt(result.substring(1, result.length() - 1), dc.getDECODEKEY());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return deData;
    }

    /**
     * 获取rtu数据接口
     *
     * @return
     */
    @GetMapping("/rtuData")
    public String getRtuData(@RequestHeader Map<String, String> headers) {
        String deData = "";
        HttpClient client = HttpClientBuilder.create().build();
        try {
//		    //参数json传字符串
//		    String queryString = JSONObject.toJSONString(queryMap);  	    
//		    //参数加密
//		    String param = NetworkCrypt.encrypt(""+queryString+"", DataConfig.ENCODEKEY);

            HttpGet get = new HttpGet(dc.getUrl() + "/zhjc/rtu/all");
            get.addHeader("Content-type", "application/json; charset=utf-8");
            String token = "";
            if (null == headers.get("X-Authorization")) {
                token = headers.get("x-authorization").trim();
            } else {
                token = headers.get("X-Authorization").trim();
            }
            get.setHeader("X-Access-Token", token.substring(7, token.length()));
            HttpResponse res = client.execute(get);
            if (res.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                HttpEntity entity = res.getEntity();
                String result = EntityUtils.toString(entity);
                deData = NetworkCrypt.decrypt(result.substring(1, result.length() - 1), dc.getDECODEKEY());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return deData;
    }


    /**
     * 数据权限更新接口
     *
     * @return
     */
    @PostMapping("/updateRolePermission")
    public ResponseResult updateRolePermission(@RequestBody Map<String, String> rolePermissionList) {
        rolePermissionService.updateRolePermission(rolePermissionList);
        return ResponseResult.success("数据权限更新成功");
    }


    /**
     * 根据角色Id获取数据权限
     *
     * @return
     */
    @GetMapping("/getRolePermissionByRoleId")
    public List<Map<String, String>> getRolePermissionByRoleId(@RequestParam(value = "roleId", required = false) String roleId) {
        return rolePermissionService.getRolePermissionByRoleId(roleId);
    }


    /**
     * 根据用户Id获取数据权限
     *
     * @return
     */
    @GetMapping("/getRolePermissionByUserId")
    public List<Map<String, String>> getRolePermissionByUserId(@RequestParam(value = "userId", required = false) String userId) {
        return rolePermissionService.getRolePermissionByUserId(userId);
    }

}