package com.tofly.base.controller.user;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.tofly.base.controller.AbstractUserAccount;
import com.tofly.base.entity.user.SysUserEntity;
import com.tofly.base.entity.user.User;
import com.tofly.base.server.common.data.Customer;
import com.tofly.base.server.common.data.EntityInfo;
import com.tofly.base.server.common.data.EntityType;
import com.tofly.base.server.common.data.common.AddGroup;
import com.tofly.base.server.common.data.common.DeleteGroup;
import com.tofly.base.server.common.data.common.UpdateGroup;
import com.tofly.base.server.common.data.core.exception.ThingsboardException;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.core.message.ErrorMessage;
import com.tofly.base.server.common.data.dto.DeleteDTO;
import com.tofly.base.server.common.data.dto.UserDTO;
import com.tofly.base.server.common.data.dto.UserInfoDTO;
import com.tofly.base.server.common.data.dto.request.AccountReqDTO;
import com.tofly.base.server.common.data.dto.request.RoleOrOrganizationReqDTO;
import com.tofly.base.server.common.data.enums.ActionType;
import com.tofly.base.server.common.data.enums.OrderTypeEnum;
import com.tofly.base.server.common.data.enums.RoleEnum;
import com.tofly.base.server.common.data.id.CustomerId;
import com.tofly.base.server.common.data.id.EntityId;
import com.tofly.base.server.common.data.id.TenantId;
import com.tofly.base.server.common.data.id.UserId;
import com.tofly.base.server.common.data.security.Authority;
import com.tofly.base.server.common.data.utils.tools.ResponseResult;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.server.utils.AccountProperties;
import com.tofly.base.service.user.TkUserService;
import com.tofly.base.service.user.UserService;
import com.tofly.base.utils.MqUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

import static com.tofly.base.server.common.data.constant.QueryConstant.*;

@RestController
@RequestMapping("api/yt/user")
@Api(value = "用户接口")
public class TkUserController extends AbstractUserAccount {

    @Autowired
    private TkUserService userService;
    @Autowired
    private AccountProperties accountProperties;
    @Autowired
    private UserService tbUserService;

    @Autowired
    private MqUtils mqUtils;


//  public TkUserController(
//      UserService tbUserService,
//      ApplicationEventPublisher eventPublisher,
//      SystemSecurityService systemSecurityService,
//      BCryptPasswordEncoder passwordEncoder,
//      TkUserService userService,
//      AccountProperties accountProperties
//      ) {
//    super(tbUserService, eventPublisher, systemSecurityService, passwordEncoder);
//    this.userService = userService;
//    this.accountProperties = accountProperties;
//  }

    @PostMapping("/info")
    @ApiResponses({@ApiResponse(code = 200, message = "OK", response = UserInfoDTO.class)})
    @ApiOperation(value = "获取当前用户信息")
    public UserDTO getUserById(@RequestParam("id") String id) throws ThingsboardException {
        final Optional<UserDTO> user = userService.getUser(
                id, true, null);
        if(user !=null){
            return  user.get();
        }else{
            return  new UserDTO();
        }
    }

    @PostMapping("/infos")
    @ApiResponses({@ApiResponse(code = 200, message = "OK", response = UserInfoDTO.class)})
    @ApiOperation(value = "获取当前用户信息")
    public List<UserDTO> getUserByuesrname(@RequestParam("uesrname") String uesrname) throws ThingsboardException {
        final Optional<List<UserDTO>> userByUsername = userService.getUserByUsername(uesrname);


        return userByUsername.get();
    }

    @ApiResponses({@ApiResponse(code = 200, message = "OK", response = UserInfoDTO.class)})
    @ApiOperation(value = "获取当前用户信息")
    @PostMapping("/getUserIdByRealName")
    public List<UserDTO> getUserIdByRealName(@RequestBody List<String> nameList) {
        return userService.getUserIdByRealName(nameList);

    }

    @GetMapping("me/info")
    @ApiResponses({@ApiResponse(code = 200, message = "OK", response = UserInfoDTO.class)})
    @ApiOperation(value = "获取当前用户信息")
    public UserInfoDTO myInfo() throws ThingsboardException {
        return userService.me(
                getCurrentUser().getCurrentUserId(),
                getCurrentUser().getCurrentTenantId(),
                getCurrentUser().getRoles());
    }

    @PreAuthorize("hasAnyAuthority('SYS_ADMIN','PLATFORM_ADMIN')")
    @GetMapping(
            path = "/tenant/page",
            params = {PAGE_SIZE, PAGE})
    @ApiOperation(value = "获取租户管理员分页数据")
    public TkPageData<UserDTO> tenantPage(
            @RequestParam(PAGE_SIZE) int pageSize,
            @RequestParam(PAGE) int page,
            @RequestParam(value = ORDER_FILED, required = false) String orderBy,
            @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType,
            @RequestParam(TENANT_ID) String tenantId)
            throws ThingsboardException {
        HashMap<String, Object> queryMap = new HashMap<>();
        queryMap.put(PAGE_SIZE, pageSize);
        queryMap.put(PAGE, page);
        if (orderType != null) {
            queryMap.put(ORDER_TYPE, orderType.name());
        }
        queryMap.put(ORDER_FILED, orderBy);
        if (StringUtils.isEmpty(tenantId)) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        return userService.tenantPage(queryMap, tenantId);
    }


    @PutMapping("/center")
    @ApiOperation(value = "修改个人中心")
    public UserInfoDTO updatePersonalCenter(
            @Validated(UpdateGroup.class) @RequestBody UserDTO userDTO) throws ThingsboardException {
        userService.updatePersonalCenter(userDTO);
        return userService.me(
                getCurrentUser().getCurrentUserId(),
                getCurrentUser().getCurrentTenantId(),
                getCurrentUser().getRoles());
    }


    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','PLATFORM_ADMIN'},{'api:yt:user:saveTenantAdmin:post'})")
    @PostMapping("save_tenant_admin")
    public UserDTO saveTenantAdmin(@Validated(AddGroup.class) @RequestBody UserDTO userDTO)
            throws ThingsboardException {
        if (StringUtils.isEmpty(userDTO.getTenantId())) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        userService.validateUserNameAndPhoneNumberAndEmail(userDTO);
        userService.validateTenantRole(userDTO.getTenantId());
        // 租户角色不存在，会留存脏数据
        TenantId tenantId = TenantId.fromUUID(UUID.fromString(userDTO.getTenantId()));
        try {
            // 创建TB的租户管理员
            CustomerId customerId = new CustomerId(EntityId.NULL_UUID);
            User tbUser = new User();
            // 只有新增才调用TB
            if (null == userDTO.getId()) {
                tbUser = createTBUser(tbUser, userDTO, tenantId, customerId, Authority.TENANT_ADMIN);
                // 激活租户管理员
                activeTBUser(tbUser.getId(), accountProperties.getDefaultPassword(), tbUser.getTenantId(), true);
            }
        } catch (Exception e) {
            throw handleException(e);
        }

        UserDTO user = userService.saveTenantAdmin(
                userDTO, getCurrentUser().isPtSysadmin(), tenantId.getId().toString());


        // ------安全测评加入------
        try {
            boolean isCreate = null == userDTO.getId();
            EntityId entityId = new EntityId() {
                @Override
                public UUID getId() {
                    return UUID.fromString(user.getId());
                }

                @Override
                public EntityType getEntityType() {
                    return EntityType.USER_ASSET;
                }
            };

            EntityInfo entity = new EntityInfo(entityId, user.getUsername());

            logEntityAction(
                    getCurrentUser(),
                    entityId,
                    entity,
                    new CustomerId(EntityId.NULL_UUID),
                    isCreate ? ActionType.ADDED : ActionType.UPDATED,
                    null);
        } catch (Exception e) {
            // ignore
        }

        return user;
    }

    @GetMapping("get_group_user/{groupId}")
    public ResponseEntity<List<UserDTO>> getGroupUserByGroupId(
            @PathVariable("groupId") String groupId) throws ThingsboardException {
        return ResponseEntity.of(
                userService.getOrganizationUserByOrganizationId(
                        groupId, getCurrentUser().getCurrentTenantId()));
    }

    /**
     * 修改密码
     *
     * @param accountReqDTO
     * @return
     * @throws ThingsboardException
     */
    @PostMapping("/reset")
    @ApiOperation(value = "修改密码")
    public ResponseResult<Boolean> changePassword(@RequestBody AccountReqDTO accountReqDTO)
            throws ThingsboardException {
        SysUserEntity user = userService.validateChangePasswordAccount(accountReqDTO);
        user.setIsFirst(1);
        return ResponseResult.success(userService.changePassword(user));
    }


    /**
     * 重置密码
     *
     * @return
     */
    @PostMapping("/passwdset")
    public ResponseResult<Boolean> passwdset(@Validated @RequestBody AccountReqDTO accountReqDTO) {
        List<String> userIdList = accountReqDTO.getUserIdList();
        if (CollectionUtil.isEmpty(userIdList)) {
            return ResponseResult.failed("用户ID不能为空");
        }
        for (String userId : userIdList) {
            SysUserEntity user = new SysUserEntity();
            user.setId(userId);
            user.setPassword("tfkj028");
            user.setIsFirst(1);
            userService.changePassword(user);
        }

        return ResponseResult.success("重置密码完成");
    }


    @PreAuthorize("@check.checkPermissions({'TENANT_ADMIN'},{})")
    @ApiOperation(value = "获取租户下对应组织的所有客户")
    @GetMapping("/customers/{organizationId}")
    public ResponseEntity<List<UserDTO>> getMyCustomers(@PathVariable String organizationId)
            throws ThingsboardException {
        return ResponseEntity.ok(
                userService.getMyCustomers(
                        organizationId,
                        getCurrentUser().getCurrentTenantId(),
                        getCurrentUser().isPtTenantAdmin()));
    }

    /**
     * 创建租户用户
     *
     * @param title 标题
     * @throws ThingsboardException tb运行异常
     */
    private Customer createCustomer(String title) throws ThingsboardException {
        Customer customer = new Customer();
        try {
            customer.setTitle(title);
            customer.setTenantId(getCurrentUser().getTenantId());
//      checkEntity(customer.getId(), customer, Resource.CUSTOMER);

            Customer savedCustomer = checkNotNull(customerService.saveCustomer(customer));

            logEntityAction(
                    savedCustomer.getId(), savedCustomer, savedCustomer.getId(), ActionType.ADDED, null);

            if (customer.getId() != null) {
//        sendEntityNotificationMsg(
//            savedCustomer.getTenantId(), savedCustomer.getId(), EdgeEventActionType.UPDATED);
            }
            return savedCustomer;
        } catch (Exception e) {
            logEntityAction(emptyId(EntityType.CUSTOMER), customer, null, ActionType.ADDED, e);
            throw handleException(e);
        }
    }

    /**
     * 激活用户
     *
     * @param userId 用户ID
     * @throws ThingsboardException tb运行异常
     */
    private void activeTBUser(UserId userId, String password, TenantId tenantId, boolean isPtAdmin) throws ThingsboardException {
//    try {
//      // 1、获取UserCredentials 并获取activateToken
//      if(!isPtAdmin){
//        User user = checkUserId(userId, Operation.READ);
//        userId = user.getId();
//        SecurityUser authUser = getCurrentUser();
//        tenantId = authUser.getTenantId();
//      }
//      UserCredentials userCredentials =
//          tbUserService.findUserCredentialsByUserId(tenantId, userId);
//      // 2、进行激活
//      String encodedPassword = passwordEncoder.encode(password);
//      UserCredentials credentials =
//          tbUserService.activateUserCredentials(
//              TenantId.SYS_TENANT_ID, userCredentials.getActivateToken(), encodedPassword);
//      User currentUser =
//          tbUserService.findUserById(TenantId.SYS_TENANT_ID, credentials.getUserId());
//      tbUserService.setUserCredentialsEnabled(currentUser.getTenantId(), currentUser.getId(), true);
//    } catch (Exception e) {
//      throw handleException(e);
//    }
    }

    /**
     * 创建TB的用户
     *
     * @param userDTO    基础用户信息
     * @param authority  用户角色权限
     * @param tenantId   租户ID
     * @param customerId 客户ID
     * @return 用户
     * @throws ThingsboardException tb运行异常
     */
    private User createTBUser(
            User tbUser, UserDTO userDTO, TenantId tenantId, CustomerId customerId, Authority authority)
            throws ThingsboardException {
        try {
            tbUser.setAuthority(authority);
            tbUser.setTenantId(tenantId);
            tbUser.setCustomerId(customerId);
            tbUser.setEmail(userDTO.getUsername() + "@" + accountProperties.getEmailSuffix());
            tbUser = tbUserService.saveUser(tbUser);
            userDTO.setTbUser(tbUser.getId().getId().toString());
            logEntityAction(
                    tbUser.getId(),
                    tbUser,
                    tbUser.getCustomerId(),
                    userDTO.getId() == null ? ActionType.ADDED : ActionType.UPDATED,
                    null);
//      sendEntityNotificationMsg(
//          tenantId,
//          tbUser.getId(),
//          userDTO.getId() == null ? EdgeEventActionType.ADDED : EdgeEventActionType.UPDATED);
            return tbUser;
        } catch (Exception e) {
            throw handleException(e);
        }
    }

    /**
     * 删除租户下的客户
     *
     * @param strCustomerId 客户ID
     * @throws ThingsboardException tb运行异常
     */
    private void deleteCustomer(String strCustomerId) throws ThingsboardException {
        try {
            CustomerId customerId = new CustomerId(toUUID(strCustomerId));
//      Customer customer = checkCustomerId(customerId, Operation.DELETE);

//      List<EdgeId> relatedEdgeIds = findRelatedEdgeIds(getTenantId(), customerId);
            customerService.deleteCustomer(getTenantId(), customerId);

//      logEntityAction(
//          customerId, customer, customer.getId(), ActionType.DELETED, null, strCustomerId);
//      sendDeleteNotificationMsg(getTenantId(), customerId, relatedEdgeIds);

//      tbClusterService.broadcastEntityStateChangeEvent(
//          getTenantId(), customerId, ComponentLifecycleEvent.DELETED);
        } catch (Exception e) {
            throw handleException(e);
        }
    }

    /**
     * 密码输入错误多次后锁定，解锁(平台用户)
     *
     * @param userId 用户ID
     * @return
     * @throws ThingsboardException
     */
    @PreAuthorize("hasAnyAuthority('SYS_ADMIN')")
    @PostMapping("/{userId}/unlock")
    @ApiOperation(value = "解锁用户")
    public ResponseResult<Boolean> unlock(@PathVariable("userId") String userId)
            throws ThingsboardException {
        return ResponseResult.success(tbUserService.unlock(userId));
    }


    @GetMapping("{userId}")
    @PreAuthorize("@check.checkPermissions({},{'api:yt:user:get'})")
    public ResponseEntity<UserDTO> getUser(@PathVariable("userId") String userId) {
        return ResponseEntity.of(
                userService.getUser(
                        userId, true, null));
    }

//  @GetMapping("me/info")
//  @ApiResponses({@ApiResponse(code = 200, message = "OK", response = UserInfoDTO.class)})
//  @ApiOperation(value = "获取当前用户信息")
//  public UserInfoDTO myInfo() throws ThingsboardException {
//    return userService.me(
//        getCurrentUser().getCurrentUserId(),
//        getCurrentUser().getCurrentTenantId(),
//        getCurrentUser().getRoles());
//  }

    @PreAuthorize("hasAnyAuthority('SYS_ADMIN','TENANT_ADMIN')")
    @GetMapping(path = "page", params = {PAGE_SIZE, PAGE})
    @ApiOperation(value = "获取用户分页数据")
    public TkPageData<UserDTO> pageUser(@RequestParam(PAGE_SIZE) int pageSize, @RequestParam(PAGE) int page,
                                        @RequestParam(value = "tenantId", required = false) String tenantId,
                                        @RequestParam(value = "organizationId", required = false) String organizationId,
                                        @RequestParam(value = "roleType", required = false) RoleEnum roleType,
                                        @RequestParam(value = "realName", required = false) String realName,
                                        @RequestParam(value = "username", required = false) String username,
                                        @RequestParam(value = ORDER_FILED, required = false) String orderBy,
                                        @RequestParam(value = ORDER_TYPE, required = false) OrderTypeEnum orderType) {
        HashMap<String, Object> queryMap = new HashMap<>();
        queryMap.put(PAGE_SIZE, pageSize);
        queryMap.put(PAGE, page);
        queryMap.put(ORDER_FILED, orderBy);
        queryMap.put("realName", realName);
        queryMap.put("username", username);

        TkPageData<UserDTO> list = null;

        try {

            if (null != roleType && roleType.equals(RoleEnum.TENANT_ADMIN)) {
                queryMap.put("roleType", roleType.name());
            } else {
                tenantId = getCurrentUser().getCurrentTenantId();
            }

            if (organizationId != null) {
                queryMap.put("organizationId", organizationId);
            }

            if (orderType != null) {
                queryMap.put(ORDER_TYPE, orderType.name());
            }

            queryMap.put("currentUserId", getCurrentUser().getCurrentUserId());
            queryMap.put("tenantId", tenantId);
            list = userService.page(queryMap, true, false);
            List<UserDTO> collect = list.getItems().stream().sorted(Comparator.comparing(UserDTO::getUsername)).collect(Collectors.toList());
            list.setItems(collect);
        } catch (Exception e) {
            e.printStackTrace();
        }


        return list;
    }

    @PostMapping
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','TENANT_ADMIN'},{'api:yt:user:post'})")
    @ApiOperation(value = "新增用户")
    public ResponseEntity<UserDTO> addUser(@RequestParam(value = "sendEmail", required = false, defaultValue = "false") boolean sendEmail,
                                           @RequestParam(value = "sendMsg", required = false, defaultValue = "false") boolean sendMsg,
                                           @Validated({AddGroup.class}) @RequestBody UserDTO userDTO) throws Exception {
        // 检查用户手机号、邮箱
        userService.validateUserNameAndPhoneNumberAndEmail(userDTO);
        UserDTO user = null;

        user = userService.saveAccount(userDTO,
                sendEmail,
                sendMsg,
                false,
                getCurrentUser().getCurrentTenantId());

        //---------------用户新增消息传入队列------------------------------------------

        HashMap<String, String> messageMap = new HashMap<String, String>();
        messageMap.put("method", "add");
        messageMap.put("record", JSON.toJSONString(user));
        mqUtils.createMessage("user", JSON.toJSONString(messageMap));


        return ResponseEntity.ok(user);
    }

    @PutMapping
    @ApiOperation(value = "修改用户信息")
    @PreAuthorize("@check.checkPermissions({},{'api:yt:user:update'})")
    public UserDTO updateUser(@Validated(UpdateGroup.class) @RequestBody UserDTO userDTO) {
        UserDTO user = null;
        try {
            user = userService.updateUser(userDTO, true, getCurrentUser().getCurrentTenantId());
        } catch (ThingsboardException e) {
            e.printStackTrace();
        }

        //---------------用户修改消息传入队列------------------------------------------

//        HashMap<String, String> messageMap = new HashMap<String, String>();
//        messageMap.put("method", "update");
//        messageMap.put("record", JSON.toJSONString(user));
//        mqUtils.createMessage("user", JSON.toJSONString(messageMap));

        return user;
    }

    @DeleteMapping
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','TENANT_ADMIN'},{'api:yt:user:delete'})")
    public void deleteUser(@Validated({DeleteGroup.class}) @RequestBody DeleteDTO deleteDTO) {

        try {
            userService.deleteUser(deleteDTO.getIds(), true, getCurrentUser().getCurrentTenantId());


            //---------------用户删除消息传入队列------------------------------------------


            HashMap<String, Object> messageMap = new HashMap<String, Object>();
            messageMap.put("method", "delete");
            messageMap.put("record", deleteDTO.getIds().toArray());
            mqUtils.createMessage("user", JSON.toJSONString(messageMap));

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @GetMapping("/account_exist/{username}")
    public ResponseResult<UserDTO> accountExist(@PathVariable String username) {
        return ResponseResult.success(userService.accountExist(username));
    }

    /**
     * 通讯录
     *
     * @param username
     * @return
     */
    @GetMapping("/addressBook")
    public ResponseResult<List<Map<String, String>>> getAddressBook() {
        return ResponseResult.success(userService.getAddressBook());
    }


    @ApiOperation(value = "新增判断电话是否重复")
    @GetMapping("/getUserPhone")
    public ResponseResult<SysUserEntity> findUserByPhoneNumber(
            @RequestParam(value = "phoneNumber", required = false) String phoneNumber,
            @RequestParam(value = "userId", required = false, defaultValue = "") String userId) {
        return ResponseResult.success(userService.findUserByPhoneNumber(phoneNumber, userId));
    }


    @PostMapping("/relation")
    public String[] getUserRoleOrGroup(
            @Validated @RequestBody RoleOrOrganizationReqDTO roleOrGroupReqDTO) {
        return userService.getUserRoleOrOrganization(roleOrGroupReqDTO);
    }
}