package com.tofly.base.ding.handle;

import com.alibaba.fastjson.JSON;
import com.dingtalk.api.DefaultDingTalkClient;
import com.dingtalk.api.DingTalkClient;
import com.dingtalk.api.request.OapiV2UserListRequest;
import com.dingtalk.api.response.OapiV2UserListResponse;
import com.taobao.api.ApiException;
import com.tofly.base.ding.base.entity.DingUserVo;
import com.tofly.base.ding.base.vo.DingUserResponseVo;
import com.tofly.base.entity.user.SysUserEntity;
import com.tofly.base.server.common.data.utils.SpringBeanUtils;
import com.tofly.base.service.user.TkUserService;
import lombok.SneakyThrows;
import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.security.crypto.password.PasswordEncoder;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class DingUserHandle {
    private static List<DingUserVo> pullUser(long deptId,String accessToken) {
        try {
            DingTalkClient client = new DefaultDingTalkClient("https://oapi.dingtalk.com/topapi/v2/user/list");
            OapiV2UserListRequest req = new OapiV2UserListRequest();
            req.setDeptId(deptId);
            req.setCursor(0L);
            req.setSize(99L);
            OapiV2UserListResponse rsp = client.execute(req, accessToken);

            final DingUserResponseVo userData
                    = JSON.parseObject(rsp.getBody(), DingUserResponseVo.class);
            if(userData.getResult()!=null){
                if(userData.getResult().getList()!=null){
                    final List<DingUserVo> list = userData.getResult().getList();
                    return list;
                }
            }


        } catch (ApiException e) {
            e.printStackTrace();
        }
        return  Collections.emptyList();
    }

    @SneakyThrows
    public static List<SysUserEntity> pullSysUserEntityFromDing(long deptId){
        final TkUserService bean = SpringBeanUtils.getBean(TkUserService.class);

        final String loinToken = DingLoginHandle.getLoinToken();

        final List<DingUserVo> dingUserVos = pullUser(deptId, loinToken);
        List<SysUserEntity> list = new ArrayList<>();
        for (DingUserVo dingUserVo : dingUserVos) {

            final SysUserEntity userByDingId = bean.findUserByDingId(dingUserVo.getUserid());

            final SysUserEntity sysUserEntity = switchUser(userByDingId, dingUserVo);
            list.add(sysUserEntity);
        }
        return  list;
    }

    /**
     * 将钉钉获取的用户信息转换为当前系统对应的数据格式
     * @param dingUserVo
     * @return
     */
    public static SysUserEntity    switchUser(SysUserEntity sysUserEntity,DingUserVo dingUserVo){
       final PasswordEncoder passwordEncoder = SpringBeanUtils.getBean(PasswordEncoder.class);
        if(sysUserEntity==null){
            //新增用户 需要设置初始密码
            sysUserEntity = new SysUserEntity();
            sysUserEntity.setPassword(passwordEncoder.encode("tfkj028"));
            sysUserEntity.setIsFirst(1);
            sysUserEntity.setActivateToken(RandomStringUtils.randomAlphabetic(10));

            sysUserEntity.setUsername(convertToPinyin(dingUserVo.getName()));
            sysUserEntity.setDingId(dingUserVo.getUserid());
            sysUserEntity.setTenantId("13814000-1dd2-11b2-8080-808080808080");
            sysUserEntity.setStatus("1");
        }

        sysUserEntity.setDeptId(dingUserVo.getDeptIdList());
        if(dingUserVo.isAdmin()){
            sysUserEntity.setIsAdmin("1");
        }else{
            sysUserEntity.setIsAdmin("0");
        }
        sysUserEntity.setAvatar(dingUserVo.getAvatar());
        sysUserEntity.setRealName(dingUserVo.getName());

        sysUserEntity.setEmail(dingUserVo.getEmail());
        sysUserEntity.setLevel(2);

        return  sysUserEntity;
    }



    @SneakyThrows
    public static String convertToPinyin(String chinese) {
        StringBuilder pinyin = new StringBuilder();
        char[] chars = chinese.toCharArray();
        HanyuPinyinOutputFormat format = new HanyuPinyinOutputFormat();
        format.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        for (char c : chars) {
            String[] pinyinArray = PinyinHelper.toHanyuPinyinStringArray(c,format);

            if (pinyinArray != null) {
                pinyin.append(pinyinArray[0]);
            } else {
                pinyin.append(c);
            }
        }

        return pinyin.toString();
    }


}
