package com.tofly.base.server.common.data.core;

import com.tofly.base.server.common.data.core.message.ErrorMessage;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.util.Assert;

import java.io.Serializable;

@Data
@AllArgsConstructor
@NoArgsConstructor
public class Result implements Serializable {
  private static final long serialVersionUID = -7460365055472356522L;
  private String msg = "success";
  private int code = -1;
  private Object data;

  public static Result ok(Object data) {
    return new Result("success", 0, data);
  }

  public static Result error() {
    return error(ErrorMessage.OPERATION_FAILED.getCode(), ErrorMessage.OPERATION_FAILED);
  }

  public static Result error(ErrorMessage ntlErrorMessage) {
    return error(ntlErrorMessage.getCode(), ntlErrorMessage);
  }

  public static Result error(int code, ErrorMessage ntlErrorMessage) {
    Assert.isTrue(code != 0, "error code should not be zero in error response!");
    return new Result(ntlErrorMessage.getMessage(), code, null);
  }

  public static Result error(String msg) {
    return new Result(msg, -1, null);
  }

  public static Result error(int code, String msg) {
    Assert.isTrue(code != 0, "error code should not be zero in error response!");
    return new Result(msg, code, null);
  }
}
