package com.tofly.base.server.common.data.core.cache;

import org.springframework.cache.Cache;
import org.springframework.cache.CacheManager;
import org.springframework.stereotype.Component;

import java.util.Objects;
import java.util.Optional;

@Component
public class CacheUtils {

  private final String COMMON_STORE_AREA = "yun-teng-iot-common";

  private final CacheManager cacheManager;

  public CacheUtils(CacheManager cacheManager) {
    this.cacheManager = cacheManager;
  }

  public void put(String key, Object value) {
    Objects.requireNonNull(cacheManager.getCache(COMMON_STORE_AREA)).put(key, value);
  }

  public void put(String cacheName, String key, Object value) {
    Cache cache = cacheManager.getCache(cacheName);
    if (null != cache) {
      cache.put(key, value);
    }
  }

  public void invalidateCacheName(String cacheName) {
    Cache cache = cacheManager.getCache(cacheName);
    if (null != cache) {
      cache.invalidate();
    }
  }

  public void invalidate(String key) {
    Cache cache = cacheManager.getCache(COMMON_STORE_AREA);
    if (null != cache) {
      cache.evictIfPresent(key);
    }
  }

  public void invalidate(String cacheName, String key) {
    Cache cache = cacheManager.getCache(cacheName);
    if (null != cache) {
      cache.evictIfPresent(key);
    }
  }

  @SuppressWarnings("unchecked")
  public <T> Optional<T> get(String key) {
    return Optional.ofNullable(cacheManager.getCache(COMMON_STORE_AREA))
        .map(cache -> Objects.requireNonNull(cache).get(key))
        .map(v -> (T) v.get());
  }

  @SuppressWarnings("unchecked")
  public <T> Optional<T> get(String cacheName, String key) {
    return Optional.ofNullable(cacheManager.getCache(cacheName))
        .map(cache -> Objects.requireNonNull(cache).get(key))
        .map(v -> (T) v.get());
  }
}
