package com.tofly.base.server.common.data.dto;


import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.DateSerializer;
import com.tofly.base.server.common.data.common.UpdateGroup;
import com.tofly.base.server.common.data.constant.ModelConstants;
import com.tofly.base.server.common.data.core.exception.EntityCreationException;
import com.tofly.base.server.common.data.utils.ReflectUtils;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.beans.BeanUtils;

import javax.validation.constraints.NotEmpty;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;

@Data
@JsonInclude(JsonInclude.Include.NON_NULL)
public class BaseDTO {
  @NotEmpty(message = "ID不能为空或者空字符串",groups = {UpdateGroup.class})
  @ApiModelProperty(value = "ID")
  private String id;
  /**创建用户*/
  @ApiModelProperty(value = "创建用户")
  @TableField(fill = FieldFill.INSERT)
  private String creator;
  /**创建时间*/
  @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
  @ApiModelProperty(value = "创建时间")
  @JsonSerialize(using = DateSerializer.class)
  @TableField(fill = FieldFill.INSERT)
  private Date createTime;
  /**更新用户*/
  @ApiModelProperty(value = "更新用户")
  @TableField(fill = FieldFill.INSERT_UPDATE)
  private String updater;
  /**更新时间*/
  @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
  @ApiModelProperty(value = "更新时间")
  @JsonSerialize(using = DateSerializer.class)
  @TableField(fill = FieldFill.INSERT_UPDATE)
  private Date updateTime;
  @JsonIgnore
  public <E> void copyToEntity(E entity, String... ignoreProperties) {
    if (ignoreProperties.length == 0) {
      BeanUtils.copyProperties(
          this,
          entity,
          ModelConstants.TablePropertyMapping.CREATE_TIME,
          ModelConstants.TablePropertyMapping.UPDATE_TIME,
          ModelConstants.TablePropertyMapping.CREATOR,
          ModelConstants.TablePropertyMapping.UPDATER);
    } else {
      BeanUtils.copyProperties(this, entity, ignoreProperties);
    }
  }

  @JsonIgnore
  @SuppressWarnings("unchecked")
  public <E> E getEntity(Class<E> clazz) {
    Constructor<?> constructor = ReflectUtils.getEmptyParameterConstructor(clazz);
    if (constructor != null) {
      try {
        E e = (E) constructor.newInstance();
        BeanUtils.copyProperties(this, e);
        return e;
      } catch (InstantiationException | IllegalAccessException | InvocationTargetException e) {
        throw new EntityCreationException(
            "try to get entity from class [" + clazz.getName() + "] failed " + e.getMessage());
      }
    } else {
      throw new EntityCreationException(
          "try to get entity from class [" + clazz.getName() + "] failed");
    }
  }
}
