package com.tofly.base.server.common.data.dto;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.JsonNode;
import com.tofly.base.server.common.data.enums.MenuTypeEnum;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@EqualsAndHashCode(callSuper = true)
@Data
public class MenuDTO extends BaseDTO implements Comparable<MenuDTO> {
    @ApiModelProperty(value = "菜单名称", required = true)
    private String name;

    @ApiModelProperty(value = "上级ID")
    private String parentId;

    @ApiModelProperty(value = "子节点")
    private List<MenuDTO> children;

    @ApiModelProperty(value = "路由地址", required = true)
    private String path;

    @ApiModelProperty(value = "菜单类型")
    private MenuTypeEnum type;

    @ApiModelProperty(value = "权限标识")
    private String permission;

    @ApiModelProperty(value = "排序字段")
    private Integer sort;

    @ApiModelProperty(value = "租户ID")
    @JsonIgnore
    private String tenantId;

    @ApiModelProperty(value = "组件路径", required = true)
    private String component;

    @ApiModelProperty(value = "元数据")
    private JsonNode meta;

    @ApiModelProperty(value = "别名")
    @JsonIgnore
    private String alias;
    
    @ApiModelProperty(value = "菜单子系统")
    private String menuSys;
    
    @ApiModelProperty(value = "菜单别属 1、本地 2、外地")
    private String menuOut;

    @JsonInclude(JsonInclude.Include.NON_NULL)
    @ApiModelProperty(value = "重定向地址")
    private String redirect;

    @JsonInclude(JsonInclude.Include.NON_NULL)
    @ApiModelProperty(value = "区分大小写")
    private Boolean caseSensitive;

    @ApiModelProperty(value = "是否显示图片")
    private Integer isshowimg;

    @ApiModelProperty(value = "图片的Base64字符串")
    private String imgbase64;


    private List<Map<String, String>> fieldMap;

    private List<Map<String, String>> visibleFieldMap;
    public MenuDTO() {
        this.children = new ArrayList<>();
    }

    @Override
    public int compareTo(MenuDTO o) {
        int t1 = this.getSort() == null ? 0 : this.getSort();
        int t2 = o.getSort() == null ? 0 : o.getSort();
        return t1 - t2;
    }
}
