package com.tofly.base.server.common.data.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.tofly.base.server.common.data.common.AddGroup;
import com.tofly.base.server.common.data.enums.UserStatusEnum;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.NotEmpty;
import java.util.Date;
import java.util.Set;

@EqualsAndHashCode(callSuper = false)
@Data
public class UserDTO extends BaseDTO {

  @ApiModelProperty(value = "用户名")
  @NotEmpty(message = "用户名不能为空或字符串", groups = AddGroup.class)
  private String username;

  @ApiModelProperty(value = "真实名称")
  @NotEmpty(message = "姓名不能为空或字符串", groups = AddGroup.class)
  private String realName;

  @ApiModelProperty(value = "密码")
  private transient String password;

  @ApiModelProperty(value = "激活token")
  @JsonInclude(JsonInclude.Include.NON_NULL)
  private String activateToken;

  @ApiModelProperty(value = "客户ID")
  private String customerId;

  @ApiModelProperty(value = "所辖组织ids")
  private String[] organizationIds;

  @ApiModelProperty(value = "角色ids")
  private String[] roleIds;

  @ApiModelProperty(value = "电话号码")
  @NotEmpty(message = "电话号码不能为空或字符串", groups = AddGroup.class)
  private String phoneNumber;

  @ApiModelProperty(value = "邮箱")
  private String email;

  @ApiModelProperty(value = "头像")
  private String avatar;

  @ApiModelProperty(value = "TB的UserId")
  private String tbUser;

  @ApiModelProperty(value = "账号状态")
  private boolean enabled;

  @ApiModelProperty(value = "账号失效时间")
  @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
  private Date accountExpireTime;

  @ApiModelProperty(value = "是否已重置密码")
  private boolean hasPassword;

  @ApiModelProperty(value = "租户ID")
  @JsonInclude(JsonInclude.Include.NON_NULL)
  private String tenantId;

  @ApiModelProperty(value = "租户名称")
  @JsonInclude(JsonInclude.Include.NON_NULL)
  private String tenantName;

  @ApiModelProperty(value = "用户状态")
  @JsonInclude(JsonInclude.Include.NON_NULL)
  private UserStatusEnum userStatusEnum;

  @ApiModelProperty(value = "角色信息列表")
  @JsonInclude(JsonInclude.Include.NON_NULL)
  private Set<PlainRolesDTO> roles;

  @ApiModelProperty(value = "部门列表")
  @JsonInclude(JsonInclude.Include.NON_NULL)
  private Set<String> departments;

  @ApiModelProperty(value = "角色级别：0：超级管理员；1：平台管理员 2：租户账号 3：租户下的账号")
  private Integer level;

  @ApiModelProperty(value = "备注")
  private String remark;

  @ApiModelProperty(value = "账号锁定状态")
  private Boolean locked;
  
  @ApiModelProperty(value = "是否第一次登录 0是 1否")
  private Integer isFirst;

  /**
   *  1在职  2离职
   */
  @ApiModelProperty(value = "1在职  2离职")
  private String status;

  /**
   * 是否为管理员  1 是 0否
   */
  @ApiModelProperty(value = "是否为管理员  1 是 0否")
  private String isAdmin;
  
}
