package com.tofly.base.server.common.data.dto;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.converters.Converter;
import com.alibaba.excel.converters.WriteConverterContext;
import com.alibaba.excel.enums.CellDataTypeEnum;
import com.alibaba.excel.metadata.data.WriteCellData;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.util.Date;
import java.util.Set;

@Data
public class UserInfoDTO {
  @ExcelProperty("用户Id")
  @ApiModelProperty(value = "用户ID")
  private String userId;

  @ApiModelProperty(value = "姓名")
  private String realName;

  @ApiModelProperty(value = "用户名")
  private String username;

  @ApiModelProperty(value = "手机")
  private String phoneNumber;

  @ApiModelProperty(value = "邮箱")
  private String email;

  @ApiModelProperty(value = "创建时间")
  @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
  private Date createTime;

  @ApiModelProperty(value = "过期时间")
  @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
  private Date accountExpireTime;

  @ExcelIgnore
  @ApiModelProperty(value = "头像")
  private String avatar;

  @ApiModelProperty(value = "租户ID")
  private String tenantId;

  @ApiModelProperty(value = "租户名称")
  private String tenantName;

  @ApiModelProperty(value = "主页地址")
  private String homePath;
  /** default: true */
  @ApiModelProperty(value = "是否需要重置密码")
  private boolean needSetPwd = true;

  @ApiModelProperty(value = "钉钉id")
  private String dingId;
  /**
   *  1在职  2离职
   */
  @ApiModelProperty(value = "1在职  2离职")
  private String status;

  /**
   * 是否为管理员  1 是 0否
   */
  @ApiModelProperty(value = "是否为管理员  1 是 0否")
  private String isAdmin;


  /** 添加SetConverter,将Set字符串或者对象转成字符串，对象转成字符串会调用其toString方法，请重写toString方法 */
  @ApiModelProperty(value = "角色ID")
  @ExcelProperty(converter = SetConverter.class)
  private Set<String> roles;

  @ApiModelProperty(value = "角色信息")
  @ExcelProperty(converter = SetConverter.class)
  private Set<PlainRolesDTO> plainRoles;




  public static class SetConverter implements Converter<Set<?>> {
    @Override
    public Class<?> supportJavaTypeKey() {
      return Set.class;
    }

    @Override
    public WriteCellData<?> convertToExcelData(WriteConverterContext<Set<?>> context)
        throws Exception {
      return new WriteCellData<>(
          CellDataTypeEnum.STRING, StringUtils.join(context.getValue(), ","));
    }
  }
}
