/**
 * Copyright © 2016-2022 The Thingsboard Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.tofly.base.server.common.data.id;


import com.tofly.base.server.common.data.EntityType;

import java.util.UUID;


/**
 * Created by ashvayka on 25.04.17.
 */
public class EntityIdFactory {

    public static EntityId getByTypeAndId(String type, String uuid) {
        return getByTypeAndUuid(EntityType.valueOf(type), UUID.fromString(uuid));
    }

    public static EntityId getByTypeAndUuid(String type, UUID uuid) {
        return getByTypeAndUuid(EntityType.valueOf(type), uuid);
    }

    public static EntityId getByTypeAndUuid(EntityType type, String uuid) {
        return getByTypeAndUuid(type, UUID.fromString(uuid));
    }

    public static EntityId getByTypeAndUuid(EntityType type, UUID uuid) {
        switch (type) {
            case TENANT:
                return TenantId.fromUUID(uuid);
            case CUSTOMER:
                return new CustomerId(uuid);
            case USER:
                return new UserId(uuid);
//            case DASHBOARD:
//                return new DashboardId(uuid);
//            case DEVICE:
//                return new DeviceId(uuid);
//            case ASSET:
//                return new AssetId(uuid);
//            case ALARM:
//                return new AlarmId(uuid);
//            case RULE_CHAIN:
//                return new RuleChainId(uuid);
//            case RULE_NODE:
//                return new RuleNodeId(uuid);
//            case ENTITY_VIEW:
//                return new EntityViewId(uuid);
//            case WIDGETS_BUNDLE:
//                return new WidgetsBundleId(uuid);
//            case WIDGET_TYPE:
//                return new WidgetTypeId(uuid);
//            case DEVICE_PROFILE:
//                return new DeviceProfileId(uuid);
//            case TENANT_PROFILE:
//                return new TenantProfileId(uuid);
//            case API_USAGE_STATE:
//                return new ApiUsageStateId(uuid);
//            case TB_RESOURCE:
//                return new TbResourceId(uuid);
//            case OTA_PACKAGE:
//                return new OtaPackageId(uuid);
//            case EDGE:
//                return new EdgeId(uuid);
//            case RPC:
//                return new RpcId(uuid);
        }
        throw new IllegalArgumentException("EntityType " + type + " is not supported!");
    }

//    public static EntityId getByEdgeEventTypeAndUuid(EdgeEventType edgeEventType, UUID uuid) {
//        switch (edgeEventType) {
//            case CUSTOMER:
//                return new CustomerId(uuid);
//            case USER:
//                return new UserId(uuid);
//            case DASHBOARD:
//                return new DashboardId(uuid);
//            case DEVICE:
//                return new DeviceId(uuid);
//            case DEVICE_PROFILE:
//                return new DeviceProfileId(uuid);
//            case ASSET:
//                return new AssetId(uuid);
//            case ALARM:
//                return new AlarmId(uuid);
//            case RULE_CHAIN:
//                return new RuleChainId(uuid);
//            case ENTITY_VIEW:
//                return new EntityViewId(uuid);
//            case WIDGETS_BUNDLE:
//                return new WidgetsBundleId(uuid);
//            case WIDGET_TYPE:
//                return new WidgetTypeId(uuid);
//            case EDGE:
//                return new EdgeId(uuid);
//        }
//        throw new IllegalArgumentException("EdgeEventType " + edgeEventType + " is not supported!");
//    }

}
