/**
 * Copyright © 2016-2022 The Thingsboard Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.tofly.base.server.common.data.security;


import com.tofly.base.entity.user.User;
import com.tofly.base.server.common.data.id.UserId;
import lombok.Getter;
import lombok.Setter;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class SecurityUser extends User {

  private static final long serialVersionUID = -797397440703066079L;

  private Collection<GrantedAuthority> authorities = new ArrayList<>();
  private boolean enabled;
  private UserPrincipal userPrincipal;
  @Getter @Setter private String currentTenantId;
  @Getter @Setter private String currentUserId;
  @Getter @Setter private LocalDateTime userExpireTime;
  @Getter @Setter private LocalDateTime tenantExpireTime;
  @Getter @Setter private boolean ptSysadmin = false;
  @Getter @Setter private boolean isAdmin = false;
  @Getter @Setter private boolean ptTenantAdmin = false;
  @Getter @Setter private boolean ptAdmin = false;
  @Getter @Setter private Set<String> roles;
  @Getter @Setter private String platformUserName;

  public SecurityUser() {
    super();
  }

  public SecurityUser(UserId id) {
    super(id);
  }

  public SecurityUser(User user, boolean enabled, UserPrincipal userPrincipal) {
    super(user);
    this.enabled = enabled;
    this.userPrincipal = userPrincipal;
  }

  public Collection<GrantedAuthority> getAuthorities() {
    if (authorities == null) {
      if (roles == null || roles.isEmpty()) {
        authorities =
            Stream.of(SecurityUser.this.getAuthority())
                .map(authority -> new SimpleGrantedAuthority(authority.name()))
                .collect(Collectors.toList());
      } else {
        authorities =
            this.roles.stream().map(SimpleGrantedAuthority::new).collect(Collectors.toList());
      }
    }
    return authorities;
  }

  public boolean isEnabled() {
    return enabled;
  }

  public UserPrincipal getUserPrincipal() {
    return userPrincipal;
  }

  public void setEnabled(boolean enabled) {
    this.enabled = enabled;
  }

  public void setUserPrincipal(UserPrincipal userPrincipal) {
    this.userPrincipal = userPrincipal;
  }
}
