package com.tofly.base.server.common.data.utils;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

public class ExcelUtil {
  private ExcelUtil() {}

  public static void exportExcel(
      HttpServletResponse response,
      String fileName,
      String sheetName,
      List<?> list,
      Class<?> pojoClass)
      throws IOException {
    if (StringUtils.isBlank(fileName)) {
      fileName = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
    }

    response.setContentType("application/vnd.ms-excel");
    response.setCharacterEncoding("UTF-8");
    fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
    response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
    EasyExcel.write(response.getOutputStream(), pojoClass).sheet(sheetName).doWrite(list);
  }

  @SuppressWarnings("all")
  public static void exportExcelToTarget(
      HttpServletResponse response,
      String fileName,
      String sheetName,
      List<?> sourceList,
      Class<?> targetClass)
      throws Exception {
    List<Object> targetList = new ArrayList<>(sourceList.size());

    for (Object source : sourceList) {
      Object target = targetClass.newInstance();
      BeanUtils.copyProperties(source, target);
      targetList.add(target);
    }

    exportExcel(response, fileName, sheetName, targetList, targetClass);
  }

  public static ByteArrayOutputStream noModelWrite(
      String fileName, List<List<String>> heads, List<?> data) {
    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
    EasyExcel.write(byteArrayOutputStream)
        .head(heads)
        .sheet(fileName)
        // 自适应列宽
        .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
        .doWrite(data);
    return byteArrayOutputStream;
  }

  public static void noModelWrite(
      ExcelWriter excelWriter,
      List<List<String>> heads,
      List<?> data,
      int sheetNo,
      String newSheetName) {
    WriteSheet writeSheet = EasyExcel.writerSheet(sheetNo, newSheetName).head(heads).build();
    excelWriter.write(data, writeSheet);
  }

}
