package com.tofly.base.server.common.data.utils;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

@Slf4j
public class ReflectUtils {

  public static <T> Constructor<?> getEmptyParameterConstructor(Class<T> clazz) {
    Constructor<?> constructor = null;
    Constructor<?>[] constructors = clazz.getConstructors();
    for (Constructor<?> con : constructors) {
      if (con.getParameterCount() == 0) {
        constructor = con;
        break;
      }
    }
    return constructor;
  }

  @SuppressWarnings("unchecked")
  public static <T> List<T> sourceToTarget(
      Collection<?> sourceList, Class<T> target, String... ignoreProperties) {
    if (sourceList == null) {
      return null;
    }

    List<T> targetList = new ArrayList<>(sourceList.size());
    Constructor<?> constructor = getEmptyParameterConstructor(target);
    if (constructor != null) {
      try {
        for (Object source : sourceList) {
          T targetObject = (T) constructor.newInstance();
          BeanUtils.copyProperties(source, targetObject, ignoreProperties);
          targetList.add(targetObject);
        }
      } catch (Exception e) {
        log.error("convert error ", e);
      }
    }
    return targetList;
  }




  /**
   * 获取类的所有属性，包含父类的【public private protected】
   * @param clazz 不确定类型
   * @return 返回List<Field>
   */
  public static List<Field> getAllDeclaredFields(Class<?> clazz){
    List<Field> fieldList = new ArrayList<>();
    for (;clazz !=null;clazz=clazz.getSuperclass()){
      fieldList.addAll(Arrays.asList(clazz.getDeclaredFields()));
    }
    return fieldList;
  }
}
