package com.tofly.base.server.common.data.utils.tools;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.http.HttpStatus;

import java.io.Serializable;

@Data
@NoArgsConstructor
public class ResponseResult<T> implements Serializable {

    private static final long serialVersionUID = -8043533414600873987L;

    /**
     * 状态码
     */
    private Integer code;

    /**
     * 消息
     */
    private String message;

    /**
     * 数据
     */
    private T data;

    /**
     * 成功返回数据
     * @param message 消息
     * @return Result<T>
     */
    public static <T> ResponseResult<T> success(String message){
        ResponseResult<T> responseResult = new ResponseResult<>();
        responseResult.setMessage(message);
        responseResult.setCode(HttpStatus.OK.value());
        return responseResult;
    }

    /**
     * 成功返回数据
     * @param data 数据
     * @return Result<T>
     */
    public static <T> ResponseResult<T> success(T data){
        ResponseResult<T> responseResult = new ResponseResult<>();
        responseResult.setCode(HttpStatus.OK.value());
        responseResult.setMessage(HttpStatus.OK.getReasonPhrase());
        responseResult.setData(data);
        return responseResult;
    }

    /**
     * 成功返回数据
     * @param data 数据
     * @return Result<T>
     */
    public static <T> ResponseResult<T> success(T data,String message){
        ResponseResult<T> responseResult = new ResponseResult<>();
        responseResult.setCode(HttpStatus.OK.value());
        responseResult.setMessage(message);
        responseResult.setData(data);
        return responseResult;
    }

    /**
     * 失败返回数据
     * @param message 消息
     * @return Result<T>
     */
    public static  <T> ResponseResult<T> failed(String message){
        ResponseResult<T> responseResult = new ResponseResult<>();
        responseResult.setMessage(message);
        responseResult.setCode(HttpStatus.INTERNAL_SERVER_ERROR.value());
        return responseResult;
    }

    /**
     * 失败返回数据
     * @param code 状态码
     * @param message 消息
     * @return Result<T>
     */
    public static <T> ResponseResult<T> failed(int code, String message){
        ResponseResult<T> responseResult = new ResponseResult<>();
        responseResult.setMessage(message);
        responseResult.setCode(code);
        return responseResult;
    }
}
