package com.tofly.base.service.area.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.tofly.base.entity.area.SysAreaEntity;
import com.tofly.base.mapper.area.SysAreaMapper;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.dto.SysAreaDTO;
import com.tofly.base.server.common.data.utils.ReflectUtils;
import com.tofly.base.service.area.SysAreaService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.CacheManager;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author: 徐浩然
 * @desc: 中国省市区县
 * @date: 2021/12/16-11:13
 */
@Service
@RequiredArgsConstructor
public class SysAreaServiceImpl implements SysAreaService {

  private final SysAreaMapper sysAreaMapper;
  private final CacheManager cacheManager;

  @Override
  public List<SysAreaDTO> list(SysAreaDTO sysAreaDTO) {
    List<SysAreaDTO> result = cacheManager.getCache(FastIotConstants.CacheConfigKey.AREA).get(sysAreaDTO.getParentId(),List.class);
    if(result == null || result.isEmpty()){
      result = ReflectUtils.sourceToTarget(
              sysAreaMapper.selectList(
                      new LambdaQueryWrapper<SysAreaEntity>()
                              .eq(sysAreaDTO.getLevel() != null, SysAreaEntity::getLevel, sysAreaDTO.getLevel())
                              .eq(
                                      sysAreaDTO.getParentId() != null,
                                      SysAreaEntity::getParentId,
                                      sysAreaDTO.getParentId())
                              .eq(sysAreaDTO.getCode() != null, SysAreaEntity::getCode, sysAreaDTO.getCode())
                              .like(
                                      StringUtils.isNotEmpty(sysAreaDTO.getName()),
                                      SysAreaEntity::getName,
                                      sysAreaDTO.getName())),
              SysAreaDTO.class);
      cacheManager.getCache(FastIotConstants.CacheConfigKey.AREA).put(sysAreaDTO.getParentId(),result);
    }


    return result;
  }

  @Override
  public SysAreaDTO getSysAreaInfoByCode(Long code) {
    if (null == code) {
      return null;
    }
    SysAreaDTO target = new SysAreaDTO();
    SysAreaEntity source =
        sysAreaMapper.selectOne(new LambdaQueryWrapper<SysAreaEntity>().eq(SysAreaEntity::getCode, code));
    if (null != source) {
      BeanUtils.copyProperties(source, target);
    }
    return target;
  }
}
