package com.tofly.base.service.dic.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.tofly.base.config.FileTableEnum;
import com.tofly.base.entity.dic.SysDictItemEntity;
import com.tofly.base.entity.miniofile.File;
import com.tofly.base.mapper.dic.SysDictItemMapper;
import com.tofly.base.server.common.data.core.exception.DataValidationException;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.core.message.ErrorMessage;
import com.tofly.base.server.common.data.dto.SysDictItemDTO;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.server.utils.FilePathBean;
import com.tofly.base.service.AbstractBaseService;
import com.tofly.base.service.dic.SysDictItemService;
import com.tofly.base.service.miniofile.IFileInfoService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Slf4j
@Service
@RequiredArgsConstructor
public class SysDictItemServiceImpl extends AbstractBaseService<SysDictItemMapper, SysDictItemEntity>
        implements SysDictItemService {

    @Autowired
    private IFileInfoService iFileInfoService;

    @Autowired
    private FilePathBean filePathBean;


    @Override
    public TkPageData<SysDictItemDTO> page(Map<String, Object> queryMap) {
        if (StringUtils.isEmpty((CharSequence) queryMap.get("dictId"))) {
            throw new DataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        IPage<SysDictItemEntity> iPage =
                baseMapper.selectPage(
                        getPage(queryMap, "create_time", false),
                        new QueryWrapper<SysDictItemEntity>()
                                .lambda()
                                .like(
                                        queryMap.get("itemText") != null,
                                        SysDictItemEntity::getItemText,
                                        String.valueOf(queryMap.get("itemText")))
                                .eq(
                                        queryMap.get("dictId") != null,
                                        SysDictItemEntity::getDictId,
                                        String.valueOf(queryMap.get("dictId")))
                                .orderByAsc(SysDictItemEntity::getSort)
                );
        TkPageData<SysDictItemDTO> tkPageData = getPageData(iPage, SysDictItemDTO.class);
        Collection<SysDictItemDTO> collection = tkPageData.getItems();
        collection.forEach(item -> {
            List<File> fileList = iFileInfoService.list(
                    new QueryWrapper<File>().lambda()
                            .eq(File::getTableName, FileTableEnum.Sys_Dict_Item.getTableName())
                            .eq(File::getTableId, item.getId())
            );
            item.setFileList(fileList);
        });
        tkPageData.setItems(collection);

        return tkPageData;
    }

    @Override
    @Transactional
    public SysDictItemDTO saveSysDictItem(HttpServletRequest servletRequest, SysDictItemDTO sysDictItemDTO, String tenantId) {


		List<SysDictItemEntity> existCodeList = baseMapper.selectList(new QueryWrapper<SysDictItemEntity>().lambda()
				  .eq(SysDictItemEntity::getItemValue, sysDictItemDTO.getItemValue())
				  .eq(SysDictItemEntity::getDictId, sysDictItemDTO.getDictId()));
		
		if (existCodeList.size()>=1) {
		    throw new TkDataValidationException("已存在相同字典值");
		}
		
		List<SysDictItemEntity> existNameList = baseMapper.selectList(new QueryWrapper<SysDictItemEntity>().lambda()
				  .eq(SysDictItemEntity::getItemText, sysDictItemDTO.getItemText())
				  .eq(SysDictItemEntity::getDictId, sysDictItemDTO.getDictId()));
		
		if (existNameList.size()>=1) {
		    throw new TkDataValidationException("已存在相同字典文本");
		}
		
		SysDictItemEntity sysDictItem = new SysDictItemEntity();
        sysDictItemDTO.copyToEntity(sysDictItem);
        sysDictItem.setTenantId(tenantId);
        baseMapper.insert(sysDictItem);
        iFileInfoService.savafile(servletRequest, filePathBean.getSysDict(), sysDictItem.getId(), FileTableEnum.Sys_Dict_Item.getTableName());
        sysDictItem.copyToDTO(sysDictItemDTO);
        List<File> fileList = iFileInfoService.list(
                new QueryWrapper<File>().lambda()
                        .eq(File::getTableName, FileTableEnum.Sys_Dict_Item.getTableName())
                        .eq(File::getTableId, sysDictItem.getId())
        );
        sysDictItemDTO.setFileList(fileList);
        return sysDictItemDTO;
    }

    @Override
    @Transactional
    public boolean deleteSysDictItem(Set<String> sysDictItemIds) {
        return baseMapper.deleteBatchIds(sysDictItemIds) > 0;
    }

    @Override
    public List<String> findDictCodeByIds(Set<String> sysDictItemIds) {

        return baseMapper.getItemCodes(sysDictItemIds);



    }

    @Override
    @Transactional
    public SysDictItemDTO updateSysDictItem(HttpServletRequest servletRequest, SysDictItemDTO sysDictItemDTO, String tenantId) {

		List<SysDictItemEntity> existCodeList = baseMapper.selectList(new QueryWrapper<SysDictItemEntity>().lambda()
			  .eq(SysDictItemEntity::getItemValue, sysDictItemDTO.getItemValue())
			  .eq(SysDictItemEntity::getDictId, sysDictItemDTO.getDictId())
			  .ne(SysDictItemEntity::getId, sysDictItemDTO.getId()));
		
		if (existCodeList.size()>=1) {
		      throw new TkDataValidationException("已存在相同字典值");
		}
		  
		List<SysDictItemEntity> existNameList = baseMapper.selectList(new QueryWrapper<SysDictItemEntity>().lambda()
				  .eq(SysDictItemEntity::getItemText, sysDictItemDTO.getItemText())
				  .eq(SysDictItemEntity::getDictId, sysDictItemDTO.getDictId())
				  .ne(SysDictItemEntity::getId, sysDictItemDTO.getId()));
		  
		if (existNameList.size()>=1) {
		      throw new TkDataValidationException("已存在相同字典文本");
		}
		
		
		SysDictItemEntity sysDictItem = baseMapper.selectById(sysDictItemDTO.getId());
        sysDictItemDTO.copyToEntity(sysDictItem);
        baseMapper.updateById(sysDictItem);
        iFileInfoService.savafile(servletRequest, filePathBean.getSysDict(), sysDictItem.getId(), FileTableEnum.Sys_Dict_Item.getTableName());
        sysDictItem.copyToDTO(sysDictItemDTO);
        List<File> fileList = iFileInfoService.list(
                new QueryWrapper<File>().lambda()
                        .eq(File::getTableName, FileTableEnum.Sys_Dict_Item.getTableName())
                        .eq(File::getTableId, sysDictItem.getId())
        );
        sysDictItemDTO.setFileList(fileList != null ? fileList : sysDictItemDTO.getFileList());
        return sysDictItemDTO;
    }

    @Override
    public List<SysDictItemDTO> findDictItemByDictCode(String dictCode) {
        if (StringUtils.isEmpty(dictCode)) {
            throw new DataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }



        List<SysDictItemDTO> sysDictItemDTOList = baseMapper.getDictItemInfo(null, dictCode);


        for (int i = 0; i < sysDictItemDTOList.size(); i++) {
            SysDictItemDTO sysDictItemDTO = sysDictItemDTOList.get(i);
            sysDictItemDTOList.set(i,sysDictItemDTO);
        }
        return sysDictItemDTOList;
    }
}
