package com.tofly.base.service.dic.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.tofly.base.entity.dic.SysDictEntity;
import com.tofly.base.mapper.dic.SysDictItemMapper;
import com.tofly.base.mapper.dic.SysDictMapper;
import com.tofly.base.server.common.data.constant.FastIotConstants;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.core.message.ErrorMessage;
import com.tofly.base.server.common.data.dto.SysDictDTO;
import com.tofly.base.server.common.data.dto.SysDictItemDTO;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.server.utils.FilePathBean;
import com.tofly.base.service.AbstractBaseService;
import com.tofly.base.service.dic.SysDictService;
import com.tofly.base.service.miniofile.IFileInfoService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Slf4j
@Service
@RequiredArgsConstructor
public class SysDictServiceImpl extends AbstractBaseService<SysDictMapper, SysDictEntity>
        implements SysDictService {

    private final SysDictItemMapper sysDictItemMapper;

    @Autowired
    private IFileInfoService iFileInfoService;

    @Autowired
    private FilePathBean filePathBean;

    @Override
    public TkPageData<SysDictDTO> page(Map<String, Object> queryMap) {
        IPage<SysDictEntity> iPage =
                baseMapper.selectPage(
                        getPage(queryMap, "create_time", false),
                        new QueryWrapper<SysDictEntity>()
                                .lambda()
                                .like(
                                        queryMap.get("dictName") != null,
                                        SysDictEntity::getDictName,
                                        String.valueOf(queryMap.get("dictName")))
                                .like(
                                        queryMap.get("dictCode") != null,
                                        SysDictEntity::getDictCode,
                                        String.valueOf(queryMap.get("dictCode")))

                );

        TkPageData<SysDictDTO> tkPageData= getPageData(iPage, SysDictDTO.class);
//        Collection<SysDictDTO> collection = tkPageData.getItems();
//        collection.forEach(item->{
//            List<File> fileList = iFileInfoService.list(
//                    new QueryWrapper<File>().lambda()
//                            .eq(File::getTableName, FileTableEnum.Sys_Dict.getTableName())
//                            .eq(File::getTableId, item.getId())
//            );
//            item.setFileList(fileList);
//        });
//        tkPageData.setItems(collection);
        return tkPageData;
    }

    @Override
    @Transactional
    public SysDictDTO saveSysDict(HttpServletRequest servletRequest, SysDictDTO sysDictDTO, String tenantId) {
        // 新增之前先判断该租户是否已添加
        SysDictDTO querySysDict = baseMapper.getDictInfoByCode(tenantId, sysDictDTO.getDictCode());
        if (querySysDict != null) {
            throw new TkDataValidationException("已存在相同字典编码");
        }
        
        SysDictEntity existDitc = baseMapper.selectOne(new QueryWrapper<SysDictEntity>().lambda()
    			  .eq(SysDictEntity::getDictName, sysDictDTO.getDictName()));
        if (existDitc != null) {
            throw new TkDataValidationException("已存在相同字典名称");
        }
        
        SysDictEntity sysDict = new SysDictEntity();
        sysDictDTO.copyToEntity(sysDict);
        sysDict.setTenantId(tenantId);
        
        baseMapper.insert(sysDict);
//        iFileInfoService.savafile(servletRequest, filePathBean.getSysDict(), sysDict.getId(), FileTableEnum.Sys_Dict.getTableName());
//        sysDict.copyToDTO(sysDictDTO);
//        List<File> fileList = iFileInfoService.list(
//                new QueryWrapper<File>().lambda()
//                        .eq(File::getTableName, FileTableEnum.Sys_Dict.getTableName())
//                        .eq(File::getTableId, sysDictDTO.getId())
//        );
//        sysDictDTO.setFileList(fileList);
        return sysDictDTO;
    }

    @Override
    @Transactional
    public boolean deleteSysDict(Set<String> sysDictIds) {
        for (String id : sysDictIds) {
            List<SysDictItemDTO> sysDictItemDTO = sysDictItemMapper.getDictItemInfo(id, null);
            if (sysDictItemDTO != null && sysDictItemDTO.size() > FastIotConstants.MagicNumber.ZERO) {
                throw new TkDataValidationException(ErrorMessage.EXIST_LEADER_MEMBER_RELATION.getMessage());
            }
        }
//        List<File> fileList = iFileInfoService.list(new QueryWrapper<File>()
//                .lambda()
//                .select(File::getId,File::getFilePath)
//                .in(File::getTableId, sysDictIds)
//                .eq(File::getTableName, FileTableEnum.Sys_Dict.getTableName())
//        );
//        if (Objects.nonNull(fileList) && fileList.size() > 0) {
//            iFileInfoService.removeFileListByFile(fileList);
//        }
        return baseMapper.deleteBatchIds(sysDictIds) > 0;
    }

    @Override
    @Transactional
    public SysDictDTO updateSysDict(HttpServletRequest servletRequest, SysDictDTO sysDictDTO, String tenantId) {
        
    	List<SysDictEntity> existCodeList = baseMapper.selectList(new QueryWrapper<SysDictEntity>().lambda()
  			  .eq(SysDictEntity::getDictCode, sysDictDTO.getDictCode())
  			  .ne(SysDictEntity::getId, sysDictDTO.getId()));
    	
        if (existCodeList.size()==1) {
            throw new TkDataValidationException("已存在相同字典编码");
        }
        
        List<SysDictEntity> existNameList = baseMapper.selectList(new QueryWrapper<SysDictEntity>().lambda()
    			  .eq(SysDictEntity::getDictName, sysDictDTO.getDictName())
    			  .ne(SysDictEntity::getId, sysDictDTO.getId()));
        
        if (existNameList.size()==1) {
            throw new TkDataValidationException("已存在相同字典名称");
        }
    	    	
    	SysDictEntity sysDict = baseMapper.selectById(sysDictDTO.getId());
        sysDictDTO.copyToEntity(sysDict);
        sysDict.setTenantId(tenantId);
        baseMapper.updateById(sysDict);
        
//        iFileInfoService.savafile(servletRequest, filePathBean.getSysDict(), sysDict.getId(), FileTableEnum.Sys_Dict.getTableName());
//        sysDict.copyToDTO(sysDictDTO);
//        List<File> fileList = iFileInfoService.list(
//                new QueryWrapper<File>().lambda()
//                        .eq(File::getTableName, FileTableEnum.Sys_Dict.getTableName())
//                        .eq(File::getTableId, sysDictDTO.getId())
//        );
//        sysDictDTO.setFileList(fileList != null ? fileList : sysDictDTO.getFileList());
        return sysDictDTO;
    }

    @Override
    public SysDictDTO getDictInfoByCode(String dictCode) {
        return baseMapper.getDictInfoByCode(null,dictCode);
    }
}
