package com.tofly.base.service.enterprise.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.tofly.base.entity.enterprise.SysEnterpriseEntity;
import com.tofly.base.mapper.enterprise.SysEnterpriseMapper;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.core.message.ErrorMessage;
import com.tofly.base.server.common.data.dto.AreaInfoDTO;
import com.tofly.base.server.common.data.dto.SysAreaDTO;
import com.tofly.base.server.common.data.dto.SysEnterpriseDTO;
import com.tofly.base.service.AbstractBaseService;
import com.tofly.base.service.area.SysAreaService;
import com.tofly.base.service.enterprise.SysEnterpriseService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author: 徐浩然
 * @desc: 企业信息
 * @date: 2021/12/3-18:40
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class SysEnterpriseServiceImpl
    extends AbstractBaseService<SysEnterpriseMapper, SysEnterpriseEntity>
    implements SysEnterpriseService {

  private final SysAreaService sysAreaService;

  @Override
  public boolean save(SysEnterpriseDTO sysAppDesignDTO, String tenantId) {
    SysEnterpriseEntity checkSysEnterprise =
        baseMapper.selectOne(
            new LambdaQueryWrapper<SysEnterpriseEntity>().eq(SysEnterpriseEntity::getTenantId, tenantId));
    if (null != checkSysEnterprise) {
      throw new TkDataValidationException(ErrorMessage.DATA_ALREADY_EXISTS.getMessage());
    }
    SysEnterpriseEntity entity = sysAppDesignDTO.getEntity(SysEnterpriseEntity.class);
    entity.setTenantId(tenantId);
    return baseMapper.insert(entity) > 0;
  }

  @Override
  public SysEnterpriseDTO get(String tenantId) {
    SysEnterpriseDTO sysEnterpriseDTO = baseMapper.get(tenantId);
    if (null == sysEnterpriseDTO) {
      return null;
    }
    if (null != sysEnterpriseDTO.getCodeTown()) {
      AreaInfoDTO sourceArea = new AreaInfoDTO();
      SysAreaDTO town = sysAreaService.getSysAreaInfoByCode(sysEnterpriseDTO.getCodeTown());
      sourceArea.setNameTown(town.getName());
      sourceArea.setCodeTown(town.getCode());
      SysAreaDTO county = sysAreaService.getSysAreaInfoByCode(town.getParentId());
      sourceArea.setNameCoun(county.getName());
      sourceArea.setCodeCoun(county.getCode());
      SysAreaDTO city = sysAreaService.getSysAreaInfoByCode(county.getParentId());
      sourceArea.setNameCity(city.getName());
      sourceArea.setCodeCity(city.getCode());
      SysAreaDTO province = sysAreaService.getSysAreaInfoByCode(city.getParentId());
      sourceArea.setNameProv(province.getName());
      sourceArea.setCodeProv(province.getCode());
      SysAreaDTO country = sysAreaService.getSysAreaInfoByCode(province.getParentId());
      sourceArea.setNameCountry(country.getName());
      sourceArea.setCodeCountry(country.getCode());

      // 查询同级别县
      SysAreaDTO var1 = new SysAreaDTO();
      var1.setParentId(town.getParentId());
      sourceArea.setTowns(sysAreaService.list(var1));
      // 查询同级别区
      SysAreaDTO var2 = new SysAreaDTO();
      var2.setParentId(county.getParentId());
      sourceArea.setCouns(sysAreaService.list(var2));
      // 查询同级别市
      SysAreaDTO var3 = new SysAreaDTO();
      var3.setParentId(city.getParentId());
      sourceArea.setCities(sysAreaService.list(var3));
      // 查询同级别省
      SysAreaDTO var4 = new SysAreaDTO();
      var4.setParentId(province.getParentId());
      sourceArea.setProvs(sysAreaService.list(var4));
      // 查询同级别省
      SysAreaDTO var5 = new SysAreaDTO();
      var5.setParentId(country.getParentId());
      sourceArea.setCountries(sysAreaService.list(var5));
      sysEnterpriseDTO.setSysTown(sourceArea);
    }
    return sysEnterpriseDTO;
  }

  @Override
  @Transactional
  public boolean update(SysEnterpriseDTO sysAppDesignDTO, String tenantId) {
    if (sysAppDesignDTO.getId() != null) {
      return baseMapper.update(
              sysAppDesignDTO.getEntity(SysEnterpriseEntity.class),
              new LambdaQueryWrapper<SysEnterpriseEntity>().eq(SysEnterpriseEntity::getTenantId, tenantId))
          > 0;
    } else {
      return save(sysAppDesignDTO, tenantId);
    }
  }
}
