package com.tofly.base.service.log;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.tofly.base.entity.log.SysLogEntity;
import com.tofly.base.server.common.data.EntityType;
import com.tofly.base.server.common.data.dto.SysLogDTO;
import com.tofly.base.server.common.data.enums.ActionType;
import com.tofly.base.server.common.data.utils.tools.TkPageData;
import com.tofly.base.service.BaseService;

public interface TkSysLogService extends BaseService<SysLogEntity> {


    /**
     * 日志详细信息
     *
     * @param entityId 日志ID
     * @return
     */
    SysLogDTO detail(String entityId);

    /**
     * 异常日志列表
     *
     * @param pageInfrom 分页配置信息
     * @param startTime  时间过滤窗口起点
     * @param endTime    时间过滤窗口终点
     * @param tenantId   租户ID，超级管理员该值为null
     * @param customerId 客户ID，租户管理员该值为null
     * @param actionType 日志类型
     * @return
     */
    TkPageData<SysLogDTO> exceptionPage(IPage<SysLogEntity> pageInfrom, Long startTime, Long endTime, String tenantId, String customerId, ActionType actionType);

    /**
     * 操作日志列表
     *
     * @param pageInfrom 分页配置信息
     * @param startTime  时间过滤窗口起点
     * @param endTime    时间过滤窗口终点
     * @param tenantId   租户ID，超级管理员该值为null
     * @param customerId 客户ID，租户管理员该值为null
     * @param actionType 日志类型
     * @return
     */
    TkPageData<SysLogDTO> operatePage(IPage<SysLogEntity> pageInfrom, Long startTime, Long endTime, String tenantId, String customerId, EntityType entityType, ActionType actionType);

    /**
     * 登录日志列表
     *
     * @param pageInfrom 分页配置信息
     * @param startTime  时间过滤窗口起点
     * @param endTime    时间过滤窗口终点
     * @param tenantId   租户ID，超级管理员该值为null
     * @param customerId 客户ID，租户管理员该值为null
     * @param actionType 日志类型
     * @return
     */
    TkPageData<SysLogDTO> loginPage(IPage<SysLogEntity> pageInfrom, Long startTime, Long endTime, String tenantId, String customerId, ActionType actionType);


    /**
     * 备份日志列表
     *
     * @param pageInfrom 分页配置信息
     * @param startTime  时间过滤窗口起点
     * @param endTime    时间过滤窗口终点
     * @param tenantId   租户ID，超级管理员该值为null
     * @param customerId 客户ID，租户管理员该值为null
     * @param actionType 日志类型
     * @return
     */
    TkPageData<SysLogDTO> backupLogPage(IPage<SysLogEntity> pageInfrom, Long startTime, Long endTime, String tenantId, String customerId, ActionType actionType);

}
