package com.tofly.base.service.miniofile.imp;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.base.entity.miniofile.File;
import com.tofly.base.mapper.miniofile.FileMapper;
import com.tofly.base.server.common.data.dto.FileUploadResponse;
import com.tofly.base.service.miniofile.FileStorageService;
import com.tofly.base.service.miniofile.IFileInfoService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.MultiValueMap;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


@Service
@Slf4j
@RequiredArgsConstructor
public class FileInfoServiceImpl extends ServiceImpl<FileMapper, File> implements IFileInfoService {

    @Autowired
    private FileMapper fileMapper;
    @Autowired
    private FileStorageService fileStorageService;


    @Override
    public List<String> getFilePathList(List<BigDecimal> fileIdList) {
        List<String> filePathList = new ArrayList<>();
        fileIdList.forEach(fileId -> {
                    filePathList.add(this.getById(fileId).getFilePath());
                }
        );
        return filePathList;
    }

    @Override
    public List<File> getFileList(List<BigDecimal> fileIdList) {
        QueryWrapper<File> wrapper = new QueryWrapper();
        wrapper.lambda().in(fileIdList.size() > 0, File::getId, fileIdList);
        return this.list(wrapper);
    }


    public StringBuilder savafile(HttpServletRequest servletRequest, String filePathBean, String tableID, String tableName) {
        StringBuilder fileIds = new StringBuilder("");
        if(servletRequest instanceof  MultipartHttpServletRequest){
            MultiValueMap<String, MultipartFile> multipartFileMap = ((MultipartHttpServletRequest) servletRequest).getMultiFileMap();

            if (multipartFileMap.size() > 0) {
                multipartFileMap.forEach((k, v) -> {
                    //web端所有文件
                    if (k.contains("file") || k.isEmpty()) {
                        v.forEach(f -> {
                            String fileName = f.getOriginalFilename();
                            FileUploadResponse fileUploadResponse = fileStorageService.upload(f, filePathBean);
                            File localFile = new File();
                            //localFile.setId(BigDecimal.valueOf(fileMapper.getTotal()));
                            localFile.setFilePath(fileUploadResponse.getFileName());
                            localFile.setFileName(f.getOriginalFilename());
                            localFile.setFileType("0");
                            //关联表名称
                            localFile.setTableId(tableID);
                            localFile.setTableName(tableName);
                            //获取后缀
                            String suffix = fileName.substring(fileName.lastIndexOf("."));
                            localFile.setFileFormat(suffix);
                            localFile.setFileSize(f.getSize() + "kb");
                            localFile.setOtherName(fileUploadResponse.getFileName().replace(filePathBean, ""));
                            localFile.setStatusFlag("0");
                            localFile.setCreateTime(new Date());
                            //localFile.setCreateUser();
                            this.save(localFile);
                            fileIds.append(localFile.getId()).append(",");
                        });
                    }
                    //
                    if (k.contains("fyptzyz")) {
                        v.forEach(f -> {
                            String fileName = f.getOriginalFilename();
                            FileUploadResponse fileUploadResponse = fileStorageService.upload(f, filePathBean);
                            File localFile = new File();
                            //localFile.setId(BigDecimal.valueOf(fileMapper.getTotal()));
                            localFile.setFilePath(fileUploadResponse.getFileName());
                            localFile.setFileName(f.getOriginalFilename());
                            localFile.setFileType("1");
                            //关联表名称
                            localFile.setTableId(tableID);
                            localFile.setTableName(tableName);
                            //获取后缀
                            String suffix = fileName.substring(fileName.lastIndexOf("."));
                            localFile.setFileFormat(suffix);
                            localFile.setFileSize(f.getSize() + "kb");
                            localFile.setOtherName(fileUploadResponse.getFileName().replace(filePathBean, ""));
                            localFile.setStatusFlag("0");
                            localFile.setCreateTime(new Date());
                            //localFile.setCreateUser();
                            this.save(localFile);
                            fileIds.append(localFile.getId()).append(",");
                        });
                    }
                });
            }
        }

        return fileIds;
    }

    public List<File> savafile(HttpServletRequest servletRequest, String filePathBean) {
        MultiValueMap<String, MultipartFile> multipartFileMap = ((MultipartHttpServletRequest) servletRequest).getMultiFileMap();
        List<File> fileList = new ArrayList<>();
        if (multipartFileMap.size() > 0) {
            multipartFileMap.forEach((k, v) -> {
                //web端所有文件
                v.forEach(f -> {
                    String fileName = f.getOriginalFilename();
                    FileUploadResponse fileUploadResponse = fileStorageService.upload(f, filePathBean);
                    File localFile = new File();
                    //localFile.setId(BigDecimal.valueOf(fileMapper.getTotal()));
                    localFile.setFilePath(fileUploadResponse.getFileName());
                    localFile.setFileName(f.getOriginalFilename());
                    localFile.setFileType("0");
                    //关联表名称
                    localFile.setTableId("0");
                    localFile.setTableName("other");
                    //获取后缀
                    String suffix = fileName.substring(fileName.lastIndexOf("."));
                    localFile.setFileFormat(suffix);
                    localFile.setFileSize(f.getSize() + "kb");
                    localFile.setOtherName(fileUploadResponse.getFileName().replace(filePathBean, ""));
                    localFile.setStatusFlag("0");
                    localFile.setCreateTime(new Date());
                    //localFile.setCreateUser();
                    this.save(localFile);
                    fileList.add(localFile);
                });
            });
        }
        return fileList;
    }

    @Override
    public Boolean removeFileListById(List<BigDecimal> fileIdList) {
        if (fileIdList.size() > 0) {
            List<File> files = this.listByIds(fileIdList);
            // 删除远程文件
            files.forEach(k -> {
                try {
                    fileStorageService.removeFile(k.getFilePath());
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            });
            return this.removeByIds(fileIdList);
        }
        return false;
    }

    @Override
    public Boolean removeFileListByFile(List<File> fileList) {
        if (fileList.size() > 0) {
            // 删除远程文件
            List<BigDecimal> idslist = new ArrayList<>();
            fileList.forEach(k -> {
                try {
                    idslist.add(k.getId());
                    fileStorageService.removeFile(k.getFilePath());
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            });
            return this.removeByIds(idslist);
        }
        return false;
    }

}
