package com.tofly.base.service.organization.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import com.tofly.base.ding.base.entity.DingDeptVo;
import com.tofly.base.entity.organization.TkOrganizationEntity;
import com.tofly.base.entity.organization.TkUserOrganizationMappingEntity;
import com.tofly.base.entity.user.SysUserEntity;
import com.tofly.base.mapper.organization.OrganizationMapper;
import com.tofly.base.mapper.organization.UserOrganizationMappingMapper;
import com.tofly.base.mapper.user.UserMapper;
import com.tofly.base.server.common.data.core.exception.TkDataValidationException;
import com.tofly.base.server.common.data.core.message.ErrorMessage;
import com.tofly.base.server.common.data.dto.DeleteDTO;
import com.tofly.base.server.common.data.dto.OrganizationDTO;
import com.tofly.base.server.common.data.utils.ReflectUtils;
import com.tofly.base.server.common.data.utils.tree.TreeUtils;
import com.tofly.base.service.AbstractBaseService;
import com.tofly.base.service.organization.TkOrganizationService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import java.util.*;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class TkOrganizationServiceImpl extends AbstractBaseService<OrganizationMapper, TkOrganizationEntity>
        implements TkOrganizationService {

    private final UserOrganizationMappingMapper userOrganizationMappingMapper;
    private final UserMapper userMapper;
// private final DeviceMapper deviceMapper;
//  private final SceneLinkageMapper sceneMapper;
//  private final AlarmContactMapper contactMapper;

    @Override
    @Transactional
    public OrganizationDTO saveOrganization(
            OrganizationDTO organizationDTO,
            String tenantId,
            boolean isCustomerOrPlatform,
            String userId) {
        Assert.notNull(organizationDTO.getName(), "organization name cannot be null");
        if (StringUtils.isNotBlank(organizationDTO.getParentId())) {
            // 新增有父组织,拿到父组织
            TkOrganizationEntity organization = baseMapper.selectById(organizationDTO.getParentId());
            if (organization == null) {
                // 父组织不存在
                throw new TkDataValidationException("parent organization not exist!");
            } else {
                if (!organization.getTenantId().equals(tenantId)) {
                    throw new TkDataValidationException("Different current tenants");
                }
            }
        }
        TkOrganizationEntity organization = new TkOrganizationEntity();
        organizationDTO.setTenantId(tenantId);
        organizationDTO.copyToEntity(organization, "id");
        // 如果当前账号为平台管理员或者用户
        if (isCustomerOrPlatform) {
            if (StringUtils.isEmpty(organizationDTO.getParentId())) {
                // 获取当前用户的最顶级组织ID的父ID
                List<OrganizationDTO> myOrganizations = getMyOrganizations(false, false, tenantId, userId);
                if (null != myOrganizations && myOrganizations.size() > 0) {
                    OrganizationDTO organizationDto = myOrganizations.get(0);
                    String parentId = organizationDto.getParentId();
                    organization.setParentId(parentId);
                }
            }
        }
        baseMapper.insert(organization);
        if (isCustomerOrPlatform) {
            // 当前用户组织数组
            List<String> currentUserOrganization =
                    userOrganizationMappingMapper
                            .selectList(
                                    new LambdaQueryWrapper<TkUserOrganizationMappingEntity>()
                                            .eq(TkUserOrganizationMappingEntity::getUserId, userId))
                            .stream()
                            .map(TkUserOrganizationMappingEntity::getOrganizationId)
                            .collect(Collectors.toList());
            currentUserOrganization.add(organization.getId());
            // 绑定用户组织关系
            bindUserToOrganization(
                    tenantId,
                    userId,
                    currentUserOrganization.toArray(new String[currentUserOrganization.size()]));
        }
        organization.copyToDTO(organizationDTO);
        return organizationDTO;
    }

    @Override
    @Transactional
    public boolean deleteOrganizations(DeleteDTO deleteDTO, String tenantId) {
        if (null == deleteDTO || deleteDTO.getIds().isEmpty()) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        
        String[] ids = deleteDTO.getIds().toArray(new String[deleteDTO.getIds().size()]);
        Set<String> idToDelete =
                baseMapper.deletefindOrganizationTreeList(tenantId, Arrays.asList(ids)).stream()
                        .map(OrganizationDTO::getId)
                        .collect(Collectors.toSet());
        if (idToDelete.size() != ids.length) {
            throw new TkDataValidationException("待删除数据存在子项,不能删除!");
        }
        
        List<TkUserOrganizationMappingEntity> userOrganizationMappingList =
                userOrganizationMappingMapper.selectList(
                        new QueryWrapper<TkUserOrganizationMappingEntity>()
                                .lambda()
                                .in(TkUserOrganizationMappingEntity::getOrganizationId, idToDelete));
        
        if (!userOrganizationMappingList.isEmpty()) {
            throw new TkDataValidationException("待删除数据存在关联用户,不能删除!");
        }
    /*if (sceneMapper.selectCount(
            new QueryWrapper<TkSceneLinkageEntity>().lambda().in(TkSceneLinkageEntity::getOrganizationId, ids))
        > 0) {
      throw new TkDataValidationException(
          ErrorMessage.ORGANIZATION_EXISTED_SCENE_REACT.getMessage());
    }
    if (contactMapper.selectCount(
            new QueryWrapper<TkAlarmContactEntity>().lambda().in(TkAlarmContactEntity::getOrganizationId, ids))
        > 0) {
      throw new TkDataValidationException(
          ErrorMessage.ORGANIZATION_EXISTED_ALARM_CONTACT.getMessage());
    }

    // 查询是否有设备使用该组织
    for (String id : ids) {
      List<TkDeviceEntity> deviceList =
          deviceMapper.selectList(
              new QueryWrapper<TkDeviceEntity>()
                  .lambda()
                  .eq(TkDeviceEntity::getTenantId, tenantId)
                  .eq(TkDeviceEntity::getOrganizationId, id));
      if (!deviceList.isEmpty()) {
        throw new TkDataValidationException("待删除数据存在关联设备,不能删除!");
      }
    }*/
        doDeleteUserOrganizationMapping(idToDelete);
        return baseMapper.deleteBatchIds(idToDelete) > 0;
    }

    private void doDeleteUserOrganizationMapping(Collection<String> organizationIds) {
        userOrganizationMappingMapper.delete(
                new QueryWrapper<TkUserOrganizationMappingEntity>()
                        .lambda()
                        .in(TkUserOrganizationMappingEntity::getOrganizationId, organizationIds));
    }

    @Override
    @Transactional
    public OrganizationDTO updateOrganization(
            OrganizationDTO organizationDTO,
            String tenantId,
            boolean isCustomerOrPlatform,
            String userId) {
        Assert.notNull(organizationDTO.getName(), "organization name cannot be null");
        // 判断自己父id不能为本身的id
        if (organizationDTO.getParentId() != null
                && organizationDTO.getParentId().equals(organizationDTO.getId())) {
            throw new TkDataValidationException(ErrorMessage.IT_CANT_EQUAL_ITSELF.name());
        }
        // 选择的组织
        TkOrganizationEntity organization = baseMapper.selectById(organizationDTO.getId());
        TkOrganizationEntity parent = null;
        // 父级组织不为空,则查询到父级组织
        if (StringUtils.isNotBlank(organizationDTO.getParentId())&&(!"1".equals(organizationDTO.getParentId()))) {
            TkOrganizationEntity parentOrganization = baseMapper.selectById(organizationDTO.getParentId());

            if (parentOrganization.getTenantId().equals(tenantId)) {
                parent = parentOrganization;
            }


        }
        if (organization == null) {
            return null;
        }
        // 数据一致则返回给前端
//        if (organization.getName().equals(organizationDTO.getName())
//                && organization.getSort() == organizationDTO.getSort()
//                && (organization.getRemark() != null
//                && organization.getRemark().equals(organizationDTO.getRemark()))
//                && Objects.equals(organization.getParentId(), organizationDTO.getParentId())) {
//            return organizationDTO;
//        }
        organization.setName(organizationDTO.getName());
        organization.setRemark(organizationDTO.getRemark());
        organization.setSort(organizationDTO.getSort());
        organization.setAdminUser(organizationDTO.getAdminUser());
        if (parent != null) {
            organization.setParentId(parent.getId());
        } else {
            if(("1".equals(organizationDTO.getParentId()))){
                organization.setParentId("1");
            }else{
                organization.setParentId(null);
            }
        }
        baseMapper.updateById(organization);
        organization.copyToDTO(organizationDTO);
        return organizationDTO;
    }

    @Override
    public List<OrganizationDTO> getMyOrganizations(
            boolean isPtSysAdmin, boolean isPtTenantAdmin, String tenantId, String currentUserId) {
        // 如果是租户管理员,则返回该租户id下的所有组织
        if (isPtTenantAdmin) {
            return findOrganizationTree(tenantId);
        } else {
            // 返回当前用户组织id集
            Set<String> organizationIds =
                    userOrganizationMappingMapper
                            .selectList(
                                    new QueryWrapper<TkUserOrganizationMappingEntity>()
                                            .lambda()
                                            .in(TkUserOrganizationMappingEntity::getUserId, currentUserId)
                            )
                            .stream()
                            .map(TkUserOrganizationMappingEntity::getOrganizationId)
                            .collect(Collectors.toSet());
            List<TkOrganizationEntity> organizations;
            if (organizationIds.size() > 0 || isPtSysAdmin) {
                organizations =
                        baseMapper.selectList(
                                new QueryWrapper<TkOrganizationEntity>()
                                        .lambda()
                                        .eq(TkOrganizationEntity::getTenantId, tenantId)

                                        .in(organizationIds.size() > 0 &&!isPtSysAdmin, TkOrganizationEntity::getId, organizationIds)
                                        .orderByAsc(TkOrganizationEntity::getSort)
                        );
                if(CollectionUtil.isNotEmpty(organizations)){
                    for (TkOrganizationEntity organization : organizations) {
                        String userIdStr = organization.getAdminUser();
                        if(StringUtils.isNotBlank(userIdStr)){
                            String[] userIds = userIdStr.split(",");
                            List<SysUserEntity> sysUserEntities = userMapper.selectBatchIds(Arrays.asList(userIds));
                            String usernameStr = sysUserEntities.stream().map(SysUserEntity::getRealName).collect(Collectors.joining(","));
                            organization.setAdminUsername(usernameStr);
                        }
                    }
                }
            } else {
                return null;
            }
            return TreeUtils.buildTree(ReflectUtils.sourceToTarget(organizations, OrganizationDTO.class));
        }
    }

    /**
     * 查询租户id下的所有组织
     *
     * @param tenantId 租户ID
     * @return 组织列表
     */
    private List<OrganizationDTO> findOrganizationTree(String tenantId) {
        List<OrganizationDTO> organizationTreeList =
                baseMapper.findOrganizationTreeList(tenantId, null);
        return buildOrganizationDTOTree(organizationTreeList);
    }

    private List<OrganizationDTO> buildOrganizationDTOTree(
            List<OrganizationDTO> organizationTreeList) {
        Map<String, OrganizationDTO> organizationDTOMap =
                new LinkedHashMap<>(organizationTreeList.size());
        HashSet<OrganizationDTO> organizationDTOSet = Sets.newHashSet(organizationTreeList);
        organizationDTOSet.forEach(
                organizationDTO -> organizationDTOMap.put(organizationDTO.getId(), organizationDTO));
        List<OrganizationDTO> result = Lists.newArrayList();
        for (OrganizationDTO node : organizationDTOMap.values()) {
            OrganizationDTO parent = organizationDTOMap.get(node.getParentId());
            if (parent != null && !(node.getId().equals(parent.getId()))) {
                parent.getChildren().add(node);
                continue;
            }
            result.add(node);
        }
        Collections.sort(result);
        sortOrganization(result);
        return result;
    }

    private void sortOrganization(List<OrganizationDTO> result) {
        result.forEach(node -> {
            if (node.getChildren().size() > 0) {
                List<OrganizationDTO> children = node.getChildren();
                Collections.sort(children);
                sortOrganization(children);
            }
        });
    }

  


    @Override
    @Transactional
    public void bindUserToOrganization(String tenantId, String userId, String[] organizationIds) {
        SysUserEntity user = userMapper.selectById(userId);
        if (user == null) {
            throw new TkDataValidationException("所选用户不存在");
        }
        if (!user.getTenantId().equals(tenantId)) {
            throw new TkDataValidationException("所选用户不可用");
        }
        if (null != organizationIds && organizationIds.length > 0) {
        	
       	 userOrganizationMappingMapper.delete(
       	            new QueryWrapper<TkUserOrganizationMappingEntity>()
       	                .lambda()
       	                .eq(TkUserOrganizationMappingEntity::getUserId, userId));
       	 
       	 Set<String> organizationIdSet = new HashSet<String>(Arrays.asList(organizationIds));
       	
       	 organizationIdSet.stream()
               .map(organizationId -> new TkUserOrganizationMappingEntity(userId, organizationId))
               .forEach(userOrganizationMappingMapper::insert);    
        }
    }

    @Override
    @Transactional
    public void unBindUserToOrganization(Set<String> userIds) {
        for (String userId : userIds) {
            SysUserEntity user = userMapper.selectById(userId);
            if (user == null) {
                throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
            }
            userOrganizationMappingMapper.delete(
                    new QueryWrapper<TkUserOrganizationMappingEntity>()
                            .lambda()
                            .eq(TkUserOrganizationMappingEntity::getUserId, userId));
        }
    }

    @Override
    public OrganizationDTO findOrganizationById(String id, String tenantId) {
        TkOrganizationEntity organization =
                baseMapper.selectOne(
                        new LambdaQueryWrapper<TkOrganizationEntity>()
                                .eq(TkOrganizationEntity::getTenantId, tenantId)
                                .eq(TkOrganizationEntity::getId, id));
        if (null == organization) {
            throw new TkDataValidationException(ErrorMessage.INVALID_PARAMETER.getMessage());
        }
        return organization.getDTO(OrganizationDTO.class);
    }

    @Override
    public Map<String, String> findOrganizationMap(Set<String> ids) {
        Map<String, String> map = new LinkedHashMap<>();
        if (ids == null || ids.size() == 0) return map;
        List<TkOrganizationEntity> list = baseMapper.selectList(
                new QueryWrapper<TkOrganizationEntity>().lambda().in(TkOrganizationEntity::getId, ids));
        if (list != null) {
            list.forEach(e -> {
                map.put(e.getId(), e.getName());
            });
        }
        return map;
    }

    @Override
    @Transactional
    public void saveOrUpdateOrganizationByDingDept(List<DingDeptVo> list) {


        list.sort(new Comparator<DingDeptVo>() {
            @Override
            public int compare(DingDeptVo o1, DingDeptVo o2) {
                return (int)( o1.getParentId()-o2.getParentId());
            }
        });
        Map<String,String>  parentIdMap = new HashMap<>();
        //查询根目录  id为1的数据
        TkOrganizationEntity tkOrganizationEntity1 = baseMapper.selectOne(Wrappers.<TkOrganizationEntity>lambdaQuery()
                .eq(TkOrganizationEntity::getDingId, "1"));
        parentIdMap.put("1",tkOrganizationEntity1.getId());
        List<DingDeptVo> updateList = new ArrayList<>();
        for (DingDeptVo dingDeptVo : list) {
            final Long dingDeptId = dingDeptVo.getDeptId();
            //通过钉钉id查询 部门
             TkOrganizationEntity tkOrganizationEntity = baseMapper.selectOne(Wrappers.<TkOrganizationEntity>lambdaQuery()
                    .eq(TkOrganizationEntity::getDingId, String.valueOf(dingDeptId)));
            if(tkOrganizationEntity!=null){

               //暂时不进行更新
            }else{
                //生成部门信息存入数据库
                tkOrganizationEntity = new TkOrganizationEntity();
                tkOrganizationEntity.setName(dingDeptVo.getName());
                tkOrganizationEntity.setParentId(parentIdMap.get(dingDeptVo.getParentId()));
                //TODO  租户id暂时写死
                tkOrganizationEntity.setTenantId("13814000-1dd2-11b2-8080-808080808080");
                tkOrganizationEntity.setDingId(String.valueOf(dingDeptId));
                baseMapper.insert(tkOrganizationEntity);
                if(StringUtils.isEmpty(tkOrganizationEntity.getParentId())){
                    updateList.add(dingDeptVo);
                }
            }
            parentIdMap.put(String.valueOf(dingDeptId),tkOrganizationEntity.getId());

        }

        int index = 0;
        // 循环处理  10次   填充 上一阶段  parentid 未填充的数据   还可以优化
        while (index<10){
            if (updateList.size()>0){
                for (DingDeptVo dingDeptVo : updateList) {
                    final Long dingDeptId = dingDeptVo.getDeptId();
                    TkOrganizationEntity tkOrganizationEntity = baseMapper.selectOne(Wrappers.<TkOrganizationEntity>lambdaQuery()
                            .eq(TkOrganizationEntity::getDingId, String.valueOf(dingDeptId)).isNull(TkOrganizationEntity::getParentId));
                    if(tkOrganizationEntity==null){
                        continue;
                    }
                    final String s = parentIdMap.get(dingDeptVo.getParentId());
                    if(StringUtils.isNotEmpty(s)){
                        tkOrganizationEntity.setParentId(s);
                        baseMapper.updateById(tkOrganizationEntity);
                        parentIdMap.put(String.valueOf(dingDeptId),tkOrganizationEntity.getId());
                    }

                }
            }
            index++;
        }

    }

    @Override
    public List<String> getChildrenId(String id) {
        Assert.notNull(id, "id can not be null");
        List<TkOrganizationEntity> orgList = baseMapper.selectList(Wrappers.lambdaQuery(TkOrganizationEntity.class)
                .select(TkOrganizationEntity::getId,TkOrganizationEntity::getParentId));
        List<String> childrenIdList = new ArrayList<>();
        childrenIdList.add(id);
        return getChildrenIdByParentId(id,orgList,childrenIdList);
    }

    @Override
    public List<String> getByUserId(String userId) {
        List<TkOrganizationEntity> orgList = baseMapper.selectList(Wrappers.lambdaQuery(TkOrganizationEntity.class).like(TkOrganizationEntity::getAdminUser, userId)
                .select(TkOrganizationEntity::getId));
        if(CollectionUtil.isEmpty(orgList)){
            return new ArrayList<>();
        }
        return orgList.stream().map(TkOrganizationEntity::getId).collect(Collectors.toList());
    }

    @Override
    public List<TkOrganizationEntity> getAllOrganization() {
        return baseMapper.selectList(Wrappers.lambdaQuery(TkOrganizationEntity.class));
    }

    private List<String> getChildrenIdByParentId(String id, List<TkOrganizationEntity> orgList,List<String> childrenIdList) {
        Assert.notNull(id, "id can not be null");
        List<String> orgIdList = orgList.stream().filter(org -> id.equals(org.getParentId()))
                .map(TkOrganizationEntity::getId)
                .collect(Collectors.toList());

        if(CollectionUtil.isNotEmpty(orgIdList)){
            for (String orgId : orgIdList) {
                childrenIdList.add(orgId);
                getChildrenIdByParentId(orgId,orgList,childrenIdList);
            }
        }
        return childrenIdList;
    }
}
