package com.tofly.base.service.user;



import com.tofly.base.entity.user.SysTenantRoleEntity;
import com.tofly.base.entity.user.SysUserEntity;
import com.tofly.base.server.common.data.dto.UserDTO;
import com.tofly.base.server.common.data.dto.UserDetailsDTO;
import com.tofly.base.server.common.data.dto.UserInfoDTO;
import com.tofly.base.server.common.data.dto.request.AccountReqDTO;
import com.tofly.base.server.common.data.dto.request.RoleOrOrganizationReqDTO;
import com.tofly.base.server.common.data.dto.request.SendResetPasswordEmailMsg;
import com.tofly.base.server.common.data.query.TsValue;
import com.tofly.base.server.common.data.utils.tools.TkPageData;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

public interface TkUserService {
  List<UserDetailsDTO> findUserDetailsByUsername(String username,String tenantId);

  String getActivationLink(String userId, boolean isPtSysadmin, String tenantId);

  UserInfoDTO me(String userId, String tenantId, Set<String> currentRoles);

  TkPageData<UserDTO> tenantPage(Map<String, Object> queryMap, String tenantId);


  UserDTO saveTenantAdmin(UserDTO userDTO, boolean isPtSysadmin, String tenantId);

  void resetPassword(String userId, String tenantId,String password);  

  void forgetPassword(String phoneNumber,AccountReqDTO forget);

  List<UserDetailsDTO> getUserByPhoneNumber(String phoneNumber);

  /**
   * 验证租户角色是否有效
   * @param tenantId
   */
  List<SysTenantRoleEntity> validateTenantRole(String tenantId);

  UserDTO findUserInfoById(String id);
  /**
   * 发送密码重置新消息
   *
   * @param msg 请求信息
   */
  void sendRestPasswordMsg(String tenantId,SendResetPasswordEmailMsg msg);

  SysUserEntity validateChangePasswordAccount(AccountReqDTO accountReqDTO);

  Optional<List<UserDTO>> getOrganizationUserByOrganizationId(
      String organizationId, String tenantId);

  /**
   * 修改密码
   *
   * @param user 账号信息
   * @return 修改成功、失败
   */
  boolean changePassword(SysUserEntity user);

  /**
   * 通过用户ID查找TB的CustomerId
   *
   * @param id 用户ID
   * @return CustomerId
   */
  String findUserCustomerIdById(String id);

  /**
   * 修改个人中心
   * @param userDTO 修改信息
   * @return 修改后的信息
   */
  UserDTO updatePersonalCenter(UserDTO userDTO);

  /**
   * 获取当前用户的客户
   * @param organizationId 组织ID
   * @param tenantId 用户ID
   * @param isTenantAdmin 是否是租户管理员
   * @return 租户创建的所有用户
   */
  List<UserDTO> getMyCustomers(String organizationId,String tenantId,boolean isTenantAdmin);

  /**
   * 异步获取用户列表
   * @param startTs 开始时间
   * @param endTs 结束时间
   * @param ts 时间
   * @return 客户信息
   */
  CompletableFuture<TsValue> findUsersAsyncByTs(LocalDateTime startTs, LocalDateTime endTs, long ts);

  /**
   * 检查账号是否存在
   * @param userId 用户ID
   * @param level 级别：0：超级管理员；1：平台管理员；2：租户账号；3：租户下的账号
   * @return true 存在 false 不存在
   */
  UserDTO checkAccount(String userId,Integer level);

  /**
   * 安全测评加入
   * @param ids
   * @return
   */
  Map<String, String> findUserMap(Set<String> ids);

  UserDTO getUserByPhoneNumberOrUsername(String principalValue);

  Set<String> getUserRole(String id);


  TkPageData<UserDTO> page(
          Map<String, Object> queryMap,
          boolean isPtSysadmin,
          boolean isTenantAdmin);

  UserDTO saveAccount(
          UserDTO userDTO, boolean sendEmail, boolean sendMsg, boolean isTenantAdmin, String tenantId);
  Optional<UserDTO> getUser(String userId, boolean isPtSysadmin, String tenantId);


  Optional<List<UserDTO>>  getUserByUsername(String username);


  UserDTO updateUser(UserDTO userDTO, boolean isPtSysadmin, String tenantId);

  boolean deleteUser(Set<String> userIds, boolean isPtSysadmin, String tenantId);

  /**
   * 判断账号是否注册过
   *
   * @param username 账号
   * @return 账号信息
   */
  UserDTO accountExist(String username);

  /**
   * 判断电话是否注册过
   * @param phoneNumber 手机号
   * @return
   */
  SysUserEntity findUserByPhoneNumber(String phoneNumber,String userId);


  SysUserEntity findUserByDingId(String dingId);

  void validateUserNameAndPhoneNumberAndEmail(UserDTO userDTO);

  /**
   * 查询用户关联的Role或者Group
   *
   * @param roleOrOrganizationReqDTO 参数
   * @return 返回关联的Role或者Group
   */
  String[] getUserRoleOrOrganization(RoleOrOrganizationReqDTO roleOrOrganizationReqDTO);
  
  
  
	/**
	 * 通讯录
	 * @return
	 */
	List<Map<String,String>> getAddressBook();	
  

	public  void pullUserFromDing(long deptId);

  List<UserDTO> getUserIdByRealName(List<String> nameList);

    List<String> getOrgIdListByUserId(String id);
}
