package com.tofly.base.service.user;

import java.util.List;

/**
 * @author: 徐浩然
 * @desc: 组织用户关联表
 * @date: 2021/12/3-10:17
 */
public interface UserOrganizationMappingService {

    List<String> getUserIdByOrganizationIds(String tenantId, String... ids);

    /**
     * 添加或更新用户组织关系
     *
     * @param userId          用户ID
     * @param organizationIds 组织列表
     * @param isUpdate        true更新 false新增
     */
    void addOrUpdateUserOrganizationMapping(
            String userId, List<String> organizationIds, boolean isUpdate);


    /**
     * 根据用户ID查询其所拥有的组织ID列表
     *
     * @param userId 用户ID
     * @return 组织ID集合
     */
    List<String> getOrganizationIdsByUserId(String userId);
    List<String> getOrganizationNamesByUserId(String id);


    /**
     * 当前用户可用的所有组织ID
     *
     * @param isPtTenantAdmin 当前用户是否为平台管理员
     * @param tenantId        租户ID
     * @param userId          用户ID
     * @param organizationId  组织ID
     * @return
     */
    List<String> getEnableOrganizationIds(boolean isPtTenantAdmin, String tenantId, String userId, String organizationId);

    /**
     * 比较一个组织与另一个用户查出来的组织相同的数据结果
     * @param organizationIds 源组织
     * @param userId 用户ID
     * @return 返回相同的组织
     */
    List<String> compareOrganizationIdsByCustomerId(List<String> organizationIds,String userId);


}
