package com.tofly.base.utils;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.concurrent.TimeoutException;


@ConfigurationProperties(prefix = "message.mq")
@Component
@Data
public class MqUtils {
	
	public String url;
	public int port;
	public String username;
	public String password;
			
	public void createMessage(String queueName,String messageBody) {
        // 创建一个连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        factory.setHost(url);
        factory.setPort(port);
        factory.setUsername(username);
        factory.setPassword(password);
        try {
            Connection connection = factory.newConnection();
            Channel channel = connection.createChannel();
            /**
             * 生成一个队列
             * 1.队列名称
             * 2.队列里面的消息是否持久化，默认消息存储在内存中
             * 3.该队列是否只供一个消费者进行消费，是否进行共享，可以多个消费者消费
             * 4.是否自动删除，最后一个消费者断开连接以后，该队列是否自动删除，true 自动删除
             * 5.其他参数
             */
            channel.queueDeclare(queueName,false,false,false,null);
            String message = messageBody;
            /**
             * 发送消息
             * 1.发送到哪个交换机
             * 2.路由的 key
             * 3.其他参数
             * 4.发送的消息体
             */
            channel.basicPublish("",queueName,null,message.getBytes());
        } catch (IOException e) {
            e.printStackTrace();
        } catch (TimeoutException e) {
            e.printStackTrace();
        }
    }
}