package com.tofly.report.jmreport.config;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.modules.jmreport.api.JmReportTokenServiceI;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

/**
 * 自定义积木报表鉴权(如果不进行自定义，则所有请求不做权限控制)
 * 1.自定义获取登录token
 * 2.自定义获取登录用户
 */
@Slf4j
@Component
public class JimuReportTokenService implements JmReportTokenServiceI {


    private static String tokenSigningKey = "1qaz2wsx3edc4rfv5tgb6yhn7ujm1qaz2wsx3edc4rfv5tgb6yhn7ujm1qaz2wsx3edc4rfv5tgb6yhn7ujm1qaz2wsx3edc4rfv5tgb6yhn7ujm";

    /**
     * 通过请求获取Token
     * @param request
     * @return
     */
    @Override
    public String getToken(HttpServletRequest request) {
        try {
            String token = request.getParameter("token");
            if (!StringUtils.isEmpty(token)) {
                if (token.startsWith("Bearer")) {
                    token = token.replace("Bearer", "").trim();
                }
                if (token.startsWith("bearer")) {
                    token = token.replace("bearer", "").trim();
                }
                return token;
            } else {
                token = request.getHeader("X-Authorization");
                if (!StringUtils.isEmpty(token)) {
                    if (token.startsWith("Bearer")) {
                        token = token.replace("Bearer", "").trim();
                    }
                    if (token.startsWith("bearer")) {
                        token = token.replace("bearer", "").trim();
                    }
                    return token;
                }
            }
            return null;
        } catch (Exception e) {
            log.info(e.getMessage());
            return null;
        }
//        return "123456";
    }

    /**
     * 自定义获取租户
     *
     * @return
     */
    @Override
    public String getTenantId() {
//        String headerTenantId = null;
//        HttpServletRequest request = JimuSpringContextUtils.getHttpServletRequest();
//        if (request != null) {
//            headerTenantId = request.getHeader(JmConst.HEADER_TENANT_ID);
//            if(OkConvertUtils.isEmpty(headerTenantId)){
//                headerTenantId = request.getParameter(JmConst.TENANT_ID);
//            }
//        }
//        return headerTenantId;
        return "1";
    }


    /**
     * 通过Token获取登录人用户名
     * @param token
     * @return
     */
    @Override
    public String getUsername(String token) {
        try {
            String jwt = getToken();
            if (!StringUtils.isEmpty(jwt)) {
                Claims claims = Jwts.parser()
                        .setSigningKey(tokenSigningKey)
                        .parseClaimsJws(jwt)
                        .getBody();
                return claims.get("sub", String.class);
            }
            return null;
        } catch (Exception e) {
            //e.printStackTrace();
            throw new RuntimeException("token is null");
        }
    }

    /**
     * 自定义用户拥有的角色
     *
     * @param token
     * @return
     */
    @Override
    public String[] getRoles(String token) {
        return new String[]{"admin"};
    }

    /**
     * Token校验
     * @param token
     * @return
     */
    @Override
    public Boolean verifyToken(String token) {
        String username = getUsername(token);
        return StringUtils.isNotBlank(username);
//        return true;
    }

    /**
     *  自定义请求头
     * @return
     */
    @Override
    public HttpHeaders customApiHeader() {
        HttpHeaders header = new HttpHeaders();
        header.add("custom-header1", "Please set a custom value 1");
        header.add("token", "token value 2");
        return header;
    }
}