package com.tofly.pms.base.controller;
 

public class Result<T> {
    private String code;
 
    private String msg;
 
    private Boolean successFlag;
 
    private T data;
 
    public static <T> Result<T> success() {
        Result<T> result = new Result<>();
        result.successFlag(true);
        return result;
    }
 
    public static <T> Result<T> success(T data) {
        Result<T> result = new Result<>();
        result.successFlag(true).data(data);
        return result;
    }
 
    public static <T> Result<T> fail(String code, String errorMsg) {
        Result<T> result = new Result<>();
        result.successFlag(false).code(code).msg(errorMsg);
        return result;
    }
 
    public static <T> Result<T> fail(ExceptionEnum exceptionEnum) {
        Result<T> result = new Result<>();
        result.successFlag(false).code(exceptionEnum.getCode()).msg(exceptionEnum.getErrorMsg());
        return result;
    }
 
    public Result<T> code(String code) {
        this.code = code;
        return this;
    }
 
    public Result<T> msg(String msg) {
        this.msg = msg;
        return this;
    }
 
    public Result<T> successFlag(Boolean successFlag) {
        this.successFlag = successFlag;
        return this;
    }
 
    public Result<T> data(T data) {
        this.data = data;
        return this;
    }
 
    public String getCode() {
        return code;
    }
 
    public String getMsg() {
        return msg;
    }
 
    public T getData() {
        return data;
    }
 
    public Boolean getSuccessFlag() {
        return successFlag;
    }
}