package com.tofly.pms.generate;

import com.baomidou.mybatisplus.generator.AutoGenerator;
import com.baomidou.mybatisplus.generator.config.*;
import com.google.common.base.CaseFormat;
import com.tofly.pms.glossary.constants.GenerateConstant;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.time.LocalDate;
import java.util.*;


/**
 * Desc:快速生成
 */
@Slf4j
public class FastAutoGenerator {

    /**
     * 数据源配置
     */
    private  static final DataSourceConfig DATA_SOURCE_CONFIG = new DataSourceConfig
            .Builder("jdbc:postgresql://221.182.8.141:14100/thingskit","postgres","tofly@028..")
            .build();
    // mapper文件的生成路径
    private static final String MAPPER_PATH = "";
    // 基础包名
    private static final String packageName = "com.tofly.yxBase";
    // 模块名
    private static final String moduleName = "yx-base-service";
    // 表名
    private static final String tableName = "";

    public static void main(String[] args) {
        AutoGenerator generator = new AutoGenerator(DATA_SOURCE_CONFIG);
        generator.strategy(strategyConfig().build());
        generator.injection(injectionConfig().build());
        generator.global(globalConfig().build());
        generator.template(templateConfig().build());
        generator.packageInfo(packageConfig().build());
        generator.execute();
    }

    /**
     * 全局配置
     */
    private static GlobalConfig.Builder globalConfig() {
        String projectPath = System.getProperty("user.dir");
        return  new GlobalConfig.Builder()
//                .fileOverride() // 覆盖已生成文件
                .disableOpenDir() // 禁止打开输出目录	默认值:true
                .author("Mu") //作者名
                .outputDir(projectPath + "/" + moduleName + "/src/main/java") // 指定输出目录
                .enableSwagger(); // 开启 swagger 模式	默认值:false
    }

    /**
     * 模板配置
     */
    private static TemplateConfig.Builder templateConfig() {
        return  new TemplateConfig.Builder()
                .entity("/templates/vm/entity.java")
                .mapper("/templates/vm/mapper.java")
                .service("/templates/vm/service.java")
                .serviceImpl("/templates/vm/serviceimpl.java")
                .controller("/templates/vm/controller.java")
                .mapperXml("/templates/vm/mapper.xml");
    }

    /**
     * 策略配置
     */
    private static StrategyConfig.Builder strategyConfig() {
        return new StrategyConfig.Builder()
                .addInclude(tableName);
    }


    /**
     * 包配置
     */
    private static PackageConfig.Builder packageConfig() {
        return  new PackageConfig.Builder()
                .parent(packageName)
                .entity("domain")
                .mapper("mapper")
                .service("service")
                .serviceImpl("service.impl")
                .controller("controller")
                .pathInfo(Collections.singletonMap(OutputFile.mapperXml, MAPPER_PATH));
    }


    /**
     * 注入配置
     */
    private static InjectionConfig.Builder injectionConfig(){
        Map<String, Object> map = new HashMap<>();
        setAttr(tableName, DATA_SOURCE_CONFIG, map);
        return new InjectionConfig.Builder().customMap(map);

    }


    /**
     * 组装模板属性
     *
     * @param tableName        表名
     * @param dataSourceConfig 数据源
     * @param map              模板里面 自定义的属性
     * @param
     */
    private static void setAttr(String tableName, DataSourceConfig dataSourceConfig, Map<String, Object> map){
        List<Map<String, Object>> columns = new ArrayList<>();
        // 获取表信息sql
        String tableSql = "select table_name  from information_schema.tables " +
                "where table_schema = 'public' and table_name = '" + tableName + "'";
        // 获取字段信息sql
        String columnSql = "SELECT a.attname as column_name,format_type(a.atttypid,-1) as data_type, col_description(a.attrelid,a.attnum) as column_comment " +
                "'FROM pg_class as c,pg_attribute as a where a.attrelid = c.oid and a.attnum>0 and atttypid != 0 and c.relname = ''" + tableName + "'";

        // 利用现有的dataSourceConfig来获取数据库连接，查询表字段及备注等信息
        try(
                Connection conn = dataSourceConfig.getConn();
                PreparedStatement psTable = conn.prepareStatement(tableSql);
                ResultSet rsTable = psTable.executeQuery();
                PreparedStatement pscolumns= conn.prepareStatement(columnSql);
                ResultSet rscolumns = pscolumns.executeQuery();
        ){
            if(rsTable.next()){
                String table_name = rsTable.getString("table_name");
                map.put("tableName",table_name);
//                map.put("comment",rsTable.getString("table_comment"));
                // 类名 大驼峰
                map.put("upperClassName", CaseFormat.UPPER_UNDERSCORE.to(CaseFormat.UPPER_CAMEL, table_name));

                // 对象名 小驼峰
                map.put("lowerClassName", CaseFormat.LOWER_UNDERSCORE.to(CaseFormat.LOWER_CAMEL, table_name));
            }
            while (rscolumns.next()){
                Map<String, Object> columnMap = new HashMap<>();
                // 列名字、数据类型、java属性名字、java属性类型、备注
                columnMap.put("column_name",rscolumns.getString("column_name"));
                columnMap.put("data_type",rscolumns.getString("data_type"));
                columnMap.put("javaLowerAttrName",CaseFormat.LOWER_UNDERSCORE.to(CaseFormat.LOWER_CAMEL, rscolumns.getString("column_name")));
                columnMap.put("javaAttrType",columnTypeToJavaType(rscolumns.getString("data_type")));
                columnMap.put("column_comment", rscolumns.getString("column_comment"));
                columns.add(columnMap);
            }
        } catch (Exception e) {
            log.info("----------sql执行出错");
            e.printStackTrace();
        }
        map.put("columns",columns);
        map.put("datetime", LocalDate.now());
        map.put("packageName", packageName);
    }

    /**
     * 数据库类型转换为java类型
     *
     * @param columnType 数据库类型
     * @return java类型
     */
    private static String  columnTypeToJavaType(String columnType) {
        if(StringUtils.isNotEmpty(columnType)){
            if(Arrays.asList(GenerateConstant.COLUMN_TYPE_STR).contains(columnType)){
                return GenerateConstant.TYPE_STRING;
            }
            if(Arrays.asList(GenerateConstant.COLUMN_TYPE_TIME).contains(columnType)){
                return GenerateConstant.TYPE_DATE;
            }
            if (Arrays.asList(GenerateConstant.COLUMN_TYPE_NUMBER).contains(columnType)) {
                return GenerateConstant.TYPE_INTEGER;
            }
            if (Arrays.asList(GenerateConstant.COLUMN_TYPE_BIGINT).contains(columnType)) {
                return GenerateConstant.TYPE_LONG;
            }
            if (Arrays.asList(GenerateConstant.COLUMN_TYPE_FLOAT).contains(columnType)) {
                return GenerateConstant.TYPE_DOUBLE;
            }
            if (Arrays.asList(GenerateConstant.COLUMN_TYPE_DOUBLE).contains(columnType)) {
                return GenerateConstant.TYPE_DOUBLE;
            }
            if (Arrays.asList(GenerateConstant.COLUMN_TYPE_DECIMAL).contains(columnType)) {
                return GenerateConstant.TYPE_BIGDECIMAL;
            }
        }
        return  null;
    }

}

