package com.tofly.pms.project.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.base.controller.Result;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.project.vo.ProjectNameAndId;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.entity.HoursW;
import com.tofly.pms.working.service.HoursWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;
import java.util.function.Function;

/**
 * 项目基础信息表;(TF_PM_PROJECT_W)表控制层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "项目基础信息表对象功能接口")
@RestController
@RequestMapping("/tfPmProjectW")
public class TfPmProjectWController {
    @Autowired
    private ProjectWService projectWService;
    @Autowired
    private ProjectTeamWService projectTeamWService;
    @Autowired
    private HoursWService hoursService;

    /**
     * 通过ID查询单条数据
     * 2、项目成员【即项目团队中设置的项目成员】：可以查看自己参与的项目，但是不允许编辑、新增、删除。
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<ProjectW> queryById(@PathVariable("id") Integer id) {
        return ResponseEntity.ok(projectWService.queryById(id));
    }

    /**
     * 分页查询
     * 2、项目成员【即项目团队中设置的项目成员】：可以查看自己参与的项目，但是不允许编辑、新增、删除。
     *
     * @param projectW 筛选条件
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<ProjectW>> paginQuery(ProjectW projectW, Page page) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "用户不合法");
        List<Integer> projectIdList = projectTeamWService.listObjs(Wrappers.lambdaQuery(ProjectTeamW.class)
                .like(ProjectTeamW::getAdminId, userID)
                .or()
                .like(ProjectTeamW::getManageId, userID)
                .or()
                .like(ProjectTeamW::getSoftHeadUserId, userID)
                .or()
                .like(ProjectTeamW::getSoftGroupUserId, userID)
                .or()
                .like(ProjectTeamW::getSoftProjectManageId, userID)
                .or()
                .like(ProjectTeamW::getDevId, userID)
                .or()
                .like(ProjectTeamW::getWebId, userID)
                .or()
                .like(ProjectTeamW::getBackendId, userID)
                .or()
                .like(ProjectTeamW::getMobileId, userID)
                .or()
                .like(ProjectTeamW::getTestId, userID)
                .or()
                .like(ProjectTeamW::getImplementionId, userID)
                .or()
                .like(ProjectTeamW::getDataId, userID)
                .or()
                .like(ProjectTeamW::getUiId, userID)
                .or()
                .like(ProjectTeamW::getOhterId, userID)
                .select(ProjectTeamW::getProjectId), item -> {
            if (item instanceof Integer) {
                return (Integer) item;
            }
            throw new RuntimeException("类型转换异常");
        });

        RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
        Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
        String name = projectW.getName();
        projectW.setName(null);
        projectW.setContractName(null);
        projectW.setCustomName(null);
        LambdaQueryWrapper<ProjectW> projectWLambdaQueryWrapper = new QueryWrapper<>(projectW)
                .lambda()
                .eq(ProjectW::getDeletedFlag, 0)
                .orderByDesc(ProjectW::getCreateTime);
        if (!has && CollectionUtil.isNotEmpty(projectIdList)) {
            projectWLambdaQueryWrapper.in(ProjectW::getId, projectIdList);
        } else if (!has && CollectionUtil.isEmpty(projectIdList)) {
            return ResponseEntity.ok(new Page<>());
        }
        projectWLambdaQueryWrapper.like(StringUtils.isNotBlank(name), ProjectW::getName, name)
                .or().like(StringUtils.isNotBlank(name), ProjectW::getContractName, name)
                .or().like(StringUtils.isNotBlank(name), ProjectW::getCustomName, name);
        final Page page1 = projectWService.page(page, projectWLambdaQueryWrapper);
        final List<ProjectW> records = page1.getRecords();
        for (ProjectW record : records) {
            final Integer id = record.getId();

            final ProjectTeamW one = projectTeamWService.getOne(Wrappers.<ProjectTeamW>lambdaQuery()
                    .eq(ProjectTeamW::getProjectId, id));
            if (one != null) {
                record.setTotalUser(one.getTotalUser());
            } else {
                record.setTotalUser(0);
            }


        }

        return ResponseEntity.ok(page1);
    }

    /**
     * 新增数据
     * 2、项目成员【即项目团队中设置的项目成员】：可以查看自己参与的项目，但是不允许编辑、新增、删除。
     *
     * @param projectW 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<ProjectW> add(@RequestBody ProjectW projectW) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "用户不合法");
        RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
        Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
        if (!has) {
            throw new RuntimeException("没有权限");
        }
        projectWService.save(projectW);
        if(StringUtils.isNotBlank(projectW.getAdminId())){
            projectTeamWService.save(ProjectTeamW.builder()
                    .projectId(projectW.getId())
                    .adminId(projectW.getAdminId())
                    .build());
        }
        return ResponseEntity.ok(projectW);
    }

    /**
     * 更新数据
     * 2、项目成员【即项目团队中设置的项目成员】：可以查看自己参与的项目，但是不允许编辑、新增、删除。
     *
     * @param projectW 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<ProjectW> edit(@RequestBody ProjectW projectW) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "用户不合法");
        ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectW.getId())
                .like(ProjectTeamW::getAdminId, userID));
        if (Objects.isNull(one)) {
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if (!has) {
                throw new RuntimeException("没有权限");
            }
        }
        projectWService.updateById(projectW);
        if(StringUtils.isNotBlank(projectW.getAdminId())) {
            projectTeamWService.update(Wrappers.lambdaUpdate(ProjectTeamW.class)
                    .set(ProjectTeamW::getAdminId, projectW.getAdminId())
                    .eq(ProjectTeamW::getProjectId, projectW.getId()));
        }
        return ResponseEntity.ok(projectW);
    }

    /**
     * 通过主键删除数据
     * 2、项目成员【即项目团队中设置的项目成员】：可以查看自己参与的项目，但是不允许编辑、新增、删除。
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public ResponseEntity<Boolean> deleteById(String ids) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "用户不合法");
        RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
        Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
        final String[] split = ids.split(",");
        for (String s : split) {
            if (!has) {
                ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                        .eq(ProjectTeamW::getProjectId, Integer.valueOf(s))
                        .like(ProjectTeamW::getAdminId, userID));
                Assert.notNull(one, "没有权限");
            }
            long count = hoursService.count(Wrappers.lambdaQuery(HoursW.class).eq(HoursW::getProjectId, Integer.valueOf(s)));
            Assert.isTrue(count == 0, "项目下存在工时登记，不能删除");

            projectWService.update(Wrappers.lambdaUpdate(ProjectW.class)
                    .set(ProjectW::getDeletedFlag, 1)
                    .eq(ProjectW::getId, Integer.valueOf(s)));
            projectTeamWService.remove(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, Integer.valueOf(s)));
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

    @ApiOperation("分页查询")
    @GetMapping("/getCustomNameList")
    public Result<List<String>> getCustomNameList() {
        return Result.success(projectWService.getCustomNameList());
    }

    @ApiOperation("项目筛选控件中只显示当前账号是项目成员的项目")
    @GetMapping("/getMyProjectList")
    public Result<List<ProjectW>> getMyProjectList() {
        return Result.success(projectWService.getMyProjectList());
    }

    @ApiOperation("根据项目id查询项目成员")
    @GetMapping("/getUserByProjectId")
    public Result<List<UserVo>> getUserByProjectId(Integer projectId) {
        return Result.success(projectWService.getUserByProjectId(projectId));
    }

    @ApiOperation("查询系统中所有的项目")
    @GetMapping("/getAllProject")
    public Result<List<ProjectNameAndId>> getAllProject() {
        return Result.success(projectWService.getAllProject());
    }

}