package com.tofly.pms.project.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.project.entity.ProjectdevW;
import com.tofly.pms.project.service.ProjectDevWService;
import com.tofly.pms.project.service.ProjectTeamWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

 /**
 * 项目研发信息表;(TF_PM_PROJECTDEV_W)表控制层
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "项目研发信息表对象功能接口")
@RestController
@RequestMapping("/tfPmProjectdevW")
@AllArgsConstructor
public class TfPmProjectdevWController{

    private final ProjectDevWService projectDevWService;
    private final ProjectTeamWService projectTeamWService;
    /** 
     * 通过ID查询单条数据 
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<ProjectdevW> queryById(Integer id){
        return ResponseEntity.ok(projectDevWService.queryById(id));
    }
    
    /** 
     * 分页查询
     *
     * @param projectdevW 筛选条件
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<ProjectdevW>> paginQuery(ProjectdevW projectdevW, Page<ProjectdevW> page){
        String name = projectdevW.getName();
        projectdevW.setName(null);
        String label = projectdevW.getLabel();
        projectdevW.setLabel(null);

        LambdaQueryWrapper<ProjectdevW> wrapper = new QueryWrapper<>(projectdevW)
                .lambda()
                .like(StringUtils.isNotBlank(name), ProjectdevW::getName, name)
                .like(StringUtils.isNotBlank(label),ProjectdevW::getLabel,label);
        final Page<ProjectdevW> page1 = projectDevWService.page(page,wrapper);




        return ResponseEntity.ok(page1);
    }
    /**
     * 新增数据
     *
     * @param projectdevW 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<ProjectdevW> add(@RequestBody  ProjectdevW projectdevW){
        projectDevWService.save(projectdevW);
        return ResponseEntity.ok(projectdevW);
    }
    
    /** 
     * 更新数据
     *
     * @param projectdevW 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<ProjectdevW> edit(@RequestBody  ProjectdevW projectdevW){
        projectDevWService.updateById(projectdevW);
        return ResponseEntity.ok(projectdevW);
    }

     /**
      * 通过主键删除数据
      *
      * @param ids 主键
      * @return 是否成功
      */
     @ApiOperation("通过主键删除数据")
     @DeleteMapping
     public ResponseEntity<Boolean> deleteById(String ids){

         final String[] split = ids.split(",");
         for (String s : split) {
             projectDevWService.removeById(Integer.valueOf(s));
         }
         return ResponseEntity.ok(Boolean.TRUE);
     }
}