package com.tofly.pms.project.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.project.entity.ProjecTresW;
import com.tofly.pms.project.service.ProjecTresWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

 /**
 * 项目成果信息表;(TF_PM_PROJECTRES_W)表控制层
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "项目成果信息表对象功能接口")
@RestController
@RequestMapping("/tfPmProjectresW")
public class TfPmProjectresWController{
    @Autowired
    private ProjecTresWService projecTresWService;
    
    /** 
     * 通过ID查询单条数据 
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<ProjecTresW> queryById(Integer id){
        return ResponseEntity.ok(projecTresWService.queryById(id));
    }
    
    /** 
     * 分页查询
     *
     * @param projecTresW 筛选条件
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<ProjecTresW>> paginQuery(ProjecTresW projecTresW, Page page){
        final Page page1 = projecTresWService.page(page, new QueryWrapper<>(projecTresW));
        return ResponseEntity.ok(page1);

    }
    
    /** 
     * 新增数据
     *
     * @param projecTresW 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<ProjecTresW> add(@RequestBody  ProjecTresW projecTresW){
        projecTresWService.save(projecTresW);
        return ResponseEntity.ok(projecTresW);
    }
    
    /** 
     * 更新数据
     *
     * @param projecTresW 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<ProjecTresW> edit(@RequestBody  ProjecTresW projecTresW){
        projecTresWService.updateById(projecTresW);
        return ResponseEntity.ok(projecTresW);
    }

     /**
      * 通过主键删除数据
      *
      * @param ids 主键
      * @return 是否成功
      */
     @ApiOperation("通过主键删除数据")
     @DeleteMapping
     public ResponseEntity<Boolean> deleteById(String ids){

         final String[] split = ids.split(",");
         for (String s : split) {
             projecTresWService.removeById(Integer.valueOf(s));
         }
         return ResponseEntity.ok(Boolean.TRUE);
     }
}