package com.tofly.pms.project.controller;

import cn.hutool.core.exceptions.ValidateException;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.excellistener.ProjectTeamListener;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.project.vo.ProjectTeamVo;
import com.tofly.pms.utils.ExcelUtil;
import com.tofly.pms.utils.UserTokenUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Objects;

/**
 * 项目团队表;(TF_PM_PROJECTTEAM_W)表控制层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "项目团队表对象功能接口")
@RestController
@RequestMapping("/tfPmProjectteamW")
public class TfPmProjectteamWController {
    @Autowired
    private ProjectTeamWService projectTeamWService;
    @Autowired
    private ProjectWService projectService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<ProjectTeamW> queryById(@PathVariable("id") Integer id) {
        return ResponseEntity.ok(projectTeamWService.queryById(id));
    }

    /**
     * 分页查询
     *
     * @param projectTeamW 筛选条件
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<ProjectTeamW>> paginQuery(ProjectTeamW projectTeamW, Page page) {
        final Page page1 = projectTeamWService.page(page, new QueryWrapper<>(projectTeamW));
        return ResponseEntity.ok(page1);
    }

    /**
     * 新增数据
     *
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<ProjectTeamW> add(@RequestBody ProjectTeamW projectTeamW) {
        //判断项目是否已经存在团队
        final ProjectTeamW one = projectTeamWService.getOne(Wrappers.<ProjectTeamW>lambdaQuery()
                .eq(ProjectTeamW::getProjectId, projectTeamW.getProjectId()));
        if (one != null) {
            throw new ValidateException("该项目已经存在团队信息");
        }

        projectTeamWService.save(projectTeamW);
        return ResponseEntity.ok(projectTeamW);
    }

    /**
     * 更新数据
     *
     * @param projectTeamW 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<ProjectTeamW> edit(@RequestBody ProjectTeamW projectTeamW) {
//        String userID = UserTokenUtil.getTokenUserID();
//        Assert.notBlank(userID, "用户不合法");
//        ProjectTeamW dbProjectTeam = projectTeamWService.getById(projectTeamW.getId());
//        if (Objects.isNull(dbProjectTeam)) {
//            throw new RuntimeException("没有权限");
//        }
//        String adminId = dbProjectTeam.getAdminId();
//        assert userID != null;
//        if (!adminId.contains(userID)) {
//            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
//            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
//            if (!has) {
//                throw new RuntimeException("没有权限");
//            }
//        }
        ProjectTeamW dbProjectTeam = projectTeamWService.getById(projectTeamW.getId());
        projectTeamWService.updateById(projectTeamW);
        if(ObjectUtil.notEqual(dbProjectTeam.getAdminId(), projectTeamW.getAdminId())){
            projectService.update(Wrappers.lambdaUpdate(ProjectW.class)
                    .set(ProjectW::getAdminId, projectTeamW.getAdminId())
                    .eq(ProjectW::getId, dbProjectTeam.getProjectId())
            );
        }
        return ResponseEntity.ok(projectTeamW);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public ResponseEntity<Boolean> deleteById(String ids) {
//        String userID = UserTokenUtil.getTokenUserID();
//        Assert.notBlank(userID, "用户不合法");
//        RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
//        Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
//        if (!has) {
//            throw new RuntimeException("没有权限");
//        }
        final String[] split = ids.split(",");
        for (String s : split) {
            projectTeamWService.removeById(Integer.valueOf(s));
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }


    @ApiOperation("团队人员导入")
    @PostMapping("/import")
    public ResponseEntity<Boolean> userImport(@RequestPart("file") MultipartFile file) {

        ExcelUtil.importExcel(file, ProjectTeamW.class, new ProjectTeamListener(projectTeamWService));


        return null;
    }

    @ApiOperation("快捷方式增加项目团队成员")
    @PostMapping("/addOrUpdate")
    public ResponseEntity<Boolean> addOrUpdate(@RequestBody ProjectTeamVo projectTeamVo) {
        return ResponseEntity.ok(projectTeamWService.addOrUpdate(projectTeamVo));
    }
}