package com.tofly.pms.project.entity;

import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.utils.easyListener.converter.CustomStringToUserIdsConverter;
import com.tofly.pms.utils.vocast.VoCast;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 项目团队表;
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Builder
@Accessors(chain = true)
@ApiModel(value = "项目团队表",description = "")
@TableName(value = "TF_PM_PROJECTTEAM_W",schema = "business")
@NoArgsConstructor
@AllArgsConstructor
@Data
public class ProjectTeamW implements Serializable,Cloneable{
    /** id */
    @ApiModelProperty(value = "id",name = "id")
    @TableId(type = IdType.AUTO)
    private Integer id ;
    /** 项目id */
    @ApiModelProperty(value = "项目id",name = "projectId")
    @VoCast(serviceClass = ProjectWService.class,valueClass = ProjectW.class)
    private Integer projectId ;
    @TableField(exist = false)
    @ExcelProperty("项目")
    private String projectName;
    /** 项目管理员ID */
    @ApiModelProperty(value = "项目管理员ID",name = "adminId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty(value = "项目管理员",converter = CustomStringToUserIdsConverter.class)
    private String adminId ;
    /** 项目经理ID */
    @ApiModelProperty(value = "项目经理ID",name = "manageId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("项目经理")
    private String manageId ;
    /** 软件负责人ID */
    @ApiModelProperty(value = "软件负责人ID",name = "softHeadUserId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("软件负责人")
    private String softHeadUserId ;
    /** 研发组长ID */
    @ApiModelProperty(value = "研发组长ID",name = "softGroupUserId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("研发组长")
    private String softGroupUserId ;
    /** 产品经理ID */
    @ApiModelProperty(value = "产品经理ID",name = "softProjectManageId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("产品经理")
    private String softProjectManageId ;
    /** 开发工程师ID */
    @ApiModelProperty(value = "开发工程师ID",name = "devId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("开发工程师")
    private String devId ;
    /** 前端工程师ID */
    @ApiModelProperty(value = "前端工程师ID",name = "webId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("前端工程师")
    private String webId ;
    /** 后端工程师ID */
    @ApiModelProperty(value = "后端工程师ID",name = "backendId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("后端工程师")
    private String backendId ;
    /** 安卓工程师ID */
    @ApiModelProperty(value = "安卓端工程师ID",name = "mobileId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("安卓工程师")
    private String mobileId ;
    /** 测试工程师ID */
    @ApiModelProperty(value = "测试工程师ID",name = "testId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("测试工程师")
    private String testId ;
    /** 实施工程师ID */
    @ApiModelProperty(value = "实施工程师ID",name = "implementionId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("实施工程师")
    private String implementionId ;
    /** 数据工程师ID */
    @ApiModelProperty(value = "数据工程师ID",name = "dataId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("数据工程师")
    private String dataId ;
    /** UI工程师ID */
    @ApiModelProperty(value = "UI工程师ID",name = "uiId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty(" UI工程师")
    private String uiId ;
    /** 其他ID */
    @ApiModelProperty(value = "其他ID",name = "ohterId")
    @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
    @ExcelProperty("其他")
    private String ohterId ;

     @ApiModelProperty(value = "总人数",name = "totalUser")
     @TableField(exist = false)
    private Integer totalUser;


     public Integer getTotalUser() {

         StringBuffer  totalUserbuffer = new StringBuffer();
         totalUserbuffer.append(ohterId).append(",");
         totalUserbuffer.append(uiId).append(",");
         totalUserbuffer.append(dataId).append(",");
         totalUserbuffer.append(implementionId).append(",");
         totalUserbuffer.append(testId).append(",");
         totalUserbuffer.append(mobileId).append(",");
         totalUserbuffer.append(backendId).append(",");
         totalUserbuffer.append(webId).append(",");
         totalUserbuffer.append(devId).append(",");
         totalUserbuffer.append(softProjectManageId).append(",");
         totalUserbuffer.append(softGroupUserId).append(",");
         totalUserbuffer.append(softHeadUserId).append(",");
         totalUserbuffer.append(manageId).append(",");
         totalUserbuffer.append(adminId);
         final String[] split = totalUserbuffer.toString().split(",");
         final List<String> collect = Arrays.stream(split)
                 .filter(StringUtils::isNotBlank).distinct()
                 .filter(f->ObjectUtils.notEqual("null",f)).collect(Collectors.toList());
         return collect.size();
     }
 }