package com.tofly.pms.project.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.utils.dict.Dict;
import com.tofly.pms.utils.vocast.VoCast;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;

 /**
 * 项目基础信息表;
 * @author : Xu'J
 * @date : 2023-12-19
 */
@ApiModel(value = "项目基础信息表",description = "")
@TableName(value = "TF_PM_PROJECT_W",schema = "business")
@Data
public class ProjectW implements Serializable,Cloneable{
    /** ID */
    @ApiModelProperty(value = "ID",name = "id")
    @TableId(type = IdType.AUTO)
    private Integer id ;
    /** 项目唯一编码 */
    @ApiModelProperty(value = "项目唯一编码",name = "code")

    private String code ;
    /** 项目名称 */
    @ApiModelProperty(value = "项目名称",name = "name")
    private String name ;
    /** 客户名称 */
    @ApiModelProperty(value = "客户名称",name = "customName")
    private String customName ;
    /** 项目类型 */
    @ApiModelProperty(value = "项目类型 XMLX",name = "type")
    @Dict(code = "XMLX")
    private String type ;
    /** 合同名称 */
    @ApiModelProperty(value = "合同名称",name = "contractName")
    private String contractName ;
    /** 立项年份 */
    @ApiModelProperty(value = "立项年份",name = "approvalYear")
    private String approvalYear ;
    /** 项目要求启动日期 */
    @ApiModelProperty(value = "项目要求启动日期",name = "startDate")
    private String startDate ;
    /** 项目要求完成日期 */
    @ApiModelProperty(value = "项目要求完成日期",name = "endDate")
    @JsonFormat(pattern="yyyy-MM-dd ",timezone = "GMT+8")
    @DateTimeFormat(pattern="yyyy-MM-dd ")
    private String endDate ;
    /** 项目创建日期 */
    @ApiModelProperty(value = "项目创建日期",name = "createDate")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date createDate ;
    /** 合同工期(天) */
    @ApiModelProperty(value = "合同工期(天)",name = "contractDuration")
    private Integer contractDuration ;
    /** 合同开始日期 */
    @ApiModelProperty(value = "合同开始日期",name = "contractStartDate")
    private String contractStartDate ;
    /** 合同结束日期 */
    @ApiModelProperty(value = "合同结束日期",name = "contractEndDate")
    private String contractEndDate ;
    /** 质保期（天） */
    @ApiModelProperty(value = "质保期（天）",name = "warrantyPeriod")
    private Integer warrantyPeriod ;
    /** 质保（运维）开始日期 */
    @ApiModelProperty(value = "质保（运维）开始日期",name = "warrantyStartDate")
    private String warrantyStartDate ;
    /** 质保（运维）结束日期 */
    @ApiModelProperty(value = "质保（运维）结束日期",name = "warrantyEndDate")
    private String warrantyEndDate ;
    /** 备注 */
    @ApiModelProperty(value = "备注",name = "note")
    private String note ;
    /** 创建人标识 */
    @ApiModelProperty(value = "创建人标识",name = "createUser")
    @VoCast(serviceClass = UserClient.class,valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT)
    private String createUser ;
    /** 创建人名称 */
    @ApiModelProperty(value = "创建人名称",name = "createUserName")
    private String createUserName ;
    /** 创建时间 */
    @ApiModelProperty(value = "创建时间",name = "createTime")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime ;
    /** 项目状态 */
    @ApiModelProperty(value = "项目状态",name = "status")
    private String status ;
     @ApiModelProperty(value = "项目逻辑删除标志（0：未删除；1：已删除）",name = "deletedFlag")
    private Integer deletedFlag;
     @ApiModelProperty(value = "工期说明",name = "deletedFlag")
     private String durationDesc;
     @ApiModelProperty(value = "付款方式",name = "payMethod")
     private String payMethod;
     @ApiModelProperty(value = "工作内容",name = "workContent")
     private String workContent;
    /** 更新人id */
    @ApiModelProperty(value = "更新人id",name = "updateUser")
    @VoCast(serviceClass = UserClient.class,isMulti =false,valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT)
    private String updateUser ;
    /** 更新时间 */
    @ApiModelProperty(value = "更新时间",name = "updateTime")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss",timezone = "GMT+8")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT)
    private Date updateTime ;


     @TableField(exist = false)
     @ApiModelProperty(value = "团队人数",name = "uiUrl")
     private Integer totalUser;

     @ApiModelProperty(value = "预估人日",name = "workUnit")
     private Double workUnit;

     @VoCast(serviceClass = UserClient.class,isMulti =true,valueClass = UserVo.class)
     @ApiModelProperty(value = "项目管理员ID",name = "adminId")
     private String adminId ;


}