package com.tofly.pms.project.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.conditions.update.LambdaUpdateChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.pms.project.entity.ProjectdevW;
import com.tofly.pms.project.mapper.ProjectDevWMapper;
import com.tofly.pms.project.service.ProjectDevWService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
 /**
 * 项目研发信息表;(TF_PM_PROJECTDEV_W)表服务实现类
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Service
public class ProjectDevWServiceImpl  extends ServiceImpl<ProjectDevWMapper, ProjectdevW>
         implements ProjectDevWService {
    @Autowired
    private ProjectDevWMapper projectDevWMapper;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    public ProjectdevW queryById(Integer id){
        return projectDevWMapper.selectById(id);
    }

    /**
     * 分页查询
     *
     * @param projectdevW 筛选条件
     * @param current 当前页码
     * @param size  每页大小
     * @return
     */
    public Page<ProjectdevW> paginQuery(ProjectdevW projectdevW, long current, long size){
        //1. 构建动态查询条件
        LambdaQueryWrapper<ProjectdevW> queryWrapper = new LambdaQueryWrapper<>();
        if(StrUtil.isNotBlank(projectdevW.getName())){
            queryWrapper.eq(ProjectdevW::getName, projectdevW.getName());
        }
        if(StrUtil.isNotBlank(projectdevW.getVisitAddress())){
            queryWrapper.eq(ProjectdevW::getVisitAddress, projectdevW.getVisitAddress());
        }
        if(StrUtil.isNotBlank(projectdevW.getAppQrcode())){
            queryWrapper.eq(ProjectdevW::getAppQrcode, projectdevW.getAppQrcode());
        }
        if(StrUtil.isNotBlank(projectdevW.getLoginUser())){
            queryWrapper.eq(ProjectdevW::getLoginUser, projectdevW.getLoginUser());
        }
        if(StrUtil.isNotBlank(projectdevW.getLoginPassword())){
            queryWrapper.eq(ProjectdevW::getLoginPassword, projectdevW.getLoginPassword());
        }
        if(StrUtil.isNotBlank(projectdevW.getHeadId())){
            queryWrapper.eq(ProjectdevW::getHeadId, projectdevW.getHeadId());
        }
        if(StrUtil.isNotBlank(projectdevW.getDescription())){
            queryWrapper.eq(ProjectdevW::getDescription, projectdevW.getDescription());
        }
        if(StrUtil.isNotBlank(projectdevW.getTechDescription())){
            queryWrapper.eq(ProjectdevW::getTechDescription, projectdevW.getTechDescription());
        }
        if(StrUtil.isNotBlank(projectdevW.getEnvironmentConfig())){
            queryWrapper.eq(ProjectdevW::getEnvironmentConfig, projectdevW.getEnvironmentConfig());
        }
        if(StrUtil.isNotBlank(projectdevW.getNote())){
            queryWrapper.eq(ProjectdevW::getNote, projectdevW.getNote());
        }
        if(StrUtil.isNotBlank(projectdevW.getLabel())){
            queryWrapper.eq(ProjectdevW::getLabel, projectdevW.getLabel());
        }
        if(StrUtil.isNotBlank(projectdevW.getIsOpen())){
            queryWrapper.eq(ProjectdevW::getIsOpen, projectdevW.getIsOpen());
        }
        if(StrUtil.isNotBlank(projectdevW.getIsRegular())){
            queryWrapper.eq(ProjectdevW::getIsRegular, projectdevW.getIsRegular());
        }
        if(StrUtil.isNotBlank(projectdevW.getIsSupportExternalVisit())){
            queryWrapper.eq(ProjectdevW::getIsSupportExternalVisit, projectdevW.getIsSupportExternalVisit());
        }
        //2. 执行分页查询
        Page<ProjectdevW> pagin = new Page<>(current , size , true);
        IPage<ProjectdevW> selectResult = projectDevWMapper.selectByPage(pagin , queryWrapper);
        pagin.setPages(selectResult.getPages());
        pagin.setTotal(selectResult.getTotal());
        pagin.setRecords(selectResult.getRecords());
        //3. 返回结果
        return pagin;
    }

    /**
     * 新增数据
     *
     * @param projectdevW 实例对象
     * @return 实例对象
     */
    public ProjectdevW insert(ProjectdevW projectdevW){
        projectDevWMapper.insert(projectdevW);
        return projectdevW;
    }

    /**
     * 更新数据
     *
     * @param projectdevW 实例对象
     * @return 实例对象
     */
    public ProjectdevW update(ProjectdevW projectdevW){
        //1. 根据条件动态更新
        LambdaUpdateChainWrapper<ProjectdevW> chainWrapper = new LambdaUpdateChainWrapper<ProjectdevW>(projectDevWMapper);
        if(StrUtil.isNotBlank(projectdevW.getName())){
            chainWrapper.eq(ProjectdevW::getName, projectdevW.getName());
        }
        if(StrUtil.isNotBlank(projectdevW.getVisitAddress())){
            chainWrapper.eq(ProjectdevW::getVisitAddress, projectdevW.getVisitAddress());
        }
        if(StrUtil.isNotBlank(projectdevW.getAppQrcode())){
            chainWrapper.eq(ProjectdevW::getAppQrcode, projectdevW.getAppQrcode());
        }
        if(StrUtil.isNotBlank(projectdevW.getLoginUser())){
            chainWrapper.eq(ProjectdevW::getLoginUser, projectdevW.getLoginUser());
        }
        if(StrUtil.isNotBlank(projectdevW.getLoginPassword())){
            chainWrapper.eq(ProjectdevW::getLoginPassword, projectdevW.getLoginPassword());
        }
        if(StrUtil.isNotBlank(projectdevW.getHeadId())){
            chainWrapper.eq(ProjectdevW::getHeadId, projectdevW.getHeadId());
        }
        if(StrUtil.isNotBlank(projectdevW.getDescription())){
            chainWrapper.eq(ProjectdevW::getDescription, projectdevW.getDescription());
        }
        if(StrUtil.isNotBlank(projectdevW.getTechDescription())){
            chainWrapper.eq(ProjectdevW::getTechDescription, projectdevW.getTechDescription());
        }
        if(StrUtil.isNotBlank(projectdevW.getEnvironmentConfig())){
            chainWrapper.eq(ProjectdevW::getEnvironmentConfig, projectdevW.getEnvironmentConfig());
        }
        if(StrUtil.isNotBlank(projectdevW.getNote())){
            chainWrapper.eq(ProjectdevW::getNote, projectdevW.getNote());
        }
        if(StrUtil.isNotBlank(projectdevW.getLabel())){
            chainWrapper.eq(ProjectdevW::getLabel, projectdevW.getLabel());
        }
        if(StrUtil.isNotBlank(projectdevW.getIsOpen())){
            chainWrapper.eq(ProjectdevW::getIsOpen, projectdevW.getIsOpen());
        }
        if(StrUtil.isNotBlank(projectdevW.getIsRegular())){
            chainWrapper.eq(ProjectdevW::getIsRegular, projectdevW.getIsRegular());
        }
        if(StrUtil.isNotBlank(projectdevW.getIsSupportExternalVisit())){
            chainWrapper.eq(ProjectdevW::getIsSupportExternalVisit, projectdevW.getIsSupportExternalVisit());
        }
        //2. 设置主键，并更新
        chainWrapper.set(ProjectdevW::getId, projectdevW.getId());
        boolean ret = chainWrapper.update();
        //3. 更新成功了，查询最最对象返回
        if(ret){
            return queryById(projectdevW.getId());
        }else{
            return projectdevW;
        }
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    public boolean deleteById(Integer id){
        int total = projectDevWMapper.deleteById(id);
        return total > 0;
    }
}