package com.tofly.pms.project.service.impl;

import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.mapper.ProjectTeamWMapper;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.vo.ProjectTeamVo;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.lang.reflect.Field;
import java.util.Objects;

/**
 * 项目团队表;(TF_PM_PROJECTTEAM_W)表服务实现类
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Service
public class ProjectTeamWServiceImpl extends ServiceImpl<ProjectTeamWMapper, ProjectTeamW>
        implements ProjectTeamWService {

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    public ProjectTeamW queryById(Integer id) {
        return baseMapper.selectById(id);
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean addOrUpdate(ProjectTeamVo projectTeamVo) {
        Integer projectId = projectTeamVo.getProjectId();
        String projectRoles = projectTeamVo.getProjectRoles();
        String userId = projectTeamVo.getUserId();
        Assert.notBlank(userId, "用户id不能为空");
        Assert.notBlank(projectRoles, "项目角色不能为空");
        Assert.notNull(projectId, "项目id不能为空");
        ProjectTeamW team = this.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId));
        if (Objects.isNull(team)) {
            team = new ProjectTeamW();
            team.setProjectId(projectId);
        }
        String[] prs = projectRoles.split(",");
        for (String projectRole : prs) {
            setFieldValue(team, projectRole, userId);
        }

        return saveOrUpdate(team);
    }


    /**
     * 使用反射动态设置对象属性值
     *
     * @param team      对象实例
     * @param fieldName 属性名
     * @param value     要设置的值
     */
    private void setFieldValue(ProjectTeamW team, String fieldName, String value) {
        try {
            // 获取对象的Class对象
            Class<?> clazz = team.getClass();
            // 获取指定名称的Field对象
            Field field = clazz.getDeclaredField(fieldName);
            // 设置该属性可访问（如果需要）
            field.setAccessible(true);
            Object oldValue = field.get(team);
            if (oldValue instanceof String && !oldValue.toString().contains(value)) {
                value = oldValue + "," + value;
            } else if (Objects.nonNull(oldValue)) {
                field.setAccessible(false);
                return;
            }
            // 设置属性值
            field.set(team, value);
            field.setAccessible(false);
        } catch (Exception e) {
            log.error("设置属性值失败", e);
        }
    }
}