package com.tofly.pms.project.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.mapper.ProjectWMapper;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.project.vo.ProjectNameAndId;
import com.tofly.pms.utils.UserTokenUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 项目基础信息表;(TF_PM_PROJECT_W)表服务实现类
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Slf4j
@Service
public class ProjectWServiceImpl extends ServiceImpl<ProjectWMapper, ProjectW>
        implements ProjectWService {

    @Resource
    ProjectTeamWService projectTeamWService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    public ProjectW queryById(Integer id) {
        String userID = UserTokenUtil.getTokenUserID();
        ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, id)
                .and(wr -> wr.like(ProjectTeamW::getAdminId, userID)
                        .or()
                        .like(ProjectTeamW::getManageId, userID)
                        .or()
                        .like(ProjectTeamW::getSoftHeadUserId, userID)
                        .or()
                        .like(ProjectTeamW::getSoftGroupUserId, userID)
                        .or()
                        .like(ProjectTeamW::getSoftProjectManageId, userID)
                        .or()
                        .like(ProjectTeamW::getDevId, userID)
                        .or()
                        .like(ProjectTeamW::getWebId, userID)
                        .or()
                        .like(ProjectTeamW::getBackendId, userID)
                        .or()
                        .like(ProjectTeamW::getMobileId, userID)
                        .or()
                        .like(ProjectTeamW::getTestId, userID)
                        .or()
                        .like(ProjectTeamW::getImplementionId, userID)
                        .or()
                        .like(ProjectTeamW::getDataId, userID)
                        .or()
                        .like(ProjectTeamW::getUiId, userID)
                        .or()
                        .like(ProjectTeamW::getOhterId, userID))

        );
        if (Objects.isNull(one)) {
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if (!has) {
                return new ProjectW();
            }
        }
        return this.baseMapper.selectById(id);
    }


    @Override
    public List<String> getCustomNameList() {
        return baseMapper.getCustomNameList();
    }

    @Override
    public List<ProjectW> getMyProjectList() {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前访问不合法");
        RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
        Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
        if(has){
            return this.list();
        }
        List<Integer> projectIdList = projectTeamWService.listObjs(Wrappers.lambdaQuery(ProjectTeamW.class)
                        .select(ProjectTeamW::getProjectId)
                        .like(ProjectTeamW::getAdminId, userID)
                        .or()
                        .like(ProjectTeamW::getManageId, userID)
                        .or()
                        .like(ProjectTeamW::getSoftHeadUserId, userID)
                        .or()
                        .like(ProjectTeamW::getSoftGroupUserId, userID)
                        .or()
                        .like(ProjectTeamW::getSoftProjectManageId, userID)
                        .or()
                        .like(ProjectTeamW::getDevId, userID)
                        .or()
                        .like(ProjectTeamW::getWebId, userID)
                        .or()
                        .like(ProjectTeamW::getBackendId, userID)
                        .or()
                        .like(ProjectTeamW::getMobileId, userID)
                        .or()
                        .like(ProjectTeamW::getTestId, userID)
                        .or()
                        .like(ProjectTeamW::getImplementionId, userID)
                        .or()
                        .like(ProjectTeamW::getDataId, userID)
                        .or()
                        .like(ProjectTeamW::getUiId, userID)
                        .or()
                        .like(ProjectTeamW::getOhterId, userID), item -> {
                    if (item instanceof Integer) {
                        return (Integer) item;
                    }
                    throw new RuntimeException("类型转换异常");
                }
        );
        if (CollectionUtil.isEmpty(projectIdList)) {
            return null;
        }
        projectIdList = projectIdList.stream().distinct().collect(Collectors.toList());
        return this.list(Wrappers.lambdaQuery(ProjectW.class)
                        .eq(ProjectW::getDeletedFlag, 0)
                .in(ProjectW::getId, projectIdList));
    }

    @Override
    public List<UserVo> getUserVoByUserId(List<String> userIdList) {
        return baseMapper.getUserVoByUserId(userIdList);
    }

    @Override
    public List<ProjectNameAndId> getAllProject() {
        List<ProjectW> list = this.list(Wrappers.lambdaQuery(ProjectW.class)
                        .eq(ProjectW::getDeletedFlag,0)
                .orderByDesc(ProjectW::getCreateTime));
        if (CollectionUtil.isNotEmpty(list)) {
            return list.stream().map(projectW -> {
                ProjectNameAndId projectNameAndId = new ProjectNameAndId();
                BeanUtils.copyProperties(projectW, projectNameAndId);
                projectNameAndId.setProjectId(projectW.getId());
                projectNameAndId.setProjectName(projectW.getName().concat("||").concat(projectW.getCustomName()));
                return projectNameAndId;
            }).collect(Collectors.toList());
        }
        return null;
    }

    @Override
    public List<Integer> getSelfProjectId() {
        String userID = UserTokenUtil.getTokenUserID();
        return getProjectIdByUserId(userID);
    }

    @Override
    public List<Integer> getProjectIdByUserId(String userID) {
        RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
        log.info(String.format("the userId is:%s",userID));
        Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
        if(has){
            return this.listObjs(Wrappers.lambdaQuery(ProjectW.class).select(ProjectW::getId),item->{
                if(item instanceof Integer){
                    return (Integer) item;
                }
                throw new RuntimeException("类型转换异常");
            });
        }
        List<Integer> projectIdList = projectTeamWService.listObjs(Wrappers.lambdaQuery(ProjectTeamW.class)
                        .select(ProjectTeamW::getProjectId)
                        .like(ProjectTeamW::getAdminId, userID)
                        .or()
                        .like(ProjectTeamW::getManageId, userID)
                        .or()
                        .like(ProjectTeamW::getSoftHeadUserId, userID)
                        .or()
                        .like(ProjectTeamW::getSoftGroupUserId, userID)
                        .or()
                        .like(ProjectTeamW::getSoftProjectManageId, userID)
                        .or()
                        .like(ProjectTeamW::getDevId, userID)
                        .or()
                        .like(ProjectTeamW::getWebId, userID)
                        .or()
                        .like(ProjectTeamW::getBackendId, userID)
                        .or()
                        .like(ProjectTeamW::getMobileId, userID)
                        .or()
                        .like(ProjectTeamW::getTestId, userID)
                        .or()
                        .like(ProjectTeamW::getImplementionId, userID)
                        .or()
                        .like(ProjectTeamW::getDataId, userID)
                        .or()
                        .like(ProjectTeamW::getUiId, userID)
                        .or()
                        .like(ProjectTeamW::getOhterId, userID), item -> {
                    if (item instanceof Integer) {
                        return (Integer) item;
                    }
                    throw new RuntimeException("类型转换异常");
                }
        );
        return projectIdList.stream().distinct().collect(Collectors.toList());
    }

    public static HashMap<String, String> PROJECT_ROLE_MAP = new HashMap<>() {
        {
            put("adminId", "项目管理员");
            put("manageId", "项目经理");
            put("softHeadUserId", "软件负责人");
            put("softGroupUserId", "研发组长");
            put("softProjectManageId", "产品经理");
            put("devId", "开发工程师");
            put("webId", "前端工程师");
            put("backendId", "后端工程师");
            put("mobileId", "安卓工程师");
            put("testId", "测试工程师");
            put("implementionId", "实施工程师");
            put("dataId", "数据工程师");
            put("uiId", "UI工程师");
            put("ohterId", "其他");
        }
    };
    @Override
    public List<UserVo> getUserByProjectId(Integer projectId) {
        HashMap<String, String> map = baseMapper.getUserIdMap(projectId);
        if(CollectionUtil.isEmpty(map)){
            return new ArrayList<>();
        }
        String idstr = map.values().stream()
                .filter(StringUtils::isNotBlank)
                .collect(Collectors.joining(","));
        if (StringUtils.isBlank(idstr)) {
            return new ArrayList<>();
        }
        List<String> userIdList = Arrays.stream(idstr.split(",")).distinct().collect(Collectors.toList());
        List<UserVo> result = baseMapper.getUserVoByUserId(userIdList);
        for (UserVo userVo : result) {
            String userId = userVo.getId();
            Set<String> collect = map.keySet().stream()
                    .map(map::get)
                    .filter(Objects::nonNull)
                    .filter(key ->key.contains(userId))
                    .map(key -> CaseFormat.LOWER_UNDERSCORE.to(CaseFormat.LOWER_CAMEL, key))
                    .map(key -> PROJECT_ROLE_MAP.get(key))
                    .collect(Collectors.toSet());
            userVo.setProjectRoles(collect);
        }
        return result;
    }
}