package com.tofly.pms.testmanage.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.testmanage.entity.BugInfo;
import com.tofly.pms.testmanage.entity.BugLog;
import com.tofly.pms.testmanage.service.BugLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.Collectors;

/**
 * BUG处理日志表;(TF_PM_BUGLOG_W)表控制层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "BUG处理日志表对象功能接口")
@RestController
@RequestMapping("/tfPmBuglogW")
public class BugLogWController {
    @Autowired
    private BugLogService bugLogService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键testSystemService
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<BugLog> queryById(Integer id) {
        return ResponseEntity.ok(bugLogService.queryById(id));
    }

    /**
     * 分页查询
     *
     * @param bugLog 筛选条件
     * @param page   分页对象
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<BugLog>> paginQuery(BugLog bugLog, Page<BugLog> page) {
        final Page<BugLog> page1 = bugLogService.page(page, new LambdaQueryWrapper<>(bugLog)
                .orderByAsc(BugLog::getHandleTime));
        List<BugLog> records = page1.getRecords().stream()
                .peek(item -> {
                    item.setBugInfo(JSON.parseObject(item.getJsonData(), BugInfo.class));
                    item.getBugInfo().setProcessingModeAnnotation();
                })
                .peek(BugLog::setProcessingModeAnnotation)
                .collect(Collectors.toList());
        page1.setRecords(records);
        return ResponseEntity.ok(page1);
    }

    /**
     * 新增数据
     *
     * @param bugLog 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<BugLog> add(@RequestBody BugLog bugLog) {
        bugLogService.save(bugLog);
        return ResponseEntity.ok(bugLog);
    }

    /**
     * 更新数据
     *
     * @param bugLog 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<BugLog> edit(@RequestBody BugLog bugLog) {
        bugLogService.updateById(bugLog);
        return ResponseEntity.ok(bugLog);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public ResponseEntity<Boolean> deleteById(String ids) {
        final String[] split = ids.split(",");
        for (String s : split) {
            bugLogService.removeById(Integer.valueOf(s));
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

}