package com.tofly.pms.testmanage.controller;

import com.tofly.pms.base.controller.Result;
import com.tofly.pms.testmanage.entity.StatisticsBug;
import com.tofly.pms.testmanage.service.BugInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author sheldon
 * @Description
 * @create 2024-04-11 13:33
 */
@Api(tags = "BUG统计模块")
@RestController
@RequestMapping("/statisticsBug")
public class StatisticsController {

    @Autowired
    private BugInfoService bugInfoService;

    @ApiOperation("总体统计")
    @GetMapping("/getStatisticalEnsemble")
    public Result<List<StatisticsBug>> getStatisticalEnsemble(@RequestParam(value = "projectId") Integer projectId,
                                                              @RequestParam(value = "startSearchTime", required = false) String startSearchTime,
                                                              @RequestParam(value = "endSearchTime", required = false) String endSearchTime,
                                                              @RequestParam(value = "startHandleTime", required = false) String startHandleTime,
                                                              @RequestParam(value = "endHandleTime", required = false) String endHandleTime,
                                                              @RequestParam(value = "startVerifyTime", required = false) String startVerifyTime,
                                                              @RequestParam(value = "endVerifyTime", required = false) String endVerifyTime) {
        return Result.success(bugInfoService.getStatisticalEnsemble(projectId, startSearchTime, endSearchTime, startHandleTime, endHandleTime, startVerifyTime, endVerifyTime));
    }

    @ApiOperation("分系统统计")
    @GetMapping("/getStatisticalBySystem")
    public Result<List<StatisticsBug>> getStatisticalBySystem(@RequestParam(value = "projectId") Integer projectId,
                                                              @RequestParam(value = "startSearchTime", required = false) String startSearchTime,
                                                              @RequestParam(value = "endSearchTime", required = false) String endSearchTime,
                                                              @RequestParam(value = "fix") Integer fix,
                                                              @RequestParam(value = "startHandleTime", required = false) String startHandleTime,
                                                              @RequestParam(value = "endHandleTime", required = false) String endHandleTime,
                                                              @RequestParam(value = "startVerifyTime", required = false) String startVerifyTime,
                                                              @RequestParam(value = "endVerifyTime", required = false) String endVerifyTime) {
        return Result.success(bugInfoService.getStatisticalBySystem(projectId, startSearchTime, endSearchTime, fix, startHandleTime, endHandleTime, startVerifyTime, endVerifyTime));
    }

    @ApiOperation("人员缺陷统计")
    @GetMapping("/getStatisticalByHead")
    public Result<List<StatisticsBug>> getStatisticalByHead(@RequestParam(value = "projectId") Integer projectId,
                                                            @RequestParam(value = "startSearchTime", required = false) String startSearchTime,
                                                            @RequestParam(value = "endSearchTime", required = false) String endSearchTime,
                                                            @RequestParam(value = "fix") Integer fix,
                                                            @RequestParam(value = "startHandleTime", required = false) String startHandleTime,
                                                            @RequestParam(value = "endHandleTime", required = false) String endHandleTime,
                                                            @RequestParam(value = "startVerifyTime", required = false) String startVerifyTime,
                                                            @RequestParam(value = "endVerifyTime", required = false) String endVerifyTime) {
        return Result.success(bugInfoService.getStatisticalByHead(projectId, startSearchTime, endSearchTime, fix, startHandleTime, endHandleTime, startVerifyTime, endVerifyTime));
    }

    @ApiOperation("人员验证统计")
    @GetMapping("/getStatisticalBySure")
    public Result<List<StatisticsBug>> getStatisticalBySure(@RequestParam(value = "projectId") Integer projectId,
                                                            @RequestParam(value = "startSearchTime", required = false) String startSearchTime,
                                                            @RequestParam(value = "endSearchTime", required = false) String endSearchTime,
                                                            @RequestParam(value = "fix") Integer fix,
                                                            @RequestParam(value = "startHandleTime", required = false) String startHandleTime,
                                                            @RequestParam(value = "endHandleTime", required = false) String endHandleTime,
                                                            @RequestParam(value = "startVerifyTime", required = false) String startVerifyTime,
                                                            @RequestParam(value = "endVerifyTime", required = false) String endVerifyTime) {
        return Result.success(bugInfoService.getStatisticalBySure(projectId, startSearchTime, endSearchTime, fix, startHandleTime, endHandleTime, startVerifyTime, endVerifyTime));
    }
}
