package com.tofly.pms.testmanage.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.exceptions.ValidateException;
import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.base.controller.Result;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.testmanage.entity.BugInfo;
import com.tofly.pms.testmanage.entity.SystemItem;
import com.tofly.pms.testmanage.entity.SystemItemExcelModel;
import com.tofly.pms.testmanage.listener.SystemItemListener;
import com.tofly.pms.testmanage.service.BugInfoService;
import com.tofly.pms.testmanage.service.SystemItemService;
import com.tofly.pms.testmanage.vo.ItemStatusVo;
import com.tofly.pms.utils.ImportExcelHelper;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.dto.SystemItemBatchDto;
import com.tofly.pms.working.vo.TestTypeCountVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 测试系统功能清单表;(TF_PM_TESTSYSTEMITEM_W)表控制层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "测试系统功能清单表对象功能接口")
@RestController
@RequestMapping("/tfPmTestsystemitemW")
public class SystemItemWController {
    @Autowired
    private SystemItemService systemItemService;
    @Autowired
    private BugInfoService bugInfoService;
    @Autowired
    private ProjectTeamWService projectTeamWService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键testSystemService
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<SystemItem> queryById(Integer id) {
        return ResponseEntity.ok(systemItemService.queryById(id));
    }

    /**
     * 分页查询
     * <p>
     * <p>
     * select *
     * from business.tf_pm_testsystemitem_w
     * where number ~ '^[0-9]+(\.[0-9]+)*$'
     * ORDER BY CASE WHEN split_part(number, '.', 1) ~ '^[0-9]+$' THEN split_part(number, '.', 1) ::integer ELSE 0 END,
     * CASE WHEN split_part(number, '.', 2) ~ '^[0-9]+$' THEN split_part(number, '.', 2) ::integer ELSE 0 END,
     * CASE WHEN split_part(number, '.', 3) ~ '^[0-9]+$' THEN split_part(number, '.', 3) ::integer ELSE 0 END,
     * CASE WHEN split_part(number, '.', 4) ~ '^[0-9]+$' THEN split_part(number, '.', 4) ::integer ELSE 0 END,
     * CASE WHEN split_part(number, '.', 5) ~ '^[0-9]+$' THEN split_part(number, '.', 5) ::integer ELSE 0 END,
     * CASE WHEN split_part(number, '.', 6) ~ '^[0-9]+$' THEN split_part(number, '.', 6) ::integer ELSE 0 END,
     * CASE WHEN split_part(number, '.', 7) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,
     * CASE WHEN split_part(number, '.', 8) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,
     * CASE WHEN split_part(number, '.', 9) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END
     *
     * @param systemItem 筛选条件
     * @param page       分页对象
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<SystemItem>> paginQuery(SystemItem systemItem, Page<SystemItem> page) {

        final Integer queryGroup = systemItem.getQueryGroup();
        Page<SystemItem> page1;
        if (systemItem.getQueryGroup() != null && systemItem.getQueryGroup() != 0) {
            List<Integer> collect = new ArrayList<>();
            final List<ItemStatusVo> itemStatusVos = systemItemService.itemStatusCount(systemItem.getProjectId());
            if (queryGroup == 1) {
                //完成
                collect = itemStatusVos.stream()
                        .filter(f -> "1".equals(f.getStatus()))
                        .map(ItemStatusVo::getFunctionId).collect(Collectors.toList());

            } else if (queryGroup == 2) {
                //修复中

                collect = itemStatusVos.stream()
                        .filter(f -> "2".equals(f.getStatus()))
                        .map(ItemStatusVo::getFunctionId).collect(Collectors.toList());


            } else if (queryGroup == 3) {
                // 0缺陷
                //查找所有的item
                collect = systemItemService.list(Wrappers.<SystemItem>lambdaQuery()
                                .select(SystemItem::getId).in(SystemItem::getProjectId, systemItem.getProjectId())).stream()
                        .map(SystemItem::getId).collect(Collectors.toList());
                collect.removeAll(itemStatusVos.stream().map(ItemStatusVo::getFunctionId).collect(Collectors.toList()));


            }
            if (collect.size() < 1) {
                collect.add(999999);
            }
            String name = systemItem.getName();
            systemItem.setName(null);
            page1 = systemItemService.page(page, new QueryWrapper<>(systemItem)
                    .lambda()
                            .like(StringUtils.isNotBlank(name), SystemItem::getName, name)
                    .in(SystemItem::getId, collect)
                    .last("ORDER BY CASE WHEN split_part(number, '.', 1) ~ '^[0-9]+$' THEN split_part(number, '.', 1) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 2) ~ '^[0-9]+$' THEN split_part(number, '.', 2) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 3) ~ '^[0-9]+$' THEN split_part(number, '.', 3) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 4) ~ '^[0-9]+$' THEN split_part(number, '.', 4) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 5) ~ '^[0-9]+$' THEN split_part(number, '.', 5) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 6) ~ '^[0-9]+$' THEN split_part(number, '.', 6) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 7) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 8) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 9) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END"));
        } else {
            String name = systemItem.getName();
            systemItem.setName(null);
            page1 = systemItemService.page(page, new QueryWrapper<>(systemItem)
                    .lambda()
                    .like(StringUtils.isNotBlank(name), SystemItem::getName, name)
                    .last("ORDER BY CASE WHEN split_part(number, '.', 1) ~ '^[0-9]+$' THEN split_part(number, '.', 1) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 2) ~ '^[0-9]+$' THEN split_part(number, '.', 2) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 3) ~ '^[0-9]+$' THEN split_part(number, '.', 3) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 4) ~ '^[0-9]+$' THEN split_part(number, '.', 4) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 5) ~ '^[0-9]+$' THEN split_part(number, '.', 5) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 6) ~ '^[0-9]+$' THEN split_part(number, '.', 6) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 7) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 8) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,\n" +
                            "         CASE WHEN split_part(number, '.', 9) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END"));

        }
        //todo 查找每一个系统的bug统计    Page<SystemItem> page1;
        List<SystemItem> records = page1.getRecords();
        if (CollectionUtil.isNotEmpty(records)) {
            for (SystemItem record : records) {
                Integer id = record.getId();
                List<TestTypeCountVo> data = bugInfoService.getTestTypeCount(id);
                record.setData(data);
            }
        }
        return ResponseEntity.ok(page1);
    }

    /**
     * 新增数据
     *
     * @param systemItem 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<SystemItem> add(@RequestBody SystemItem systemItem) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前用户不合法");
        SystemItem byId = systemItemService.getById(systemItem);
        long one = projectTeamWService.count(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, byId.getProjectId())
                .and(wr -> wr.like(ProjectTeamW::getAdminId, userID)
                        .or()
                        .like(ProjectTeamW::getTestId, userID)));
        if (one==0) {
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if (!has) {
                throw new ValidateException("当前用户没有权限操作");
            }
        }
        systemItemService.save(systemItem);
        return ResponseEntity.ok(systemItem);
    }

    /**
     * 批量新增数据
     *
     * @param systemItemBatchDto 实例对象
     * @return 实例对象
     */
    @ApiOperation("批量新增修改数据")
    @PostMapping("saveOrUpdateBatch")
    public ResponseEntity<SystemItemBatchDto> saveOrUpdateBatch(@RequestBody SystemItemBatchDto systemItemBatchDto) {
        List<SystemItem> datas = systemItemBatchDto.getDatas();
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前用户不合法");
        List<SystemItem> hasId = datas.stream().filter(data -> Objects.nonNull(data.getId())).collect(Collectors.toList());
        for (SystemItem data : hasId) {
            SystemItem byId = systemItemService.getById(data);
            long one = projectTeamWService.count(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, byId.getProjectId())
                    .and(wr -> wr.like(ProjectTeamW::getAdminId, userID)
                            .or()
                            .like(ProjectTeamW::getTestId, userID)));
            if (one==0) {
                RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                if (!has) {
                    throw new ValidateException("当前用户没有权限操作");
                }
            }
            systemItemService.updateById(data);
        }
        List<SystemItem> noId = datas.stream().filter(data -> Objects.isNull(data.getId())).collect(Collectors.toList());
        for (SystemItem data : noId) {
            long one = projectTeamWService.count(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, data.getProjectId())
                    .and(wr -> wr.like(ProjectTeamW::getAdminId, userID)
                            .or()
                            .like(ProjectTeamW::getTestId, userID)));
            if (one == 0) {
                RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                if (!has) {
                    throw new ValidateException("当前用户没有权限操作");
                }
            }
            systemItemService.save(data);
        }
        return ResponseEntity.ok(systemItemBatchDto);
    }

    /**
     * 更新数据
     *
     * @param systemItem 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<SystemItem> edit(@RequestBody SystemItem systemItem) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前用户不合法");
        SystemItem byId = systemItemService.getById(systemItem);
        long one = projectTeamWService.count(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, byId.getProjectId())
                .and(wr -> wr.like(ProjectTeamW::getAdminId, userID)
                        .or()
                        .like(ProjectTeamW::getTestId, userID)));
        if (one==0) {
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if (!has) {
                throw new ValidateException("当前用户没有权限操作");
            }
        }
        systemItemService.updateById(systemItem);
        return ResponseEntity.ok(systemItem);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    @Transactional(rollbackFor = Exception.class)
    public ResponseEntity<Boolean> deleteById(String ids) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前用户不合法");

        final String[] split = ids.split(",");
        for (String s : split) {
            SystemItem byId = systemItemService.getById(Integer.valueOf(s));
            long one = projectTeamWService.count(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, byId.getProjectId())
                    .and(wr -> wr.like(ProjectTeamW::getAdminId, userID)
                            .or()
                            .like(ProjectTeamW::getTestId, userID)));
            if (one==0) {
                RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                if (!has) {
                    throw new ValidateException("当前用户没有权限操作");
                }
            }
            final long count = bugInfoService.count(Wrappers.<BugInfo>lambdaQuery()
                    .eq(BugInfo::getFunctionId, Integer.valueOf(s))
                    .eq(BugInfo::getDeleted,0));
            if (count > 0) {
                throw new ValidateException("该功能存在关联的bug信息，无法删除");
            }
            systemItemService.removeById(Integer.valueOf(s));
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

    @ApiOperation("获取唯一code")
    @GetMapping("/getSystemItemCode")
    public Result<String> getSystemItemCode() {
        return Result.success(systemItemService.getSystemItemCode());
    }

    @ApiOperation("导出excel")
    @GetMapping("/export")
    public void export(@RequestParam("systemId") Integer systemId) throws IOException {
        systemItemService.export(systemId);
    }

    @Transactional(rollbackFor = Exception.class)
    @ApiOperation("批量导入")
    @PostMapping("/import")
    public ResponseEntity<Boolean> feedbackImport(@RequestParam("file") MultipartFile file,
                                                  @RequestParam("projectId") Integer projectId,
                                                  @RequestParam("systemId") Integer systemId) throws Exception {

        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前用户不合法");
        long one = projectTeamWService.count(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId)
                .and(wr -> wr.like(ProjectTeamW::getAdminId, userID)
                        .or()
                        .like(ProjectTeamW::getTestId, userID)));
        if (one==0) {
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if (!has) {
                throw new ValidateException("当前用户没有权限操作");
            }
        }
        ImportExcelHelper<SystemItemExcelModel> helper = new ImportExcelHelper<>(new SystemItemListener(1));
        List<SystemItemExcelModel> importData = helper.getList(file, SystemItemExcelModel.class, 0, 1);
        if (CollectionUtil.isEmpty(importData)) {
            return ResponseEntity.ok(Boolean.FALSE);
        }
        List<SystemItem> systemItemList = new ArrayList<>(importData.size());
        for (SystemItemExcelModel model : importData) {
            SystemItem systemItem = model.toSystemItem();
            systemItem.setProjectId(projectId);
            systemItem.setSystemId(systemId);
            systemItemList.add(systemItem);
        }
        Map<String, SystemItem> systemItemMap = systemItemList.stream().collect(Collectors.toMap(SystemItem::getNumber, item -> item, (k1, k2) -> k1));
        for (SystemItem item : systemItemList) {
            item.setCode(systemItemService.getSystemItemCode());
        }
        for (SystemItem item : systemItemList) {
            String number = item.getNumber();
            String parentNumber = this.getParentNumber(number);
            if (StringUtils.isNotBlank(parentNumber)) {
                SystemItem parentSystemItem = systemItemMap.get(parentNumber);
                if (ObjectUtils.isNotNull(parentSystemItem)) {
                    item.setParentCode(parentSystemItem.getCode());
                }
            }
        }

        systemItemService.remove(Wrappers.lambdaQuery(SystemItem.class)
                .eq(SystemItem::getProjectId, projectId)
                .eq(SystemItem::getSystemId, systemId));
        boolean b = systemItemService.saveBatch(systemItemList);
        return ResponseEntity.ok(b);
    }

    private String getParentNumber(String number) {
        if (number.contains(".")) {
            return number.substring(0, number.lastIndexOf("."));
        }
        return null;
    }
}