package com.tofly.pms.testmanage.controller;

import cn.hutool.core.exceptions.ValidateException;
import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.testmanage.entity.BugInfo;
import com.tofly.pms.testmanage.entity.TestSystem;
import com.tofly.pms.testmanage.service.BugInfoService;
import com.tofly.pms.testmanage.service.SystemItemService;
import com.tofly.pms.testmanage.service.TestSystemService;
import com.tofly.pms.testmanage.vo.BugGroupCountVo;
import com.tofly.pms.testmanage.vo.ProjectBugCountVo;
import com.tofly.pms.utils.UserTokenUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.Objects;

/**
 * 测试系统名称表;(TF_PM_TESTSYSTEM_W)表控制层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "测试系统名称表对象功能接口")
@RestController
@RequestMapping("/tfPmTestsystemW")
public class TestSystemWController {
    @Autowired
    private TestSystemService testSystemService;
    @Autowired
    private SystemItemService systemItemService;
    @Autowired
    private ProjectTeamWService projectTeamWService;
    @Autowired
    private BugInfoService bugInfoService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键testSystemService
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<TestSystem> queryById(Integer id) {
        return ResponseEntity.ok(testSystemService.queryById(id));
    }

    /**
     * 分页查询
     *
     * @param testSystem 筛选条件
     * @param page       分页对象
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<TestSystem>> paginQuery(TestSystem testSystem, Page page) {


        final Page page1 = testSystemService.page(page, new QueryWrapper<>(testSystem));
        return ResponseEntity.ok(page1);
    }

    /**
     * 新增数据
     *
     * @param testSystem 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<TestSystem> add(@RequestBody TestSystem testSystem) {
        Integer projectId = testSystem.getProjectId();
        Assert.notNull(projectId, "项目id不能为空");
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前操作用户不合法");
        ProjectTeamW projectTeam = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId)
                .and(wr -> wr.like(ProjectTeamW::getTestId, userID)
                        .or()
                        .like(ProjectTeamW::getAdminId, userID))
        );
        if (Objects.isNull(projectTeam)) {
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if (!has) {
                throw new RuntimeException("当前用户没有权限");
            }
        }
        testSystemService.save(testSystem);
        return ResponseEntity.ok(testSystem);
    }

    /**
     * 更新数据
     *
     * @param testSystem 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<TestSystem> edit(@RequestBody TestSystem testSystem) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前操作用户不合法");
        TestSystem dbTestSystem = testSystemService.queryById(testSystem.getId());
        if (Objects.isNull(dbTestSystem)) {
            return ResponseEntity.ok().build();
        }
        Integer projectId = dbTestSystem.getProjectId();
        ProjectTeamW projectTeam = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId)
                .and(wr -> wr.like(ProjectTeamW::getTestId, userID).or().like(ProjectTeamW::getAdminId, userID))
        );
        if (Objects.isNull(projectTeam)) {
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if (!has) {
                throw new RuntimeException("当前用户没有权限");
            }
        }
        testSystemService.updateById(testSystem);
        return ResponseEntity.ok(testSystem);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public ResponseEntity<Boolean> deleteById(String ids) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前操作用户不合法");
        final String[] split = ids.split(",");
        for (String s : split) {
            //判断该数据 是否存在关联的bug
            final long count = bugInfoService.count(Wrappers.<BugInfo>lambdaQuery()
                    .eq(BugInfo::getSystemId, Integer.valueOf(s))
                    .eq(BugInfo::getDeleted, 0));
            if (count > 0) {
                throw new ValidateException("该系统存在bug,无法删除");
            }

            TestSystem dbTestSystem = testSystemService.queryById(Integer.valueOf(s));
            if (Objects.isNull(dbTestSystem)) {
                return ResponseEntity.ok().build();
            }
            Integer projectId = dbTestSystem.getProjectId();
            ProjectTeamW projectTeam = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, projectId)
                    .and(wr -> wr.like(ProjectTeamW::getTestId, userID).or().like(ProjectTeamW::getAdminId, userID))
            );
            if (Objects.isNull(projectTeam)) {
                RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                if (!has) {
                    throw new RuntimeException("当前用户没有权限");
                }
            }

            testSystemService.removeById(Integer.valueOf(s));
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }


    /**
     * 系项目 以及系统提测率通过率统计
     *
     * @param projectId 主键testSystemService
     * @return 实例对象
     */
//    @ApiOperation("项目 以及系统提测率通过率统计")
//    @GetMapping("/systemProjectCont")
//    public ResponseEntity<ProjectBugCountVo> systemProjectCont(Integer projectId) {
//
//        return ResponseEntity.ok(testSystemService.projectCont(projectId));
//    }
    @ApiOperation("项目 以及系统提测率通过率统计")
    @GetMapping("/systemProjectCont")
    public ResponseEntity<ProjectBugCountVo> systemProjectCont(Integer projectId) {

        return ResponseEntity.ok(testSystemService.projectContByProjectId(projectId));
    }

    /**
     * 缺陷分类统计
     *
     * @param systemId 主键testSystemService
     * @return 实例对象
     */
    @ApiOperation("项目 以及系统提测率通过率统计")
    @GetMapping("/systemBugGroupCount")
    public ResponseEntity<BugGroupCountVo> systemBugGroupCount(Integer systemId) {

        return ResponseEntity.ok(testSystemService.systemBugGroupCount(systemId));
    }

}