package com.tofly.pms.testmanage.entity;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.file.entity.File;
import com.tofly.pms.file.service.FileService;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.testmanage.service.SystemItemService;
import com.tofly.pms.testmanage.service.TestSystemService;
import com.tofly.pms.utils.dict.Dict;
import com.tofly.pms.utils.vocast.VoCast;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;
import java.util.Date;
import java.util.Map;

/**
 * 测试BUG表;
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Slf4j
@ApiModel(value = "测试BUG表", description = "")
@TableName(value = "TF_PM_BUG_W", schema = "business")
@Data
public class BugInfo implements Serializable, Cloneable {
    /**
     * id
     */
    @ApiModelProperty(value = "id", name = "id")
    @TableId(type = IdType.AUTO)
    private Integer id;
    /**
     * BUG编号
     */
    @ApiModelProperty(value = "BUG编号", name = "code")
    private String code;
    /**
     * 项目id
     */
    @ApiModelProperty(value = "项目id", name = "projectId")
    @VoCast(serviceClass = ProjectWService.class, valueClass = ProjectW.class)
    private Integer projectId;
    /**
     * 系统id
     */
    @ApiModelProperty(value = "系统id", name = "systemId")
    @VoCast(serviceClass = TestSystemService.class, valueClass = TestSystem.class)
    private Integer systemId;
    /**
     * 功能唯一编码
     */
    @ApiModelProperty(value = "功能id", name = "functionId")
    @VoCast(serviceClass = SystemItemService.class, valueClass = SystemItem.class)
    private Integer functionId;
    @ApiModelProperty(value = "功能id", name = "functionId")
    @TableField(exist = false)
    private String fullFunctionName;
    /**
     * 缺陷标题
     */
    @ApiModelProperty(value = "缺陷标题", name = "title")
    private String title;
    /**
     * 缺陷简述
     */
    @ApiModelProperty(value = "缺陷简述", name = "shortDescription")
    private String shortDescription;
    /**
     * 缺陷描述
     */
    @ApiModelProperty(value = "缺陷描述", name = "description")
    private String description;
    /**
     * 系统版本
     */
    @ApiModelProperty(value = "系统版本", name = "version")
    private String version;
    /**
     * 测试环境
     */
    @ApiModelProperty(value = "测试环境", name = "enviroment")
    private String enviroment;
    /**
     * 测试类型
     */
    @ApiModelProperty(value = "测试类型 CSLX", name = "testType")
    @Dict(code = "CSLX")
    private String testType;
    /**
     * 缺陷类型
     */
    @ApiModelProperty(value = "缺陷类型 BUGLX", name = "type")
    @Dict(code = "BUGLX")
    private String type;
    /**
     * 严重程度
     */
    @ApiModelProperty(value = "严重程度 YZCD", name = "severity")
    @Dict(code = "YZCD")
    private String severity;
    /**
     * 紧急程度
     */
    @ApiModelProperty(value = "紧急程度 JJCD", name = "urgency")
    @Dict(code = "JJCD")
    private String urgency;
    /**
     * 缺陷类别
     */
    @ApiModelProperty(value = "缺陷类别 BUGLB", name = "defectCategory")
    @Dict(code = "BUGLB")
    private String defectCategory;
    /**
     * 负责人ID
     */
    @ApiModelProperty(value = "负责人ID", name = "headId")
    @VoCast(serviceClass = UserClient.class, isMulti = true, valueClass = UserVo.class)
    private String headId;
    /**
     * 确认人ID，有多个确认人用逗号连接
     */
    @ApiModelProperty(value = "待确认人ID", name = "sureId")
    @VoCast(serviceClass = UserClient.class, isMulti = true, valueClass = UserVo.class)
    private String sureId;
    /**
     * 已经确认人ID
     */
    @ApiModelProperty(value = "已经确认人ID", name = "alreadySureId")
    @VoCast(serviceClass = UserClient.class, isMulti = true, valueClass = UserVo.class)
    private String alreadySureId;
    /**
     * 关注人ID
     */
    @ApiModelProperty(value = "关注人ID", name = "careId")
    @VoCast(serviceClass = UserClient.class, isMulti = true, valueClass = UserVo.class)
    private String careId;
    /**
     * 创建人标识
     */
    @ApiModelProperty(value = "创建人标识", name = "createUser")
    @VoCast(serviceClass = UserClient.class, isMulti = false, valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT)
    private String createUser;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间", name = "createTime")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;
    /**
     * 建议完成时间
     */
    @ApiModelProperty(value = "建议完成时间", name = "adviseTime")
    private String adviseTime;
    /**
     * 缺陷状态
     */
    @ApiModelProperty(value = "缺陷状态 RWZT", name = "status ")
    @Dict(code = "RWZT")
    private String status;
    /**
     * 缺陷状态
     */
    @ApiModelProperty(value = "缺陷处理方式 CLFS 或者 验证结果 YZJG", name = "processingMode")
    @Dict(code = "YZJG")
    private String processingMode;
    @ApiModelProperty(value = "文件id,多个以,号分割")
    @VoCast(serviceClass = FileService.class, isMulti = true, valueClass = File.class)
    private String fileUrl;
    @ApiModelProperty(value = "图片id,多个以,号分割")
    @VoCast(serviceClass = FileService.class, isMulti = true, valueClass = File.class)
    private String pictureUrl;

    @ApiModelProperty(value = "验证之后上传图片id,多个以,号分割")
    @VoCast(serviceClass = FileService.class, isMulti = true, valueClass = File.class)
    private String verifyUrl;
    @ApiModelProperty(value = "备注")
    private String notes;
    @TableField(fill = FieldFill.INSERT)
    @ApiModelProperty(value = "是否删除")
    private Integer deleted;

    /**
     * 完成时间  即状态变更为已完成的时候的操作时间
     */
    @ApiModelProperty(value = "完成时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date completeTime;


    @ApiModelProperty(value = "是否逾期 1是 0否")
    //结束时间 <当前时间
    private Integer timeStatus;
    @ApiModelProperty(value = "是否正在处理：0：未处理，1：正在处理")
    private Integer processing;

    public void setProcessingModeAnnotation() {
        try {
            Field field = this.getClass().getDeclaredField("processingMode");
            Dict annotation = field.getAnnotation(Dict.class);
            InvocationHandler invocationHandler = Proxy.getInvocationHandler(annotation);
            Field hf = invocationHandler.getClass().getDeclaredField("memberValues");
            hf.setAccessible(true);
            Map<String, Object> memberValues = (Map<String, Object>) hf.get(invocationHandler);
            if ("4".equals(this.status)) {
                memberValues.put("code", "YZJG");
                hf.setAccessible(false);
            } else {
                memberValues.put("code", "CLFS");
                hf.setAccessible(false);
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }

    public BugLog createLog() {
        BugLog bugLog = new BugLog();
        bugLog.setBugId(this.getId());
        bugLog.setStatus(this.status);
        bugLog.setProcessingMode(this.processingMode);
        bugLog.setJsonData(JSON.toJSONString(this));
        return bugLog;

    }
}