package com.tofly.pms.testmanage.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.file.entity.File;
import com.tofly.pms.file.service.FileService;
import com.tofly.pms.utils.dict.Dict;
import com.tofly.pms.utils.vocast.VoCast;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;
import java.util.Date;
import java.util.Map;

/**
 * BUG处理日志表;
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Slf4j
@ApiModel(value = "BUG处理日志表", description = "")
@TableName(value = "TF_PM_BUGLOG_W", schema = "business")
@Data
public class BugLog implements Serializable, Cloneable {
    /**
     * id
     */
    @ApiModelProperty(value = "id", name = "id")
    @TableId(type = IdType.AUTO)
    private Integer id;
    /**
     * 缺陷id
     */
    @ApiModelProperty(value = "缺陷id", name = "bugId")
    private Integer bugId;
    /**
     * 处理时间
     */
    @ApiModelProperty(value = "处理时间", name = "handleTime")
    private Date handleTime;
    /**
     * 处理流程
     */
    @ApiModelProperty(value = "处理流程", name = "flow")
    private String flow;
    /**
     * 处理描述
     */
    @ApiModelProperty(value = "处理描述", name = "description")
    private String description;
    /**
     * 处理状态
     */
    @ApiModelProperty(value = "处理状态 RWZT", name = "status")
    @Dict(code = "RWZT")
    private String status;
    @ApiModelProperty(value = "缺陷处理方式 CLFS", name = "processingMode")
    @Dict(code = "YZJG")
    private String processingMode;
    /**
     * 处理人员
     */
    @ApiModelProperty(value = "处理人员", name = "handleUser")
    @VoCast(serviceClass = UserClient.class, isMulti = false, valueClass = UserVo.class)
    private String handleUser;
    /**
     * 流程序号
     */
    @ApiModelProperty(value = "流程序号", name = "flowSort")
    private String flowSort;

    @ApiModelProperty(value = "文件id,多个以,号分割")
    @VoCast(serviceClass = FileService.class, isMulti = true, valueClass = File.class)
    private String fileUrl;

    /**
     * bug的json对象
     */
    @ApiModelProperty(value = "bug的json对象", name = "JsonData")
    private String JsonData;

    @ApiModelProperty(value = "日志主要内容")
    private String content;

    @TableField(exist = false)
    private BugInfo bugInfo;

    public void setProcessingModeAnnotation() {
        try {
            Field field = this.getClass().getDeclaredField("processingMode");
            Dict annotation = field.getAnnotation(Dict.class);
            InvocationHandler invocationHandler = Proxy.getInvocationHandler(annotation);
            Field hf = invocationHandler.getClass().getDeclaredField("memberValues");
            hf.setAccessible(true);
            Map<String, Object> memberValues = (Map<String, Object>) hf.get(invocationHandler);
            if ("4".equals(this.status)) {
                memberValues.put("code", "YZJG");
                hf.setAccessible(false);
            }  else {
                memberValues.put("code", "CLFS");
                hf.setAccessible(false);
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
    }
}