package com.tofly.pms.testmanage.entity;

import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.service.ProjectDevWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.testmanage.service.SystemItemService;
import com.tofly.pms.testmanage.service.TestSystemService;
import com.tofly.pms.utils.dict.Dict;
import com.tofly.pms.utils.vocast.VoCast;
import com.tofly.pms.working.vo.TestTypeCountVo;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 测试系统功能清单表;
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@ApiModel(value = "测试系统功能清单表", description = "")
@TableName(value = "TF_PM_TESTSYSTEMITEM_W", schema = "business")
@Data
public class SystemItem implements Serializable, Cloneable {
    /**
     * id
     */
    @ApiModelProperty(value = "id", name = "id")
    @TableId(type = IdType.AUTO)
    private Integer id;
    /**
     * 项目id
     */
    @ApiModelProperty(value = "项目id", name = "projectId")
    @VoCast(serviceClass = ProjectWService.class, valueClass = ProjectW.class)
    private Integer projectId;
    /**
     * 系统id
     */
    @ApiModelProperty(value = "系统id", name = "systemId")
    @VoCast(serviceClass = TestSystemService.class, valueClass = TestSystem.class)
    private Integer systemId;
    /**
     * 功能唯一编码
     */
    @ApiModelProperty(value = "功能唯一编码", name = "code")
    private String code;
    /**
     * 功能名称
     */
    @ApiModelProperty(value = "功能名称", name = "name")
    private String name;
    /**
     * 上级功能编码
     */
    @ApiModelProperty(value = "上级功能编码", name = "parentCode")
    private String parentCode;

    /**
     * 测试要求
     */
    @ApiModelProperty(value = "测试要求", name = "demand")
    private String demand;

    @ApiModelProperty(value = "负责人", name = "headUser")
    @VoCast(serviceClass = UserClient.class, isMulti = true, valueClass = UserVo.class)
    private String headUser;


    @ApiModelProperty(value = "测试状态 TESTSTATUS ", name = "status")
    @Dict(code = "TESTSTATUS")
    private String status;

    @ApiModelProperty(value = "是否开始测试 BOOL", name = "isStartTest")
    //@Dict(code = "BOOL")
    private String isStartTest;

    @ApiModelProperty(value = "是否提测 BOOL", name = "isCommitTest")
    // @Dict(code = "BOOL")
    private String isCommitTest;

    @ApiModelProperty(value = "排序", name = "isStartTest")
    private Integer sortNum;

    /**
     * 0 全部 1 完成  2 修复中  3 0缺陷
     **/
    @ApiModelProperty(value = "0 全部 1 完成  2 修复中  3 0缺陷", name = "queryGroup")
    @TableField(exist = false)
    private Integer queryGroup;
    @ApiModelProperty(value = "序号", name = "number")
    private String number;

    @ApiModelProperty(value = "测试类型统计", name = "data")
    @TableField(exist = false)
    private List<TestTypeCountVo> data;

    /**
     * 创建人标识
     */
    @ApiModelProperty(value = "创建人标识")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = true, valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT)
    private String createUser;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT)
    private Date createDate;



    /**
     * 创建人标识
     */
    @ApiModelProperty(value = "创建人标识")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = true, valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateUser;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    @ApiModelProperty(value = "级别", name = "level")
    private Integer level;

    public SystemItemExcelModel toExcelModel() {
        SystemItemExcelModel excelModel = new SystemItemExcelModel();
        excelModel.setId(this.id);
        excelModel.setName(this.name);
        if (StringUtils.isNotBlank(this.parentCode)) {
            SystemItemService systemItemService = SpringUtil.getBean(SystemItemService.class);
            SystemItem parent = systemItemService.getOne(Wrappers.lambdaQuery(SystemItem.class).eq(SystemItem::getCode, this.parentCode));
            excelModel.setParentName(parent.name);
        }
        excelModel.setDemand(this.demand);
        if ("true".equalsIgnoreCase(this.isCommitTest)) {
            excelModel.setIsCommitTest("是");
        } else {
            excelModel.setIsCommitTest("否");
        }

        if ("true".equalsIgnoreCase(this.isStartTest)) {
            excelModel.setIsStartTest("是");
        } else {
            excelModel.setIsStartTest("否");
        }
        excelModel.setNumber(this.number);
        return excelModel;
    }
}