package com.tofly.pms.testmanage.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.pms.fegin.DictItemClient;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.testmanage.entity.*;
import com.tofly.pms.testmanage.mapper.BugInfoMapper;
import com.tofly.pms.testmanage.service.BugInfoService;
import com.tofly.pms.testmanage.service.BugLogService;
import com.tofly.pms.testmanage.vo.*;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.entity.Editorlog;
import com.tofly.pms.working.service.EditorlogService;
import com.tofly.pms.working.vo.TestTypeCountVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 测试BUG表;(TF_PM_BUG_W)表服务实现类
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Service
public class BugInfoServiceImpl extends ServiceImpl<BugInfoMapper, BugInfo>
        implements BugInfoService {

    @Resource
    private BugLogService bugLogService;
    @Resource
    private EditorlogService editorlogService;
    @Resource
    HttpServletResponse response;
    @Resource
    ProjectWService projectWService;
    @Resource
    ProjectTeamWService projectTeamWService;

    @Override
    public BugInfo queryById(Integer id) {
        return super.getById(id);
    }

    @Override
    public List<BugStatusCountVo> bugStatusOverCount(BugInfo bugInfo) {
        List<Integer> selfProjectId = projectWService.getSelfProjectId();
        return baseMapper.bugStatusOverCount(bugInfo,selfProjectId);
    }

    @Override
    public ToResolveCountVo gettoResolveCount(BugInfo bugInfo) {
        String tokenUserID = UserTokenUtil.getTokenUserID();
        String severity = bugInfo.getSeverity();
        String urgency = bugInfo.getUrgency();
        Integer systemId = bugInfo.getSystemId();
        String title = bugInfo.getTitle();
        bugInfo.setSeverity(null);
        bugInfo.setUrgency(null);
        bugInfo.setSystemId(null);
        bugInfo.setTitle(null);
        bugInfo.setShortDescription(null);
        bugInfo.setDescription(null);
        LambdaQueryWrapper<BugInfo> wrapper = Wrappers
                .lambdaQuery(bugInfo)
                .eq(BugInfo::getDeleted, 0)
                .like(BugInfo::getHeadId, tokenUserID)
                .eq(com.baomidou.mybatisplus.core.toolkit.StringUtils.isNotBlank(severity), BugInfo::getSeverity, severity)
                .eq(com.baomidou.mybatisplus.core.toolkit.StringUtils.isNotBlank(urgency), BugInfo::getUrgency, urgency)
                .eq(Objects.nonNull(systemId), BugInfo::getSystemId, systemId);
        if (com.baomidou.mybatisplus.core.toolkit.StringUtils.isNotBlank(title)) {
            wrapper.and(w -> w.like(BugInfo::getTitle, title)
                    .or().like(BugInfo::getShortDescription, title)
                    .or().like(BugInfo::getDescription, title)
            );
        }
        wrapper.and(w -> w.eq(BugInfo::getStatus, "1")
                .or()
                .eq(BugInfo::getStatus, "4"));
        ToResolveCountVo toResolveCountVo = new ToResolveCountVo();
        final long count = this.count(wrapper);
        toResolveCountVo.setTotal(count);
        bugInfo.setHeadId(tokenUserID);
        bugInfo.setTitle(title);
        bugInfo.setSystemId(systemId);
        bugInfo.setUrgency(urgency);
        bugInfo.setSeverity(severity);

        final List<ToResolveOverTypeVo> toResolveOverTypeVos = baseMapper.gettoResolveCount(bugInfo);
        toResolveCountVo.setDatas(toResolveOverTypeVos);
        return toResolveCountVo;
    }

    @Override
    public List<BugStatusCountProjectVo> BugStatusCountOverProject(BugInfo bugInfo) {

        List<BugStatusCountProjectVo> list = new ArrayList<>();
        final List<BugStatusCountOverProjectVo> datas =
                baseMapper.BugStatusCountOverProject(bugInfo);

        final Map<Integer, List<BugStatusCountOverProjectVo>> collect =
                datas.stream().collect(Collectors.groupingBy(BugStatusCountOverProjectVo::getProjectId));
        for (Map.Entry<Integer, List<BugStatusCountOverProjectVo>> entry : collect.entrySet()) {
            BugStatusCountProjectVo bugStatusCountProjectVo = new BugStatusCountProjectVo();
            bugStatusCountProjectVo.setProjectId(entry.getKey());
            bugStatusCountProjectVo.setDatas(entry.getValue());
            list.add(bugStatusCountProjectVo);
        }

        return list;
    }

    @Override
    public List<ToSureLeftVo> toSureCountLeft(String userId) {
        //查询当前成员所属的项目


        //统计项目中的bug，
        // this.li
        return baseMapper.toSureLeft(null, userId);
    }

    @Override
    @Transactional
    public void saveOrUpdateWithLog(BugInfo bugInfo) {
        BugInfo oldInfo = null;
        if (bugInfo.getId() != null) {
            oldInfo = this.queryById(bugInfo.getId());
        }
        BugLog bugLog = new BugLog();


    }

    @Override
    public void saveOrUpdateBatchWithLog(List<BugInfo> bugInfos) {

    }

    @Override
    public List<TestTypeCountVo> getTestTypeCount(Integer functionId) {
        return baseMapper.getTestTypeCount(functionId);
    }

    @Override
    public List<StatisticsBug> getStatisticalEnsemble(Integer projectId, String startSearchTime, String endSearchTime,
                                                      String startHandleTime,
                                                      String endHandleTime,
                                                      String startVerifyTime,
                                                      String endVerifyTime) {
        Assert.notNull(projectId, "项目id不能为空");
        LocalDateTime startTime = null;
        LocalDateTime endTime = null;
        if (StringUtils.isNotBlank(startSearchTime)) {
            startTime = LocalDateTime.parse(startSearchTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endSearchTime)) {
            endTime = LocalDateTime.parse(endSearchTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        LocalDateTime startHandle = null;
        LocalDateTime endHandle = null;
        if (StringUtils.isNotBlank(startHandleTime)) {
            startHandle = LocalDateTime.parse(startHandleTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endHandleTime)) {
            endHandle = LocalDateTime.parse(endHandleTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        LocalDateTime startVerify = null;
        LocalDateTime endVerify = null;
        if (StringUtils.isNotBlank(startVerifyTime)) {
            startVerify = LocalDateTime.parse(startVerifyTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endVerifyTime)) {
            endVerify = LocalDateTime.parse(endVerifyTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        return baseMapper.getStatisticalEnsemble(projectId, startTime, endTime, startHandle, endHandle, startVerify, endVerify);
    }

    @Override
    public List<StatisticsBug> getStatisticalBySystem(Integer projectId, String startSearchTime, String endSearchTime, Integer fix,
                                                      String startHandleTime,
                                                      String endHandleTime,
                                                      String startVerifyTime,
                                                      String endVerifyTime) {
        Assert.notNull(projectId, "项目id不能为空");
        Assert.notNull(fix, "修复状态不能为空");
        LocalDateTime startTime = null;
        LocalDateTime endTime = null;
        if (StringUtils.isNotBlank(startSearchTime)) {
            startTime = LocalDateTime.parse(startSearchTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endSearchTime)) {
            endTime = LocalDateTime.parse(endSearchTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        LocalDateTime startHandle = null;
        LocalDateTime endHandle = null;
        if (StringUtils.isNotBlank(startHandleTime)) {
            startHandle = LocalDateTime.parse(startHandleTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endHandleTime)) {
            endHandle = LocalDateTime.parse(endHandleTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        LocalDateTime startVerify = null;
        LocalDateTime endVerify = null;
        if (StringUtils.isNotBlank(startVerifyTime)) {
            startVerify = LocalDateTime.parse(startVerifyTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endVerifyTime)) {
            endVerify = LocalDateTime.parse(endVerifyTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        return baseMapper.getStatisticalBySystem(projectId, startTime, endTime, fix, startHandle, endHandle, startVerify, endVerify);
    }

    @Override
    public List<StatisticsBug> getStatisticalByHead(Integer projectId, String startSearchTime, String endSearchTime, Integer fix,
                                                    String startHandleTime,
                                                    String endHandleTime,
                                                    String startVerifyTime,
                                                    String endVerifyTime) {
        Assert.notNull(projectId, "项目id不能为空");
        Assert.notNull(fix, "修复状态不能为空");
        LocalDateTime startTime = null;
        LocalDateTime endTime = null;
        if (StringUtils.isNotBlank(startSearchTime)) {
            startTime = LocalDateTime.parse(startSearchTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endSearchTime)) {
            endTime = LocalDateTime.parse(endSearchTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        LocalDateTime startHandle = null;
        LocalDateTime endHandle = null;
        if (StringUtils.isNotBlank(startHandleTime)) {
            startHandle = LocalDateTime.parse(startHandleTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endHandleTime)) {
            endHandle = LocalDateTime.parse(endHandleTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        LocalDateTime startVerify = null;
        LocalDateTime endVerify = null;
        if (StringUtils.isNotBlank(startVerifyTime)) {
            startVerify = LocalDateTime.parse(startVerifyTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endVerifyTime)) {
            endVerify = LocalDateTime.parse(endVerifyTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        return baseMapper.getStatisticalByHead(projectId, startTime, endTime, fix, startHandle, endHandle, startVerify, endVerify);
    }

    @Override
    public List<StatisticsBug> getStatisticalBySure(Integer projectId, String startSearchTime, String endSearchTime, Integer fix,
                                                    String startHandleTime,
                                                    String endHandleTime,
                                                    String startVerifyTime,
                                                    String endVerifyTime) {
        Assert.notNull(projectId, "项目id不能为空");
        Assert.notNull(fix, "修复状态不能为空");
        LocalDateTime startTime = null;
        LocalDateTime endTime = null;
        if (StringUtils.isNotBlank(startSearchTime)) {
            startTime = LocalDateTime.parse(startSearchTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endSearchTime)) {
            endTime = LocalDateTime.parse(endSearchTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        LocalDateTime startHandle = null;
        LocalDateTime endHandle = null;
        if (StringUtils.isNotBlank(startHandleTime)) {
            startHandle = LocalDateTime.parse(startHandleTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endHandleTime)) {
            endHandle = LocalDateTime.parse(endHandleTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        LocalDateTime startVerify = null;
        LocalDateTime endVerify = null;
        if (StringUtils.isNotBlank(startVerifyTime)) {
            startVerify = LocalDateTime.parse(startVerifyTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        if (StringUtils.isNotBlank(endVerifyTime)) {
            endVerify = LocalDateTime.parse(endVerifyTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        }
        return baseMapper.getStatisticalBySure(projectId, startTime, endTime, fix, startHandle, endHandle, startVerify, endVerify);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateBugInfo(BugInfo bugInfo) {
        String userID = UserTokenUtil.getTokenUserID();
        Integer id = bugInfo.getId();
        BugInfo dbBugInfo = baseMapper.selectById(id);
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        if(ObjectUtil.notEqual(userID,dbBugInfo.getCreateUser())){
            Integer projectId = dbBugInfo.getProjectId();
            ProjectTeamW projectTeam = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, projectId)
                    .and(wr->wr.like(ProjectTeamW::getAdminId, userID)
                            .or()
                            .like(ProjectTeamW::getTestId,userID))
            );
            if (Objects.isNull(projectTeam)) {
                RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                if (!has) {
                    throw new RuntimeException("当前用户没有权限");
                }
            }
        }
        Editorlog editorlog = new Editorlog();
        editorlog.setDataId(id);
        editorlog.setDataType("bug");
        editorlog.setSource(JSON.toJSONString(dbBugInfo));
        editorlog.setChangedData(JSON.toJSONString(bugInfo));
        UserVo userInfo = userClient.getUserInfo(userID);
        String content = "[" + userInfo.getRealName() + "]编辑缺陷";
        String dbHeadId = dbBugInfo.getHeadId();
        String headId = bugInfo.getHeadId();
        editorlog.setSourceHeadId(dbHeadId);
        editorlog.setChangedHeadId(headId);
        if (ObjectUtil.notEqual(dbHeadId, headId)) {
            if("null".equals(headId)){
                bugInfo.setHeadId(dbHeadId);
                editorlog.setChangedHeadId(dbHeadId);
            }else {
                UserVo dbHeadUser = userClient.getUserInfo(dbHeadId);
                UserVo headUser = userClient.getUserInfo(headId);
                content += "，负责人由[" + dbHeadUser.getRealName() + "]变更为[" + headUser.getRealName() + "]";
            }
        }
        String dbSureId = dbBugInfo.getSureId();
        String sureId = bugInfo.getSureId();
        editorlog.setSourceSureId(dbSureId);
        editorlog.setChangedSureId(sureId);
        if (ObjectUtil.notEqual(dbSureId, sureId)) {
            if("null".equals(sureId)){
                bugInfo.setSureId(dbSureId);
                editorlog.setChangedSureId(sureId);
            }else{
                StringBuilder dbHeadUserName = new StringBuilder();
                StringBuilder headUserName = new StringBuilder();
                if (StringUtils.isNotBlank(dbSureId)) {
                    String[] split = dbSureId.split(",");
                    for (String suserId : split) {
                        UserVo dbHeadUser = userClient.getUserInfo(suserId);
                        dbHeadUserName.append(dbHeadUser.getRealName()).append(",");
                    }
                }
                if (StringUtils.isNotBlank(sureId)) {
                    String[] split = sureId.split(",");
                    for (String suserId : split) {
                        UserVo headUser = userClient.getUserInfo(suserId);
                        headUserName.append(headUser.getRealName()).append(",");
                    }
                }
                content += "，确认人由[" + dbHeadUserName + "]变更为[" + headUserName + "]";
            }
        }
        editorlog.setContent(content);
        editorlogService.save(editorlog);
        bugInfo.setAlreadySureId(null);
        bugInfo.setCompleteTime(dbBugInfo.getCompleteTime());
        if (StringUtils.isNotBlank(bugInfo.getHeadId()) && "0".equals(dbBugInfo.getStatus())) {
            bugInfo.setStatus("1");
        } else if (StringUtils.isBlank(bugInfo.getHeadId())) {
            bugInfo.setStatus("0");
        } else {
            bugInfo.setStatus(dbBugInfo.getStatus());
        }
        baseMapper.updateById(bugInfo);
    }

    @Override
    public void export(Integer projectId, Integer systemId, String headId, String createUser) throws IOException {
        List<BugInfoExcelModel> data = baseMapper.getExportData(projectId, systemId, headId, createUser);
        DictItemClient bean = SpringUtil.getBean(DictItemClient.class);
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        for (BugInfoExcelModel datum : data) {
            String status = bean.getDicItemText("RWZT", datum.getStatus());
            String severity = bean.getDicItemText("YZCD", datum.getSeverity());
            String urgency = bean.getDicItemText("JJCD", datum.getUrgency());
            String testType = bean.getDicItemText("CSLX", datum.getTestType());
            String type = bean.getDicItemText("BUGLX", datum.getType());
            datum.setStatus(status);
            datum.setSeverity(severity);
            datum.setUrgency(urgency);
            datum.setTestType(testType);
            datum.setType(type);
            String sureUser = datum.getSureUser();
            if (StringUtils.isNotBlank(sureUser)) {
                String[] split = sureUser.split(",");
                sureUser = Arrays.stream(split).filter(StringUtils::isNotBlank)
                        .map(userClient::getUserInfo)
                        .map(UserVo::getRealName)
                        .collect(Collectors.joining(","));
                datum.setSureUser(sureUser);

            }
        }

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = URLEncoder.encode("缺陷清单", "UTF-8").replaceAll("\\+", "%20");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
        EasyExcel.write(response.getOutputStream(), BugInfoExcelModel.class)
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .sheet("缺陷清单").doWrite(data);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean save(BugInfo entity) {
        String headId = entity.getHeadId();
        if (StringUtils.isNotBlank(headId)) {
            entity.setStatus("1");
        } else {
            entity.setStatus("0");
        }
        super.save(entity);
        BugLog bugLog = entity.createLog();
        String userID = UserTokenUtil.getTokenUserID();
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        UserVo userInfo = userClient.getUserInfo(userID);
        String realName = userInfo.getRealName();
        String content = "[ " + realName + " ] bug 登记";
        bugLog.setContent(content);
        return bugLogService.save(bugLog);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateById(BugInfo entity) {
        String userId = UserTokenUtil.getTokenUserID();
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        UserVo userInfo = userClient.getUserInfo(userId);
        String realName = userInfo.getRealName();
        String content = "[ " + realName + " ]";
        Integer id = entity.getId();
        BugInfo dbEntity = baseMapper.selectById(id);
        content = getContentByStatus(entity.getStatus(), dbEntity.getStatus(),entity.getProcessingMode(), content);
        String status = entity.getStatus();
        if ("3".equals(status)) {

            String alreadySureId = dbEntity.getAlreadySureId();
            alreadySureId = StringUtils.isNotBlank(alreadySureId) ? alreadySureId : "";
            if ("0".equals(entity.getProcessingMode())) {
//                String notes = entity.getNotes();
//                String verifyUrl = StringUtils.isNotBlank(entity.getVerifyUrl()) ? entity.getVerifyUrl() : "";
//                entity.setNotes(dbEntity.getNotes().concat(";").concat(notes));
//                String dbverifyUrl = StringUtils.isNotBlank(dbEntity.getVerifyUrl()) ? dbEntity.getVerifyUrl() : "";
//                entity.setVerifyUrl(dbverifyUrl.concat(",").concat(verifyUrl));

                String sureId = dbEntity.getSureId();
                sureId = removeUserId(sureId, userId);
                if (StringUtils.isNotBlank(sureId)) {
                    entity.setStatus("2");
                }
                entity.setSureId(sureId);
                assert userId != null;
                if(StringUtils.isNotBlank(alreadySureId)){
                    entity.setAlreadySureId(userId.concat(",").concat(alreadySureId));
                }else{
                    entity.setAlreadySureId(userId);
                }

            } else {
                if(StringUtils.isNotBlank(alreadySureId)){
                    entity.setSureId(dbEntity.getSureId().concat(",").concat(alreadySureId));
                }else{
                    entity.setSureId(dbEntity.getSureId());
                }

            }
        }
        if (StringUtils.isNotBlank(entity.getHeadId())) {
            if (StringUtils.isBlank(entity.getStatus()) || "0".equals(entity.getStatus())) {
                entity.setStatus("1");
            }
        }
        super.updateById(entity);
        BugLog bugLog = entity.createLog();
        bugLog.setContent(content);
        return bugLogService.save(bugLog);
    }

    private String getContentByStatus(String status,String dbStatus, String processingMode, String content) {
        if (StringUtils.isNotBlank(status) && StringUtils.isNotBlank(processingMode)) {
            if("1".equals(status) && "1".equals(dbStatus)){
                if ("5".equals(processingMode)) {
                    content = content.concat("转派");
                }
            }else if ("2".equals(status) && "1".equals(dbStatus)) {
                if ("1".equals(processingMode)) {
                    content = content.concat("处理完成");
                } else if ("2".equals(processingMode)) {
                    content = content.concat("设计如此");
                } else if ("3".equals(processingMode)) {
                    content = content.concat("无法重现");
                } else if ("4".equals(processingMode)) {
                    content = content.concat("其他");
                } else if ("5".equals(processingMode)) {
                    content = content.concat("转派");
                }
            }else if ("2".equals(status) && "4".equals(dbStatus)) {
                if ("1".equals(processingMode)) {
                    content = content.concat("处理完成");
                } else if ("2".equals(processingMode)) {
                    content = content.concat("设计如此");
                } else if ("3".equals(processingMode)) {
                    content = content.concat("无法重现");
                } else if ("4".equals(processingMode)) {
                    content = content.concat("其他");
                } else if ("5".equals(processingMode)) {
                    content = content.concat("转派");
                }
            } else if ("3".equals(status)) {
                content = content.concat("通过");
            } else if ("4".equals(status)) {
                if ("0".equals(processingMode)) {
                    content = content.concat("通过");
                } else if ("1".equals(processingMode)) {
                    content = content.concat("驳回");
                } else if ("2".equals(processingMode)) {
                    content = content.concat("转派他人验证");
                }
            }else if("2".equals(status) && "2".equals(dbStatus)){
                if ("0".equals(processingMode)) {
                    content = content.concat("通过");
                } else if ("1".equals(processingMode)) {
                    content = content.concat("驳回");
                }else if ("2".equals(processingMode)) {
                    content = content.concat("转派他人验证");
                }
            }
        }
        return content;
    }

    private String removeUserId(String resource, String userId) {
        Assert.notBlank(resource, "原始字符串不能为空");
        Assert.notBlank(userId, "被移除的字符串不能为空");
        if (StrUtil.contains(resource, userId)) {
            String replace = resource.replace(userId, "");
            if (StrUtil.isBlank(replace)) {
                return replace;
            } else {
                String[] split = replace.split(",");
                return Arrays.stream(split).filter(StringUtils::isNotBlank).collect(Collectors.joining(","));
            }
        } else {
            return resource;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveOrUpdateBatch(Collection<BugInfo> entityList) {
        for (BugInfo bugInfo : entityList) {
            bugInfo.setCompleteTime(new Date());
            super.saveOrUpdate(bugInfo);
//            this.updateBugInfo(bugInfo);
        }
        return true;
    }
}