package com.tofly.pms.testmanage.service.impl;

import cn.hutool.core.lang.Assert;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.pms.testmanage.entity.Code;
import com.tofly.pms.testmanage.entity.SystemItem;
import com.tofly.pms.testmanage.entity.SystemItemExcelModel;
import com.tofly.pms.testmanage.mapper.CodeMapper;
import com.tofly.pms.testmanage.mapper.SystemItemWMapper;
import com.tofly.pms.testmanage.service.BugInfoService;
import com.tofly.pms.testmanage.service.SystemItemService;
import com.tofly.pms.testmanage.vo.ItemPassCountVo;
import com.tofly.pms.testmanage.vo.ItemStatusVo;
import com.tofly.pms.testmanage.vo.SystemCommitContVo;
import com.tofly.pms.testmanage.vo.SystemPassCountVo;
import com.tofly.pms.working.vo.TestTypeCountVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 测试系统功能清单表;(TF_PM_TESTSYSTEMITEM_W)表服务实现类
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Service
public class SystemItemServiceImpl extends ServiceImpl<SystemItemWMapper, SystemItem> implements SystemItemService {
    @Resource
    CodeMapper codeMapper;
    @Resource
    HttpServletResponse response;
    @Autowired
    private BugInfoService bugInfoService;

    @Override
    public SystemItem queryById(Integer id) {
        return baseMapper.selectById(id);
    }

    @Override
    public List<ItemPassCountVo> itemThrowCount(List<Integer> itemIds) {
        return baseMapper.itemThrowCount(itemIds);
    }

    @Override
    public List<SystemPassCountVo> systemPassCount(Integer projectId) {
        return baseMapper.systemPassCount(projectId);
    }

    @Override
    public List<SystemCommitContVo> sytemCommitRate(Integer projectId) {
        return baseMapper.sytemCommitRate(projectId);
    }

    @Override
    public List<ItemStatusVo> itemStatusCount(Integer projectId) {
        return baseMapper.itemStatusCount(projectId);
    }

    @Override
    public String getFullNameByFunctionId(Integer functionId) {
        String fullName = "";
        if (Objects.isNull(functionId) || functionId == 0) {
            return fullName;
        }
        SystemItem systemItem = this.baseMapper.selectById(functionId);
        if(Objects.isNull(systemItem)){
            return fullName;
        }
        fullName = systemItem.getName();
        String parentCode = systemItem.getParentCode();
        if (StringUtils.isNotBlank(parentCode)) {
            String name = getFullNameByCode(parentCode);
            if (StringUtils.isNotBlank(name)) {
                fullName = name + " -> " + fullName;
            }

        }
        return fullName;
    }

    @Override
    public String getSystemItemCode() {
        Code code = new Code();
        codeMapper.insert(code);
        return code.getId();
    }

    @Override
    public List<String> getSystemItemCodeList(int length) {

        if (length > 0) {
            List<String> codeList = new ArrayList<>(length);
            for (int i = 0; i < length; i++) {
                String code = this.getSystemItemCode();
                codeList.add(code);
            }
            return codeList;
        }

        return null;
    }

    @Override
    public void export(Integer systemId) throws IOException {
        Assert.notNull(systemId, "系统id不能为空");

        List<SystemItem> itemList = baseMapper.selectList(Wrappers.lambdaQuery(SystemItem.class)
                .eq(SystemItem::getSystemId, systemId)
                .last("ORDER BY CASE WHEN split_part(number, '.', 1) ~ '^[0-9]+$' THEN split_part(number, '.', 1) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 2) ~ '^[0-9]+$' THEN split_part(number, '.', 2) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 3) ~ '^[0-9]+$' THEN split_part(number, '.', 3) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 4) ~ '^[0-9]+$' THEN split_part(number, '.', 4) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 5) ~ '^[0-9]+$' THEN split_part(number, '.', 5) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 6) ~ '^[0-9]+$' THEN split_part(number, '.', 6) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 7) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 8) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 9) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END"));
        List<SystemItemExcelModel> data = new ArrayList<>(itemList.size());
        for (SystemItem item : itemList) {
            SystemItemExcelModel excelModel =  item.toExcelModel();
            List<TestTypeCountVo> countData=  bugInfoService.getTestTypeCount(item.getId());
            Integer count = countData.stream().map(TestTypeCountVo::getCount).reduce(0, Integer::sum);
            Integer completeCount = countData.stream().map(TestTypeCountVo::getCompleteNum).reduce(0, Integer::sum);
            excelModel.setBugCount(count);
            excelModel.setCompleteCount(completeCount);
            Map<String, TestTypeCountVo> countVoMap = countData.stream().collect(Collectors.toMap(TestTypeCountVo::getTestType, it -> it));
            TestTypeCountVo countVo0 = countVoMap.get("0");
            excelModel.setCount0(countVo0.getCount());
            excelModel.setCompleteCount0(countVo0.getCompleteNum());

            TestTypeCountVo countVo1 = countVoMap.get("1");
            excelModel.setCount1(countVo1.getCount());
            excelModel.setCompleteCount1(countVo1.getCompleteNum());

            TestTypeCountVo countVo2 = countVoMap.get("2");
            excelModel.setCount2(countVo2.getCount());
            excelModel.setCompleteCount2(countVo2.getCompleteNum());

            TestTypeCountVo countVo3 = countVoMap.get("3");
            excelModel.setCount3(countVo3.getCount());
            excelModel.setCompleteCount3(countVo3.getCompleteNum());

            TestTypeCountVo countVo5 = countVoMap.get("5");
            excelModel.setCount5(countVo5.getCount());
            excelModel.setCompleteCount5(countVo5.getCompleteNum());
            data.add(excelModel);
        }
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = URLEncoder.encode("功能清单", "UTF-8").replaceAll("\\+", "%20");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
        EasyExcel.write(response.getOutputStream(), SystemItemExcelModel.class)
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .sheet("功能清单").doWrite(data);
    }

    private String getFullNameByCode(String code) {
        String name = "";
        if (StringUtils.isBlank(code)) {
            return name;
        }
        SystemItem systemItem = baseMapper.selectOne(Wrappers.lambdaQuery(SystemItem.class)
                .eq(SystemItem::getCode, code));
        if (Objects.nonNull(systemItem)) {
            name = systemItem.getName();
            String parentCode = systemItem.getParentCode();
            if (StringUtils.isNotBlank(parentCode)) {
                String name1 = getFullNameByCode(parentCode);
                if (StringUtils.isNotBlank(name1)) {
                    name = name1 + "->" + name;
                }

            }
        }
        return name;
    }
}