package com.tofly.pms.testmanage.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.pms.testmanage.entity.BugInfo;
import com.tofly.pms.testmanage.entity.SystemItem;
import com.tofly.pms.testmanage.entity.TestSystem;
import com.tofly.pms.testmanage.mapper.TestSystemWMapper;
import com.tofly.pms.testmanage.service.BugInfoService;
import com.tofly.pms.testmanage.service.SystemItemService;
import com.tofly.pms.testmanage.service.TestSystemService;
import com.tofly.pms.testmanage.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 测试系统名称表;(TF_PM_TESTSYSTEM_W)表服务实现类
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Service
public class TestSystemServiceImpl extends ServiceImpl<TestSystemWMapper, TestSystem> implements TestSystemService {

    @Autowired
    private SystemItemService systemItemService;
    @Autowired
    private BugInfoService bugInfoService;

    @Override
    public boolean save(TestSystem entity) {
//        Assert.notNull(entity, "添加的测试系统信息不能为空");
//        Assert.notBlank(entity.getName(), "测试系统名称不能为空");
//        Assert.notBlank(entity.getSystemCode(), "测试系统代码不能为空");
//        TestSystem dbEntity = this.getOne(Wrappers.lambdaQuery(TestSystem.class).eq(TestSystem::getSystemCode, entity.getSystemCode())
//                .or()
//                .eq(TestSystem::getName, entity.getName()));
//        Assert.isNull(dbEntity, "名称或者代码重复，请检查");
        return super.save(entity);
    }

    @Override
    public boolean updateById(TestSystem entity) {
//        Assert.notNull(entity, "修改的测试系统信息不能为空");
//        Assert.notBlank(entity.getName(), "测试系统名称不能为空");
//        Assert.notBlank(entity.getSystemCode(), "测试系统代码不能为空");
//        TestSystem dbEntity = this.getOne(Wrappers.lambdaQuery(TestSystem.class).eq(TestSystem::getSystemCode, entity.getSystemCode())
//                .or()
//                .eq(TestSystem::getName, entity.getName()));
//        Assert.isTrue(Objects.isNull(dbEntity) || Objects.equals(dbEntity.getId(), entity.getId()),
//                "名称或者代码重复，请检查");
        return super.updateById(entity);
    }

    @Override
    public TestSystem queryById(Integer id) {
        return baseMapper.selectById(id);
    }

    @Override
    public ProjectBugCountVo projectCont(Integer projectId) {
        ProjectBugCountVo projectBugCountVo = new ProjectBugCountVo();
        projectBugCountVo.setProjectId(projectId);
        //TODO 提测率  通过率  如何计算
        //统计项目的提测率  通过率
        final long projectCount = systemItemService.count(Wrappers.<SystemItem>lambdaQuery().eq(SystemItem::getProjectId, projectId));
        final long projectCommitCount = systemItemService.count(
                Wrappers.<SystemItem>lambdaQuery().eq(SystemItem::getProjectId, projectId)
                        .eq(SystemItem::getIsCommitTest, "true"));
        //通过测试状态 统计 通过的数量
        if (projectCount != 0) {
            double testRate = Double.valueOf(projectCommitCount) / projectCount;
            projectBugCountVo.setTestRate(testRate);
        } else {
            projectBugCountVo.setTestRate(0d);
        }

        //统计通过率
        //统计 无bug 以及 bug的状态 为已完成 已关闭的数量
        //查询已提测的功能
        final List<Integer> collect = systemItemService.list(Wrappers.<SystemItem>lambdaQuery()
                .eq(SystemItem::getProjectId, projectId)
                .eq(SystemItem::getIsCommitTest, "true")
        ).stream().map(SystemItem::getId).collect(Collectors.toList());
        //查看功能下 所有bug 已完成已关闭的数量
        final List<ItemPassCountVo> itemPassCountVos
                = systemItemService.itemThrowCount(collect);
        int throwItemCount = 0;
        for (ItemPassCountVo itemPassCountVo : itemPassCountVos) {
            if (itemPassCountVo.getCompleteCount() == itemPassCountVo.getTotal()) {
                throwItemCount++;
            }
        }
        if (collect.size() > 0) {
            double throwRate = Double.valueOf(throwItemCount) / collect.size();
            projectBugCountVo.setPassRate(throwRate);
        } else {
            projectBugCountVo.setPassRate(0d);
        }

        //统计项目下各子系统的提测率  通过率
        //获取提测率
        final List<SystemCommitContVo> list = systemItemService.sytemCommitRate(projectId);
        Map<Integer, SystemCommitContVo> commitCommitContVoMap = list.stream().collect(Collectors.toMap(SystemCommitContVo::getSystemId, r -> r));
        List<TestSystem> testSystemList = this.list(Wrappers.lambdaQuery(TestSystem.class).eq(TestSystem::getProjectId, projectId));
        final List<SystemPassCountVo> systemPassCountVos = systemItemService.systemPassCount(projectId);
        final Map<Integer, SystemPassCountVo> collectMap = systemPassCountVos.stream().collect(Collectors.toMap(SystemPassCountVo::getSystemId, r -> r));

        List<SystemBugCountVo> systemBugCountVos = new ArrayList<>();

        for (TestSystem system : testSystemList) {
            SystemCommitContVo systemCommitContVo = commitCommitContVoMap.get(system.getId());
            SystemBugCountVo systemBugCountVo = new SystemBugCountVo();
            if (Objects.isNull(systemCommitContVo)) {
                systemBugCountVo.setSystemId(system.getId());
                systemBugCountVo.setTestRate(0d);
                systemBugCountVo.setPassRate(0d);
                systemBugCountVos.add(systemBugCountVo);
                continue;
            }

            systemBugCountVo.setSystemId(systemCommitContVo.getSystemId());
            if (systemCommitContVo.getItemTotal() != 0) {
                double testRate = Double.valueOf(systemCommitContVo.getCommitCount()) / systemCommitContVo.getItemTotal();
                systemBugCountVo.setTestRate(testRate);
            }
            final SystemPassCountVo systemPassCountVo = collectMap.get(systemBugCountVo.getSystemId());

            if (systemPassCountVo != null) {
                if (systemPassCountVo.getTotal() != 0) {
                    double passRate = Double.valueOf(systemPassCountVo.getCompleteCount()) / systemPassCountVo.getTotal();
                    systemBugCountVo.setPassRate(passRate);
                } else {
                    systemBugCountVo.setPassRate(0d);
                }
            }
            systemBugCountVos.add(systemBugCountVo);

        }
        projectBugCountVo.setDatas(systemBugCountVos);

        return projectBugCountVo;
    }

    @Override
    public BugGroupCountVo systemBugGroupCount(Integer projectId) {
        //查询系统下所有缺陷的状态
        final List<ItemStatusVo> itemStatusVos = systemItemService.itemStatusCount(projectId);
        //分状态统计
        final Map<String, Long> collect = itemStatusVos.stream().collect(Collectors.groupingBy(ItemStatusVo::getStatus, Collectors.counting()));
        //查询缺陷总数
        final long count = systemItemService.count(Wrappers.<SystemItem>lambdaQuery().eq(SystemItem::getProjectId, projectId));
        //
        BugGroupCountVo bugGroupCountVo = new BugGroupCountVo();
        bugGroupCountVo.setAll(count);
        //获取已完成数据
        Long aLong = collect.get("1");
        if (aLong == null) {
            aLong = 0l;
            bugGroupCountVo.setComplete(0);
        } else {
            bugGroupCountVo.setComplete(aLong);

        }
        Long aLong1 = collect.get("2");
        if (aLong1 == null) {
            aLong1 = 0l;
            bugGroupCountVo.setRepairing(0);
        } else {
            bugGroupCountVo.setRepairing(aLong1);
        }
        bugGroupCountVo.setRepairing(aLong1);
        // 0缺陷

        bugGroupCountVo.setZeroNu(count - aLong - aLong1);
        // 完成修复的缺陷  关联的bug状态 为已关闭 或者 已确认

        //查询系统下  正在修复中的  状态为
        //查询系统下  未提测的数据


        return bugGroupCountVo;
    }

    /**
     * 根据项目id查询项目下各子系统的提测率  通过率
     *
     * @param projectId 项目id
     * @return  ProjectBugCountVo
     * 1、查询整个项目的提测情况 和 通过情况
     * 2、查询项目下面每一个子系统提测情况 和通过情况
     */
    @Override
    public ProjectBugCountVo projectContByProjectId(Integer projectId) {
        if (Objects.isNull(projectId)) {
            return null;
        }
        ProjectBugCountVo projectBugCountVo = new ProjectBugCountVo();
        projectBugCountVo.setProjectId(projectId);
        //项目下面的所有的子系统：系统清单
        List<Integer> systemList = this.listObjs(Wrappers.lambdaQuery(TestSystem.class)
                        .eq(TestSystem::getProjectId, projectId)
                        .select(TestSystem::getId),
                item -> (Integer) item);
        if(CollectionUtil.isEmpty(systemList)){
            return projectBugCountVo;
        }
        //项目下面的所有的测试项，因为删除清单的时候，没有同步删除测试项目，所以需要.in(SystemItem::getSystemId, systemIdList)
        List<SystemItem> systemItemList = systemItemService.list(Wrappers.lambdaQuery(SystemItem.class)
                        .eq(SystemItem::getProjectId, projectId)
                        .in(SystemItem::getSystemId, systemList)
                        .select(SystemItem::getId, SystemItem::getSystemId));
        //项目下面已提交测试的测试项
        List<SystemItem> commitSystemItemList = systemItemService.list(Wrappers.lambdaQuery(SystemItem.class)
                        .eq(SystemItem::getProjectId, projectId)
                        .eq(SystemItem::getIsCommitTest, "true")
                        .in(SystemItem::getSystemId, systemList)
                        .select(SystemItem::getId, SystemItem::getSystemId));
        if (CollectionUtil.isEmpty(commitSystemItemList)) {
            //如果没有提交测试的测试项，则提测率=0 直接返回
            projectBugCountVo.setTestRate(0d);
            projectBugCountVo.setPassRate(0d);
        }
        //计算提测率
        double testRate = ((double) commitSystemItemList.size()) / systemItemList.size();
        projectBugCountVo.setTestRate(testRate);
        //计算bug通过率
        List<Integer> commitSystemItemIdList = commitSystemItemList.stream()
                .map(SystemItem::getId)
                .distinct()
                .collect(Collectors.toList());
        List<BugInfo> bugList = bugInfoService.list(Wrappers.lambdaQuery(BugInfo.class)
                .eq(BugInfo::getProjectId, projectId)
                .in(CollectionUtil.isNotEmpty(commitSystemItemList),BugInfo::getFunctionId, commitSystemItemIdList)
                .eq(CollectionUtil.isEmpty(commitSystemItemList),BugInfo::getFunctionId, -1)
                .eq(BugInfo::getDeleted, 0)
                .select(BugInfo::getId,BugInfo::getSystemId,BugInfo::getFunctionId));
        List<BugInfo> completedBugList = bugInfoService.list(Wrappers.lambdaQuery(BugInfo.class)
                .eq(BugInfo::getProjectId, projectId)
                .in(CollectionUtil.isNotEmpty(commitSystemItemList),BugInfo::getFunctionId, commitSystemItemIdList)
                .eq(CollectionUtil.isEmpty(commitSystemItemList),BugInfo::getFunctionId, -1)
                .eq(BugInfo::getStatus, "3")
                .eq(BugInfo::getDeleted, 0)
                .select(BugInfo::getId,BugInfo::getSystemId,BugInfo::getFunctionId));
        if(CollectionUtil.isEmpty(completedBugList)){
            projectBugCountVo.setPassRate(0d);
        }
        double passRate = ((double) completedBugList.size()) / bugList.size();
        projectBugCountVo.setPassRate(passRate);
        //计算每一个子系统的提测率 通过率
        Map<Integer, Long> systemItemIdMap = systemItemList.stream().collect(Collectors.toMap(SystemItem::getSystemId,
                item -> systemItemList.stream().filter(systemItem -> Objects.equals(systemItem.getSystemId(), item.getSystemId())).count(),
                (k1, k2) -> k1));
        Map<Integer, Long> commitSystemItemIdMap = commitSystemItemList.stream().collect(Collectors.toMap(SystemItem::getSystemId,
                item -> commitSystemItemList.stream().filter(systemItem -> Objects.equals(systemItem.getSystemId(), item.getSystemId())).count(),
                (k1, k2) -> k1));
        Map<Integer, Long> systemIdMap = bugList.stream().collect(Collectors.toMap(BugInfo::getSystemId,
                item ->bugList.stream().filter(bugInfo -> Objects.equals(item.getSystemId(), bugInfo.getSystemId())).count(),
                (k1, k2) -> k1));
        Map<Integer, Long> completedSystemIdMap = completedBugList.stream().collect(Collectors.toMap(BugInfo::getSystemId,
                item ->completedBugList.stream().filter(bugInfo -> Objects.equals(item.getSystemId(), bugInfo.getSystemId())).count(),
                (k1, k2) -> k1));
        List<SystemBugCountVo> datas = new ArrayList<>();
        for (Integer systemId : systemList) {
            SystemBugCountVo vo = new SystemBugCountVo();
            vo.setSystemId(systemId);
            //提测率
            Long countSystemItem = systemItemIdMap.get(systemId);
            if(Objects.isNull(countSystemItem)){
                vo.setTestRate(0d);
                vo.setPassRate(0d);
                datas.add(vo);
                continue;
            }
            long countCommitSystemItem = Objects.isNull(commitSystemItemIdMap.get(systemId))?0:commitSystemItemIdMap.get(systemId);
            vo.setTestRate(((double) countCommitSystemItem) / countSystemItem);
            //通过率
            Long countSystem = systemIdMap.get(systemId);
            if(Objects.isNull(countSystem)){
                vo.setPassRate(0d);
                datas.add(vo);
                continue;
            }
            long countCompletedSystem = Objects.isNull(completedSystemIdMap.get(systemId))?0:completedSystemIdMap.get(systemId);
            vo.setPassRate(((double) countCompletedSystem) / countSystem);
            datas.add(vo);
        }
        projectBugCountVo.setDatas(datas);
        return projectBugCountVo;
    }
}