package com.tofly.pms.utils;

import com.tofly.common.core.util.StringUtil;
import com.tofly.pms.exception.BusinessException;
import com.tofly.pms.glossary.constants.Constant;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

@Slf4j
public class UserTokenUtil {

    private static String tokenSigningKey = "1qaz2wsx3edc4rfv5tgb6yhn7ujm1qaz2wsx3edc4rfv5tgb6yhn7ujm1qaz2wsx3edc4rfv5tgb6yhn7ujm1qaz2wsx3edc4rfv5tgb6yhn7ujm";

    private static String getAuthorization() throws BusinessException {
        try {
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            String token = request.getHeader("authorization");
            if (!StringUtil.isEmpty(token)) {
                if (token.startsWith("Bearer")) {
                    token = token.replace("Bearer", "").trim();
                }
                if (token.startsWith("bearer")) {
                    token = token.replace("bearer", "").trim();
                }
                return token;
            } else {
                token = request.getHeader("X-Authorization");
                if (!StringUtil.isEmpty(token)) {
                    if (token.startsWith("Bearer")) {
                        token = token.replace("Bearer", "").trim();
                    }
                    if (token.startsWith("bearer")) {
                        token = token.replace("bearer", "").trim();
                    }
                    return token;
                }
            }
            return null;
        } catch (Exception e) {
            log.info(e.getMessage());
            return null;
        }

    }

    /**
     * 解析token
     */
    public static String getTokenUserID() throws BusinessException {
        try {
            String jwt = getAuthorization();
            if (!StringUtil.isEmpty(jwt)) {
                Claims claims = Jwts.parser()
                        .setSigningKey(tokenSigningKey)
                        .parseClaimsJws(jwt)
                        .getBody();
                return claims.get("userId", String.class);
            }
            return null;
        } catch (BusinessException e) {
            //e.printStackTrace();
            throw new BusinessException(Constant.TOKEN_NULL_CODE, Constant.TOKEN_NULL);
        }
    }

    /**
     * 解析token
     */
    public static Claims getTokenUser() throws BusinessException {
        try {
            String jwt = getAuthorization();
            if (!StringUtil.isEmpty(jwt)) {
                Claims claims = Jwts.parser()
                        .setSigningKey(tokenSigningKey)
                        .parseClaimsJws(jwt)
                        .getBody();
                return claims;
            }
            return null;
        } catch (BusinessException e) {
            //e.printStackTrace();
            log.info(e.getMessage());
            log.info("获取token信息失败");
            throw new BusinessException(Constant.TOKEN_NULL_CODE, Constant.TOKEN_NULL);
        }
    }
}
