package com.tofly.pms.utils.dict;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import com.tofly.pms.fegin.DictItemClient;
import com.tofly.pms.utils.dict.vo.DictItemVo;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 字典自定序列化类
 */
@Slf4j
public class DictSerializer extends StdSerializer<Object> implements ContextualSerializer {

    private static final long serialVersionUID = -6157558261755426448L;

    private String dictCode;


    public DictSerializer() {
        super(Object.class);
    }

    public DictSerializer(String dictCode) {
        super(Object.class);
        this.dictCode = dictCode;
    }

    @Override
    public JsonSerializer<?> createContextual(SerializerProvider prov, BeanProperty property) {
        Dict annotation = property.getAnnotation(Dict.class);
        String code = annotation.code();
        log.info(code);
        return new DictSerializer(code);
    }

    @SneakyThrows
    @Override
    public void serialize(Object code, JsonGenerator gen, SerializerProvider provider) {
        /*
        获取键值对，这里我处理成map的方式返回给前端
        这部分要自己处理，我这里是通过一个业务类从redis中获取字典
         */
        Map<String, Object> map = new HashMap<>();
        map.put("value", String.valueOf(code));
        try {

            DictItemClient service = SpringUtil.getBean(DictItemClient.class);
            final Object dicInfo = service.getDicInfo(dictCode, String.valueOf(code));

            DictItemVo dictItemVo = JSON.parseObject(JSON.toJSONString(dicInfo), DictItemVo.class);
            if (Objects.nonNull(dictItemVo)) {
                map.put("info", dictItemVo);
            }
        }catch (Exception e){
            log.error("字典序列化异常",e);
        }finally {
            gen.writeObject(map);
        }

    }
}