package com.tofly.pms.utils.file;

import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import com.tofly.pms.file.entity.File;
import com.tofly.pms.file.service.FileService;
import lombok.SneakyThrows;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 字典自定序列化类
 */
public class ToflyFileSerializer extends StdSerializer<Object> implements ContextualSerializer {

    private static final long serialVersionUID = -6157558261755426448L;

    private String fieldName;

    private String tableName;
    /**一对多**/
    private boolean oneToMany;

    public ToflyFileSerializer() {
        super(Object.class);
    }

    public ToflyFileSerializer(String tableName,boolean oneToMany,String fieldName) {
        super(Object.class);
        this.tableName = tableName;
        this.oneToMany=oneToMany;
        this.fieldName=fieldName;
    }

    @Override
    public JsonSerializer<?> createContextual(SerializerProvider prov, BeanProperty property) {
        ToflyFile annotation = property.getAnnotation(ToflyFile.class);
        return new ToflyFileSerializer(annotation.tableName(),annotation.oneToMany(),annotation.fieldName());
    }

    @SneakyThrows
    @Override
    public void serialize(Object code, JsonGenerator gen, SerializerProvider provider) {
        final FileService service = SpringUtil.getBean(FileService.class);
        //获取valueId的值
        final String codeString = String.valueOf(code);
        if(oneToMany){
            final List<Integer> collect = Arrays.stream(codeString.split(",")).map(s -> Integer.parseInt(s)).collect(Collectors.toList());
            final List<File> list = service.list(Wrappers.<File>lambdaQuery().
                    eq(StringUtils.isNotBlank(tableName),File::getTableName, tableName).in(File::getTableId, collect));
            gen.writeObjectField("files",list);
        }else{
            File file = service.getOne(Wrappers.<File>lambdaQuery().
                    eq(StringUtils.isNotBlank(tableName),File::getTableName, tableName).eq(File::getTableId, codeString));
            gen.writeObjectField("file",file);
        }

    }
}