package com.tofly.pms.utils.vocast;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import com.tofly.pms.base.service.BaseService;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import org.springframework.data.redis.core.RedisTemplate;

import java.io.IOException;
import java.util.*;

/**
 * 字典自定序列化类
 */
public class VoCastSerializer extends StdSerializer<Object> implements ContextualSerializer {

    private static final long serialVersionUID = -6157558261755426448L;

   private Class serviceClass;
    private boolean isMulti;
    private  Class valueClass;


    public VoCastSerializer() {
        super(Object.class);
    }

    public VoCastSerializer(Class serviceClass,boolean isMulti,Class valueClass) {
        super(Object.class);
        this.serviceClass = serviceClass;
        this.isMulti = isMulti;
        this.valueClass = valueClass;

    }

    @Override
    public JsonSerializer<?> createContextual(SerializerProvider prov, BeanProperty property) {
        VoCast annotation = property.getAnnotation(VoCast.class);
        return new VoCastSerializer(annotation.serviceClass(),annotation.isMulti(),annotation.valueClass());
    }

    @Override
    public void serialize(Object code, JsonGenerator gen, SerializerProvider provider) throws IOException {
        /*
        获取键值对，这里我处理成map的方式返回给前端
        这部分要自己处理，我这里是通过一个业务类从redis中获取字典
         */
        final RedisTemplate redisTemplate = SpringUtil.getBean("redisTemplate");
        Map<String,Object>  map = new HashMap<>();
        map.put("value",String.valueOf(code));
        if(valueClass!=null){

            if(valueClass == UserVo.class){
                //从缓存获取用户信息
             String userIds =    String.valueOf(code);
             //支持多个
             if(StringUtils.isNotBlank(userIds)){
                 final String[] userIdArr = userIds.split(",");
                List<UserVo> datas = new ArrayList<>();
                 for (String userId : userIdArr) {
                     if(StringUtils.isBlank(userId)){
                         continue;
                     }
                     final String userInfoStr  =(String) redisTemplate.opsForHash().get("pms:userInfo",userId);
                     UserVo userInfo;
                     if(StringUtils.isNotBlank(userInfoStr) && (!"null".equals(userInfoStr))){
                         userInfo = JSON.parseObject(userInfoStr,UserVo.class);
                     }else{
                         UserClient userClient = (UserClient) SpringUtil.getBean(UserClient.class);
                      try{
                          userInfo = userClient.getUserInfo(userId);
                      }catch ( Exception e ){
                          userInfo =null;
                      }

                         //redisTemplate.opsForValue().set(userId,JSON.toJSONString(userInfo));
                         redisTemplate.opsForHash().put("pms:userInfo",userId,JSON.toJSONString(userInfo));
                         //通过队列来进行用户信息的刷新
                     }
                     datas.add(userInfo);

                 }
                 if(isMulti){
                     map.put("value",userIds);
                     map.put("info",datas);
                     try {
                         gen.writeObject(map);
                     } catch (IOException e) {
                         e.printStackTrace();
                     }
                     // gen.writeObject(datas);
                 }else{
                     if(datas.size()>0){
                         map.put("info",datas.get(0));
                         try {
                             gen.writeObject(map);
                         } catch (IOException e) {
                             e.printStackTrace();
                         }
                     }
                 }

             }else{
                 gen.writeNull();
             }

                //多个用户 返回数组
            }else{
                BaseService baseService = (BaseService) SpringUtil.getBean(serviceClass);
                if(isMulti){
                    List list = new ArrayList();
                    if(StringUtils.isNotBlank(String.valueOf(code))){
                        for (String codeSingle : String.valueOf(code).split(",")) {
                            Object o = baseService.queryById(Integer.valueOf(String.valueOf(codeSingle)));
                            final Object o1 = JSON.parseObject(JSON.toJSONString(o), valueClass);
                            list.add(o1);
                        }

                        map.put("datas",list);
                    }else{
                        map.put("datas", Collections.emptyList());
                    }


                    try {
                        gen.writeObject(map);
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }else{

                    Object o = baseService.queryById(Integer.valueOf(String.valueOf(code)));
                    final Object o1 = JSON.parseObject(JSON.toJSONString(o), valueClass);
                    map.put("data",o1);

                    try {

                        gen.writeObject(map);
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }

            }

        }else{
            gen.writeNull();
        }






    }


}