package com.tofly.pms.working.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.utils.ImportExcelHelper;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.entity.FeedBackPlan;
import com.tofly.pms.working.entity.Feedback;
import com.tofly.pms.working.entity.FeedbackExcelModel;
import com.tofly.pms.working.listener.FeedBackListener;
import com.tofly.pms.working.service.FeedBackPlanService;
import com.tofly.pms.working.service.FeedBackService;
import io.jsonwebtoken.lang.Objects;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Api(tags = "反馈信息")
@RestController
@RequestMapping("/feedback")
public class FeedBackController {
    @Autowired
    private FeedBackService feedBackService;
    @Autowired
    private FeedBackPlanService feedBackPlanService;
    @Autowired
    private ProjectTeamWService projectTeamWService;
    @Autowired
    private ProjectWService projectWService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<Feedback> queryById(Integer id) {
        return ResponseEntity.ok(feedBackService.queryById(id));
    }

    /**
     * 分页查询
     *
     * @param feedback 筛选条件
     * @param page     分页对象
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<Feedback>> paginQuery(Feedback feedback, Page<Feedback> page) {
        List<Integer> selfProjectId = projectWService.getSelfProjectId();
        if(CollectionUtil.isEmpty(selfProjectId)){
            return ResponseEntity.ok(page);
        }
        final Page<Feedback> page1 = feedBackService.page(page, new QueryWrapper<>(feedback)
                .lambda()
                .eq(Feedback::getDeleted, 0)
                        .in(Feedback::getProjectId,selfProjectId)
                .orderByDesc(Feedback::getCreateDate));
        List<Feedback> records = page1.getRecords();
        if (CollectionUtil.isNotEmpty(records)) {
            for (Feedback record : records) {
                FeedBackPlan feedBackPlan = feedBackPlanService.getOne(
                        Wrappers.lambdaQuery(FeedBackPlan.class).eq(FeedBackPlan::getFeedbackId, record.getId())
                );
                record.setIsDispatchTask(feedBackPlan.getIsDispatchTask());
            }
        }
        return ResponseEntity.ok(page1);
    }

    /**
     * 新增数据
     *
     * @param feedback 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<Feedback> add(@RequestBody Feedback feedback) {
        feedBackService.save(feedback);
        return ResponseEntity.ok(feedback);
    }

    /**
     * 更新数据
     *
     * @param feedback 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<Feedback> edit(@RequestBody Feedback feedback) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前登录用户不合法");
            Feedback dbFeedback = feedBackService.getById(feedback.getId());
            if (ObjectUtils.notEqual(dbFeedback.getCreateUser(), userID)) {
                Integer projectId = dbFeedback.getProjectId();
                if (ObjectUtils.isNotEmpty(projectId)) {
                    ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                            .eq(ProjectTeamW::getProjectId, projectId)
                            .like(ProjectTeamW::getAdminId, userID));
                    if (ObjectUtils.isEmpty(one)) {
                        RoleClient client = SpringUtil.getBean(RoleClient.class);
                        Boolean has = client.getHasRoleByUserId(userID, "所有功能");
                        if (!has) {
                            throw new RuntimeException("当前用户没有权限");
                        }
                    }
                } else {
                    RoleClient client = SpringUtil.getBean(RoleClient.class);
                    Boolean has = client.getHasRoleByUserId(userID, "所有功能");
                    if (!has) {
                        throw new RuntimeException("当前用户没有权限");
                    }
                }

            }
        feedBackService.updateById(feedback);
        return ResponseEntity.ok(feedback);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public ResponseEntity<Boolean> deleteById(String ids) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前登录用户不合法");
        final String[] split = ids.split(",");
        for (String s : split) {
            Integer id = Integer.valueOf(s);
            Feedback feedback = feedBackService.getById(id);
            if (ObjectUtils.notEqual(feedback.getCreateUser(), userID)) {
                Integer projectId = feedback.getProjectId();

                if (ObjectUtils.isNotEmpty(projectId)) {
                    ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                            .eq(ProjectTeamW::getProjectId, projectId)
                            .like(ProjectTeamW::getAdminId, userID));
                    if (ObjectUtils.isEmpty(one)) {
                        RoleClient client = SpringUtil.getBean(RoleClient.class);
                        Boolean has = client.getHasRoleByUserId(userID, "所有功能");
                        if (!has) {
                            throw new RuntimeException("当前用户没有权限");
                        }
                    }
                } else {
                    RoleClient client = SpringUtil.getBean(RoleClient.class);
                    Boolean has = client.getHasRoleByUserId(userID, "所有功能");
                    if (!has) {
                        throw new RuntimeException("当前用户没有权限");
                    }
                }

            }
            feedback.setDeleted(1);
            feedBackService.updateById(feedback);
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

    @ApiOperation("意见批量导入")
    @PostMapping("/import")
    @Transactional(rollbackFor = Exception.class)
    public ResponseEntity<Boolean> feedbackImport(@RequestParam("file") MultipartFile file,
                                                  @RequestParam("projectId") Integer projectId) throws Exception {

        ImportExcelHelper<FeedbackExcelModel> helper = new ImportExcelHelper<>(new FeedBackListener(1));
        List<FeedbackExcelModel> importData = helper.getList(file, FeedbackExcelModel.class, 0, 1);
        if (CollectionUtil.isEmpty(importData)) {
            return ResponseEntity.ok(Boolean.FALSE);
        }
        List<Feedback> feedbackList = new ArrayList<>();
        for (FeedbackExcelModel model : importData) {
            Feedback feedback = model.toFeedback();
            feedback.setProjectId(projectId);
            String resolverPerson = feedback.getResolverPerson();
            if (StringUtils.isNotBlank(resolverPerson) && resolverPerson.contains(",")) {
                String[] split = resolverPerson.split(",");

                for (String s : split) {
                    Feedback newFeedback = feedback.clone();
                    newFeedback.setResolverPerson(s);
                    feedbackList.add(newFeedback);
                }
            } else {
                feedbackList.add(feedback);
            }

        }
        boolean b = feedBackService.saveBatch(feedbackList);
        String userID = UserTokenUtil.getTokenUserID();
        if (b) {
            feedbackList.stream()
                    .filter(feedback -> StringUtils.isNotBlank(feedback.getResolverPerson()))
                    .map(feedback -> {
                        FeedBackPlan feedBackPlan = new FeedBackPlan();
                        feedBackPlan.setIsDispatchTask(Boolean.TRUE);
                        feedBackPlan.setFeedbackId(feedback.getId());
                        feedBackPlan.setResolverPerson(feedback.getResolverPerson());
                        feedBackPlan.setHandlePlan(feedback.getHandlePlan());
                        feedBackPlan.setDescription(feedback.getResolverDescription());
                        feedBackPlan.setConfirmPerson(userID);
                        return feedBackPlan;
                    }).forEach(feedBackPlan -> feedBackPlanService.saveOrUpdateFeedBackPlan(feedBackPlan));

        }
        return ResponseEntity.ok(b);
    }

}
