package com.tofly.pms.working.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.working.dto.FeedBackPlanDto;
import com.tofly.pms.working.dto.FeedBackQueryDto;
import com.tofly.pms.working.entity.FeedBackPlan;
import com.tofly.pms.working.entity.Feedback;
import com.tofly.pms.working.entity.TaskW;
import com.tofly.pms.working.service.FeedBackPlanService;
import com.tofly.pms.working.service.FeedBackService;
import com.tofly.pms.working.service.TaskWService;
import com.tofly.pms.working.vo.FeedBackWithPlanVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@Api(tags = "意见处理")
@RestController
@RequestMapping("/feedbackplan")
public class FeedBackPlanController {
    @Autowired
    private FeedBackPlanService feedBackPlanService;
    @Autowired
    private FeedBackService feedBackService;
    @Autowired
    private TaskWService taskWService;
    @Autowired
    private ProjectWService projectWService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @PreAuthorize("@check.checkPermissions({'api:tf:management:details'})")
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<FeedBackPlan> queryById(Integer id) {
        return ResponseEntity.ok(feedBackPlanService.queryById(id));
    }

    /**
     * 分页查询
     *
     * @param feedBackPlan 筛选条件
     * @param page         分页对象
     * @return 查询结果
     */
//    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','TENANT_ADMIN'},{'api:yt:user:post'})")
    @PreAuthorize("@check.checkPermissions({'api:tf:management:details'})")
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<FeedBackPlan>> paginQuery(FeedBackPlan feedBackPlan, Page<FeedBackPlan> page) {
        final Page<FeedBackPlan> page1 = feedBackPlanService.page(page,
                new QueryWrapper<>(feedBackPlan)
                        .lambda()
                        .orderByDesc(FeedBackPlan::getResolveDate));
        return ResponseEntity.ok(page1);
    }

    @ApiOperation("根据状态统计feedback")
    @GetMapping("/groupByStatus")
    public ResponseEntity<Map<String, Long>> groupByStatus(FeedBackQueryDto feedBackQueryDto) {
        Map<String, Long> result = new HashMap<>(4);
        List<Integer> selfProjectId = projectWService.getSelfProjectId();
        if(CollectionUtil.isEmpty(selfProjectId)){
            result.put("1",0L);
            result.put("2",0L);
            result.put("3",0L);
            result.put("4",0L);
            return ResponseEntity.ok(result);
        }
        LambdaQueryWrapper<Feedback> feedbackLambdaQueryWrapper = Wrappers.<Feedback>lambdaQuery()
                .eq(Feedback::getDeleted, 0)
                .eq(Objects.nonNull(feedBackQueryDto.getProjectId()), Feedback::getProjectId, feedBackQueryDto.getProjectId())
                .like(StringUtils.isNotBlank(feedBackQueryDto.getTitle()), Feedback::getTitle, feedBackQueryDto.getTitle())
                .like(StringUtils.isNotBlank(feedBackQueryDto.getFeedUser()), Feedback::getFeedUser, feedBackQueryDto.getFeedUser())
                .in(Feedback::getProjectId,selfProjectId)
                .eq(StringUtils.isNotBlank(feedBackQueryDto.getCreateUser()), Feedback::getCreateUser, feedBackQueryDto.getCreateUser());
        LambdaQueryWrapper<Feedback> status2 = feedbackLambdaQueryWrapper.clone();
        LambdaQueryWrapper<Feedback> status3 = feedbackLambdaQueryWrapper.clone();
        LambdaQueryWrapper<Feedback> status4 = feedbackLambdaQueryWrapper.clone();
        long count = feedBackService.count(feedbackLambdaQueryWrapper);
        result.put("1", count);
        List<Integer> feedBackIdList = taskWService.listObjs(Wrappers.lambdaQuery(TaskW.class)
                .isNotNull(TaskW::getFeedbackId)
                .select(TaskW::getFeedbackId), item -> {
            if (item instanceof Integer) {
                return (Integer) item;
            }
            throw new RuntimeException("类型转换异常");
        });
        status2.notIn(CollectionUtil.isNotEmpty(feedBackIdList), Feedback::getId, feedBackIdList);
        long count2 = feedBackService.count(status2);
        result.put("2", count2);
        feedBackIdList = taskWService.listObjs(Wrappers.lambdaQuery(TaskW.class)
                .isNotNull(TaskW::getFeedbackId)
                .select(TaskW::getFeedbackId), item -> {
            if (item instanceof Integer) {
                return (Integer) item;
            }
            throw new RuntimeException("类型转换异常");
        });
        status3.in(CollectionUtil.isNotEmpty(feedBackIdList), Feedback::getId, feedBackIdList)
                .isNull(CollectionUtil.isEmpty(feedBackIdList), Feedback::getId);
        long count3 = feedBackService.count(status3);
        result.put("3", count3);
        feedBackIdList = taskWService.listObjs(Wrappers.lambdaQuery(TaskW.class)
                .eq(TaskW::getStatus,  "3")
                .isNotNull(TaskW::getFeedbackId)
                .select(TaskW::getFeedbackId), item -> {
            if (item instanceof Integer) {
                return (Integer) item;
            }
            throw new RuntimeException("类型转换异常");
        });
        status4.in(CollectionUtil.isNotEmpty(feedBackIdList), Feedback::getId, feedBackIdList)
                .isNull(CollectionUtil.isEmpty(feedBackIdList), Feedback::getId);

        long count4 = feedBackService.count(status4);
        result.put("4", count4);
        return ResponseEntity.ok(result);

    }


    @PreAuthorize("@check.checkPermissions({'api:tf:management:details'})")
    @ApiOperation("分页查询feedback")
    @GetMapping("/feedbackPage")
    public ResponseEntity<Page<FeedBackWithPlanVo>> paginQueryFeedBack(FeedBackQueryDto feedBackQueryDto, Page page) {
        List<Integer> selfProjectId = projectWService.getSelfProjectId();
        if(CollectionUtil.isEmpty(selfProjectId)){
            return ResponseEntity.ok(new Page<>());
        }
        LambdaQueryWrapper<Feedback> feedbackLambdaQueryWrapper = Wrappers.<Feedback>lambdaQuery()
                .eq(Feedback::getDeleted, 0)
                .eq(Objects.nonNull(feedBackQueryDto.getProjectId()), Feedback::getProjectId, feedBackQueryDto.getProjectId())
                .like(StringUtils.isNotBlank(feedBackQueryDto.getTitle()), Feedback::getTitle, feedBackQueryDto.getTitle())
                .like(StringUtils.isNotBlank(feedBackQueryDto.getFeedUser()), Feedback::getFeedUser, feedBackQueryDto.getFeedUser())
                .eq(StringUtils.isNotBlank(feedBackQueryDto.getCreateUser()), Feedback::getCreateUser, feedBackQueryDto.getCreateUser())
                .in(Feedback::getProjectId, selfProjectId)
                .orderByDesc(Feedback::getCreateDate);
        String status = feedBackQueryDto.getStatus();
        if (StringUtils.isNotBlank(status)) {
            List<Integer> feedBackIdList;
            switch (status) {
                case "2":
                    feedBackIdList = taskWService.listObjs(Wrappers.lambdaQuery(TaskW.class)
                            .isNotNull(TaskW::getFeedbackId)
                            .select(TaskW::getFeedbackId), item -> {
                        if (item instanceof Integer) {
                            return (Integer) item;
                        }
                        throw new RuntimeException("类型转换异常");
                    });
                    feedbackLambdaQueryWrapper.notIn(CollectionUtil.isNotEmpty(feedBackIdList), Feedback::getId, feedBackIdList);
                    break;
                case "3":
                    feedBackIdList = taskWService.listObjs(Wrappers.lambdaQuery(TaskW.class)
                            .isNotNull(TaskW::getFeedbackId)
                            .select(TaskW::getFeedbackId), item -> {
                        if (item instanceof Integer) {
                            return (Integer) item;
                        }
                        throw new RuntimeException("类型转换异常");
                    });
                    feedbackLambdaQueryWrapper.in(CollectionUtil.isNotEmpty(feedBackIdList), Feedback::getId, feedBackIdList)
                            .isNull(CollectionUtil.isEmpty(feedBackIdList), Feedback::getId);
                    break;
                case "4":
                    feedBackIdList = taskWService.listObjs(Wrappers.lambdaQuery(TaskW.class)
                            .eq(TaskW::getStatus,  "3")
                            .isNotNull(TaskW::getFeedbackId)
                            .select(TaskW::getFeedbackId), item -> {
                        if (item instanceof Integer) {
                            return (Integer) item;
                        }
                        throw new RuntimeException("类型转换异常");
                    });
                    feedbackLambdaQueryWrapper.in(CollectionUtil.isNotEmpty(feedBackIdList), Feedback::getId, feedBackIdList)
                            .isNull(CollectionUtil.isEmpty(feedBackIdList), Feedback::getId);
                    break;
                default:
            }

        }


        final Page page1 = feedBackService.page(page, feedbackLambdaQueryWrapper);
        List<FeedBackWithPlanVo> datas = new ArrayList<>();
        for (Object obj : page1.getRecords()) {
            Feedback record = (Feedback) obj;
            FeedBackWithPlanVo feedBackWithPlanVo = new FeedBackWithPlanVo();
            BeanUtils.copyProperties(record, feedBackWithPlanVo);
            List<FeedBackPlan> plans = feedBackPlanService.list(Wrappers.<FeedBackPlan>lambdaQuery()
                    .eq(FeedBackPlan::getFeedbackId, record.getId()).orderByDesc(FeedBackPlan::getId));
            if (plans != null && plans.size() > 0) {
                feedBackWithPlanVo.setFeedBackPlan(plans.get(0));

            }
            //意见关联的任务数量统计
            Integer id = record.getId();
            long tatalTask = taskWService.count(Wrappers.<TaskW>lambdaQuery().eq(TaskW::getFeedbackId, id)
                    .eq(TaskW::getDeleted, 0));
            long completedTask = taskWService.count(Wrappers.<TaskW>lambdaQuery().eq(TaskW::getFeedbackId, id)
                    .eq(TaskW::getDeleted, 0)
                    .eq(TaskW::getStatus, '3'));
            feedBackWithPlanVo.setTotalTask(tatalTask);
            feedBackWithPlanVo.setCompletedTask(completedTask);
            datas.add(feedBackWithPlanVo);
        }
        page1.setRecords(datas);
        return ResponseEntity.ok(page1);
    }

    /**
     * 新增数据
     *
     * @param feedBackPlan 实例对象
     * @return 实例对象
     */
    @PreAuthorize("@check.checkPermissions({'api:tf:management:add'})")
    @ApiOperation("新增或者修改数据")
    @PostMapping
    public ResponseEntity<FeedBackPlan> saveOrUpdateFeedBackPlan(@RequestBody FeedBackPlan feedBackPlan) {

        feedBackPlanService.saveOrUpdateFeedBackPlan(feedBackPlan);
        return ResponseEntity.ok(feedBackPlan);
    }


    @Transactional(rollbackFor = Exception.class)
    @PreAuthorize("@check.checkPermissions({'api:tf:management:add'})")
    @ApiOperation("批量新增或者修改数据")
    @PostMapping("/saveOrUpdateBatchFeedBackPlan")
    public ResponseEntity<Boolean> saveOrUpdateBatchFeedBackPlan(@RequestBody FeedBackPlanDto feedBackPlanDto) {
        List<FeedBackPlan> feedBackPlanList = feedBackPlanDto.getFeedBackPlanList();
        if (CollectionUtil.isNotEmpty(feedBackPlanList)) {
            for (FeedBackPlan feedBackPlan : feedBackPlanList) {
                feedBackPlanService.saveOrUpdateFeedBackPlan(feedBackPlan);
            }
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @PreAuthorize("@check.checkPermissions({'SYS_ADMIN','TENANT_ADMIN'},{'api:tf:management:del','api:tf:management:batchDel'})")
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public ResponseEntity<Boolean> deleteById(String ids) {
        final String[] split = ids.split(",");
        for (String s : split) {
            feedBackPlanService.removeById(Integer.valueOf(s));
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }
}
