package com.tofly.pms.working.controller;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.base.controller.Result;
import com.tofly.pms.working.entity.Holidays;
import com.tofly.pms.working.service.HolidaysService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author sheldon
 * @Description
 * @create 2024-04-15 9:21
 */
@Api(tags = "节假日管理")
@RestController
@RequestMapping("/holidays")
public class HolidaysController {

    @Resource
    private HolidaysService holidaysService;

    /**
     * 新增数据
     *
     * @param holidays 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<Holidays> add(@RequestBody Holidays holidays) {
        holidaysService.save(holidays);
        return ResponseEntity.ok(holidays);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public Result<Boolean> deleteById(@RequestParam String ids) {
        final String[] split = ids.split(",");
        for (String s : split) {
            holidaysService.removeById(s);
        }
        return Result.success(Boolean.TRUE);
    }

    /**
     * 分页查询
     *
     * @param holidays 筛选条件
     * @param page     分页对象
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<Holidays>> pageQuery(Holidays holidays, Page<Holidays> page) {
        return ResponseEntity.ok(holidaysService.page(page, Wrappers.lambdaQuery(holidays)
                .orderByDesc(Holidays::getHolidayDate)));
    }

    @ApiOperation("根据开始日期和结束日期查询节假日")
    @GetMapping("getByDate")
    public ResponseEntity<List<Holidays>> getByDate(@RequestParam("startDate") String startDate, @RequestParam("endDate") String endDate) {
        Date start = DateUtil.parse(startDate, "yyyy-MM-dd");
        Date end = DateUtil.parse(endDate, "yyyy-MM-dd");
        return ResponseEntity.ok(holidaysService.list(Wrappers.lambdaQuery(Holidays.class)
                .between(Holidays::getHolidayDate, start, end)
                .orderByDesc(Holidays::getHolidayDate)));
    }
}
