package com.tofly.pms.working.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.base.controller.Result;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.entity.HoursW;
import com.tofly.pms.working.service.HoursWService;
import com.tofly.pms.working.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 工时登记表;(TF_PM_WORKINGHOURS_W)表控制层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "工时登记表对象功能接口")
@RestController
@RequestMapping("/tfPmWorkinghoursW")
public class HoursWController {
    @Autowired
    private HoursWService hoursWService;
    @Autowired
    private ProjectWService projectWService;
    @Autowired
    private ProjectTeamWService projectTeamWService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<HoursW> queryById(Integer id) {
        return ResponseEntity.ok(hoursWService.queryById(id));
    }

    /**
     * 分页查询
     *
     * @param hoursW 筛选条件
     * @param page   分页对象
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<HoursW>> paginQuery(HoursW hoursW, Page<HoursW> page) {
        String tokenUserID = UserTokenUtil.getTokenUserID();
        hoursW.setCreateUser(tokenUserID);
        final Page<HoursW> page1 = hoursWService.page(page, new QueryWrapper<>(hoursW));
        List<HoursW> records = page1.getRecords();
        if (CollectionUtil.isNotEmpty(records)) {
            for (HoursW record : records) {
                Integer projectId = record.getProjectId();
                record.setProjectW(projectWService.getOne(Wrappers.lambdaQuery(ProjectW.class).eq(ProjectW::getId, projectId)));
            }
        }
        return ResponseEntity.ok(page1);
    }

    /**
     * 新增数据
     *
     * @param hoursW 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<HoursW> add(@RequestBody HoursW hoursW) {
        hoursWService.save(hoursW);
        String userID = UserTokenUtil.getTokenUserID();
        Integer projectId = hoursW.getProjectId();
        List<UserVo> teamUser = projectWService.getUserByProjectId(projectId);
        if (CollectionUtil.isNotEmpty(teamUser)) {
            List<String> userIdList = teamUser.stream().map(UserVo::getId).collect(Collectors.toList());
            if (!userIdList.contains(userID)) {
                ProjectTeamW teamW = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class).eq(ProjectTeamW::getProjectId, projectId));
                assert userID != null;
                teamW.setOhterId(teamW.getOhterId().concat(",").concat(userID));
                projectTeamWService.updateById(teamW);
            }
        } else {
            ProjectTeamW teamW = new ProjectTeamW();
            teamW.setProjectId(projectId);
            teamW.setAdminId(userID);
            teamW.setOhterId(userID);
            projectTeamWService.save(teamW);
        }

        return ResponseEntity.ok(hoursW);
    }

    /**
     * 更新数据
     *
     * @param hoursW 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<HoursW> edit(@RequestBody HoursW hoursW) {
        hoursWService.updateById(hoursW);
        String userID = UserTokenUtil.getTokenUserID();
        Integer projectId = hoursW.getProjectId();
        List<UserVo> teamUser = projectWService.getUserByProjectId(projectId);
        if (CollectionUtil.isNotEmpty(teamUser)) {
            List<String> userIdList = teamUser.stream().map(UserVo::getId).collect(Collectors.toList());
            if (!userIdList.contains(userID)) {
                ProjectTeamW teamW = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class).eq(ProjectTeamW::getProjectId, projectId));
                assert userID != null;
                teamW.setOhterId(teamW.getOhterId().concat(",").concat(userID));
                projectTeamWService.updateById(teamW);
            }
        } else {
            ProjectTeamW teamW = new ProjectTeamW();
            teamW.setProjectId(projectId);
            teamW.setAdminId(userID);
            teamW.setOhterId(userID);
            projectTeamWService.save(teamW);
        }
        return ResponseEntity.ok(hoursW);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public ResponseEntity<Boolean> deleteById(String ids) {
        final String[] split = ids.split(",");
        for (String s : split) {
            hoursWService.removeById(Integer.valueOf(s));
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

    /**
     * 统计指定月份的每一天工作时长（不区分工作日和非工作日）
     *
     * @param workMonth 工作月份
     * @return 当月每一天的工作时长
     */
    @ApiOperation("统计指定月份的每一天工作时长")
    @GetMapping("/workHoursList")
    public ResponseEntity<List<WorkHoursVo>> getWorkHoursList(@RequestParam(required = false) String workMonth) {
        List<WorkHoursVo> workHoursVoList = hoursWService.getWorkHoursList(workMonth);
        return ResponseEntity.ok(workHoursVoList);
    }

    @ApiOperation("项目工时统计")
    @GetMapping("/projectWorkingCount")
    public Result<Page<ProjectWorkingCountVo>> projectWorkingCount(
            @RequestParam(value = "projectId", required = false) Integer projectId,
            @RequestParam(value = "customName", required = false) String customName,
            @RequestParam(value = "userId", required = false) String userId,
            @RequestParam(value = "startWorkTime", required = false) String startWorkTime,
            @RequestParam(value = "endWorkTime", required = false) String endWorkTime,
            @RequestParam(value = "page") Integer page,
            @RequestParam(value = "size") Integer size
    ) {
        return Result.success(hoursWService.projectWorkingCount(projectId, customName, userId, startWorkTime, endWorkTime, page, size));
    }

    @ApiOperation("项目工时统计导出excel")
    @GetMapping("/exportProject")
    public void exportProject(@RequestParam(value = "projectId", required = false) Integer projectId,
                              @RequestParam(value = "customName", required = false) String customName,
                              @RequestParam(value = "userId", required = false) String userId,
                              @RequestParam(value = "startWorkTime", required = false) String startWorkTime,
                              @RequestParam(value = "endWorkTime", required = false) String endWorkTime) throws IOException {
        hoursWService.exportProject(projectId, customName, userId, startWorkTime, endWorkTime);
    }

    @ApiOperation("工人工时统计导出excel")
    @GetMapping("/exportWorker")
    public void exportWorker(@RequestParam(value = "projectId", required = false) Integer projectId,
                             @RequestParam(value = "customName", required = false) String customName,
                             @RequestParam(value = "userId", required = false) String userId,
                             @RequestParam(value = "startWorkTime", required = false) String startWorkTime,
                             @RequestParam(value = "endWorkTime", required = false) String endWorkTime) throws IOException {
        hoursWService.exportWorker(projectId, customName, userId, startWorkTime, endWorkTime);
    }

    @ApiOperation("工人工时统计")
    @GetMapping("/workerWorkingCount")
    public Result<Page<WorkerWorkingCountVo>> workerWorkingCount(
            @RequestParam(value = "projectId", required = false) Integer projectId,
            @RequestParam(value = "customName", required = false) String customName,
            @RequestParam(value = "userId", required = false) String userId,
            @RequestParam(value = "startWorkTime", required = false) String startWorkTime,
            @RequestParam(value = "endWorkTime", required = false) String endWorkTime,
            @RequestParam(value = "page") Integer page,
            @RequestParam(value = "size") Integer size
    ) {
        return Result.success(hoursWService.workerWorkingCount(projectId, customName, userId, startWorkTime, endWorkTime, page, size));
    }


    @ApiOperation("员工工时稽核")
    @GetMapping("/check")
    public ResponseEntity<List<CheckHoursVo>> check(@RequestParam(value = "deptId", required = false) String deptId,
                                                    @RequestParam(value = "realName", required = false) String realName,
                                                    @RequestParam(value = "month") String month) {
        return ResponseEntity.ok(hoursWService.check(deptId, realName, month));
    }

    @ApiOperation("员工工时稽核导出excel")
    @GetMapping("/exportCheck")
    public void exportWorker(@RequestParam(value = "deptId", required = false) String deptId,
                             @RequestParam(value = "realName", required = false) String realName,
                             @RequestParam(value = "month") String month) throws IOException {
        hoursWService.exportCheck(deptId, realName, month);
    }

    /**
     * 工时：选择项目后，自动统计该项目今日任务、bug的完成情况，如果今日没有bug或者任务，则不自动生成相应文本。
     * 待处理：任务状态为待处理且任务结束时间大于等于今天。
     * 今日完成：今日处理完成的任务。
     * 逾期：任务状态为处理中且任务结束时间小于今天。
     * 被驳回：任务状态为被驳回的任务。
     * 内容参考如下：
     * 今日任务情况：今日完成0个，逾期1个，被驳回1个，待处理1个。今日创建*个，确认*个，驳回*个，待确认*个。
     * <p>
     * 今日bug情况：今日完成0个，逾期0个，被驳回0个，待处理1个。今日创建*个，验证*个，驳回*个，待验证*个。
     * <p>
     * 今日任务情况：今日完成3个，逾期1个，被驳回1个，待处理2个。
     * 今日bug情况：今日完成5个，逾期4个，被驳回2个，待处理1个。
     * 【任务完成明细】
     * ·【投入4h，预计8h】这里是任务标题
     * 。
     * ·【投入4h，预计8h】这里是任务标题
     * 。
     * 【bug完成明细】
     * ·这里是bug标题
     * ·这里是bug标题
     *
     * @param projectId
     * @return
     */
    @ApiOperation("根据项目id查询当日任务完成情况")
    @GetMapping("/getTaskAchievement")
    public Result<TodayTaskCompleteVo> getTaskAchievement(@RequestParam Integer projectId) {
        return Result.success(hoursWService.getTaskAchievement(projectId));
    }
}