package com.tofly.pms.working.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.exceptions.ValidateException;
import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.utils.ImportExcelHelper;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.dto.PlanBatchVo;
import com.tofly.pms.working.dto.PlanDispatchDto;
import com.tofly.pms.working.entity.Plan;
import com.tofly.pms.working.entity.PlanExcelModel;
import com.tofly.pms.working.entity.TaskW;
import com.tofly.pms.working.listener.PlanListener;
import com.tofly.pms.working.service.PlanService;
import com.tofly.pms.working.service.TaskWService;
import com.tofly.pms.working.vo.ProjectCountVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 项目计划信息表;(TF_PM_PLAN_W)表控制层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "项目计划信息表对象功能接口")
@RestController
@RequestMapping("/tfPmPlanW")
public class PlanController {
    @Autowired
    private PlanService planService;
    @Autowired
    private TaskWService taskWService;
    @Autowired
    private ProjectTeamWService projectTeamWService;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<Plan> queryById(Integer id) {
        return ResponseEntity.ok(planService.queryById(id));
    }


    /**
     * 项目统计
     */
    @ApiOperation("统计项目工作量")
    @GetMapping("/overCount")
    public ResponseEntity<ProjectCountVo> projectOverCount(Integer projectId) {

        return ResponseEntity.ok(planService.projectOverCount(projectId));
    }


    /**
     * 分页查询
     *
     * @param plan 筛选条件
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<Plan>> paginQuery(Plan plan, Page<Plan> page) {
        String name = plan.getName();
        String headId = plan.getHeadId();
        plan.setName(null);
        plan.setHeadId(null);
        QueryWrapper<Plan> planQueryWrapper = new QueryWrapper<>(plan);
        planQueryWrapper.lambda().like(StringUtils.isNotBlank(name),Plan::getName, name)
                .like(StringUtils.isNotBlank(headId),Plan::getHeadId, headId)
                .last("ORDER BY CASE WHEN split_part(number, '.', 1) ~ '^[0-9]+$' THEN split_part(number, '.', 1) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 2) ~ '^[0-9]+$' THEN split_part(number, '.', 2) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 3) ~ '^[0-9]+$' THEN split_part(number, '.', 3) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 4) ~ '^[0-9]+$' THEN split_part(number, '.', 4) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 5) ~ '^[0-9]+$' THEN split_part(number, '.', 5) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 6) ~ '^[0-9]+$' THEN split_part(number, '.', 6) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 7) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 8) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END,\n" +
                        "         CASE WHEN split_part(number, '.', 9) ~ '^[0-9]+$' THEN split_part(number, '.', 7) ::integer ELSE 0 END");
        final Page<Plan> page1 = planService.page(page, planQueryWrapper);
        //todo 需要返回关联任务的数据，包括总数和已完成的任务数量
        List<Plan> records = page1.getRecords();
        if (CollectionUtil.isNotEmpty(records)) {
            for (Plan record : records) {
                Integer planId = record.getId();
                long totalTask = taskWService.count(Wrappers.lambdaQuery(TaskW.class)
                        .eq(TaskW::getPlanId, planId)
                        .eq(TaskW::getDeleted,0)
                );
                long completeTask = taskWService.count(Wrappers.lambdaQuery(TaskW.class)
                        .eq(TaskW::getPlanId, planId)
                        .eq(TaskW::getDeleted,0)
                        .eq(TaskW::getStatus, "3")
                );
                record.setTotalTask(totalTask);
                record.setCompleteTask(completeTask);
            }
        }
        return ResponseEntity.ok(page1);
    }

    @ApiOperation("获取id")
    @GetMapping("/getMaxId")
    public ResponseEntity<Integer> getMaxId() {

        return ResponseEntity.ok(planService.getMaxId());
    }

    /**
     * 新增数据
     *
     * @param plan 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<Plan> add(@RequestBody Plan plan) {
        Integer projectId = plan.getProjectId();
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID,"当前用户不合法");
        Assert.notNull(projectId,"项目id不能为空");
        ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId)
                .like(ProjectTeamW::getAdminId, userID)
        );
        if(Objects.isNull(one)){
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if(!has){
                throw new RuntimeException("当前用户没有权限");
            }
        }
        planService.save(plan);
        return ResponseEntity.ok(plan);
    }

    /**
     * 批量新增或者修改
     *
     * @param planBatchVo 实例对象
     * @return 实例对象
     */
    @ApiOperation("批量新增或者修改计划")
    @PostMapping("saveOrUpdateBatch")
    public ResponseEntity<PlanBatchVo> saveOrUpdateBatch(@RequestBody PlanBatchVo planBatchVo) {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID,"当前用户不合法");
        List<Plan> datas = planBatchVo.getDatas();
        int sort = 1;
        for (Plan data : datas) {
            data.setSort(sort++);
            Integer projectId = data.getProjectId();
            ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, projectId)
                    .like(ProjectTeamW::getAdminId, userID)
            );
            if(Objects.isNull(one)){
                RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                if(!has){
                    throw new RuntimeException("当前用户没有权限");
                }
            }
        }
        planService.saveOrUpdateBatch(datas);
        return ResponseEntity.ok(planBatchVo);
    }

    /**
     * 更新数据
     *
     * @param plan 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<Plan> edit(@RequestBody Plan plan) {
        Integer projectId = plan.getProjectId();
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID,"当前用户不合法");
        Assert.notNull(projectId,"项目id不能为空");
        ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId)
                .like(ProjectTeamW::getAdminId, userID)
        );
        if(Objects.isNull(one)){
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if(!has){
                throw new RuntimeException("当前用户没有权限");
            }
        }
        planService.updateById(plan);
        return ResponseEntity.ok(plan);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    @Transactional(rollbackFor = Exception.class)
    public ResponseEntity<Boolean> deleteById(String ids) {
        final String[] split = ids.split(",");
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID,"当前用户不合法");
        for (String s : split) {
            final Plan byId = planService.getById(Integer.valueOf(s));
            Integer projectId = byId.getProjectId();
            Assert.notNull(projectId,"项目id不能为空");
            ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, projectId)
                    .like(ProjectTeamW::getAdminId, userID)
            );
            if(Objects.isNull(one)){
                RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                if(!has){
                    throw new RuntimeException("当前用户没有权限");
                }
            }
            final long count = taskWService.count(Wrappers.<TaskW>lambdaQuery().
                    eq(TaskW::getPlanId, Integer.valueOf(s))
                    .eq(TaskW::getDeleted, 0));
            if (count > 0) {


                throw new ValidateException(byId.getName() + "存在关联的任务，无法删除");

            }


            planService.removeById(Integer.valueOf(s));
        }
        return ResponseEntity.ok(Boolean.TRUE);
        //查看该计划是否存在对应的任务
    }

    /**
     * 任务分发
     *
     * @param planDispatchDto
     * @return
     */
    @PostMapping("/dispatchPlan")
    @ApiOperation("任务分发")
    public ResponseEntity<Boolean> dispatchPlan(@RequestBody PlanDispatchDto planDispatchDto) {


        return ResponseEntity.ok(planService.dispatchPlan(planDispatchDto));

    }


    @ApiOperation("导出excel")
    @GetMapping("/export")
    public void export(@RequestParam("projectId") Integer projectId,@RequestParam("listId") Integer listId) throws IOException {
        planService.export(projectId,listId);
    }
    @Transactional(rollbackFor = Exception.class)
    @ApiOperation("计划批量导入")
    @PostMapping("/import")
    public ResponseEntity<Boolean> feedbackImport(@RequestParam("file") MultipartFile file,
                                                  @RequestParam("projectId") Integer projectId,
                                                  @RequestParam("listId") Integer listId) throws Exception {
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID,"当前用户不合法");
        ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId)
                .like(ProjectTeamW::getAdminId, userID)
        );
        if(Objects.isNull(one)){
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if(!has){
                throw new RuntimeException("当前用户没有权限");
            }
        }
        ImportExcelHelper<PlanExcelModel> helper = new ImportExcelHelper<>(new PlanListener(1));
        List<PlanExcelModel> importData = helper.getList(file, PlanExcelModel.class, 0, 1);
        if(CollectionUtil.isEmpty(importData)){
            return ResponseEntity.ok(Boolean.FALSE);
        }
        int sort = 1;
        List<Plan> planList = new ArrayList<>(importData.size());
        for (PlanExcelModel model : importData) {
            Plan plan =  model.toPlan();
            plan.setProjectId(projectId);
            plan.setListId(listId);
            plan.setSort(sort++);
            planList.add(plan);
        }
        Map<String, Plan> planMap = planList.stream().collect(Collectors.toMap(Plan::getNumber, item -> item));
        for (Plan plan : planList) {
            Integer id = plan.getId();
            if(ObjectUtils.isNotNull(id)){
                continue;
            }
            String number = plan.getNumber();
            Assert.notBlank(number,"id和编号不能同时为空");
            plan.setId(planService.getMaxId());
        }
        for (Plan plan : planList) {
            String number = plan.getNumber();
            String parentNumber = this.getParentNumber(number);
            if(StringUtils.isNotBlank(parentNumber)){
                Plan parentPlan = planMap.get(parentNumber);
                if(ObjectUtils.isNotNull(parentPlan)){
                    plan.setParentId(parentPlan.getId());
                }
            }
        }

        planService.remove(Wrappers.lambdaQuery(Plan.class).eq(Plan::getProjectId,projectId).eq(Plan::getListId,listId));
        boolean b = planService.saveBatch(planList);
        return ResponseEntity.ok(b);
    }

    private String getParentNumber(String number) {
        if(number.contains(".")){
            return number.substring(0, number.lastIndexOf("."));
        }
        return null;
    }
}

