package com.tofly.pms.working.controller;

import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.entity.PlanList;
import com.tofly.pms.working.service.PlanListService;
import com.tofly.pms.working.service.PlanService;
import com.tofly.pms.working.vo.PlanListCountVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * 计划列表
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "计划列表对象功能接口")
@RestController
@RequestMapping("/tfPmPlanTask")
public class PlanListController {

    @Autowired
    private PlanListService planListService;

    @Autowired
    private PlanService planService;
    @Autowired
    private ProjectTeamWService projectTeamWService;
    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<PlanList> queryById(Integer id){
        return ResponseEntity.ok(planListService.queryById(id));
    }

    /**
     * 分页查询
     *
     * @param planList 筛选条件
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<PlanList>> paginQuery( PlanList planList, Page page){
        final Page page1 = planListService.page(page, new QueryWrapper<>(planList)
                .lambda().orderByAsc(PlanList::getSortNum));

        return ResponseEntity.ok(page1);
    }

    /**
     * 列表查询包含统计
     *
     * @param planList 列表查询包含统计
     * @return 查询结果
     */
    @ApiOperation("列表查询包含统计")
    @GetMapping("/listOverCount")
    public ResponseEntity<List<PlanListCountVo>> planListOverCount(PlanList planList){
        final List<PlanListCountVo> planListCountVos = planListService.countPlanOverList(planList);

        return ResponseEntity.ok(planListCountVos);
    }

    /**
     * 新增数据
     *
     * @param planList 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<PlanList> add(@RequestBody  PlanList planList){
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID,"当前用户不合法");
        Integer projectId = planList.getProjectId();
        ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId)
                .like(ProjectTeamW::getAdminId, userID)
        );
        if(Objects.isNull(one)){
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if(!has){
                throw new RuntimeException("当前用户没有权限");
            }
        }
        planListService.save(planList);
        return ResponseEntity.ok(planList);
    }

    /**
     * 更新数据
     *
     * @param planList 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<PlanList> edit(@RequestBody PlanList planList){
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID,"当前用户不合法");
        Integer projectId = planList.getProjectId();
        ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId)
                .like(ProjectTeamW::getAdminId, userID)
        );
        if(Objects.isNull(one)){
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if(!has){
                throw new RuntimeException("当前用户没有权限");
            }
        }
        planListService.updateById(planList);
        return ResponseEntity.ok(planList);
    }

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public ResponseEntity<Boolean> deleteById(Integer id){

        //判断是否存在关联的计划数据
//        final long count = planService.count(Wrappers.<Plan>lambdaQuery().eq(Plan::getListId, id));

//        if(count>0){
//            throw  new ValidateException("该计划列表存在关联计划，无法删除");
//        }
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID,"当前用户不合法");
        PlanList planList = planListService.getById(id);
        Integer projectId = planList.getProjectId();
        ProjectTeamW one = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                .eq(ProjectTeamW::getProjectId, projectId)
                .like(ProjectTeamW::getAdminId, userID)
        );
        if(Objects.isNull(one)){
            RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
            Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
            if(!has){
                throw new RuntimeException("当前用户没有权限");
            }
        }

        return ResponseEntity.ok(planListService.removeById(id));
    }

}
