package com.tofly.pms.working.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.base.controller.Result;
import com.tofly.pms.fegin.DictItemClient;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.project.vo.ProjectOverTaskCountVo;
import com.tofly.pms.project.vo.ProjectVo;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.dto.TaskHandleBatchDto;
import com.tofly.pms.working.dto.TaskUpdateBatchDto;
import com.tofly.pms.working.entity.*;
import com.tofly.pms.working.mapper.TaskWMapper;
import com.tofly.pms.working.service.PlanListService;
import com.tofly.pms.working.service.PlanService;
import com.tofly.pms.working.service.TaskWService;
import com.tofly.pms.working.service.TasklogWService;
import com.tofly.pms.working.vo.TaskCountVo;
import com.tofly.pms.working.vo.TaskParamCountVo;
import com.tofly.pms.working.vo.TaskParamVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * 任务信息表;(TF_PM_TASK_W)表控制层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "任务信息表对象功能接口")
@RestController
@RequestMapping("/tfPmTaskW")
public class TaskWController {
    @Autowired
    private TaskWService taskWService;
    @Autowired
    private TasklogWService tasklogWService;
    @Autowired
    private PlanService planService;
    @Autowired
    private PlanListService planListService;
    @Autowired
    private ProjectTeamWService projectTeamWService;
    @Autowired
    private TaskWMapper taskWMapper;
    @Autowired
    private ProjectWService projectWService;
    @Autowired
    private HttpServletResponse response;

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<TaskW> queryById(Integer id) {
        return ResponseEntity.ok(taskWService.queryById(id));
    }

    @ApiOperation("任务查询条件统计")
    @GetMapping("/getTaskParamList")
    public Result<List<TaskParamVo>> getTaskParamList(TaskW taskW) {
        List<Integer> selfProjectIdList = projectWService.getSelfProjectId();
        String status = taskW.getStatus();
        Integer projectId = taskW.getProjectId();
        String title = taskW.getTitle();
        String headId = taskW.getHeadId();
        String confirmId = taskW.getConfirmId();
        String type = taskW.getType();
        String urgency = taskW.getUrgency();
        Date completeTimeStart = taskW.getCompleteTimeStart();
        Date completeTimeEnd = taskW.getCompleteTimeEnd();
        LambdaQueryWrapper<TaskW> taskWLambdaQueryWrapper = Wrappers.lambdaQuery(taskW)
                .eq(StringUtils.isNotBlank(status), TaskW::getStatus, status)
                .eq(Objects.nonNull(projectId), TaskW::getProjectId, projectId)
                .eq(StringUtils.isNotBlank(type), TaskW::getType, type)
                .eq(StringUtils.isNotBlank(urgency), TaskW::getUrgency, urgency)
                .eq(StringUtils.isNotBlank(headId), TaskW::getHeadId, headId)
                .like(StringUtils.isNotBlank(title), TaskW::getTitle, title)
                .eq(TaskW::getDeleted, 0)
                .in(CollectionUtil.isNotEmpty(selfProjectIdList), TaskW::getProjectId, selfProjectIdList)
                .eq(CollectionUtil.isEmpty(selfProjectIdList), TaskW::getId, 0)
                .between(Objects.nonNull(completeTimeStart) && Objects.nonNull(completeTimeEnd), TaskW::getCompleteTime, completeTimeStart, completeTimeEnd);
        if(StringUtils.isNotBlank(confirmId)){
            taskWLambdaQueryWrapper.and(wrapper -> wrapper.like(TaskW::getConfirmId, confirmId).or()
                    .like(TaskW::getAlreadyConfirmId, confirmId));
        }
        List<TaskW> taskWList = taskWService.list(taskWLambdaQueryWrapper);
        if (CollectionUtil.isEmpty(taskWList)) {
            return null;
        }
        List<TaskParamVo> result = new ArrayList<>();
        //统计负责人的数据

        List<String> headIdList = taskWList
                .stream()
                .map(TaskW::getHeadId)
                .filter(StringUtils::isNotBlank)
                .distinct()
                .collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(headIdList)) {
            List<UserVo> headUser = projectWService.getUserVoByUserId(headIdList);
            Map<String, String> headUserMap = headUser.stream()
                    .collect(Collectors.toMap(UserVo::getId, UserVo::getRealName));
            List<TaskParamCountVo> headUserList = headIdList.stream().map(itemId -> {
                List<Integer> projectIdList = projectWService.getProjectIdByUserId(itemId);
                TaskParamCountVo item = new TaskParamCountVo();
                item.setId(itemId);
                item.setName(headUserMap.get(itemId));
                item.setCount(taskWList.stream()
                        .filter(task -> StringUtils.isNotBlank(task.getHeadId()))
                        .filter(task -> task.getHeadId().equals(itemId))
                        .filter(task -> projectIdList.contains(task.getProjectId()))
                        .count());
                return item;
            }).filter(item -> item.getCount() > 0)
                    .collect(Collectors.toList());
            result.add(new TaskParamVo("headId", headUserList));
        } else {
            result.add(new TaskParamVo("headId", new ArrayList<>()));
        }
        //统计确认人
        String confirmIdStr = taskWList.stream().map(TaskW::getConfirmId).collect(Collectors.joining(","));
        if (StringUtils.isNotBlank(confirmIdStr)) {
            List<String> confirmIdList = Arrays.stream(confirmIdStr.split(","))
                    .filter(StringUtils::isNotBlank)
                    .distinct()
                    .collect(Collectors.toList());
            List<UserVo> confirmUser = projectWService.getUserVoByUserId(confirmIdList);
            Map<String, String> confirmUserMap = confirmUser.stream()
                    .collect(Collectors.toMap(UserVo::getId, UserVo::getRealName));
            List<TaskParamCountVo> confirmUserList = confirmIdList.stream().map(itemId -> {
                List<Integer> projectIdList = projectWService.getProjectIdByUserId(itemId);
                TaskParamCountVo item = new TaskParamCountVo();
                item.setId(itemId);
                item.setName(confirmUserMap.get(itemId));
                item.setCount(taskWList.stream()
                        .filter(task -> StringUtils.isNotBlank(task.getConfirmId()))
                        .filter(task -> task.getConfirmId().contains(itemId))
                        .filter(task -> projectIdList.contains(task.getProjectId()))
                        .count());
                return item;
            }).filter(item -> item.getCount() > 0)
                    .collect(Collectors.toList());
            result.add(new TaskParamVo("confirmId", confirmUserList));
        } else {
            result.add(new TaskParamVo("confirmId", new ArrayList<>()));
        }

        //统计任务类型
        DictItemClient dictClient = SpringUtil.getBean(DictItemClient.class);
        List<String> typeList = taskWList.stream().map(TaskW::getType)
                .filter(StringUtils::isNotBlank)
                .distinct().collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(typeList)) {
            List<TaskParamCountVo> taskTypeList = typeList.stream().map(itemType -> {
                TaskParamCountVo item = new TaskParamCountVo();
                item.setId(itemType);
                item.setName(dictClient.getDicItemText("TASKTYPE", itemType));
                item.setCount(taskWList.stream()
                        .filter(task -> StringUtils.isNotBlank(task.getType()))
                        .filter(task -> task.getType().equals(itemType))
                        .count());
                return item;
            }).collect(Collectors.toList());
            result.add(new TaskParamVo("type", taskTypeList));
        } else {
            result.add(new TaskParamVo("type", new ArrayList<>()));
        }

        //统计任务紧急程度
        List<String> urgencyList = taskWList
                .stream()
                .map(TaskW::getUrgency)
                .filter(StringUtils::isNotBlank)
                .distinct().collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(urgencyList)) {
            List<TaskParamCountVo> taskUrgencyList = urgencyList.stream().map(itemUrgency -> {
                TaskParamCountVo item = new TaskParamCountVo();
                item.setId(itemUrgency);
                item.setName(dictClient.getDicItemText("JJCD", itemUrgency));
                item.setCount(taskWList.stream()
                        .filter(task -> StringUtils.isNotBlank(task.getUrgency()))
                        .filter(task -> task.getUrgency().equals(itemUrgency))
                        .count());
                return item;
            }).collect(Collectors.toList());
            result.add(new TaskParamVo("urgency", taskUrgencyList));
        } else {
            result.add(new TaskParamVo("urgency", new ArrayList<>()));
        }
        return Result.success(result);
    }

    /**
     * 分页查询
     *
     * @param taskW 筛选条件
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<TaskW>> paginQuery(TaskW taskW, Page<TaskW> page,
                                                  String orderBy,
                                                  String ascOrDesc) {
        List<Integer> projectIdList = projectWService.getSelfProjectId();
        if (CollectionUtil.isEmpty(projectIdList)) {
            return ResponseEntity.ok(new Page<>());
        }
        List<OrderItem> orders = page.getOrders();
        page.setOrders(null);
        String title = taskW.getTitle();
        taskW.setTitle(null);
        String confirmId = taskW.getConfirmId();
        taskW.setConfirmId(null);
        Date completeTimeStart = taskW.getCompleteTimeStart();
        Date completeTimeEnd = taskW.getCompleteTimeEnd();
        LambdaQueryWrapper<TaskW> taskWLambdaQueryWrapper = Wrappers.lambdaQuery(taskW)
                .like(StringUtils.isNotBlank(title), TaskW::getTitle, title)
                .eq(TaskW::getDeleted, 0)
                .in(TaskW::getProjectId, projectIdList)
                .between(Objects.nonNull(completeTimeStart) && Objects.nonNull(completeTimeEnd), TaskW::getCompleteTime, completeTimeStart, completeTimeEnd);
        if(StringUtils.isNotBlank(confirmId)){
            taskWLambdaQueryWrapper.and(qw->qw.like(TaskW::getConfirmId,confirmId).or()
                    .like(TaskW::getAlreadyConfirmId,confirmId));
        }
        if (StringUtils.isNotBlank(orderBy)) {
            taskWLambdaQueryWrapper.last("order by " + orderBy + " " + ascOrDesc + " NULLS last ");
        } else if (CollectionUtil.isNotEmpty(orders) && StringUtils.isNotBlank(orders.get(0).getColumn())) {

            taskWLambdaQueryWrapper.last("order by " + orders.get(0).getColumn() + (orders.get(0).isAsc() ? " asc" : " desc") + " NULLS last ");

        } else {
            taskWLambdaQueryWrapper.orderByDesc(TaskW::getCreateDate).last("NULLS last ");
        }
        final Page<TaskW> page1 = taskWService.page(page,
                taskWLambdaQueryWrapper
        );
        final List<TaskW> records = page1.getRecords();
        List<TaskW> newPage = new ArrayList<>();

        final List<Integer> collect = records.stream().map(TaskW::getId).collect(Collectors.toList());
        final List<TasklogW> tasklogWS = tasklogWService.selectByMaxIdForEachTaskId(collect);
        Map<Integer, TasklogW> resultMap = new HashMap<>();
        for (TasklogW tasklogW : tasklogWS) {
            resultMap.put(tasklogW.getTaskId(), tasklogW);
        }
        for (TaskW record : records) {

            final TasklogW tasklogW = resultMap.get(record.getId());
            if (tasklogW != null) {
                record.setCurrentTaskLog(tasklogW);
            }
            // 返回关联任务的全路径
            Integer planId = record.getPlanId();
            String testList = record.getTestList();
            if (Objects.nonNull(planId)) {
                String planFullPath = planService.getFullPath(planId);
                if (StringUtils.isBlank(testList)) {
                    Plan plan = planService.getById(planId);
                    testList = plan.getListId().toString();
                }
                PlanList planList = planListService.getById(Integer.parseInt(testList));
                if (Objects.nonNull(planList)) {
                    if (StringUtils.isNotBlank(planFullPath)) {
                        planFullPath = org.apache.commons.lang3.StringUtils.isNotBlank(planList.getName()) ? planList.getName().concat("->").concat(planFullPath) : planFullPath;

                    }
                }
                record.setPlanFullPath(planFullPath);
            }


            newPage.add(record);
        }
        page1.setRecords(newPage);

        return ResponseEntity.ok(page1);
    }

    /**
     * 分页查询
     *
     * @param taskW 筛选条件
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("pageself")
    public ResponseEntity<Page<TaskW>> paginQuerySelf(TaskW taskW, Page<TaskW> page) {
        final String tokenUserID = UserTokenUtil.getTokenUserID();
        List<Integer> projectIdList = projectWService.getSelfProjectId();
        if (CollectionUtil.isEmpty(projectIdList)) {
            return ResponseEntity.ok(new Page<>());
        }
        final Page<TaskW> page1 = taskWService.page(page, new QueryWrapper<>(taskW)
                .lambda()
                .eq(TaskW::getDeleted, 0)
                .eq(TaskW::getHeadId, tokenUserID)
                .like(TaskW::getConfirmId, tokenUserID)
                .like(TaskW::getCareId, tokenUserID)
                .in(TaskW::getProjectId, projectIdList)
                .orderByDesc(TaskW::getCreateDate));


        final List<TaskW> records = page1.getRecords();
        List<TaskW> newPage = new ArrayList<>();
        for (TaskW record : records) {
            final List<TasklogW> list = tasklogWService.list(Wrappers.<TasklogW>lambdaQuery()
                    .eq(TasklogW::getTaskId, record.getId()).orderByDesc(TasklogW::getId));
            if (list != null && list.size() > 0) {
                record.setCurrentTaskLog(list.get(0));
            }
            newPage.add(record);
        }
        page1.setRecords(newPage);

        return ResponseEntity.ok(page1);
    }

    @ApiOperation("根据项目统计任务类型")
    @GetMapping("/countProjectTask")
    public ResponseEntity<List<ProjectOverTaskCountVo>> countProjectTask(TaskW taskW) {

        return ResponseEntity.ok(taskWService.getProjectOverTaskCountVo(taskW));
    }

    /**
     * 新增数据
     *
     * @param taskW 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<TaskW> add(@RequestBody TaskW taskW) {
        if (StringUtils.isNotBlank(taskW.getHeadId())) {
            //TODO 修改为常量
            taskW.setStatus("1");
        }
        taskWService.save(taskW);
        return ResponseEntity.ok(taskW);
    }

    /**
     * 任务处理
     *
     * @return 实例对象
     */
//    @ApiOperation("任务处理")
//    @PostMapping("handle")
//    public ResponseEntity<Boolean> handleTask(@RequestBody TaskHandleDto taskHandleDto) {
//        taskWService.taskHandle(taskHandleDto);
//        return ResponseEntity.ok(Boolean.TRUE);
//    }
    @ApiOperation("任务处理")
    @PostMapping("handle")
    public ResponseEntity<Boolean> handleTask(@RequestBody TaskW taskW) {
        taskWService.handleSingleTask(taskW);
        return ResponseEntity.ok(Boolean.TRUE);
    }

    /**
     * 任务处理
     *
     * @param taskHandleBatchDto 实例对象
     * @return 实例对象
     */
    @ApiOperation("任务处理(批量)")
    @PostMapping("handleBatch")
    public ResponseEntity<Boolean> handleTaskBatch(@RequestBody TaskHandleBatchDto taskHandleBatchDto) {
        taskWService.taskHandleBatch(taskHandleBatchDto);
        return ResponseEntity.ok(Boolean.TRUE);
    }

    /**
     * 更新数据
     *
     * @param taskW 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PostMapping("updateTaskInfo")
    public ResponseEntity<TaskW> updateTaskInfo(@RequestBody TaskW taskW) {
        taskWService.updateTaskInfo(taskW);
        return ResponseEntity.ok(taskW);
    }

    /**
     * 更新数据
     *
     * @param taskW 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<TaskW> edit(@RequestBody TaskW taskW) {
        taskWService.updateById(taskW);
        return ResponseEntity.ok(taskW);
    }

    /**
     * 批量更新数据
     *
     * @param taskUpdateBatchDto 实例对象
     * @return Boolean
     */
    @ApiOperation("更新数据(批量)")
    @PutMapping("/updateBatch")
    @Transactional(rollbackFor = Exception.class)
    public ResponseEntity<Boolean> editBatch(@RequestBody TaskUpdateBatchDto taskUpdateBatchDto) {

        final String ids = taskUpdateBatchDto.getIds();
        final List<Integer> collect =
                Arrays.stream(ids.split(",")).mapToInt(Integer::parseInt).boxed().collect(Collectors.toList());
        List<TaskW> taskWS = taskWService.listByIds(collect);
        if (CollectionUtil.isNotEmpty(taskWS)) {
            for (TaskW taskW : taskWS) {
                if (StringUtils.isNotBlank(taskUpdateBatchDto.getGroupname())) {
                    taskW.setGroupname(taskUpdateBatchDto.getGroupname());
                }
                if (StringUtils.isNotBlank(taskUpdateBatchDto.getConfirmId())) {
                    taskW.setConfirmId(taskUpdateBatchDto.getConfirmId());
                }
                if (StringUtils.isNotBlank(taskUpdateBatchDto.getPlanStartDate())) {
                    taskW.setPlanStartDate(taskUpdateBatchDto.getPlanStartDate());
                }
                if (StringUtils.isNotBlank(taskUpdateBatchDto.getPlanEndDate())) {
                    taskW.setPlanEndDate(taskUpdateBatchDto.getPlanEndDate());
                }
                if (StringUtils.isNotBlank(taskUpdateBatchDto.getHeadId())) {
                    taskW.setHeadId(taskUpdateBatchDto.getHeadId());
                }
                if (StringUtils.isNotBlank(taskUpdateBatchDto.getLabel())) {
                    taskW.setLabel(taskUpdateBatchDto.getLabel());
                }
                if (StringUtils.isNotBlank(taskW.getHeadId()) && (StringUtils.isBlank(taskW.getStatus()) || "0".equals(taskW.getStatus()))) {
                    taskW.setStatus("1");
                }
            }
        }
        for (TaskW taskW : taskWS) {
            taskWService.updateTaskInfo(taskW);
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

    /**
     * 通过主键删除数据
     *
     * @param ids 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    @Transactional(rollbackFor = Exception.class)
    public ResponseEntity<Boolean> deleteById(String ids) {
        final String[] split = ids.split(",");
        for (String s : split) {
            Integer id = Integer.valueOf(s);
            TaskW taskW = taskWMapper.selectById(id);
            taskW.setDeleted(1);
            taskWService.updateById(taskW);
        }
        return ResponseEntity.ok(Boolean.TRUE);
    }

    @ApiOperation("统计当前人员的待处理，已驳回任务以及项目数据")
    @GetMapping("/countCurrentUserTask")
    public ResponseEntity countCurrentUserTask(Integer projectId, String title, String urgency,
                                               String groupname, String planEndDate1, String planEndDate2) {
        final String tokenUserID = UserTokenUtil.getTokenUserID();
        LambdaQueryWrapper<TaskW> queryWrapper = Wrappers.<TaskW>lambdaQuery()
                .eq(TaskW::getDeleted, 0)
                .eq(TaskW::getHeadId, tokenUserID).in(TaskW::getStatus, "1", "4")
                .eq(projectId != null, TaskW::getProjectId, projectId)
                .like(StringUtils.isNotBlank(groupname), TaskW::getGroupname, groupname)
                .eq(StringUtils.isNotBlank(urgency), TaskW::getUrgency, urgency)
                .between(StringUtils.isNotBlank(planEndDate1) && StringUtils.isNotBlank(planEndDate2),
                        TaskW::getPlanEndDate, planEndDate1, planEndDate2);
        if (StringUtils.isNotBlank(title)) {
            Pattern pattern = Pattern.compile("^\\d+$");
            if(pattern.matcher(title).matches()){
                queryWrapper.and(qw -> qw.like(TaskW::getTitle, title).or().eq(TaskW::getId, Integer.valueOf(title)));
            }else {
                queryWrapper.like(TaskW::getTitle, title);
            }
        }
        final List<TaskW> list = taskWService.list(queryWrapper);
        if (CollectionUtil.isNotEmpty(list)) {
            Map<String, Long> collect = list.stream().collect(Collectors.groupingBy(TaskW::getStatus, Collectors.counting()));
            return ResponseEntity.ok(collect);
        }
        return ResponseEntity.ok(0);


    }

    @ApiOperation("查询当前人员的待处理，已驳回任务以及项目数据")
    @GetMapping("/getCurrentUserTask")
    public ResponseEntity getCurrentUserTask(Integer projectId, String status, String title, String urgency,
                                             String groupname, String planEndDate1, String planEndDate2) {

        final String tokenUserID = UserTokenUtil.getTokenUserID();
//         .like(StringUtils.isNotBlank(title), TaskW::getTitle, title)
        LambdaQueryWrapper<TaskW> queryWrapper = Wrappers.lambdaQuery(TaskW.class)
                .eq(TaskW::getDeleted, 0)
                .eq(TaskW::getHeadId, tokenUserID).in(TaskW::getStatus, "1", "4")
                .eq(projectId != null, TaskW::getProjectId, projectId)
                .eq(StringUtils.isNotBlank(status), TaskW::getStatus, status)
                .eq(StringUtils.isNotBlank(urgency), TaskW::getUrgency, urgency)
                .eq(StringUtils.isNotBlank(groupname), TaskW::getGroupname, groupname)
                .between(StringUtils.isNotBlank(planEndDate1) && StringUtils.isNotBlank(planEndDate2),
                        TaskW::getPlanEndDate, planEndDate1, planEndDate2);
        if (StringUtils.isNotBlank(title)) {
            Pattern pattern = Pattern.compile("^\\d+$");
            if(pattern.matcher(title).matches()){
                queryWrapper.and(qw -> qw.like(TaskW::getTitle, title).or().eq(TaskW::getId, Integer.valueOf(title)));
            }else {
                queryWrapper.like(TaskW::getTitle, title);
            }
        }
        final List<TaskW> list = taskWService.list(queryWrapper);
        List<TaskW> newList = new ArrayList<>();
        for (TaskW record : list) {

            final List<TasklogW> list2 = tasklogWService.list(Wrappers.<TasklogW>lambdaQuery()
                    .eq(TasklogW::getTaskId, record.getId()).orderByDesc(TasklogW::getId));
            if (list2 != null && list2.size() > 0) {
                record.setCurrentTaskLog(list2.get(0));
            }
            // 返回关联任务的全路径
            Integer planId = record.getPlanId();
            String testList = record.getTestList();
            if (Objects.nonNull(planId)) {
                String planFullPath = planService.getFullPath(planId);
                if (StringUtils.isBlank(testList)) {
                    Plan plan = planService.getById(planId);
                    testList = plan.getListId().toString();
                }
                PlanList planList = planListService.getById(Integer.parseInt(testList));
                if (Objects.nonNull(planList)) {
                    if (StringUtils.isNotBlank(planFullPath)) {
                        planFullPath = org.apache.commons.lang3.StringUtils.isNotBlank(planList.getName()) ? planList.getName().concat("->").concat(planFullPath) : planFullPath;

                    }
                }
                record.setPlanFullPath(planFullPath);
            }
            String planStartDate = record.getPlanStartDate();
            String planEndDate = record.getPlanEndDate();
            Date startDate = new Date();
            if (StringUtils.isNotBlank(planStartDate)) {
                startDate = DateUtil.parse(planStartDate, "yyyy-MM-dd");
            }
            Date endDate = new Date();
            if (StringUtils.isNotBlank(planEndDate)) {
                endDate = DateUtil.parse(planEndDate, "yyyy-MM-dd");
            }
// 0 正常 1 逾期 2 待开始
            Date nowDate = new Date();
            DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
            String format = df.format(nowDate);
            nowDate = DateUtil.parse(format, "yyyy-MM-dd");
            if (endDate.before(nowDate)) {
                record.setTimeStatus(1);
            } else if (startDate.after(nowDate)) {
                record.setTimeStatus(2);
            } else {
                record.setTimeStatus(0);
            }
            newList.add(record);
        }
        return ResponseEntity.ok(newList);
    }

//    @ApiOperation("统计当前人员的待确认数据")
//    @GetMapping("/countCurrentUserTaskConfirm")
//    public ResponseEntity countCurrentUserTaskConfirm(Integer projectId, boolean isAll, String title) {
//        final String tokenUserID = UserTokenUtil.getTokenUserID();
//        List<Integer> taskIdList = tasklogWService.getTaskIdList(isAll);
//        final List<TaskW> list = taskWService.list(Wrappers.<TaskW>lambdaQuery()
//                .eq(TaskW::getDeleted, 0)
//                .like(TaskW::getConfirmId, tokenUserID)
//                .in(TaskW::getStatus, "2")
//                .eq(projectId != null, TaskW::getProjectId, projectId)
//                .in(CollectionUtil.isNotEmpty(taskIdList), TaskW::getId, taskIdList)
//                .like(StringUtils.isNotBlank(title), TaskW::getTitle, title)
//        );
//        if (CollectionUtil.isNotEmpty(list)) {
//            Map<String, Long> collect = list.stream().collect(Collectors.groupingBy(TaskW::getStatus, Collectors.counting()));
//            return ResponseEntity.ok(collect);
//        }
//        return ResponseEntity.ok(0);
//    }

    @ApiOperation("统计当前人员的待确认数据")
    @GetMapping("/countCurrentUserTaskConfirm")
    public ResponseEntity<List<TaskCountVo>> countCurrentUserTaskConfirm(Integer projectId,
                                                                         boolean isAll,
                                                                         String title,
                                                                         String urgency,
                                                                         String startDate,
                                                                         String endDate,
                                                                         String groupname,
                                                                         String planEndDate1,
                                                                         String planEndDate2) throws Exception {
        final String tokenUserID = UserTokenUtil.getTokenUserID();
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date start = null;
        Date end = null;
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            System.out.println("df = " + df);
            start = df.parse(startDate);
            end = df.parse(endDate);

        }
        LambdaQueryWrapper<TaskW> queryWrapper = Wrappers.lambdaQuery(TaskW.class)
                .eq(TaskW::getDeleted, 0)
                .like(TaskW::getConfirmId, tokenUserID)
                .eq(TaskW::getStatus, "2")
                .eq(projectId != null, TaskW::getProjectId, projectId)
                .eq(StringUtils.isNotBlank(urgency), TaskW::getUrgency, urgency)
                .eq(StringUtils.isNotBlank(groupname), TaskW::getGroupname, groupname)
                .between(Objects.nonNull(start) && Objects.nonNull(end),
                        TaskW::getCompleteTime, start, end)
                .between(StringUtils.isNotBlank(planEndDate1) && StringUtils.isNotBlank(planEndDate2),
                        TaskW::getPlanEndDate, planEndDate1, planEndDate2);
        if (StringUtils.isNotBlank(title)) {
            Pattern pattern = Pattern.compile("^\\d+$");
            if(pattern.matcher(title).matches()){
                queryWrapper.and(qw -> qw.like(TaskW::getTitle, title).or().eq(TaskW::getId, Integer.valueOf(title)));
            }else {
                queryWrapper.like(TaskW::getTitle, title);
            }
        }

        //查询出当前项目的任务
        List<TaskW> taskWList = taskWService.list(
                queryWrapper
        );
        List<TaskCountVo> taskCountVos = new ArrayList<>();
        TaskCountVo vo = new TaskCountVo();
        TaskCountVo vo1 = new TaskCountVo();
        TaskCountVo vo2 = new TaskCountVo();
        vo.setStatus("all");
        vo1.setStatus("confirmed");
        vo2.setStatus("reconfirmed");
        if (CollectionUtil.isEmpty(taskWList)) {
            vo.setCount(0);
            vo1.setCount(0);
            vo2.setCount(0);
            taskCountVos.add(vo);
            taskCountVos.add(vo1);
            taskCountVos.add(vo2);
            return ResponseEntity.ok(taskCountVos);
        }
        List<Integer> taskIdList = taskWList.stream().map(TaskW::getId).collect(Collectors.toList());
        vo.setCount(taskIdList.size());
        //统计待确认数据
        List<TasklogW> tasklogWList = tasklogWService.list(Wrappers.lambdaQuery(TasklogW.class)
                .in(TasklogW::getTaskId, taskIdList)
                .eq(TasklogW::getSourceStatus, "4"));
        if (CollectionUtil.isNotEmpty(tasklogWList)) {
            List<Integer> taskIdList2 = tasklogWList.stream().map(TasklogW::getTaskId).distinct().collect(Collectors.toList());
            vo1.setCount(taskIdList.size() - taskIdList2.size());
            vo2.setCount(taskIdList2.size());
        } else {
            vo1.setCount(taskIdList.size());
            vo2.setCount(0);
        }
        taskCountVos.add(vo);
        taskCountVos.add(vo1);
        taskCountVos.add(vo2);
        return ResponseEntity.ok(taskCountVos);
    }

    /**
     * @param projectId
     * @param status
     * @param isAll     false:初次验证 true: 再次验证
     * @return
     */
    @ApiOperation("查询当前人员的待确认数据")
    @GetMapping("/getCurrentUserTaskConfirm")
    public ResponseEntity getCurrentUserTaskConfirm(Integer projectId,
                                                    String status,
                                                    Boolean isAll,
                                                    String title,
                                                    String urgency,
                                                    String startDate,
                                                    String endDate,
                                                    String groupname,
                                                    String planEndDate1,
                                                    String planEndDate2) throws Exception {
        final String tokenUserID = UserTokenUtil.getTokenUserID();
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date start = null;
        Date end = null;
        if (StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate)) {
            start = df.parse(startDate);
            end = df.parse(endDate);
        }
        List<Integer> taskIdList = tasklogWService.getTaskIdList(isAll);
//         .like(StringUtils.isNotBlank(title), TaskW::getTitle, title)
        LambdaQueryWrapper<TaskW> queryWrapper = Wrappers.lambdaQuery(TaskW.class)
                .eq(TaskW::getDeleted, 0)
                .like(TaskW::getConfirmId, tokenUserID)
                .in(TaskW::getStatus, "2")
                .eq(projectId != null, TaskW::getProjectId, projectId)
                .eq(StringUtils.isNotBlank(status), TaskW::getStatus, status)
                .in(CollectionUtil.isNotEmpty(taskIdList), TaskW::getId, taskIdList)
                .eq(StringUtils.isNotBlank(urgency), TaskW::getUrgency, urgency)
                .eq(StringUtils.isNotBlank(groupname), TaskW::getGroupname, groupname)
                .between(Objects.nonNull(start) && Objects.nonNull(end),
                        TaskW::getCompleteTime, start, end)
                .between(StringUtils.isNotBlank(planEndDate1) && StringUtils.isNotBlank(planEndDate2),
                        TaskW::getPlanEndDate, planEndDate1, planEndDate2);
        if (StringUtils.isNotBlank(title)) {
            Pattern pattern = Pattern.compile("^\\d+$");
            if(pattern.matcher(title).matches()){
                queryWrapper.and(qw -> qw.like(TaskW::getTitle, title).or().eq(TaskW::getId, Integer.valueOf(title)));
            }else {
                queryWrapper.like(TaskW::getTitle, title);
            }
        }
        final List<TaskW> list = taskWService.list(
                queryWrapper
        );
        List<TaskW> newList = new ArrayList<>();
        for (TaskW record : list) {

            final List<TasklogW> list2 = tasklogWService.list(Wrappers.<TasklogW>lambdaQuery()
                    .eq(TasklogW::getTaskId, record.getId()).orderByDesc(TasklogW::getId));
            if (list2 != null && list2.size() > 0) {
                record.setCurrentTaskLog(list2.get(0));
            }
            // 返回关联任务的全路径
            Integer planId = record.getPlanId();
            String testList = record.getTestList();
            if (Objects.nonNull(planId)) {
                String planFullPath = planService.getFullPath(planId);
                if (StringUtils.isBlank(testList)) {
                    Plan plan = planService.getById(planId);
                    testList = plan.getListId().toString();
                }
                PlanList planList = planListService.getById(Integer.parseInt(testList));
                if (Objects.nonNull(planList)) {
                    planFullPath = org.apache.commons.lang3.StringUtils.isNotBlank(planList.getName()) ? planList.getName().concat("->").concat(planFullPath) : planFullPath;
                }
                record.setPlanFullPath(planFullPath);
            }
            newList.add(record);
        }
        return ResponseEntity.ok(newList);
    }

    @ApiOperation("查询当前人员的待处理，已驳回信息")
    @GetMapping("/getCurrentUserTaskOverProject")
    public ResponseEntity<List<ProjectVo>> getCurrentUserTaskOverProject(TaskW taskW) {
        taskW.setHeadId(UserTokenUtil.getTokenUserID());
        final List<ProjectVo> projectVoList = taskWService.taskOverProject(taskW, Arrays.asList("1", "4"));
        return ResponseEntity.ok(projectVoList);
    }

    @ApiOperation("查询当前人员的待确认信息")
    @GetMapping("/getCurrentUserConfirmTask")
    public ResponseEntity<List<ProjectVo>> getCurrentUserConfirmTask(TaskW taskW) {
        taskW.setConfirmId(UserTokenUtil.getTokenUserID());
        final List<ProjectVo> projectVoList = taskWService.taskOverProject2(taskW, "2");


        return ResponseEntity.ok(projectVoList);
    }

    @ApiOperation("查询分组信息列表")
    @GetMapping("/getGroupList")
    public ResponseEntity<List<String>> getGroupList(TaskW taskW) {
        taskW.setHeadId(UserTokenUtil.getTokenUserID());
        final List<String> distinctGroupName = taskWService.getDistinctGroupName(taskW);
        return ResponseEntity.ok(distinctGroupName);
    }

    @ApiOperation("查询项目分组信息列表")
    @GetMapping("/getGroupListByProjectId")
    public ResponseEntity<List<String>> getGroupListByProjectId(@RequestParam Integer projectId) {
        List<TaskW> TaskWList = taskWService.list(Wrappers.lambdaQuery(TaskW.class)
                .eq(TaskW::getProjectId, projectId)
                .eq(TaskW::getDeleted, 0)
        );
        List<String> distinctGroupName = TaskWList.stream()
                .map(TaskW::getGroupname)
                .filter(StringUtils::isNotBlank)
                .distinct()
                .collect(Collectors.toList());
        return ResponseEntity.ok(distinctGroupName);
    }

    @ApiOperation("根据项目id查找当前登陆人未完成的任务")
    @GetMapping("/getUnfinishedTaskByProjectId")
    public Result<List<TaskW>> getUnfinishedTaskByProjectId(@RequestParam Integer projectId) {
        String tokenUserID = UserTokenUtil.getTokenUserID();
        List<TaskW> taskWList = taskWService.list(Wrappers.lambdaQuery(TaskW.class)
                .eq(TaskW::getDeleted, 0)
                .eq(TaskW::getProjectId, projectId)
                .eq(TaskW::getHeadId, tokenUserID));
        if (CollectionUtil.isNotEmpty(taskWList)) {
            for (TaskW taskW : taskWList) {
                Integer planId = taskW.getPlanId();
                if (Objects.isNull(planId)) {
                    continue;
                }
                String testList = taskW.getTestList();
                String planFullPath = planService.getFullPath(planId);
                if (StringUtils.isBlank(testList)) {
                    Plan plan = planService.getById(planId);
                    testList = plan.getListId().toString();
                }
                PlanList planList = planListService.getById(Integer.parseInt(testList));
                if (Objects.nonNull(planList)) {
                    if (StringUtils.isNotBlank(planFullPath)) {
                        planFullPath = org.apache.commons.lang3.StringUtils.isNotBlank(planList.getName()) ? planList.getName().concat("->").concat(planFullPath) : planFullPath;

                    }
                }
                taskW.setPlanFullPath(planFullPath);
            }

        }
        return Result.success(taskWList);
    }

    @ApiOperation("甘特图")
    @PostMapping("/getGanttData")
    public Result<List<GanttData>> getGanttData(@RequestBody(required = false) GanttData data) {
        return Result.success(taskWService.getGanttData(data));
    }

    @ApiOperation("甘特图每个table页面统计")
    @PostMapping("/countGanttData")
    public Result<List<ProjectOverTaskCountVo>> countGanttData(@RequestBody(required = false) GanttData data) {
        return Result.success(taskWService.countGanttData(data));
    }

    @ApiOperation("任务导出")
    @GetMapping("/export")
    public void export(TaskW taskW) throws IOException {
        List<Integer> projectIdList = projectWService.getSelfProjectId();
        if (CollectionUtil.isEmpty(projectIdList)) {
            throw new RuntimeException("");
        }

        String title = taskW.getTitle();
        taskW.setTitle(null);
        String confirmId = taskW.getConfirmId();
        taskW.setConfirmId(null);
        Date completeTimeStart = taskW.getCompleteTimeStart();
        Date completeTimeEnd = taskW.getCompleteTimeEnd();
        LambdaQueryWrapper<TaskW> taskWLambdaQueryWrapper = Wrappers.lambdaQuery(taskW)
                .like(StringUtils.isNotBlank(title), TaskW::getTitle, title)
                .eq(TaskW::getDeleted, 0)
                .in(TaskW::getProjectId, projectIdList)
                .between(Objects.nonNull(completeTimeStart) && Objects.nonNull(completeTimeEnd), TaskW::getCompleteTime, completeTimeStart, completeTimeEnd);
        if(StringUtils.isNotBlank(confirmId)){
            taskWLambdaQueryWrapper.and(qw->qw.like(TaskW::getConfirmId,confirmId).or()
                    .like(TaskW::getAlreadyConfirmId,confirmId));
        }
        List<TaskW> resultData = taskWService.list(taskWLambdaQueryWrapper);
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        DictItemClient dictItemClient = SpringUtil.getBean(DictItemClient.class);
        List<TaskExcelModel> data = resultData.stream().map(taskW1 -> {
            TaskExcelModel taskExcelModel = new TaskExcelModel();
            taskExcelModel.setTitle(taskW1.getTitle());
            if(StringUtils.isNotBlank(taskW1.getStatus())){
                String rwzt = dictItemClient.getDicItemText("RWZT", taskW1.getStatus());
                taskExcelModel.setStatus(rwzt);
            }
            if(StringUtils.isNotBlank(taskW1.getType())){
                String type = dictItemClient.getDicItemText("TASKTYPE",taskW1.getType());
                taskExcelModel.setTaskType(type);
            }
            UserVo userInfo = userClient.getUserInfo(taskW1.getHeadId());
            taskExcelModel.setHeadUser(userInfo.getRealName());
            ProjectW project = projectWService.getById(taskW1.getProjectId());
            taskExcelModel.setProjectName(project.getName());
            return taskExcelModel;

        }).collect(Collectors.toList());

        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = URLEncoder.encode("任务", "UTF-8").replaceAll("\\+", "%20");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
        EasyExcel.write(response.getOutputStream(), TaskExcelModel.class)
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .sheet("任务").doWrite(data);
    }

}