package com.tofly.pms.working.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.pms.file.constant.FileUploadTypeEnum;
import com.tofly.pms.file.entity.File;
import com.tofly.pms.file.service.FileService;
import com.tofly.pms.working.entity.TaskW;
import com.tofly.pms.working.entity.TasklogW;
import com.tofly.pms.working.service.TasklogWService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

 /**
 * 任务日志表;(TF_PM_TASKLOG_W)表控制层
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Api(tags = "任务日志表对象功能接口")
@RestController
@RequestMapping("/tfPmTasklogW")
public class TasklogWController{
    @Autowired
    private TasklogWService tasklogWService;
    @Autowired
    private FileService fileService;
    
    /** 
     * 通过ID查询单条数据 
     *
     * @param id 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("{id}")
    public ResponseEntity<TasklogW> queryById(Integer id){
        final TasklogW tasklogW = tasklogWService.queryById(id);
        final List<File> list = fileService.list(Wrappers.<File>lambdaQuery()
                .eq(File::getTableId, id)
                .eq(File::getTableName, FileUploadTypeEnum.TASK.getTableName())
                .orderByDesc(File::getCreateTime)
        );
        tasklogW.setFileList(list);
        return ResponseEntity.ok(tasklogW);
    }
    
    /** 
     * 分页查询
     *
     * @param tasklogW 筛选条件
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping
    public ResponseEntity<Page<TasklogW>> paginQuery(TasklogW tasklogW, Page<TasklogW> page){
        final Page<TasklogW> page1 = tasklogWService.page(page, new QueryWrapper<>(tasklogW)
                .lambda()
                .orderByAsc(TasklogW::getCreateDate));
        List<TasklogW> records = page1.getRecords();
        if(CollectionUtil.isNotEmpty(records)){
            for (TasklogW record : records) {
                record.setTask(JSON.parseObject(record.getJsonData(), TaskW.class));
//                record.getTask().setProcessingModeAnnotation();
//                record.setProcessingModeAnnotation();
            }
        }
        return ResponseEntity.ok(page1);
    }
    
    /** 
     * 新增数据
     *
     * @param tasklogW 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping
    public ResponseEntity<TasklogW> add(@RequestBody TasklogW tasklogW){
        throw new RuntimeException("不能走接口生成日志");
//        tasklogWService.save(tasklogW);
//        return ResponseEntity.ok(tasklogW);
    }
    
    /** 
     * 更新数据
     *
     * @param tasklogW 实例对象
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping
    public ResponseEntity<TasklogW> edit(@RequestBody TasklogW tasklogW){
        throw new RuntimeException("不能走接口更新日志");
    }
    
    /** 
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping
    public ResponseEntity<Boolean> deleteById(Integer id){
        throw new RuntimeException("不能走接口通过主键删除数据");
    }
}