package com.tofly.pms.working.controller;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.tofly.pms.base.controller.Result;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.dto.WorkingreportDto;
import com.tofly.pms.working.entity.Workingreport;
import com.tofly.pms.working.service.WorkingreportService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author sheldon
 * @Description
 * @create 2024-04-16 10:14
 */
@Api(tags = "反馈信息")
@RestController
@RequestMapping("/workingReport")
public class WorkingreportController {
    @Resource
    private WorkingreportService workingreportService;


    @ApiOperation("审核数据")
    @PostMapping("saveBatch")
    public Result<Boolean> saveOrUpdateBatch(@RequestBody WorkingreportDto workingreportDto) {
        List<Workingreport> data = workingreportDto.getWorkingreportList();
        if (CollectionUtil.isNotEmpty(data)) {
            data = data.stream().filter(item ->
                    StringUtils.isBlank(item.getStatus()) || Objects.isNull(item.getId())
            ).collect(Collectors.toList());
            List<String> deptIdList = data.stream().map(Workingreport::getDeptId).distinct().filter(StringUtils::isNotBlank).collect(Collectors.toList());
            String userID = UserTokenUtil.getTokenUserID();
            UserClient userClient = SpringUtil.getBean(UserClient.class);

            UserVo userInfo = userClient.getUserInfo(userID);
            Set<String> orgIdList = userInfo.getDepartments();
            if (CollectionUtil.isNotEmpty(orgIdList) && orgIdList.containsAll(deptIdList)) {
                for (Workingreport datum : data) {
                    datum.setId(IdUtil.getSnowflakeNextId());
                    datum.setStatus("已审核");
                }
                boolean b = workingreportService.saveBatch(data);
                if(b){
                    return Result.success(true);
                }else {
                  return Result.fail("500", "审核失败");
                }

            } else {
                return Result.fail("304", "没有权限");
            }
        }
        return Result.fail("500", "当前没有要审核的数据");
    }

    @ApiOperation("工时月报统计")
    @GetMapping("/statistics")
    public Result<List<Workingreport>> statistics(
            @RequestParam(value = "projectId", required = false) Integer projectId,
            @RequestParam(value = "deptId", required = false) String deptId,
            @RequestParam(value = "userId", required = false) String userId,
            @RequestParam(value = "month") String month

    ) {
        return Result.success(workingreportService.statistics(projectId, deptId, userId, month));
    }
    @ApiOperation("工时月报统计导出")
    @GetMapping("/export")
    public void export(
            @RequestParam(value = "projectId", required = false) Integer projectId,
            @RequestParam(value = "deptId", required = false) String deptId,
            @RequestParam(value = "userId", required = false) String userId,
            @RequestParam(value = "month") String month

    ) throws IOException {
        workingreportService.export(projectId, deptId, userId, month);
    }
}
