package com.tofly.pms.working.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.file.entity.File;
import com.tofly.pms.file.service.FileService;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.service.ProjectDevWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.utils.dict.Dict;
import com.tofly.pms.utils.vocast.VoCast;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.*;
import java.util.Date;

@ApiModel(value = "反馈信息", description = "")
@TableName(value = "tf_pm_feedback_w", schema = "business")
@Data
public class Feedback implements Serializable{
    /**
     * id
     */
    @ApiModelProperty(value = "id", name = "id")
    @TableId(type = IdType.AUTO)
    private Integer id;


    @ApiModelProperty(value = "意见类别 VIEWTYPE")
    @Dict(code = "VIEWTYPE")
    private String viewType;

    @ApiModelProperty(value = "反馈紧急程度 VIEWURGENCY")
    @Dict(code = "VIEWURGENCY")
    private String urgency;

    @ApiModelProperty(value = "意见类型 VIEWTYLE")
    @Dict(code = "VIEWTYLE")
    private String viewTyle;
    @ApiModelProperty(value = "项目")
    @VoCast(serviceClass = ProjectWService.class, valueClass = ProjectW.class)
    private Integer projectId;
    @ApiModelProperty(value = "系统")
    private String systemName;
    @ApiModelProperty(value = "模型")
    private String modelName;
    @ApiModelProperty(value = "标题")
    private String title;
    @ApiModelProperty(value = "描述")
    private String description;

    @ApiModelProperty(value = "反馈部门")
    private String dept;

    @ApiModelProperty(value = "反馈人员")
    private String feedUser;

    @ApiModelProperty(value = "图片")
    @VoCast(serviceClass = FileService.class, isMulti = true, valueClass = File.class)
    private String fileUrl;

    @ApiModelProperty(value = "是否下发任务")
    @TableField(exist = false)
    private Boolean isDispatchTask;
    @TableField(fill = FieldFill.INSERT)
    @ApiModelProperty(value = "删除")
    private Integer deleted;


    /**
     * 创建人标识
     */
    @ApiModelProperty(value = "创建人标识")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = true, valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT)
    private String createUser;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "create_date", fill = FieldFill.INSERT)
    private Date createDate;

    @ApiModelProperty(value = "提出时间")
    private String feedTime;
    @ApiModelProperty(value = "处理方案")
    private String handlePlan;
    @ApiModelProperty(value = "处理描述")
    private String resolverDescription;
    @ApiModelProperty(value = "解决时间")
    private String resolveDate;
    @ApiModelProperty(value = "处理人")
    private String resolverPerson;


    @Override
    public Feedback clone() {
        try (ByteArrayOutputStream bos = new ByteArrayOutputStream();
             ObjectOutput oos = new ObjectOutputStream(bos)) {
            oos.writeObject(this);
            oos.flush();
            byte[] byteArray = bos.toByteArray();
            try (ByteArrayInputStream bis = new ByteArrayInputStream(byteArray);
                 ObjectInputStream ois = new ObjectInputStream(bis)) {
                return (Feedback) ois.readObject();
            }
        } catch (Exception e) {
            throw new RuntimeException("对象克隆失败", e);
        }
    }

}
