package com.tofly.pms.working.entity;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.tofly.pms.fegin.DictItemClient;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sheldon
 * @Description 意见批量导入的模型
 * @create 2024-04-01 10:48
 */
@Data
public class FeedbackExcelModel {
    @ExcelProperty(value = "序号", index = 0)
    @ApiModelProperty(value = "序号")
    private Integer number;

    @ExcelProperty(value = "系统名称", index =1)
    @ApiModelProperty(value = "系统")
    private String systemName;
    @ExcelProperty(value = "功能名称", index =2)
    @ApiModelProperty(value = "模块")
    private String modelName;
    @ExcelProperty(value = "意见标题", index = 3)
    @ApiModelProperty(value = "意见标题")
    private String title;
    @ExcelProperty(value = "意见描述", index = 4)
    @ApiModelProperty(value = "描述")
    private String description;
    @ExcelProperty(value = "意见类型", index = 5)
    @ApiModelProperty(value = "意见类型 VIEWTYLE")
    private String viewTyle;
    @ExcelProperty(value = "紧急程度", index = 6)
    @ApiModelProperty(value = "反馈紧急程度 VIEWURGENCY")
    private String urgency;
    @ExcelProperty(value = "意见类别", index = 7)
    @ApiModelProperty(value = "意见类别 VIEWTYPE")
    private String viewType;
    @ExcelProperty(value = "反馈单位", index = 8)
    @ApiModelProperty(value = "反馈单位")
    private String dept;
    @ExcelProperty(value = "反馈人员", index = 9)
    @ApiModelProperty(value = "反馈人")
    private String feedUser;
    @ExcelProperty(value = "提出时间", index = 10)
    @ApiModelProperty(value = "提出时间")
    private String feedTime;
    @ExcelProperty(value = "处理方案", index = 11)
    @ApiModelProperty(value = "处理方案")
    private String handlePlan;
    @ExcelProperty(value = "处理说明", index = 12)
    @ApiModelProperty(value = "处理描述")
    private String resolverDescription;
    @ExcelProperty(value = "计划完成日期", index = 13)
    @ApiModelProperty(value = "解决时间")
    private String resolveDate;
    @ExcelProperty(value = "负责人", index = 14)
    @ApiModelProperty(value = "处理人")
    private String resolverPerson;

    public Feedback toFeedback() {
        Feedback feedback = new Feedback();
        DictItemClient bean = SpringUtil.getBean(DictItemClient.class);
        feedback.setViewTyle(bean.getCodeByValue(this.viewTyle.trim(), "VIEWTYLE"));//
        feedback.setUrgency(bean.getCodeByValue(this.urgency.trim(), "VIEWURGENCY"));
        feedback.setViewType(bean.getCodeByValue(this.viewType.trim(), "VIEWTYPE"));
        feedback.setTitle(this.title);
        feedback.setDept(this.dept);
        feedback.setFeedUser(this.feedUser);
        feedback.setSystemName(this.systemName);
        feedback.setModelName(this.modelName);
        feedback.setFeedTime(this.feedTime);
        feedback.setHandlePlan(this.handlePlan);
        feedback.setResolverDescription(this.resolverDescription);
        feedback.setResolveDate(this.resolveDate);
        if(StringUtils.isNotBlank(this.resolverPerson)){
            UserClient userClient = SpringUtil.getBean(UserClient.class);
            String[] split = this.resolverPerson.split("[,，、]");
            List<UserVo> userVoList = userClient.getUserIdByRealName(Arrays.asList(split));
            if(CollectionUtil.isNotEmpty(userVoList)){
                String userIdStr = userVoList.stream().map(UserVo::getId).collect(Collectors.joining(","));
                feedback.setResolverPerson(userIdStr);
            }
        }
        if (StringUtils.isNotBlank(this.description)) {
            feedback.setDescription("<p>".concat(this.description).concat("<p/>"));
        }
        return feedback;
    }
}
