package com.tofly.pms.working.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.service.ProjectDevWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.utils.dict.Dict;
import com.tofly.pms.utils.vocast.VoCast;
import com.tofly.pms.working.service.TaskWService;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;

/**
 * 工时登记表;
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@ApiModel(value = "工时登记表", description = "")
@TableName(value = "TF_PM_WORKINGHOURS_W", schema = "business")
@Data
public class HoursW implements Serializable, Cloneable {
    /**
     * id
     */
    @ApiModelProperty(value = "id", name = "id")
    @TableId(type = IdType.AUTO)
    private Integer id;
    /**
     * 项目id
     */
    @ApiModelProperty(value = "项目id", name = "projectId")
    @VoCast(serviceClass = ProjectWService.class, valueClass = ProjectW.class)
    private Integer projectId;
    @TableField(exist = false)
    private ProjectW projectW;
    /**
     * 任务id
     */
    @ApiModelProperty(value = "任务id", name = "taskIds")
    @VoCast(serviceClass = TaskWService.class, isMulti = true, valueClass = TaskW.class)
    private String taskIds;

    /**
     * 工作日期  yyyy-mm-dd
     */
    @ApiModelProperty(value = "工作日期  yyyy-mm-dd", name = "workDate")
    private String workDate;
    /**
     * 工作时长(小时)
     */
    @ApiModelProperty(value = "工作时长(小时)", name = "duration")
    private Integer duration;
    /**
     * 工作描述
     */
    @ApiModelProperty(value = "工作描述", name = "description")
    private String description;
    /**
     * 登记人ID
     */
    @ApiModelProperty(value = "登记人ID", name = "registerId")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = false, valueClass = UserVo.class)
    private String registerId;
    /**
     * 登记类型
     */
    @ApiModelProperty(value = "登记类型 GSDJLX", name = "registerType")
    @Dict(code = "GSDJLX")
    private String registerType;
    /**
     * 创建人ID
     */
    @ApiModelProperty(value = "创建人ID", name = "createUser")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = false, valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT)
    private String createUser;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间", name = "createDate")
    @TableField(fill = FieldFill.INSERT)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createDate;
    /**
     * 更新人id
     */
    @ApiModelProperty(value = "更新人id", name = "updateUser")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateUser;
    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间", name = "updateTime")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;


}