package com.tofly.pms.working.entity;

import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.pms.fegin.DictItemClient;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.utils.dict.Dict;
import com.tofly.pms.utils.vocast.VoCast;
import com.tofly.pms.working.service.PlanService;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.util.Date;
import java.util.Objects;

/**
 * 项目计划信息表;
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@ApiModel(value = "项目计划信息表", description = "")
@TableName(value = "TF_PM_PLAN_W", schema = "business")
@Data
public class Plan implements Serializable, Cloneable {
    /**
     * id
     */
    @ApiModelProperty(value = "id", name = "id")
    @TableId(type = IdType.INPUT)
    private Integer id;

    /**
     * 计划列表id
     */
    @ApiModelProperty(value = "listId", name = "listId")
    private Integer listId;
    /**
     * 项目id
     */
    @ApiModelProperty(value = "项目id", name = "projectId")
    @VoCast(serviceClass = ProjectWService.class, valueClass = ProjectW.class)
    private Integer projectId;
    /**
     * 计划分组id
     */
    @ApiModelProperty(value = "计划分组id", name = "groupId")
    private Integer groupId;
    /**
     * 计划唯一编码
     */
    @ApiModelProperty(value = "计划唯一编码", name = "code")
    private String code;
    /**
     * 计划名称
     */
    @ApiModelProperty(value = "计划名称", name = "name")
    private String name;
    /**
     * 计划级别
     */
    @ApiModelProperty(value = "计划级别", name = "level")
    private Integer level;
    /**
     * 父计划
     */
    @ApiModelProperty(value = "父计划", name = "parentId")
    private Integer parentId;

    @ApiModelProperty(value = "排序", name = "sort")
    private Integer sort;
    /**
     * 计划描述
     */
    @ApiModelProperty(value = "工作描述", name = "description")
    private String description;

    /**
     * 注意事项
     */
    @ApiModelProperty(value = "注意事项", name = "note")
    private String note;
    /**
     * 需求来源
     */
    @ApiModelProperty(value = "需求来源 XQLY", name = "source")
    @Dict(code = "XQLY")
    private String source;
    /**
     * 是否里程碑
     */
    @ApiModelProperty(value = "是否里程碑", name = "isMilestone")
    private String isMilestone;
    /**
     * 预计工作量（小时）;单位：人天
     */
    @ApiModelProperty(value = "预计工作量 单位：人天", name = "predictWorkload")
    private Double predictWorkload;
    /**
     * 计划开始时间
     */
    @ApiModelProperty(value = "计划开始时间", name = "planStartTime")
    private String planStartTime;
    /**
     * 计划结束时间
     */
    @ApiModelProperty(value = "计划结束时间", name = "planEndDate")
    private String planEndDate;
    /**
     * 负责人ID
     */
    @ApiModelProperty(value = "负责人ID", name = "headId")
    @VoCast(serviceClass = UserClient.class, isMulti = true, valueClass = UserVo.class)
    private String headId;

    /**
     * 更新人id
     */
    @ApiModelProperty(value = "更新人id", name = "updateUser")
    @VoCast(serviceClass = UserClient.class, isMulti = false, valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateUser;
    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间", name = "updateTime")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;


    /**
     * 更新人id
     */
    @ApiModelProperty(value = "创建人id", name = "createUser")
    @VoCast(serviceClass = UserClient.class, isMulti = false, valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT)
    private String createUser;
    /**
     * 更新时间
     */
    @ApiModelProperty(value = "创建时间", name = "createDate")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT)
    private Date createDate;
    @ApiModelProperty(value = "关联的任务总数", name = "totalTask")
    @TableField(exist = false)
    private Long totalTask;
    @ApiModelProperty(value = "完成的任务数量", name = "completeTask")
    @TableField(exist = false)
    private Long completeTask;
    @ApiModelProperty(value = "序号", name = "number")
    private String number;
    @ApiModelProperty(value = "是否派发", name = "status")
    private Integer status;

    public PlanExcelModel toExcelModel() {
        PlanExcelModel planExcelModel = new PlanExcelModel();

        planExcelModel.setId(this.getId());
        planExcelModel.setNumber(this.getNumber());
        planExcelModel.setName(this.getName());
        planExcelModel.setDescription(this.getDescription());
        planExcelModel.setNote(this.getNote());
        planExcelModel.setPlanStartTime(this.getPlanStartTime());
        planExcelModel.setPlanEndDate(this.getPlanEndDate());
        planExcelModel.setLevel(this.getLevel());
        if (StringUtils.isNotBlank(this.getIsMilestone()) && "2".equals(this.getIsMilestone())) {
            planExcelModel.setIsMilestone("是");
        } else {
            planExcelModel.setIsMilestone("否");
        }
        planExcelModel.setPredictWorkload(this.getPredictWorkload());


        PlanService planService = SpringUtil.getBean(PlanService.class);
        Plan plan = planService.getById(this.getParentId());
        if (Objects.nonNull(plan)) {
            planExcelModel.setParentName(plan.getName());
        }
        if (StringUtils.isNotBlank(this.getSource())) {
            DictItemClient dictItemClient = SpringUtil.getBean(DictItemClient.class);
            String source = dictItemClient.getDicItemText("XQLY", this.getSource());
            planExcelModel.setSource(source);
        }
        return planExcelModel;
    }
}