package com.tofly.pms.working.entity;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.HeadStyle;
import com.alibaba.excel.enums.poi.FillPatternTypeEnum;
import com.tofly.pms.fegin.DictItemClient;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import lombok.Data;
import org.apache.commons.lang3.StringUtils;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sheldon
 * @Description
 * @create 2024-04-02 10:22
 */
@Data
public class PlanExcelModel {
        @HeadStyle(fillPatternType = FillPatternTypeEnum.SOLID_FOREGROUND, fillForegroundColor = 14)
    @ExcelProperty(value = "序号", index = 0)
    private String number;

    /**
     * 计划名称
     */
    @HeadStyle(fillPatternType = FillPatternTypeEnum.SOLID_FOREGROUND, fillForegroundColor = 14)
    @ExcelProperty(value = "工作项名称", index = 1)
    private String name;

    /**
     * 计划描述
     */
    @ExcelProperty(value = "工作描述", index = 2)
    private String description;

    /**
     * 注意事项
     */
    @ExcelProperty(value = "备注", index = 3)
    private String note;
    /**
     * 需求来源  背景设置成粉红
     */
    @HeadStyle(fillPatternType = FillPatternTypeEnum.SOLID_FOREGROUND, fillForegroundColor = 14)
    @ExcelProperty(value = "需求来源", index = 4)
    private String source;
    /**
     * 是否里程碑
     */
    @ExcelProperty(value = "是否里程碑", index = 5)
    private String isMilestone;
    /**
     * 预计工作量（小时）;单位：人天
     */
    @ExcelProperty(value = "预计工作量", index = 6)
    private Double predictWorkload;
    /**
     * 计划开始时间
     */
    @ExcelProperty(value = "计划开始时间", index = 7)
    private String planStartTime;
    /**
     * 计划结束时间
     */
    @ExcelProperty(value = "计划结束时间", index = 8)
    private String planEndDate;
    /**
     * 负责人ID
     */
    @ExcelProperty(value = "负责人", index = 9)
    private String headName;
    @ExcelProperty(value = "编号(系统维护)", index = 10)
    private Integer id;
    /**
     * 级别
     */
    @ExcelProperty(value = "级别(系统维护)", index = 11)
    private Integer level;
    /**
     * 父计划
     */
    @ExcelProperty(value = "上级工作项(系统维护)", index = 12)
    private String parentName;

    public Plan toPlan() {
        Plan plan = new Plan();
        plan.setId(this.id);
        plan.setNumber(this.number);
        if (StringUtils.isNotBlank(this.number)) {
            plan.setLevel(this.number.split("\\.").length);
        }
        plan.setName(this.name);
        plan.setDescription(this.description);
        plan.setNote(this.note);
        if (StringUtils.isNotBlank(this.isMilestone) && "是".equals(this.isMilestone)) {
            plan.setIsMilestone("2");
        } else {
            plan.setIsMilestone("1");
        }
        plan.setPredictWorkload(this.predictWorkload);
        plan.setPlanStartTime(this.planStartTime);
        plan.setPlanEndDate(this.planEndDate);
        if (StringUtils.isNotBlank(this.headName)) {
            UserClient userClient = SpringUtil.getBean(UserClient.class);
            String[] names = this.headName.split("，");
            if (names.length < 2) {
                names = this.headName.split(",");
            }
            List<String> nameList = Arrays.asList(names);
            List<UserVo> userList = userClient.getUserIdByRealName(nameList);
            if (CollectionUtil.isNotEmpty(userList)) {
                String userIds = userList.stream().map(UserVo::getId).collect(Collectors.joining(","));
                plan.setHeadId(userIds);
            }
        }
        if (StringUtils.isNotBlank(this.source)) {
            DictItemClient dictItemClient = SpringUtil.getBean(DictItemClient.class);
            String source = dictItemClient.getCodeByValue(this.source, "XQLY");
            plan.setSource(source);
        }
        return plan;
    }
}
