package com.tofly.pms.working.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.file.entity.File;
import com.tofly.pms.file.service.FileService;
import com.tofly.pms.project.entity.ProjectW;
import com.tofly.pms.project.service.ProjectDevWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.utils.dict.Dict;
import com.tofly.pms.utils.vocast.VoCast;
import com.tofly.pms.working.service.PlanService;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.*;
import java.util.Date;

/**
 * 任务信息表;
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Slf4j
@ApiModel(value = "任务信息表", description = "")
@TableName(value = "TF_PM_TASK_W", schema = "business")
@Data
public class TaskW implements Serializable, Cloneable {
    /**
     * id
     */
    @ApiModelProperty(value = "id", name = "")
    @TableId(type = IdType.AUTO)
    private Integer id;
    /**
     * 项目id
     */
    @ApiModelProperty(value = "项目id", name = "projectId")
    @VoCast(serviceClass = ProjectWService.class, valueClass = ProjectW.class)
    private Integer projectId;
    /**
     * 关联工作项唯一编码;挂架计划
     */
    @ApiModelProperty(value = "关联计划")
    @VoCast(serviceClass = PlanService.class, isMulti = true, valueClass = Plan.class)
    private Integer planId;

    @ApiModelProperty(value = "关联计划全路径")
    @TableField(exist = false)
    private String planFullPath;
    /**
     * 任务唯一编码
     */
    @ApiModelProperty(value = "任务唯一编码", name = "")
    private String taskCode;
    /**
     * 任务标题
     */
    @ApiModelProperty(value = "任务标题", name = "")
    private String title;
    /**
     * 任务类型
     */
    @ApiModelProperty(value = "任务类型", name = "")
    @Dict(code = "TASKTYPE")
    private String type;
    /**
     * 任务分组
     */
    @ApiModelProperty(value = "任务分组", name = "")
    private String groupname;
    /**
     * 任务标签
     */
    @ApiModelProperty(value = "任务标签", name = "")
    private String label;
    /**
     * 任务描述
     */
    @ApiModelProperty(value = "任务描述", name = "")
    private String descroption;
    /**
     * 图片;图片存储信息
     */
    @ApiModelProperty(value = "图片")
    @VoCast(serviceClass = FileService.class, isMulti = true, valueClass = File.class)
    private String picture;
    /**
     * 附件;附件存储信息
     */
    @ApiModelProperty(value = "附件")
    @VoCast(serviceClass = FileService.class, isMulti = true, valueClass = File.class)
    private String fileUrl;

    /**
     * 预计工作量;单位：人天
     */
    @ApiModelProperty(value = "预计工作量 单位：人天")
    private Double predictWorkload;
    /**
     * 计划开始时间
     */
    @ApiModelProperty(value = "计划开始时间")
    private String planStartDate;
    /**
     * 计划结束时间
     */
    @ApiModelProperty(value = "计划结束时间")
    private String planEndDate;
    /**
     * 负责人ID
     */
    @ApiModelProperty(value = "负责人ID")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = true, valueClass = UserVo.class)
    private String headId;
    /**
     * 确认人ID
     */
    @ApiModelProperty(value = "确认人ID")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = true, valueClass = UserVo.class)
    private String confirmId;
    /**
     * 确认人ID
     */
    @ApiModelProperty(value = "已经确认人ID")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = true, valueClass = UserVo.class)
    private String alreadyConfirmId;
    /**
     * 关注人ID
     */
    @ApiModelProperty(value = "关注人ID")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = true, valueClass = UserVo.class)
    private String careId;
    /**
     * 创建人标识
     */
    @ApiModelProperty(value = "创建人标识")
    @VoCast(serviceClass = ProjectDevWService.class, isMulti = true, valueClass = UserVo.class)
    @TableField(fill = FieldFill.INSERT)
    private String createUser;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "create_date", fill = FieldFill.INSERT)
    private Date createDate;
    /**
     * 任务状态
     */
    @ApiModelProperty(value = "任务状态 RWZT")
    @Dict(code = "RWZT")
    private String status;

    @ApiModelProperty(value = "任务处理方式 RCLFS 或者 验证结果 RYZJG", name = "processingMode")
    @Dict(code = "RCLFS")
    private String processingMode;


    /**
     * 紧急程度
     */
    @ApiModelProperty(value = "紧急程度 JJCD", name = "urgency")
    @Dict(code = "JJCD")
    private String urgency;
    /**
     * 测试清单
     */
    @ApiModelProperty(value = "测试清单", name = "testList")
    private String testList;

    /**
     * 完成时间  即状态变更为已完成的时候的操作时间
     */
    @ApiModelProperty(value = "完成时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date completeTime;
    @TableField(exist = false)
    @ApiModelProperty(value = "完成时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date completeTimeStart;
    @TableField(exist = false)
    @ApiModelProperty(value = "完成时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date completeTimeEnd;
    /**
     * 累计投入时长(小时)
     */
    @ApiModelProperty(value = "累计投入时长(人天)")
    private Double accureTime;
    @ApiModelProperty(value = "是否逾期 1是 0否")
    //结束时间 <当前时间
    private Integer timeStatus;

    @TableField(exist = false)
    private TasklogW currentTaskLog;


    @ApiModelProperty(value = "耗费时间  单位 小时")
    private Double time;
    @ApiModelProperty(value = "处理说明")
    private String description;

    @ApiModelProperty(value = "逻辑删除：1 已经删除；0 未删除")
    @TableField(value = "deleted", fill = FieldFill.INSERT)
    private Integer deleted;
    @ApiModelProperty(value = "意见id")
    private Integer feedbackId;
    @ApiModelProperty(value = "是否正在处理：0：未处理，1：正在处理")
    private Integer processing;

    public TaskW deepClone() throws Exception {
        try (ByteArrayOutputStream baos = new ByteArrayOutputStream();
             ObjectOutputStream oos = new ObjectOutputStream(baos)) {
            // 序列化当前对象
            oos.writeObject(this);
            try (ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(baos.toByteArray()))) {
                // 反序列化得到一个新的对象实例
                return (TaskW) ois.readObject();
            }
        }
    }



//public void setProcessingModeAnnotation() {
//    try {
//        Field field = this.getClass().getDeclaredField("processingMode");
//        Dict annotation = field.getAnnotation(Dict.class);
//        InvocationHandler invocationHandler = Proxy.getInvocationHandler(annotation);
//        Field hf = invocationHandler.getClass().getDeclaredField("memberValues");
//        hf.setAccessible(true);
//        Map<String, Object> memberValues = (Map<String, Object>) hf.get(invocationHandler);
//        if ("4".equals(this.status)) {
//            memberValues.put("code", "RYZJG");
//            hf.setAccessible(false);
//        } else {
//            memberValues.put("code", "RCLFS");
//            hf.setAccessible(false);
//        }
//    } catch (Exception e) {
//        log.error(e.getMessage());
//    }
//}
//    public String getTimeStatus() {
//         int compare=0;
//        if(planEndDate==null){
//        return  "正常";
//        }
//         final Date date = DateUtils.parseDate(planEndDate);
//
//         final DateTime dateTime = DateUtil.endOfDay(date);
//         if(completeTime==null){
//              compare = DateUtil.compare(new Date(), dateTime);
//         }else{
//             compare =  DateUtil.compare(completeTime, dateTime);
//         }
//
//         if(compare>0){
//             return  "超时";
//         }else{
//             return  "正常";
//         }
//
//     }
}