package com.tofly.pms.working.listener;

import cn.hutool.core.lang.Assert;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.exception.ExcelDataConvertException;
import com.alibaba.fastjson.JSON;
import com.tofly.pms.utils.ImportExcelListener;
import com.tofly.pms.working.entity.FeedbackExcelModel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.Map;

/**
 * @author sheldon
 * @Description
 * @create 2024-04-01 11:12
 */
@Slf4j
public class FeedBackListener extends ImportExcelListener<FeedbackExcelModel> {
    public FeedBackListener(Integer headRowNumber) {
        super(headRowNumber);
    }


    @Override
    public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
        log.debug("解析到一条头数据:{}", JSON.toJSONString(headMap));
    }

    @Override
    public void onException(Exception exception, AnalysisContext context) {
        log.error("解析失败:{}", exception.getMessage());
        // 如果是某一个单元格的转换异常 能获取到具体行号
        // 如果要获取头的信息 配合invokeHeadMap使用
        if (exception instanceof ExcelDataConvertException) {
            ExcelDataConvertException excelDataConvertException = (ExcelDataConvertException) exception;
            throw new RuntimeException(
                    "第" + excelDataConvertException.getRowIndex() + "行，第" + excelDataConvertException.getColumnIndex() + "列解析异常"
            );
        }
        throw new RuntimeException(exception.getMessage());

    }

    @Override
    public void invoke(FeedbackExcelModel data, AnalysisContext context) {
        if (isLineNullValue(data)) {
            //读取到空白行，就停止解析数据
            return;
        }
        Assert.notBlank(data.getUrgency(), "紧急程度不能为空");
        Assert.notNull(data.getViewTyle(), "意见类别不能为空");
        Assert.notNull(data.getViewTyle(), "意见类型不能为空");
        Assert.notNull(data.getTitle(), "标题不能为空");
        Assert.notNull(data.getFeedUser(), "反馈人不能为空");
        Assert.notNull(data.getDept(), "反馈单位不能为空");
        String feedTime = data.getFeedTime();
        if (StringUtils.isNotBlank(feedTime)) {
            data.setFeedTime(feedTime
                    .replaceAll("/", "-")
                    .replaceAll("-([1-9]-)", "-0$1")
                    .replaceAll("-([1-9]$)", "-0$1"));
        }
        String resolveDate = data.getResolveDate();
        if (StringUtils.isNotBlank(resolveDate)) {
            data.setResolveDate(resolveDate
                    .replaceAll("/", "-")
                    .replaceAll("-([1-9]-)", "-0$1")
                    .replaceAll("-([1-9]$)", "-0$1"));
        }
        super.invoke(data, context);
    }
//    private void saveImageToFile(InputStream inputStream) {
//        String fileName = "test.jpg";
//        try (
//             OutputStream outputStream = new FileOutputStream(fileName)
//        ) {
//            // 文件保存逻辑
//            byte[] buffer = new byte[4096];
//            int bytesRead;
//
//            // 循环读取输入流直到结束
//            while ((bytesRead = inputStream.read(buffer)) != -1) {
//                // 将缓冲区内容写入到输出流
//                outputStream.write(buffer, 0, bytesRead);
//            }
//
//        } catch (IOException e) {
//            // 异常处理逻辑
//        }
//    }

}
