package com.tofly.pms.working.listener;

import cn.hutool.core.lang.Assert;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.exception.ExcelDataConvertException;
import com.alibaba.fastjson.JSON;
import com.tofly.pms.utils.ImportExcelListener;
import com.tofly.pms.working.entity.PlanExcelModel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import java.util.Map;

/**
 * @author sheldon
 * @Description
 * @create 2024-04-02 15:13
 */
@Slf4j
public class PlanListener  extends ImportExcelListener<PlanExcelModel> {
    public PlanListener(Integer headRowNumber) {
        super(headRowNumber);
    }

    @Override
    public void invokeHeadMap(Map<Integer, String> headMap, AnalysisContext context) {
        log.debug("解析到一条头数据:{}", JSON.toJSONString(headMap));
    }

    @Override
    public void onException(Exception exception, AnalysisContext context) {
        log.error("解析失败:{}", exception.getMessage());
        // 如果是某一个单元格的转换异常 能获取到具体行号
        // 如果要获取头的信息 配合invokeHeadMap使用
        if (exception instanceof ExcelDataConvertException) {
            ExcelDataConvertException excelDataConvertException = (ExcelDataConvertException) exception;
            throw new RuntimeException(
                    "第" + excelDataConvertException.getRowIndex() + "行，第" + excelDataConvertException.getColumnIndex() + "列解析异常"
            );
        }
        throw new RuntimeException(exception.getMessage());

    }

    @Override
    public void invoke(PlanExcelModel data, AnalysisContext context) {
        if (isLineNullValue(data)) {
            //读取到空白行，就停止解析数据
            return;
        }
        Assert.notBlank(data.getNumber(),"序号不能为空");
        String planStartTime = data.getPlanStartTime();
        String planEndDate = data.getPlanEndDate();
        if(StringUtils.isNotBlank(planStartTime)){
            data.setPlanStartTime(planStartTime
                    .replaceAll("/","-")
                    .replaceAll("-([1-9]-)", "-0$1")
                    .replaceAll("-([1-9]$)", "-0$1"));

        }
        if(StringUtils.isNotBlank(planEndDate)){
            data.setPlanEndDate(planEndDate
                    .replaceAll("/","-")
                    .replaceAll("-([1-9]-)", "-0$1")
                    .replaceAll("-([1-9]$)", "-0$1"));
        }
        super.invoke(data, context);
    }
}
