package com.tofly.pms.working.mapper;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.tofly.pms.working.entity.HoursW;
import com.tofly.pms.working.entity.Workingreport;
import com.tofly.pms.working.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.time.LocalDate;
import java.util.List;
import java.util.Set;

/**
 * 工时登记表;(TF_PM_WORKINGHOURS_W)表数据库访问层
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Mapper
public interface HoursWMapper extends BaseMapper<HoursW> {
    /**
     * 分页查询指定行数据
     *
     * @param page    分页参数
     * @param wrapper 动态查询条件
     * @return 分页对象列表
     */
    IPage<HoursW> selectByPage(IPage<HoursW> page, @Param(Constants.WRAPPER) Wrapper<HoursW> wrapper);

    /**
     * 统计指定月份的每一天工作时长（不区分工作日和非工作日）
     *
     * @param workMonth 工作月份
     * @param userId    用户id
     * @return 当月每一天的工作时长
     */
    List<WorkHoursVo> getWorkHoursList(@Param("userId") String userId, @Param("workMonth") String workMonth);

    List<ProjectWorkingCountVo> projectWorkingCountRecord(@Param("projectId") Integer projectId,
                                                          @Param("customName") String customName,
                                                          @Param("userId") String userId,
                                                          @Param("startWorkTime") String startWorkTime,
                                                          @Param("endWorkTime") String endWorkTime,
                                                          @Param("offset") Integer offset,
                                                          @Param("size") Integer size);

    long projectWorkingCountTotal(@Param("projectId") Integer projectId,
                                  @Param("customName") String customName,
                                  @Param("userId") String userId,
                                  @Param("startWorkTime") String startWorkTime,
                                  @Param("endWorkTime") String endWorkTime);

    List<WorkerWorkingCountVo> workerWorkingCountRecord(@Param("projectId") Integer projectId,
                                                        @Param("customName") String customName,
                                                        @Param("userId") String userId,
                                                        @Param("startWorkTime") String startWorkTime,
                                                        @Param("endWorkTime") String endWorkTime,
                                                        @Param("offset") Integer offset,
                                                        @Param("size") Integer size);

    long workerWorkingCountTotal(@Param("projectId") Integer projectId,
                                 @Param("customName") String customName,
                                 @Param("userId") String userId,
                                 @Param("startWorkTime") String startWorkTime,
                                 @Param("endWorkTime") String endWorkTime);

    List<CheckHoursVo> check(@Param("deptId")  List<String> deptId,
                             @Param("realName") String realName,
                             @Param("month") String month);

    List<Workingreport> statisticsByOrgIdList(@Param("orgIdList") Set<String> orgIdList, @Param("month") String month);

    Integer getCompleteTask(@Param("projectId") Integer projectId,
                            @Param("userId") String userId,
                            @Param("date") LocalDate date);

    Integer getOverdueTask(@Param("projectId") Integer projectId,
                           @Param("userId") String userId,
                           @Param("date") LocalDate date);

    Integer getRejectTask(@Param("projectId") Integer projectId,
                          @Param("userId") String userId,
                          @Param("date") LocalDate date);

    Integer getTodoTask(@Param("projectId") Integer projectId,
                        @Param("userId") String userId,
                        @Param("date") LocalDate date);

    Integer getCompleteBug(@Param("projectId") Integer projectId,
                           @Param("userId") String userId,
                           @Param("date") LocalDate date);

    Integer getOverdueBug(@Param("projectId") Integer projectId,
                          @Param("userId") String userId,
                          @Param("date") LocalDate date);

    Integer getRejectBug(@Param("projectId") Integer projectId,
                         @Param("userId") String userId,
                         @Param("date") LocalDate date);

    Integer getTodoBug(@Param("projectId") Integer projectId,
                       @Param("userId") String userId,
                       @Param("date") LocalDate date);

    List<TaskCompleteVo> getTaskCompleteVoList(@Param("projectId") Integer projectId,
                                               @Param("userId") String userId,
                                               @Param("date") LocalDate date);

    List<String> getBugTitleList(@Param("projectId") Integer projectId,
                                 @Param("userId") String userId,
                                 @Param("date") LocalDate date);

    Integer getCreateTask(@Param("projectId") Integer projectId,
                          @Param("userId") String userId,
                          @Param("date") LocalDate date);

   Integer getVerifyTask(@Param("projectId") Integer projectId,
                        @Param("userId") String userId,
                        @Param("date") LocalDate date);

    Integer getRegressionTask(@Param("projectId") Integer projectId,
                              @Param("userId") String userId,
                              @Param("date") LocalDate date);

    Integer getAwaitVerifyTask(@Param("projectId") Integer projectId,
                               @Param("userId") String userId,
                               @Param("date") LocalDate date);

    Integer getCreateBug(@Param("projectId") Integer projectId,
                         @Param("userId") String userId,
                         @Param("date") LocalDate date);

    Integer getVerifyBug(@Param("projectId") Integer projectId,
                         @Param("userId") String userId,
                         @Param("date") LocalDate date);

    Integer getRegressionBug(@Param("projectId") Integer projectId,
                             @Param("userId") String userId,
                             @Param("date") LocalDate date);

    Integer getAwaitVerifyBug(@Param("projectId") Integer projectId,
                              @Param("userId") String userId,
                              @Param("date") LocalDate date);

    List<String> getTaskTitleProcessingList(@Param("projectId")Integer projectId,
                                           @Param("userId") String userId);
}