package com.tofly.pms.working.service.impl;

import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.pms.fegin.DictItemClient;
import com.tofly.pms.working.entity.FeedBackPlan;
import com.tofly.pms.working.entity.Feedback;
import com.tofly.pms.working.entity.TaskW;
import com.tofly.pms.working.mapper.FeedBackPlanMapper;
import com.tofly.pms.working.service.FeedBackPlanService;
import com.tofly.pms.working.service.FeedBackService;
import com.tofly.pms.working.service.TaskWService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.Objects;

@Service
public class FeedBackPlanServiceImpl extends ServiceImpl<FeedBackPlanMapper, FeedBackPlan>
        implements FeedBackPlanService {

    @Autowired
    private TaskWService taskWService;
    @Autowired
    private FeedBackService feedBackService;

    @Override
    public FeedBackPlan queryById(Integer id) {
        return baseMapper.selectById(id);
    }

    /**
     * 保存反馈相关处理 以及生成task
     *
     * @param feedBackPlan
     */
    @Override
    @Transactional
    public void saveOrUpdateFeedBackPlan(FeedBackPlan feedBackPlan) {
        boolean oldDispatch = false;
        //表明数据存在
        if (feedBackPlan.getId() != null) {
            //查找原有数据是否下发任务
            final FeedBackPlan feedBackPlan1 = queryById(feedBackPlan.getId());
            if (feedBackPlan1.getIsDispatchTask()) {
                oldDispatch = true;
            }

        }
        final Feedback byId = feedBackService.getById(feedBackPlan.getFeedbackId());
        byId.setHandlePlan(feedBackPlan.getHandlePlan());
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        byId.setFeedTime(Objects.nonNull(byId.getCreateDate())?df.format(byId.getCreateDate()):df.format(new Date()));
        byId.setResolverPerson(feedBackPlan.getResolverPerson());
        byId.setResolveDate(feedBackPlan.getResolveDate());
        feedBackService.updateById(byId);
        //原数据 未下发任务  新数据下发任务
        if (!oldDispatch && feedBackPlan.getIsDispatchTask() != null && feedBackPlan.getIsDispatchTask()) {
            //下发任务


            TaskW taskW = new TaskW();
            //根据处理意见的信息 生成task

            taskW.setType("5");
            //意见的项目-》任务的项目
            taskW.setProjectId(byId.getProjectId());
            //意见标题 -》任务标题
            taskW.setTitle(byId.getTitle());
            //意见类型 -》任务标签
            DictItemClient service = SpringUtil.getBean(DictItemClient.class);
            String viewtyle = service.getDicItemText("VIEWTYLE", String.valueOf(byId.getViewTyle()));

            String viewType = service.getDicItemText("VIEWTYPE", String.valueOf(byId.getViewType()));
            taskW.setLabel(viewType.concat("【").concat(viewtyle).concat("】"));
            //意见类别 -》任务分组
//            taskW.setGroupname(viewType);
            //紧急程度 -》任务紧急程度
            taskW.setUrgency(byId.getUrgency());
            //描述 -》任务描述
//            把意见描述、处理方案、处理说明自动添加到任务描述中，形式如下：
//
//【意见描述】这里是意见描述
//
//【处理方案】这里是处理方案
//
//【处理说明】这里是处理说明
            taskW.setDescroption("<p>\n【意见描述】"
                    .concat( StringUtils.isBlank(byId.getDescription())?"":byId.getDescription())
                    .concat("\n【处理方案】")
                    .concat(StringUtils.isBlank(byId.getHandlePlan())?"":byId.getHandlePlan())
                    .concat("\n【处理说明】")
                    .concat(StringUtils.isBlank(byId.getResolverDescription())?"":byId.getResolverDescription())
                    .concat("<p/>"));
//            taskW.setDescroption(byId.getDescription());
            //预计工作量 -》任务预计工作量
            taskW.setPredictWorkload(feedBackPlan.getPlanWorkload());
            //解决时间 -》任务结束时间
            taskW.setPlanEndDate(feedBackPlan.getResolveDate());
            //负责人 -》任务负责人
            taskW.setHeadId(feedBackPlan.getResolverPerson());
            // 确认人 -》任务关注人
            taskW.setConfirmId(feedBackPlan.getConfirmPerson());
            taskW.setCareId(feedBackPlan.getFocusPerson());
            taskW.setFileUrl(byId.getFileUrl());
            taskW.setFeedbackId(feedBackPlan.getFeedbackId());
           LocalDate date = LocalDate.now();
            taskW.setPlanStartDate(date.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
            taskW.setPlanEndDate(byId.getResolveDate());

            taskWService.save(taskW);

        }


        this.saveOrUpdate(feedBackPlan);

    }
}
