package com.tofly.pms.working.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.metadata.style.WriteCellStyle;
import com.alibaba.excel.write.metadata.style.WriteFont;
import com.alibaba.excel.write.style.HorizontalCellStyleStrategy;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.excel.write.style.column.SimpleColumnWidthStyleStrategy;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.DateUtil;
import com.tofly.pms.fegin.OrgClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.utils.UserUtils;
import com.tofly.pms.working.entity.HoursW;
import com.tofly.pms.working.entity.WorkHoursExcelModel;
import com.tofly.pms.working.entity.Workingreport;
import com.tofly.pms.working.mapper.HoursWMapper;
import com.tofly.pms.working.service.HolidaysService;
import com.tofly.pms.working.service.HoursWService;
import com.tofly.pms.working.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

/**
 * 工时登记表;(TF_PM_WORKINGHOURS_W)表服务实现类
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Slf4j
@Service
public class HoursWServiceImpl extends ServiceImpl<HoursWMapper, HoursW>
        implements HoursWService {
    @Resource
    HttpServletResponse response;
    @Autowired
    private UserUtils userUtils;
    @Autowired
    private HolidaysService holidaysService;
    private static final ExecutorService executorService = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());

    @Override
    public HoursW queryById(Integer id) {
        return baseMapper.selectById(id);
    }

    /**
     * 统计指定月份的每一天工作时长（不区分工作日和非工作日）
     *
     * @param workMonth 工作月份
     * @return 当月每一天的工作时长
     */
    @Override
    public List<WorkHoursVo> getWorkHoursList(String workMonth) {
        String userId = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userId, "当前查询用户不合法");
        if (StringUtils.isBlank(workMonth)) {
            workMonth = DateUtil.getDateFormat(new Date(), "yyyy-MM");
        }
        List<WorkHoursVo> result = baseMapper.getWorkHoursList(userId, workMonth);
        return paddingWorkDate(result, workMonth);
    }

    @Override
    public Page<ProjectWorkingCountVo> projectWorkingCount(Integer projectId, String customName,
                                                           String userId, String startWorkTime,
                                                           String endWorkTime, Integer page,
                                                           Integer size) {
        Assert.isTrue(Objects.nonNull(page) && page > 0, "page不能为空");
        Assert.isTrue(Objects.nonNull(size) && size > 0, "size不能为空");
        Page<ProjectWorkingCountVo> pageInfo = new Page<>(page, size);
        Integer offset = (page - 1) * size;
        List<ProjectWorkingCountVo> record = baseMapper.projectWorkingCountRecord(projectId, customName, userId, startWorkTime, endWorkTime, offset, size);
        long total = baseMapper.projectWorkingCountTotal(projectId, customName, userId, startWorkTime, endWorkTime);
        pageInfo.setRecords(record);
        pageInfo.setTotal(total);
        return pageInfo;
    }

    @Override
    public Page<WorkerWorkingCountVo> workerWorkingCount(Integer projectId, String customName,
                                                         String userId, String startWorkTime,
                                                         String endWorkTime, Integer page,
                                                         Integer size) {
        Assert.isTrue(Objects.nonNull(page) && page > 0, "page不能为空");
        Assert.isTrue(Objects.nonNull(size) && size > 0, "size不能为空");
        Page<WorkerWorkingCountVo> pageInfo = new Page<>(page, size);
        Integer offset = (page - 1) * size;
        List<WorkerWorkingCountVo> record = baseMapper.workerWorkingCountRecord(projectId, customName, userId, startWorkTime, endWorkTime, offset, size);
        long total = baseMapper.workerWorkingCountTotal(projectId, customName, userId, startWorkTime, endWorkTime);
        pageInfo.setRecords(record);
        pageInfo.setTotal(total);
        return pageInfo;
    }

    @Override
    public void exportProject(Integer projectId, String customName, String userId, String startWorkTime, String endWorkTime) throws IOException {
        List<ProjectWorkingCountVo> data = baseMapper.projectWorkingCountRecord(projectId, customName, userId, startWorkTime, endWorkTime, null, null);
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = URLEncoder.encode("项目工时统计", "UTF-8").replaceAll("\\+", "%20");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
        EasyExcel.write(response.getOutputStream(), ProjectWorkingCountVo.class)
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .sheet("项目工时统计").doWrite(data);
    }

    @Override
    public void exportWorker(Integer projectId, String customName, String userId, String startWorkTime, String endWorkTime) throws IOException {
        List<WorkerWorkingCountVo> data = baseMapper.workerWorkingCountRecord(projectId, customName, userId, startWorkTime, endWorkTime, null, null);
        if (CollectionUtil.isNotEmpty(data)) {
            for (WorkerWorkingCountVo datum : data) {
                UserVo userVo = userUtils.getUserVoByUserId(datum.getUserId());
                datum.setUsername(userVo.getRealName());
            }
        }
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = URLEncoder.encode("工人工时统计", "UTF-8").replaceAll("\\+", "%20");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
        EasyExcel.write(response.getOutputStream(), WorkerWorkingCountVo.class)
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .sheet("工人工时统计").doWrite(data);
    }

    @Override
    public List<CheckHoursVo> check(String deptId, String realName, String month) {
        Assert.notNull(month, "月份不能为空");
        List<String> deptIdLIst = null;
        if(StringUtils.isNotBlank(deptId)){
            OrgClient orgClient = SpringUtil.getBean(OrgClient.class);
             deptIdLIst = orgClient.getChildrenId(deptId);
        }
        List<CheckHoursVo> check = baseMapper.check(deptIdLIst, realName, month);
        List<Date> workDay = getWorkDay(month);
        if (CollectionUtil.isNotEmpty(check)) {
            for (CheckHoursVo checkHoursVo : check) {
                List<WorkHoursVo> result = checkHoursVo.getResult();
                List<Date> dateList = result.stream()
                        .filter(item -> item.getDuration() >= 8)
                        .map(WorkHoursVo::getWorkDate)
                        .filter(StringUtils::isNotBlank)
                        .map(dateStr -> DateUtil.formatDate(dateStr, "yyyy-MM-dd"))
                        .collect(Collectors.toList());
                // 判断是否全勤
                checkHoursVo.setIncomplete(!allCollectionsContains(dateList, workDay));
                List<WorkHoursVo> workHoursVos = paddingWorkDate(result, month);
                checkHoursVo.setResult(workHoursVos);
            }
        }
        return check;
    }

    @Override
    public void exportCheck(String deptId, String realName, String month) throws IOException {
        List<CheckHoursVo> exportData = check(deptId, realName, month);
        WorkHoursExcelModel data = new WorkHoursExcelModel();
        List<List<String>> head = data.getHead();
        if (CollectionUtil.isNotEmpty(exportData)) {
            CheckHoursVo checkHoursVo = exportData.get(0);
            List<WorkHoursVo> result = checkHoursVo.getResult();
            for (WorkHoursVo workHoursVo : result) {
                String workDate = workHoursVo.getWorkDate();
                String workDay = workDate.split("-")[2];
                head.add(List.of(workDay));
            }
        }
        List<List<Object>> dataList = data.getDataList();
        //将数据转换
        transferData(exportData, dataList);
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = URLEncoder.encode(data.getFileName(), StandardCharsets.UTF_8).replaceAll("\\+", "%20");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
        WriteCellStyle headWriteCellStyle = new WriteCellStyle();
        WriteCellStyle contentWriteCellStyle = new WriteCellStyle();
        WriteFont headWriteFont = new WriteFont();
        headWriteFont.setFontHeightInPoints((short) 12);
        headWriteCellStyle.setWriteFont(headWriteFont);
        HorizontalCellStyleStrategy horizontalCellStyleStrategy =
                new HorizontalCellStyleStrategy(headWriteCellStyle, contentWriteCellStyle);
        SimpleColumnWidthStyleStrategy columnWidthStrategy = new SimpleColumnWidthStyleStrategy(10);
        EasyExcel.write(response.getOutputStream())
                .head(head)
                .registerWriteHandler(horizontalCellStyleStrategy)
                .registerWriteHandler(columnWidthStrategy)
                .sheet(data.getFileName())
                .doWrite(dataList);
    }

    @Override
    public List<Workingreport> statisticsByOrgIdList(Set<String> orgIdList, String month) {

        return baseMapper.statisticsByOrgIdList(orgIdList, month);
    }

    @Override
    public TodayTaskCompleteVo getTaskAchievement(Integer projectId) {
        Assert.notNull(projectId, "项目id不能为空");
        String userId = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userId, "访问不合法");
        LocalDate date = LocalDate.now();
        TodayTaskCompleteVo todayTaskCompleteVo = new TodayTaskCompleteVo();
        //查询今天完成的任务数量
        CompletableFuture<Integer> completeTaskFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getCompleteTask(projectId, userId, date), executorService);
//        Integer completeTask = baseMapper.getCompleteTask(projectId, userId, date);
        //  今日逾期任务
        CompletableFuture<Integer> overdueTaskFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getOverdueTask(projectId, userId, date), executorService);
//        Integer overdueTask = baseMapper.getOverdueTask(projectId, userId, date);
        //今日被驳回任务
        CompletableFuture<Integer> rejectTaskFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getRejectTask(projectId, userId, date), executorService);
//        Integer rejectTask = baseMapper.getRejectTask(projectId, userId, date);
        //今日待处理任务
        CompletableFuture<Integer> todoTaskFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getTodoTask(projectId, userId, date), executorService);
//        Integer todoTask = baseMapper.getTodoTask(projectId, userId, date);
        //今日创建的任务
        CompletableFuture<Integer> createTaskFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getCreateTask(projectId, userId, date), executorService);
        //今日确认任务
        CompletableFuture<Integer> verifyTaskFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getVerifyTask(projectId, userId, date), executorService);
        //今日驳回任务
        CompletableFuture<Integer> regressionTaskFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getRegressionTask(projectId, userId, date), executorService);
        //今日待确认任务
        CompletableFuture<Integer> awaitVerifyTaskFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getAwaitVerifyTask(projectId, userId, date), executorService);

        //今日完成bug
        CompletableFuture<Integer> completeBugFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getCompleteBug(projectId, userId, date), executorService);
//        Integer completeBug = baseMapper.getCompleteBug(projectId, userId, date);
        //今日逾期bug
        CompletableFuture<Integer> overdueBugFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getOverdueBug(projectId, userId, date), executorService);
//        Integer overdueBug = baseMapper.getOverdueBug(projectId, userId, date);
        //今日被驳回bug
        CompletableFuture<Integer> rejectBugFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getRejectBug(projectId, userId, date), executorService);
//        Integer rejectBug = baseMapper.getRejectBug(projectId, userId, date);
        //今日待处理bug
        CompletableFuture<Integer> todoBugFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getTodoBug(projectId, userId, date), executorService);
//        Integer todoBug = baseMapper.getTodoBug(projectId, userId, date);


        //今日创建的bug
        CompletableFuture<Integer> createBugFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getCreateBug(projectId, userId, date), executorService);
        //今日确认bug
        CompletableFuture<Integer> verifyBugFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getVerifyBug(projectId, userId, date), executorService);
        //今日驳回bug
        CompletableFuture<Integer> regressionBugFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getRegressionBug(projectId, userId, date), executorService);
        //今日待确认bug
        CompletableFuture<Integer> awaitVerifyBugFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getAwaitVerifyBug(projectId, userId, date), executorService);


        //任务完成明细
        CompletableFuture<List<TaskCompleteVo>> taskCompleteVoListFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getTaskCompleteVoList(projectId, userId, date), executorService);
//        List<TaskCompleteVo> taskCompleteVoList = baseMapper.getTaskCompleteVoList(projectId, userId, date);
        //bug完成明细
        CompletableFuture< List<String>> bugTitleListFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getBugTitleList(projectId, userId, date), executorService);
//        List<String> bugTitleList = baseMapper.getBugTitleList(projectId, userId, date);
        //正在处理的任务
        CompletableFuture< List<String>> taskTitleListFuture = CompletableFuture.supplyAsync(() ->
                baseMapper.getTaskTitleProcessingList(projectId, userId), executorService);
        CompletableFuture.allOf(completeTaskFuture,
                overdueTaskFuture,
                rejectTaskFuture,
                todoTaskFuture,
                completeBugFuture,
                overdueBugFuture,
                rejectBugFuture,
                todoBugFuture,
                taskCompleteVoListFuture,
                bugTitleListFuture,
                createTaskFuture,
                verifyTaskFuture,
                regressionTaskFuture,
                awaitVerifyTaskFuture,
                createBugFuture,
                verifyBugFuture,
                regressionBugFuture,
                awaitVerifyBugFuture,
                taskTitleListFuture
        ).join();
        todayTaskCompleteVo.setAwaitVerifyTask(awaitVerifyTaskFuture.join());
        todayTaskCompleteVo.setAwaitVerifyBug(awaitVerifyBugFuture.join());
        todayTaskCompleteVo.setVerifyTask(verifyTaskFuture.join());
        todayTaskCompleteVo.setVerifyBug(verifyBugFuture.join());
        todayTaskCompleteVo.setCreateTask(createTaskFuture.join());
        todayTaskCompleteVo.setCreateBug(createBugFuture.join());
        todayTaskCompleteVo.setRegressionTask(regressionTaskFuture.join());
        todayTaskCompleteVo.setRegressionBug(regressionBugFuture.join());
        todayTaskCompleteVo.setCompleteTask(completeTaskFuture.join());
        todayTaskCompleteVo.setOverdueTask(overdueTaskFuture.join());
        todayTaskCompleteVo.setRejectTask(rejectTaskFuture.join());
        todayTaskCompleteVo.setTodoTask(todoTaskFuture.join());
        todayTaskCompleteVo.setCompleteBug(completeBugFuture.join());
        todayTaskCompleteVo.setOverdueBug(overdueBugFuture.join());
        todayTaskCompleteVo.setRejectBug(rejectBugFuture.join());
        todayTaskCompleteVo.setTodoBug(todoBugFuture.join());
        todayTaskCompleteVo.setTaskCompleteVoList(taskCompleteVoListFuture.join());
        todayTaskCompleteVo.setBugTitleList(bugTitleListFuture.join());
        todayTaskCompleteVo.setProcessingList(taskTitleListFuture.join());
        todayTaskCompleteVo.setProcessingTask(taskTitleListFuture.join().size());
        return todayTaskCompleteVo;
    }
    private void transferData(List<CheckHoursVo> exportData, List<List<Object>> dataList) {
        if (CollectionUtil.isEmpty(exportData)) {
            return;
        }
        for (CheckHoursVo exportDatum : exportData) {
            List<Object> objeList = new ArrayList<>();
            objeList.add(exportDatum.getDeptName());
            objeList.add(exportDatum.getRealName());
            objeList.add(Objects.equals(exportDatum.getStatus(), "1") ? "在职" : "离职");
            objeList.add(exportDatum.getIncomplete() ? "是" : "否");
            List<WorkHoursVo> result = exportDatum.getResult();
            for (WorkHoursVo workHoursVo : result) {
                objeList.add(workHoursVo.getDuration());
            }
            dataList.add(objeList);
        }
    }

    private boolean allCollectionsContains(List<Date> sourceCollections, List<Date> targetCollection) {


        return new HashSet<>(sourceCollections).containsAll(targetCollection);
    }

    private List<Date> getWorkDay(String month) {
        List<Date> workDayList = new ArrayList<>();
        if (StringUtils.isNotBlank(month)) {
            Date startDate = DateUtil.formatDate(month + "-01", "yyyy-MM-dd");
            Date endDate = DateUtil.getLastDayOfMonth(startDate);
            List<Date> restList = holidaysService.getHolidays(startDate, endDate, 1);
            List<Date> workList = holidaysService.getHolidays(startDate, endDate, 2);
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(startDate);
            for (; !endDate.before(calendar.getTime()); calendar.add(Calendar.DAY_OF_MONTH, 1)) {
                int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
                boolean isWeekend = (dayOfWeek == Calendar.SATURDAY || dayOfWeek == Calendar.SUNDAY);
                //如果是周末，但是调休上班，算工作日
                if (isWeekend && workList.contains(calendar.getTime())) {
                    workDayList.add(calendar.getTime());
                }
                //如果不是周末，也不是节假日放假，算工作日
                if (!isWeekend && !restList.contains(calendar.getTime())) {
                    workDayList.add(calendar.getTime());
                }
            }
        }

        log.info("{}工作日期：{}", month, workDayList);

        return workDayList;
    }

    /**
     * 补全工作日期
     *
     * @param result    已经有的工作日期
     * @param workMonth 工作月份
     * @return 整月的数据
     */
    private List<WorkHoursVo> paddingWorkDate(List<WorkHoursVo> result, String workMonth) {
        if (CollectionUtil.isEmpty(result)) {
            result = new ArrayList<>(31);
        }
        List<Date> dateList = result.stream()
                .map(WorkHoursVo::getWorkDate)
                .filter(StringUtils::isNotBlank)
                .map(dateStr -> DateUtil.formatDate(dateStr, "yyyy-MM-dd"))
                .collect(Collectors.toList());
        Date startDate = DateUtil.formatDate(workMonth + "-01", "yyyy-MM-dd");
        Date endDate = DateUtil.getLastDayOfMonth(startDate);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);
        for (; !endDate.before(calendar.getTime()); calendar.add(Calendar.DAY_OF_MONTH, 1)) {
            Date time = calendar.getTime();
            if (dateList.contains(time)) {
                continue;
            }
            WorkHoursVo vo = new WorkHoursVo();
            vo.setWorkDate(DateUtil.getDateFormat(time, "yyyy-MM-dd"));
            vo.setDuration(0);
            result.add(vo);
        }
        return result.stream().filter(item -> Objects.nonNull(item.getWorkDate())).sorted(Comparator.comparing(WorkHoursVo::getWorkDate)).collect(Collectors.toList());
    }
}