package com.tofly.pms.working.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.exceptions.ValidateException;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.dto.PlanDispatchDto;
import com.tofly.pms.working.entity.Plan;
import com.tofly.pms.working.entity.PlanExcelModel;
import com.tofly.pms.working.entity.TaskW;
import com.tofly.pms.working.mapper.PlanMapper;
import com.tofly.pms.working.service.PlanService;
import com.tofly.pms.working.service.TaskWService;
import com.tofly.pms.working.vo.ProjectCountVo;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.time.Duration;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 项目计划信息表;(TF_PM_PLAN_W)表服务实现类
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Service
public class PlanServiceImpl extends ServiceImpl<PlanMapper, Plan> implements PlanService {
    @Autowired
    RedisTemplate redisTemplate;
    @Autowired
    TaskWService taskWService;
    @Autowired
    HttpServletResponse response;

    @Override
    public Plan queryById(Integer id) {
        return baseMapper.selectById(id);
    }

    @Override
    public ProjectCountVo projectOverCount(Integer projectId) {
        if (projectId == null) {
            return new ProjectCountVo();
        }
        return baseMapper.projectOverCount(projectId);
    }

    @Override
    public Integer getMaxId() {
        //获取redis中id最大值
        Integer redisMaxId = (Integer) redisTemplate.opsForValue().get("pms:plan:maxid");
        //获取当前数据库中id最大值
        Integer maxId = baseMapper.getMaxId();
        if (maxId == null) {
            maxId = 1;
        }
        if (maxId == null || redisMaxId == null || redisMaxId < maxId) {

            redisTemplate.opsForValue().set("pms:plan:maxid", maxId, Duration.ofDays(1));
            return maxId + 1;
        } else {
            redisTemplate.opsForValue().set("pms:plan:maxid", redisMaxId + 1, Duration.ofDays(1));
            return redisMaxId + 1;
        }
    }

    /**
     * 支持将选中的计划批量下发工作任务。若选中的工作项有多个负责人，则将任务拆成多个，默认将工作项的标题作为任务标题，确认人设置为当前登录用户，计划结束
     *
     * @param planDispatchDto
     * @return
     */
    @Override
    @Transactional
    public Boolean dispatchPlan(PlanDispatchDto planDispatchDto) {
        //获取所选取的任务
        final String planIds = planDispatchDto.getPlanIds();
        final List<String> planIdList = Arrays.asList(planIds.split(","));
        final List<Long> collect = planIdList.stream().map(Long::parseLong).collect(Collectors.toList());
        final List<Plan> plans = baseMapper.selectList(Wrappers.<Plan>lambdaQuery().in(Plan::getId, collect));
        List<TaskW> totalTask = new ArrayList<>();

        for (Plan plan : plans) {
            plan.setStatus(1);

            totalTask.addAll(plansGenerateTask(plan));
        }
        this.updateBatchById(plans);
        taskWService.saveOrUpdateBatch(totalTask);

        return Boolean.TRUE;
    }

    /**
     * 删除计划 以及关联的子计划
     *
     * @param ids
     * @return
     */
    @Override
    @Transactional
    public boolean removeChildByIds(String ids) {

        final String[] split = ids.split(",");
        for (String id : split) {
            final long count = taskWService.count(Wrappers.<TaskW>lambdaQuery().eq(TaskW::getPlanId, Integer.valueOf(id)));
            if (count > 0) {
                final Plan byId = this.getById(Integer.valueOf(id));

                throw new ValidateException(byId.getName() + "存在关联的任务，无法删除");
            }
            //查询子计划
            final List<Plan> list = this.list(Wrappers.<Plan>lambdaQuery().eq(Plan::getParentId, Integer.valueOf(id)));

            if (list.size() > 0) {

            }


            this.removeById(Integer.valueOf(id));
        }
        return false;
    }

    @Override
    public String getFullPath(Integer id) {
        Assert.notNull(id, "计划id不能为空");
        Plan plan = this.getById(id);
        if (Objects.isNull(plan)) {
            return "";
        }
        String name = plan.getName();
        if (Objects.nonNull(plan.getParentId())) {
            String parentName = this.getFullPath(plan.getParentId());
            if (StrUtil.isNotBlank(parentName)) {
                name = parentName + "->" + name;
            }
        }
        return name;
    }

    @Override
    public void export(Integer projectId, Integer listId) throws IOException {
        Assert.notNull(projectId, "项目id不能为空");
        Assert.notNull(listId, "计划清单id不能为空");
        List<Plan> plans = baseMapper.selectList(Wrappers.lambdaQuery(Plan.class)
                .eq(Plan::getProjectId, projectId)
                .eq(Plan::getListId, listId));
        List<PlanExcelModel> data = new ArrayList<>(plans.size());
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        for (Plan plan : plans) {
            PlanExcelModel planExcelModel = plan.toExcelModel();
            if (StringUtils.isNotBlank(plan.getHeadId())) {
                String[] userIds = plan.getHeadId().split(",");
                List<UserVo> userInfoList = new ArrayList<>(userIds.length);
                for (String userId : userIds) {
                    UserVo userInfo = userClient.getUserInfo(userId);
                    userInfoList.add(userInfo);
                }
                if (CollectionUtil.isNotEmpty(userInfoList)) {
                    String realName = userInfoList.stream().map(UserVo::getRealName).collect(Collectors.joining(","));
                    planExcelModel.setHeadName(realName);
                }
            }
            data.add(planExcelModel);
        }
        data = data.stream().sorted(Comparator.comparing(PlanExcelModel::getNumber)).collect(Collectors.toList());
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = URLEncoder.encode("计划列表", "UTF-8").replaceAll("\\+", "%20");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
        EasyExcel.write(response.getOutputStream(), PlanExcelModel.class)
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .sheet("计划列表").doWrite(data);
    }

    private List<TaskW> plansGenerateTask(Plan plan) {
        //验证该任务 是否已经分发过


        List<TaskW> tasks = new ArrayList<>();
        //查看责任人是否有多个

        final String headIds = plan.getHeadId();
        if (StringUtils.isNotBlank(headIds)) {

            final String[] splitHead = headIds.split(",");

            for (String headUser : splitHead) {
                TaskW taskW = new TaskW();
                taskW.setPlanId(plan.getId());
                taskW.setHeadId(headUser);
                taskW.setProjectId(plan.getProjectId());
                taskW.setConfirmId(UserTokenUtil.getTokenUserID());
                taskW.setTitle(plan.getName());
                taskW.setPlanStartDate(plan.getPlanStartTime());
                taskW.setPlanEndDate(plan.getPlanEndDate());
                taskW.setStatus("1");
                //TODO 任务状态需确认
                if(StringUtils.isNotBlank(plan.getDescription())){
                    taskW.setDescroption("<p style=\"white-space:pre-wrap\">" + plan.getDescription() + "</p>");
                }
                taskW.setTestList(plan.getListId().toString());
                taskW.setPredictWorkload(plan.getPredictWorkload());
                tasks.add(taskW);

            }


        } else {
            TaskW taskW = new TaskW();
            taskW.setPlanId(plan.getId());
            taskW.setProjectId(plan.getProjectId());
            taskW.setConfirmId(UserTokenUtil.getTokenUserID());
            taskW.setTitle(plan.getName());
            taskW.setPlanStartDate(plan.getPlanStartTime());
            taskW.setPlanEndDate(plan.getPlanEndDate());
            //TODO 任务状态需确认
            taskW.setStatus("0");
            if(StringUtils.isNotBlank(plan.getDescription())){
                taskW.setDescroption("<p style=\"white-space:pre-wrap\">" + plan.getDescription() + "</p>");
            }
            taskW.setTestList(plan.getListId().toString());
            tasks.add(taskW);
        }

        return tasks;
    }


}