package com.tofly.pms.working.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.pms.fegin.OrgClient;
import com.tofly.pms.fegin.RoleClient;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.project.entity.ProjectTeamW;
import com.tofly.pms.project.service.ProjectTeamWService;
import com.tofly.pms.project.service.ProjectWService;
import com.tofly.pms.project.vo.ProjectOverTaskCountVo;
import com.tofly.pms.project.vo.ProjectVo;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.dto.TaskHandleBatchDto;
import com.tofly.pms.working.dto.TaskHandleDto;
import com.tofly.pms.working.en.HandleEnum;
import com.tofly.pms.working.entity.Editorlog;
import com.tofly.pms.working.entity.GanttData;
import com.tofly.pms.working.entity.TaskW;
import com.tofly.pms.working.entity.TasklogW;
import com.tofly.pms.working.mapper.TaskWMapper;
import com.tofly.pms.working.service.EditorlogService;
import com.tofly.pms.working.service.TaskWService;
import com.tofly.pms.working.service.TasklogWService;
import com.tofly.pms.working.vo.TaskCountVo;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 任务信息表;(TF_PM_TASK_W)表服务实现类
 *
 * @author : Xu'J
 * @date : 2023-12-19
 */
@Slf4j
@Service
public class TaskWServiceImpl extends ServiceImpl<TaskWMapper, TaskW> implements TaskWService {
    @Autowired
    private TasklogWService tasklogWService;
    @Autowired
    private ProjectTeamWService projectTeamWService;
    @Autowired
    private ProjectWService projectWService;

    @Resource
    private OrgClient orgClient;
    @Autowired
    private EditorlogService editorlogService;


    @Transactional(rollbackFor = Exception.class)
    public boolean save(TaskW taskW) {
        String userID = UserTokenUtil.getTokenUserID();
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        UserVo userInfo = userClient.getUserInfo(userID);
        String realName = userInfo.getRealName();
        String content = "[ " + realName + " ] 创建任务";
        String headId = taskW.getHeadId();
        if (StringUtils.isNotBlank(headId)) {
            String[] split = headId.split(",");
            taskW.setStatus("1");
            for (String s : split) {
                try {
                    TaskW newTaskW = taskW.deepClone();
                    newTaskW.setHeadId(s);
                    super.save(newTaskW);
                    TasklogW tasklogW = new TasklogW();
                    tasklogW.setTaskId(newTaskW.getId());
                    tasklogW.setSourceStatus(newTaskW.getStatus());
                    tasklogW.setProcessingMode(newTaskW.getProcessingMode());
                    tasklogW.setJsonData(JSON.toJSONString(newTaskW));
                    tasklogW.setContent(content);
                    tasklogWService.save(tasklogW);
                } catch (Exception e) {
                    throw new RuntimeException("对象clone错误");
                }
            }
        } else {
            taskW.setStatus("0");
            super.save(taskW);
            TasklogW tasklogW = new TasklogW();
            tasklogW.setTaskId(taskW.getId());
            tasklogW.setSourceStatus(taskW.getStatus());
            tasklogW.setProcessingMode(taskW.getProcessingMode());
            tasklogW.setJsonData(JSON.toJSONString(taskW));
            tasklogW.setContent(content);
            tasklogWService.save(tasklogW);
        }
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void updateTaskInfo(TaskW taskW) {
        Integer id = taskW.getId();
        TaskW dbTask = baseMapper.selectById(id);
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前操作用户不合法");
        if (!dbTask.getCreateUser().equals(userID)) {
            Integer projectId = dbTask.getProjectId();
            ProjectTeamW projectTeam = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, projectId)
                    .like(ProjectTeamW::getAdminId, userID));
            if (Objects.isNull(projectTeam)) {
                RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                if (!has) {
                    throw new RuntimeException("当前用户没有权限");
                }
            }
        }


        //todo 记录编辑日志
        Editorlog editorlog = new Editorlog();
        editorlog.setDataId(id);
        editorlog.setDataType("task");
        editorlog.setSource(JSON.toJSONString(dbTask));
        editorlog.setChangedData(JSON.toJSONString(taskW));
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        UserVo userInfo = userClient.getUserInfo(userID);
        String content = "[" + userInfo.getRealName() + "]编辑任务";
        String dbHeadId = dbTask.getHeadId();
        String headId = taskW.getHeadId();
        editorlog.setSourceHeadId(dbHeadId);
        editorlog.setChangedHeadId(headId);
        if (ObjectUtil.notEqual(dbHeadId, headId)) {
            UserVo dbHeadUser = userClient.getUserInfo(dbHeadId);
            UserVo headUser = userClient.getUserInfo(headId);
            content += "，负责人由[" + dbHeadUser.getRealName() + "]变更为[" + headUser.getRealName() + "]";
        }
        String dbConfirmId = dbTask.getConfirmId();
        String confirmId = taskW.getConfirmId();
        editorlog.setSourceSureId(dbConfirmId);
        editorlog.setChangedSureId(confirmId);
        if (ObjectUtil.notEqual(dbConfirmId, confirmId)) {
            StringBuilder dbHeadUserName = new StringBuilder();
            StringBuilder headUserName = new StringBuilder();
            if (StringUtils.isNotBlank(dbConfirmId)) {
                String[] split = dbConfirmId.split(",");
                for (String suserId : split) {
                    UserVo dbHeadUser = userClient.getUserInfo(suserId);
                    dbHeadUserName.append(dbHeadUser.getRealName()).append(",");
                }
            }
            if (StringUtils.isNotBlank(confirmId)) {
                String[] split = confirmId.split(",");
                for (String suserId : split) {
                    UserVo headUser = userClient.getUserInfo(suserId);
                    headUserName.append(headUser.getRealName()).append(",");
                }
            }
            content += "，确认人由[" + dbHeadUserName + "]变更为[" + headUserName + "]";
        }
        editorlog.setContent(content);
        editorlogService.save(editorlog);
        taskW.setCompleteTime(dbTask.getCompleteTime());
        if (StringUtils.isNotBlank(taskW.getHeadId()) && "0".equals(dbTask.getStatus())) {
            taskW.setStatus("1");
        } else if (StringUtils.isBlank(taskW.getHeadId())) {
            taskW.setStatus("0");
        } else {
            taskW.setStatus(dbTask.getStatus());
        }
        baseMapper.updateById(taskW);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateById(TaskW taskW) {
        Integer id = taskW.getId();
        TaskW dbTask = baseMapper.selectById(id);
        String userID = UserTokenUtil.getTokenUserID();
        Assert.notBlank(userID, "当前操作用户不合法");
        if (!dbTask.getCreateUser().equals(userID)) {
            Integer projectId = dbTask.getProjectId();
            ProjectTeamW projectTeam = projectTeamWService.getOne(Wrappers.lambdaQuery(ProjectTeamW.class)
                    .eq(ProjectTeamW::getProjectId, projectId)
                    .like(ProjectTeamW::getAdminId, userID));
            if (Objects.isNull(projectTeam)) {
                RoleClient roleClient = SpringUtil.getBean(RoleClient.class);
                Boolean has = roleClient.getHasRoleByUserId(userID, "所有功能");
                if (!has) {
                    throw new RuntimeException("当前用户没有权限");
                }
            }
        }
        String status = taskW.getStatus();
        String headId = taskW.getHeadId();
        if (StringUtils.isNotBlank(headId)) {
            if (StringUtils.isBlank(status) || "0".equals(status)) {
                taskW.setStatus("1");
            }
        }
        TasklogW tasklogW = new TasklogW();
        tasklogW.setTaskId(taskW.getId());
        tasklogW.setSourceStatus(taskW.getStatus());
        tasklogW.setProcessingMode(taskW.getProcessingMode());
        tasklogW.setJsonData(JSON.toJSONString(taskW));
        tasklogWService.save(tasklogW);
        return super.updateById(taskW);
    }


    @Override
    public TaskW queryById(Integer id) {
        return baseMapper.selectById(id);
    }

    @Override
    public List<ProjectVo> taskOverProject2(TaskW taskW, String status) {
        List<ProjectVo> projectVoList = new ArrayList<>();
        List<TaskW> taskWList = this.baseMapper.getByStatus(taskW, status);
//        final List<TaskCountVo> taskCountVos = this.baseMapper.taskOverProject(taskW, statusList);
        if (CollectionUtil.isNotEmpty(taskWList)) {
            final Map<Integer, List<TaskW>> collect
                    = taskWList.stream()
                    .peek(task -> {
                        Integer projectId = task.getProjectId();
                        if (Objects.isNull(projectId)) {
                            task.setProjectId(0);
                        }
                    }).collect(Collectors.groupingBy(TaskW::getProjectId));
            for (Integer projectId : collect.keySet()) {
                ProjectVo projectVo = new ProjectVo();
                projectVo.setProjectId(projectId);

                List<TaskW> taskCountVos1 = collect.get(projectId);
                Integer count = taskCountVos1.size();
                List<Integer> taskIdList = taskCountVos1.stream().map(TaskW::getId).collect(Collectors.toList());
                List<TasklogW> tasklogWList = tasklogWService.list(Wrappers.lambdaQuery(TasklogW.class)
                        .eq(TasklogW::getSourceStatus, "4")
                        .in(TasklogW::getTaskId, taskIdList));
                List<Integer> collect1 = tasklogWList.stream().map(TasklogW::getTaskId).distinct().collect(Collectors.toList());
                List<TaskCountVo> taskCountVos = new ArrayList<>();
                TaskCountVo vo = new TaskCountVo();
                TaskCountVo vo1 = new TaskCountVo();
                TaskCountVo vo2 = new TaskCountVo();
                vo.setStatus("全部");
                vo.setCount(count);
                vo1.setStatus("待确认");
                vo1.setCount(count - collect1.size());
                vo2.setStatus("再次确认");
                vo2.setCount(collect1.size());
                taskCountVos.add(vo);
                taskCountVos.add(vo1);
                taskCountVos.add(vo2);
                projectVo.setTaskCountVoList(taskCountVos);
                projectVoList.add(projectVo);
            }
        } else {
            List<TaskCountVo> taskCountVos = new ArrayList<>();
            ProjectVo projectVo = new ProjectVo();
            TaskCountVo vo = new TaskCountVo();
            TaskCountVo vo1 = new TaskCountVo();
            TaskCountVo vo2 = new TaskCountVo();
            vo.setStatus("全部");
            vo.setCount(0);
            vo1.setStatus("待确认");
            vo1.setCount(0);
            vo2.setStatus("再次确认");
            vo2.setCount(0);
            taskCountVos.add(vo);
            taskCountVos.add(vo1);
            taskCountVos.add(vo2);
            projectVo.setTaskCountVoList(taskCountVos);
            projectVoList.add(projectVo);
        }

        return projectVoList;
    }

    @Override
    public List<ProjectVo> taskOverProject(TaskW taskW, List<String> statusList) {
        List<ProjectVo> projectVoList = new ArrayList<>();
        final List<TaskCountVo> taskCountVos = this.baseMapper.taskOverProject(taskW, statusList);

        final Map<String, List<TaskCountVo>> collect
                = taskCountVos.stream()
                .peek(taskCountVo -> {
                    String projectId = taskCountVo.getProjectId();
                    if (StringUtils.isBlank(projectId)) {
                        taskCountVo.setProjectId("0");
                    }
                })
                .collect(Collectors.groupingBy(TaskCountVo::getProjectId));
        for (String s : collect.keySet()) {
            ProjectVo projectVo = new ProjectVo();
            projectVo.setProjectId(Integer.parseInt(s));

            List<TaskCountVo> taskCountVos1 = collect.get(s);
            Integer count = taskCountVos1.stream().map(TaskCountVo::getCount).reduce(0, Integer::sum);
            List<String> hasStatus = taskCountVos1.stream().map(TaskCountVo::getStatus).distinct().filter(StringUtils::isNotBlank).collect(Collectors.toList());

            for (String status : statusList) {
                if (hasStatus.contains(status)) {
                    continue;
                }
                TaskCountVo vo = new TaskCountVo();
                vo.setStatus(status);
                vo.setCount(0);

                vo.setProjectId(s);

                taskCountVos1.add(vo);
            }


            TaskCountVo vo = new TaskCountVo();
            vo.setStatus("全部");
            vo.setCount(count);
            taskCountVos1.add(0, vo);
            projectVo.setTaskCountVoList(taskCountVos1);

            projectVoList.add(projectVo);
        }

        return projectVoList;
    }

    @Override
    public List<String> getDistinctGroupName(TaskW taskW) {
        return baseMapper.getDistinctGroupName(taskW);
    }

    @Override
    public List<ProjectOverTaskCountVo> getProjectOverTaskCountVo(TaskW taskW) {
        List<Integer> selfProjectId = projectWService.getSelfProjectId();
        if (CollectionUtil.isEmpty(selfProjectId)) {
            return new ArrayList<>();
        }
        List<Integer> headIdList = null;
        List<Integer> confirmIdList = null;
        String headId = taskW.getHeadId();
        if(StringUtils.isNotBlank(headId)){
            headIdList = projectWService.getProjectIdByUserId(headId);
        }
        String confirmId = taskW.getConfirmId();
        if(StringUtils.isNotBlank(confirmId)){
            confirmIdList = projectWService.getProjectIdByUserId(confirmId);
        }
        List<ProjectOverTaskCountVo> projectOverTaskCountVo = baseMapper.getProjectOverTaskCountVo(taskW, selfProjectId,headIdList,confirmIdList);
        Integer count = projectOverTaskCountVo.stream()
                .map(ProjectOverTaskCountVo::getCount)
                .reduce(0, Integer::sum);
        ProjectOverTaskCountVo vo = new ProjectOverTaskCountVo();
        vo.setCount(count);
        vo.setStatus("全部");
        projectOverTaskCountVo.add(0, vo);
        return projectOverTaskCountVo;
    }

    @Override
    @Transactional
    @Synchronized
    public void taskHandle(TaskHandleDto taskHandleDto) {
        /*
        1.处理原本的任务信息  修改状态

        2.生成tasklog数据
         */
        final String taskId = taskHandleDto.getTaskId();

        final TaskW taskW = this.getById(Integer.valueOf(taskId));
        //提交
        if (HandleEnum.COMMIT.getType().equals(taskHandleDto.getType())) {

            generateLogData(HandleEnum.COMMIT, taskW, taskHandleDto);
            //确认
        } else if (HandleEnum.CONFIRM.getType().equals(taskHandleDto.getType())) {

            generateLogData(HandleEnum.CONFIRM, taskW, taskHandleDto);
            //驳回
        } else if (HandleEnum.REJECT.getType().equals(taskHandleDto.getType())) {

            generateLogData(HandleEnum.REJECT, taskW, taskHandleDto);
        }
    }

    @Override
    public void taskHandleBatch(TaskHandleBatchDto taskHandleBatchDto) {
        final List<TaskW> datas = taskHandleBatchDto.getDatas();
        for (TaskW data : datas) {
            handleSingleTask(data);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void handleSingleTask(TaskW taskW) {
        Integer taskId = taskW.getId();
        TaskW dbTask = this.getById(taskId);
        if (StringUtils.isBlank(taskW.getHeadId())) {
            taskW.setHeadId(dbTask.getHeadId());
        }
        if (StringUtils.isBlank(taskW.getConfirmId())) {
            taskW.setConfirmId(dbTask.getConfirmId());
        }
        if (StringUtils.isBlank(taskW.getCareId())) {
            taskW.setCareId(dbTask.getCareId());
        }

        String processingMode = taskW.getProcessingMode();
        if (Objects.equals(dbTask.getStatus(), "1") || Objects.equals(dbTask.getStatus(), "4")) {
            Assert.notBlank(processingMode, "处理方式不能为空");
            // 根据处理方式 处理当前任务
            handleTaskByProcessingMode(taskW);
        } else if (Objects.equals(dbTask.getStatus(), "2")) {
            Assert.notBlank(processingMode, "验证结果不能为空");
            //根据验证的结果处理当前任务
            taskResultByProcessingMode(taskW);
        }
    }

    @Override
    public List<GanttData> getGanttData(GanttData data) {
        String deptId = data.getDeptId();
        List<String> orgIdList = null;
        if (StringUtils.isNotBlank(deptId)) {
            OrgClient orgClient = SpringUtil.getBean(OrgClient.class);
            orgIdList = orgClient.getChildrenId(deptId);
        }
        String startDate = data.getStartDate();
        String endDate = data.getEndDate();
        String status = data.getStatus();
        String headerId = data.getHeaderId();
        List<GanttData> result = baseMapper.getGanttData(startDate, endDate, status, headerId, orgIdList);
        if (CollectionUtil.isNotEmpty(result)) {
            for (GanttData ganttData : result) {
                String start = ganttData.getStartDate();
                String end = ganttData.getEndDate();
                Date complete = ganttData.getCompleteTime();
                if (StringUtils.isNotBlank(start)
                        && StringUtils.isNotBlank(end)
                        && Objects.nonNull(complete)) {
                    Date date1 = DateUtil.parse(start, "yyyy-MM-dd");
                    Date date2 = DateUtil.parse(end, "yyyy-MM-dd");
                    long between = DateUtil.between(date1, complete, DateUnit.DAY);
                    ganttData.setDuration(String.valueOf(between));
                    long between2 = DateUtil.between(date1, date2, DateUnit.DAY);
                    ganttData.setProgress((double) between / between2);
                }
            }
        }
        return result;
    }

    @Override
    public List<ProjectOverTaskCountVo> countGanttData(GanttData data) {
        String deptId = data.getDeptId();
        List<String> orgIdList = null;
        if (StringUtils.isNotBlank(deptId)) {
            OrgClient orgClient = SpringUtil.getBean(OrgClient.class);
            orgIdList = orgClient.getChildrenId(deptId);
        }
        String startDate = data.getStartDate();
        String endDate = data.getEndDate();
        String status = data.getStatus();
        String headerId = data.getHeaderId();
        return baseMapper.getCountMap(startDate, endDate, status, headerId, orgIdList);
    }


    private String removeUserId(String resource, String userId) {
        Assert.notBlank(resource, "原始字符串不能为空");
        Assert.notBlank(userId, "被移除的字符串不能为空");
        if (StrUtil.contains(resource, userId)) {
            String replace = resource.replace(userId, "");
            if (StrUtil.isBlank(replace)) {
                return replace;
            } else {
                String[] split = replace.split(",");
                return Arrays.stream(split)
                        .filter(StringUtils::isNotBlank)
                        .filter(s -> !s.equals("null"))
                        .collect(Collectors.joining(","));
            }
        } else {
            return resource;
        }
    }

    private void taskResultByProcessingMode(TaskW taskW) {
        String userID = UserTokenUtil.getTokenUserID();
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        UserVo userInfo = userClient.getUserInfo(userID);
        String realName = userInfo.getRealName();
        String content = "[ " + realName + " ]";
        String processingMode = taskW.getProcessingMode();
        Integer id = taskW.getId();
        TaskW dbEntity = baseMapper.selectById(id);
        String confirmId = dbEntity.getConfirmId();
        confirmId = removeUserId(confirmId, userID);
        if (Objects.equals(processingMode, "10")) {
            //验证结果通过-->任务完成
            content = content.concat(" 验证通过");
            if (StringUtils.isBlank(confirmId)) {
                taskW.setStatus("3");
            } else {
                taskW.setStatus(dbEntity.getStatus());
            }
            taskW.setConfirmId(confirmId);
            assert userID != null;
            String alreadyConfirmId = dbEntity.getAlreadyConfirmId();
            alreadyConfirmId = StringUtils.isNotBlank(alreadyConfirmId) ? alreadyConfirmId : "";
            taskW.setAlreadyConfirmId(userID.concat(",").concat(alreadyConfirmId));
        } else if (Objects.equals(processingMode, "11")) {
            //验证结果不通过-->任务驳回
            taskW.setStatus("4");
            content = content.concat(" 任务驳回");
            taskW.setConfirmId(StringUtils.isNotBlank(dbEntity.getConfirmId()) ? dbEntity.getConfirmId() : " " + "," +
                    (StringUtils.isNotBlank(dbEntity.getAlreadyConfirmId()) ? dbEntity.getAlreadyConfirmId() : " "));
        } else if (Objects.equals(processingMode, "12")) {
            //验证结果 转派他人继续处理-->任务待确认
            taskW.setStatus("1");
            content = content.concat(" 转派他人继续处理");
            taskW.setConfirmId(StringUtils.isNotBlank(dbEntity.getConfirmId()) ? dbEntity.getConfirmId() : " " + "," +
                    (StringUtils.isNotBlank(dbEntity.getAlreadyConfirmId()) ? dbEntity.getAlreadyConfirmId() : " "));
        }
        taskW.setCompleteTime(new Date());
        baseMapper.updateById(taskW);
        //生成日志
        TasklogW tasklogW = new TasklogW();
        tasklogW.setTaskId(taskW.getId());
        tasklogW.setSourceStatus(taskW.getStatus());
        tasklogW.setJsonData(JSON.toJSONString(taskW));
        tasklogW.setHandleTime(taskW.getTime());
        tasklogW.setDescription(taskW.getDescription());
        tasklogW.setProcessingMode(taskW.getProcessingMode());
        tasklogW.setContent(content);
        tasklogWService.save(tasklogW);

    }

    private void handleTaskByProcessingMode(TaskW taskW) {
        String userID = UserTokenUtil.getTokenUserID();
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        UserVo userInfo = userClient.getUserInfo(userID);
        String realName = userInfo.getRealName();
        String content = "[ " + realName + " ]";
        String processingMode = taskW.getProcessingMode();
        if (Objects.equals(processingMode, "3")) {
            //转派他人处理
            content = content.concat(" 转派他人处理");
            taskW.setStatus("1");
            taskW.setProcessing(0);
        } else if (Objects.equals(processingMode, "4")) {
            //正在处理，不用修改状态
            log.info("正在处理，不用修改状态");
            content = content.concat(" 正在处理");
            taskW.setProcessing(1);
        } else {
            //状态设置为待确认
            content = content.concat(" 处理，待确认结果");
            if(Objects.equals(processingMode,"0")){
                content = content.concat("[ 处理完成 ]");
            } else if(Objects.equals(processingMode, "1")){
                content = content.concat("[ 无法处理 ]");
            }else if(Objects.equals(processingMode, "2")){
                content = content.concat("[ 其他 ]");
            }
            taskW.setStatus("2");
            taskW.setProcessing(0);
        }
        taskW.setCompleteTime(new Date());
        baseMapper.updateById(taskW);
        //生成日志
        TasklogW tasklogW = new TasklogW();
        tasklogW.setTaskId(taskW.getId());
        tasklogW.setSourceStatus(taskW.getStatus());
        tasklogW.setJsonData(JSON.toJSONString(taskW));
        tasklogW.setHandleTime(taskW.getTime());
        tasklogW.setDescription(taskW.getDescription());
        tasklogW.setProcessingMode(taskW.getProcessingMode());
        tasklogW.setContent(content);
        tasklogWService.save(tasklogW);
    }

    /**
     * 生成tasklog数据  并且更新 task状态
     *
     * @param handleEnum
     * @param taskW
     * @param taskHandleDto
     */
    @Transactional
    public void generateLogData(HandleEnum handleEnum, TaskW taskW, TaskHandleDto taskHandleDto) {
        //生成tasklog数据
//        TasklogW tasklogW = new TasklogW();
//        tasklogW.setSourceStatus(taskW.getStatus());
//        taskW.setStatus(handleEnum.getTargetStatus());
//        if (HandleEnum.CONFIRM.getType().equals(handleEnum.getType())) {
//            taskW.setCompleteTime(new Date());
//        }
//        this.updateById(taskW);
//
//
//        tasklogW.setTaskId(taskW.getId());
//        tasklogW.setTargetStatus(handleEnum.getTargetStatus());
//        tasklogW.setDescription(taskHandleDto.getDescription());
//        final Integer maxSequenceAddOne = tasklogWService.getMaxSequenceAddOne(Integer.valueOf(taskW.getId()));
//        tasklogW.setHandleSequence(maxSequenceAddOne);
//        tasklogW.setHandleTime(taskHandleDto.getTime());
//        tasklogWService.save(tasklogW);
    }

    @Override
    public boolean saveOrUpdateBatch(Collection<TaskW> entityList) {
        for (TaskW taskW : entityList) {
            super.saveOrUpdate(taskW);
        }
        return true;
    }

}