package com.tofly.pms.working.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.DateUtil;
import com.tofly.pms.fegin.UserClient;
import com.tofly.pms.fegin.vo.UserVo;
import com.tofly.pms.utils.UserTokenUtil;
import com.tofly.pms.working.entity.Holidays;
import com.tofly.pms.working.entity.HoursW;
import com.tofly.pms.working.entity.Workingreport;
import com.tofly.pms.working.mapper.WorkingreportMapper;
import com.tofly.pms.working.service.HolidaysService;
import com.tofly.pms.working.service.HoursWService;
import com.tofly.pms.working.service.WorkingreportService;
import com.tofly.pms.working.vo.CheckHoursVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author sheldon
 * @Description
 * @create 2024-04-16 9:52
 */
@Service
public class WorkingreportServiceImpl extends ServiceImpl<WorkingreportMapper, Workingreport> implements WorkingreportService {


    @Resource
    private HoursWService hoursWService;
    @Autowired
    private HolidaysService holidaysService;
    @Autowired
    private HttpServletResponse response;

    @Override
    public Workingreport queryById(Integer id) {
        return null;
    }

    @Override
    public List<Workingreport> statistics(Integer projectId, String deptId, String userId, String month) {
        Assert.notBlank(month, "统计的月份不能为空");
        UserClient userClient = SpringUtil.getBean(UserClient.class);
        String userID = UserTokenUtil.getTokenUserID();
        UserVo userInfo = userClient.getUserInfo(userID);
        Set<String> orgIdList = userInfo.getDepartments();
        if (CollectionUtil.isEmpty(orgIdList)) {
            return null;
        }
        List<Workingreport> workingreportList = baseMapper.selectList(Wrappers.lambdaQuery(Workingreport.class)
                .eq(Workingreport::getMonth, month)
                .in(Workingreport::getDeptId, orgIdList)
        );
        if (CollectionUtil.isNotEmpty(workingreportList)) {
            List<String> deptIdList = workingreportList.stream()
                    .map(Workingreport::getDeptId)
                    .distinct().collect(Collectors.toList());
            deptIdList.forEach(orgIdList::remove);

        }
        List<Workingreport> workingreports = statisticsByOrgIdList(orgIdList, month);
        if (CollectionUtil.isNotEmpty(workingreports)) {
            workingreportList.addAll(workingreports);
        }
        List<CheckHoursVo> hoursVoList = hoursWService.check(null, null, month);
        if(CollectionUtil.isNotEmpty(hoursVoList)){
            Map<String, Boolean> completedMap = hoursVoList.stream()
                    .collect(Collectors.toMap(CheckHoursVo::getRealName, CheckHoursVo::getIncomplete, (k1, k2) -> k1));
            for (Workingreport workingreport : workingreportList) {
                workingreport.setIncomplete(completedMap.get(workingreport.getRealName()));
            }
        }
        if (StringUtils.isNotBlank(deptId)) {
            return workingreportList.stream()
                    .filter(item -> deptId.equals(item.getDeptId()))
                    .collect(Collectors.toList());
        }
        return workingreportList;
//        if (Objects.nonNull(projectId) && StringUtils.isNotBlank(deptId) && StringUtils.isNotBlank(userId)) {
//            return workingreportList.stream()
//                    .filter(item -> projectId.equals(item.getProjectId())
//                            && deptId.equals(item.getDeptId())
//                            && userId.equals(item.getUserId())).collect(Collectors.toList());
//        } else if (Objects.nonNull(projectId) && StringUtils.isNotBlank(deptId) && StringUtils.isBlank(userId)) {
//            return workingreportList.stream()
//                    .filter(item -> projectId.equals(item.getProjectId())
//                            && deptId.equals(item.getDeptId())).collect(Collectors.toList());
//        } else if (Objects.nonNull(projectId) && StringUtils.isBlank(deptId) && StringUtils.isNotBlank(userId)) {
//            return workingreportList.stream()
//                    .filter(item -> projectId.equals(item.getProjectId())
//                            && userId.equals(item.getUserId())).collect(Collectors.toList());
//        } else if (Objects.isNull(projectId) && StringUtils.isNotBlank(deptId) && StringUtils.isNotBlank(userId)) {
//            return workingreportList.stream()
//                    .filter(item -> deptId.equals(item.getDeptId())
//                            && userId.equals(item.getUserId())).collect(Collectors.toList());
//        } else if (Objects.isNull(projectId) && StringUtils.isBlank(deptId) && StringUtils.isNotBlank(userId)) {
//            return workingreportList.stream()
//                    .filter(item -> userId.equals(item.getUserId())).collect(Collectors.toList());
//        } else if (Objects.isNull(projectId) && StringUtils.isNotBlank(deptId) && StringUtils.isBlank(userId)) {
//            return workingreportList.stream()
//                    .filter(item -> deptId.equals(item.getDeptId())
//                    ).collect(Collectors.toList());
//        } else if (Objects.nonNull(projectId) && StringUtils.isBlank(deptId) && StringUtils.isBlank(userId)) {
//            return workingreportList.stream()
//                    .filter(item -> projectId.equals(item.getProjectId())).collect(Collectors.toList());
//        } else {
//            return workingreportList;
//        }

    }

    @Override
    public void export(Integer projectId, String deptId, String userId, String month) throws IOException {
        List<Workingreport> data = statistics(projectId, deptId, userId, month);
        for (Workingreport datum : data) {
            String status = datum.getStatus();
            if (StringUtils.isBlank(status)) {
                datum.setStatus("未审核");
            }
        }
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.setCharacterEncoding("utf-8");
        // 这里URLEncoder.encode可以防止中文乱码 当然和easyexcel没有关系
        String fileName = URLEncoder.encode("工时月报", "UTF-8").replaceAll("\\+", "%20");
        response.setHeader("Content-disposition", "attachment;filename*=utf-8''" + fileName + ".xlsx");
        EasyExcel.write(response.getOutputStream(), Workingreport.class)
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .sheet("工时月报").doWrite(data);
    }

    private List<Workingreport> statisticsByOrgIdList(Set<String> orgIdList, String month) {
        if (CollectionUtil.isEmpty(orgIdList)) {
            return new ArrayList<>();
        }
        List<Workingreport> data = hoursWService.statisticsByOrgIdList(orgIdList, month);
        //当月总天数-员工请假天数-节假日天数-周末
        Calendar calendar = Calendar.getInstance();
        Date startDate = DateUtil.formatDate(month + "-01", "yyyy-MM-dd");
        calendar.setTime(startDate);
//        int daysInMonth = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
        Date endDate = DateUtil.getLastDayOfMonth(startDate);
        int daysInMonth = this.getDaysBetween(startDate, endDate);
        long restDays = holidaysService.count(Wrappers.lambdaQuery(Holidays.class)
                .eq(Holidays::getHolidayType, 1)
                .between(Holidays::getHolidayDate, startDate, endDate));
        long workDays = holidaysService.count(Wrappers.lambdaQuery(Holidays.class)
                .eq(Holidays::getHolidayType, 2)
                .between(Holidays::getHolidayDate, startDate, endDate));

        for (Workingreport datum : data) {
            String userId = datum.getUserId();
            long count = hoursWService.count(Wrappers.lambdaQuery(HoursW.class)
                    .eq(HoursW::getCreateUser, userId)
                    .eq(HoursW::getRegisterType, "6")
                    .like(HoursW::getWorkDate, month));
            datum.setWorkDays((int) (daysInMonth + workDays - restDays - count));
            Integer workHours = data.stream().filter(item -> item.getUserId().equals(userId))
                    .map(Workingreport::getProjectWorkHours)
                    .filter(Objects::nonNull)
                    .reduce(0, Integer::sum);
            datum.setWorkHours(workHours);
            if (workHours > 0 && Objects.nonNull(datum.getProjectWorkHours())) {
                datum.setProportion(String.format("%.2f", datum.getProjectWorkHours() * 100.00 / workHours));
            } else {
                datum.setProportion("0.00");
            }
        }
        return data;
    }

    /**
     * 计算两个日期之间相差的天数
     *
     * @param startDate
     * @param endDate
     * @return
     */

    private int getDaysBetween(Date startDate, Date endDate) {
        Calendar start = Calendar.getInstance();
        start.setTime(startDate);
        Calendar end = Calendar.getInstance();
        end.setTime(endDate);
        int weekdays = 0;
        while (!start.after(end)) {
            if (start.get(Calendar.DAY_OF_WEEK) != Calendar.SATURDAY && start.get(Calendar.DAY_OF_WEEK) != Calendar.SUNDAY) {
                weekdays++;
            }
            start.add(Calendar.DATE, 1);
        }

        return weekdays;
    }
}
